		TITLE	M256X.ASM
		NAME	M256X
		PAGE	55,132

.286

INCLUDE vs2.inc

MULTIPLEX_ID	EQU	0E8h
STDOUT		EQU	1
cbStack 	EQU	1024

DGROUP		GROUP	VS2_TEXT, _DATA

VS2_TEXT	SEGMENT PARA PUBLIC 'CODE'
		ASSUME	ds:DGROUP

		EXTRN	Fn00:NEAR
		EXTRN	Fn06:NEAR
		EXTRN	Fn0C:NEAR
		EXTRN	Fn0E:NEAR
		EXTRN	Fn13:NEAR
		EXTRN	Fn4F00:NEAR
		EXTRN	Fn4F01:NEAR
		EXTRN	Fn4F02:NEAR
		EXTRN	Fn4F03:NEAR
		EXTRN	Fn4F04:NEAR
		EXTRN	Fn4F05:NEAR
		EXTRN	Fn4F06:NEAR
		EXTRN	Fn4F07:NEAR
		EXTRN	Fn4F08:NEAR

; ISR10 -- interrupt 10h service routine
;
; Notes:
;   The stack-switch scheme is not re-entrant.
;
;   All VBEFn and NonVBEFn set cf if it is necessary to
;   chain to the previous int 10h handler.

		ALIGN	DWORD
ISR10		PROC	FAR

	push	ds		; save caller's DS
	push	seg DGROUP
	pop	ds		; DS -> DGROUP
	mov	Save1, bx	; save caller's BX

	cmp	ah, 4Fh 	; trap VBE functions
	je	L010

; come here for non-VBE functions
	cmp	ah, 1Ch
	jbe	L001

; come here for non-VBE functions 1Dh-FFh
	pushf
	call	PrevISR10
	pop	ds		; restore DS and return
	iret

; come here for non-VBE functions 00h-1Ch
L001:	mov	bl, ah
	xor	bh, bh
	add	bx, bx		; BX = jump table offset
	lea	bx, NonVBEFn[bx]

	mov	bx, [bx]	; BX -> function
	xchg	Save1, bx	; restore caller's BX
	call	WORD PTR Save1	; do a non-VBE function
	pop	ds
	iret

; come here for VBE functions (AH = 4Fh)
L010:	cmp	al, 8
	jbe	L011

; come here for VBE functions 09h-FFh
	pushf			; simulate an interrupt
	call	PrevVBE
	pop	ds
	iret

; come here for VBE functions 00h-08h
L011:	mov	bl, al
	xor	bh, bh
	add	bx, bx		; BX = jump table offset
	lea	bx, VBEFn[bx]

	mov	bx, [bx]	; BX -> function
	xchg	Save1, bx	; restore caller's BX
	call	WORD PTR Save1	; do a VBE function
	pop	ds
	iret

ISR10		ENDP

; ISR2F -- interrupt 2Fh service routine
;
; Caller:   AH = multiplex ID
;	    AL = function number

		ALIGN	DWORD
ISR2F		PROC	FAR

; look for this ISR's magic ID number
	cmp	ah, MULTIPLEX_ID
	je	L101

; jump to previous int 2F handler
L100::	jmp	FAR PTR L199	; (fixed up at load time)

; come here if it's our multiplex ID
L101:	or	al, al
	jnz	L199		; jump if AL <> 0

; Function 0: get installed state
;
; Notes:
;   Returns: AL = 0FFh
;	     BX = resident PSP segment
;	     CX = resident DGROUP

	dec	al		; AL = 0FFh
	mov	cx, cs		; CX = resident data seg
	mov	bx, cs:TSRPSP	; BX = PSP segment

L199:	iret

ISR2F		ENDP

; ISRVBE -- an empty VBE service routine

ISRVBE		PROC	FAR

	mov	ah, 1	    ; return AH=1 to indicate a VBE error
	iret

ISRVBE		ENDP

; FnNV -- chains directly to the previous int 10h handler

FnNV		PROC	NEAR

; chain to the previous int 10h handler
	pushf
	call	PrevISR10
	ret

FnNV		ENDP

; SwitchStack
;
; Notes:
;   This function switches to a stack in DGROUP, so DS == SS.
;
;   This function is not reentrant.

		PUBLIC	SwitchStack
SwitchStack	PROC	NEAR

	pop	SwitchReturn	    ; save return address

	mov	OldSS, ss	    ; switch to new stack
	mov	OldSP, sp

	push	ds
	pop	ss
	mov	sp, offset StackBottom

	pusha			    ; preserve these regs
	push	es
	pushf

	push	SwitchReturn	    ; push return address
	ret

SwitchStack	ENDP

; UnswitchStack

		PUBLIC	UnswitchStack
UnswitchStack	PROC	NEAR

	pop	SwitchReturn	    ; save return address

	popf
	pop	es		    ; restore registers
	popa

	mov	ss, OldSS	    ; switch back to old stack
	mov	sp, OldSP

	push	SwitchReturn
	ret

UnswitchStack	ENDP

VS2_TEXT	ENDS


_DATA		SEGMENT WORD PUBLIC 'DATA'

		EXTRN	VMLEnd:WORD

Save1		WORD	?

VBEFn		WORD	Fn4F00, Fn4F01, Fn4F02, Fn4F03
		WORD	Fn4F04, Fn4F05, Fn4F06, Fn4F07
		WORD	Fn4F08

NonVBEFn	WORD	Fn00, FnNV, FnNV, FnNV, FnNV, FnNV, Fn06, FnNV
		WORD	FnNV, FnNV, FnNV, FnNV, Fn0C, FnNV, Fn0E, FnNV
		WORD	FnNV, FnNV, FnNV, Fn13, FnNV, FnNV, FnNV, FnNV
		WORD	FnNV, FnNV, FnNV, FnNV, FnNV

TSRPSP		WORD	?

		PUBLIC	PrevISR10
PrevISR10	LABEL	DWORD
		WORD	offset ISR10
		WORD	seg VS2_TEXT

		PUBLIC	PrevVBE
PrevVBE 	LABEL	DWORD
		WORD	offset ISRVBE
		WORD	seg VS2_TEXT

PrevISR2F	LABEL	DWORD
		WORD	offset ISR2F
		WORD	seg VS2_TEXT

SwitchReturn	WORD	?

; temporary stack
		BYTE	cbStack DUP(?)
StackBottom	LABEL	BYTE
OldSS		WORD	?
OldSP		WORD	?

_DATA		ENDS

; (end of resident portion)

TRANSIENT_TEXT	SEGMENT PARA PUBLIC 'TCODE'
		ASSUME	ds:DGROUP, ss:TRANSIENT_STACK

; LoadTSR
;
; Caller:   DS,ES -> program segment prefix
;	    SS:SP -> stack

LoadTSR 	PROC	NEAR

	cld

	call	GetUninstSwitch ; CF set if uninstall
	jnc	L501

; uninstall requested
	call	GetInstState	; CF set if installed
	jnc	L500

; come here to uninstall
; (warning: this is "bare bones" code without a check for
; subsequently-installed interrupt service routines)
	push	bx		; save previous PSP
	mov	ds, cx		; CX = resident DS

	mov	al, 10h 	; unhook int 10h
	mov	si, offset PrevISR10
	call	HookISR

	mov	al, 2Fh 	; unhook int 2Fh
	mov	si, offset PrevISR2F
	call	HookISR

; go to a BIOS-supported video mode
	mov	ax, 3
	int	10h

; free memory used by the TSR
	pop	es		; ES = PSP segment
	mov	ah, 49h 	; (free allocated memory)
	int	21h

	mov	bx, offset TRANSIENT_DATA:Msg0
	call	ConsoleOut
	jmp	L509

; uninstall requested but not previously installed
L500:	mov	bx, offset TRANSIENT_DATA:Msg1
	call	ConsoleOut
	jmp	L509

; uninstall not requested
L501:	call	GetInstState
	jnc	L510

; come here if already installed
	mov	bx, offset TRANSIENT_DATA:Msg2
	call	ConsoleOut

L509:	mov	ax, 4C00h
	int	21h

; come here to install the TSR
L510:	push	seg DGROUP
	pop	ds
	mov	TSRPSP, es  ; save the PSP segment

; install ISR10
	mov	al, 10h
	mov	si, offset PrevISR10
	call	HookISR

; install ISR2F
	mov	al, 2Fh
	mov	si, offset PrevISR2F
	call	HookISR

; do the fixup at label L100 in ISR2F
	mov	si, WORD PTR PrevISR2F
	mov	WORD PTR L100+1, si
	mov	si, WORD PTR PrevISR2F+2
	mov	WORD PTR L100+3, si

; look for a previous VBE installation
	mov	ax, 4F00h	    ; VBE function 00h
	mov	di, seg TRANSIENT_DATA
	mov	es, di
	mov	di, offset TRANSIENT_DATA:TempVIB

	pushf
	call	PrevISR10

	cmp	ax, 4Fh
	jne	L515		    ; jump if no previous VBE

; come here if a previous VBE is installed
	les	bx, PrevISR10
	mov	WORD PTR PrevVBE, bx
	mov	WORD PTR PrevVBE+2, es	; PrevVBE -> previous ISR 10h

; copy video-mode numbers from the previous VBE to the resident
;  VBEINFOBLOCK data structure
	mov	cx, ds		    ; CX = seg DGROUP

	push	seg TRANSIENT_DATA
	pop	ds
	mov	si, offset TRANSIENT_DATA:TempVIB.VideoModePtr
	lds	si, [si]	    ; DS:SI -> previous video-mode list

	mov	es, cx
	mov	di, offset VMLEnd   ; ES:DI -> end of resident list

L512:	lodsw			    ; load a video-mode number
	stosw			    ; copy it to the VBE info block
	inc	ax		    ; AX=0 if the mode number = 0FFFFh
	jnz	L512

	mov	ds, cx		    ; DS -> DGROUP

; free memory allocated to the TSR's environment block
L515:	mov	es, TSRPSP
	mov	es, es:[2Ch]	; ES = environment seg
	mov	ah, 49h 	; (free allocated memory)
	int	21h

; terminate and stay resident
	mov	bx, offset TRANSIENT_DATA:Msg3
	call	ConsoleOut

	mov	dx, cs	    ; DX = paragraph address of start
			    ;  of transient portion
	sub	dx, TSRPSP  ; DX = size of resident portion
	mov	ax, 3100h   ; AH = 31h (TSR)
			    ; AL = 0 (return code)
	int	21h

LoadTSR 	ENDP

; GetUninstSwitch
;
; Caller:   DS = PSP segment
;
; Returns:  CF set if /U switch is found in the command tail

GetUninstSwitch PROC NEAR

	mov	si, 81h     ; DS:SI -> command tail

; scan for '/', '-', or CR
L550:	lodsb		    ; AL = next byte in command tail

	cmp	al, CR
	jne	L551	    ; jump if end of command tail

; CR encountered
	clc		    ; clear CF
	jmp	L553	    ; exit

L551:	cmp	al, '/'     ; look for slash ...
	je	L552

	cmp	al, '-'     ; ... and dash
	jne	L550

; slash or dash encountered, so look at the next byte
L552:	lodsb
	or	al, 20h     ; force lowercase
	cmp	al, 'u'
	jne	L550	    ; jump (ignore) if not 'u'

	stc		    ; set CF

L553:	ret

GetUninstSwitch ENDP

; GetInstState
;
; Caller:   nothing
;
; Returns:  CF set if TSR already installed
;	    BX = previous PSP segment if TSR previously installed
;	    CX = resident DGROUP if TSR previously installed

GetInstState	PROC  NEAR

	push	es	    ; preserve ES

; look for an int 2Fh handler
	mov	ax, 352Fh
	int	21h

	mov	ax, es
	or	ax, bx
	jnz	L560

; no previous int 2Fh, so this TSR cannot have been installed
	clc
	jmp	L561

; determine whether the TSR is already installed
L560:	mov	ah, MULTIPLEX_ID
	xor	al, al
	int	2Fh
	add	al, 1	    ; CF set if AL = 0FFh

L561:	pop	es	    ; restore ES
	ret

GetInstState	ENDP

;
; HookISR
;
; Caller:   AL = interrupt number
;	    DS:SI -> storage for interrupt vector (in DGROUP)
;
; Returns:  nothing

HookISR 	PROC	NEAR

	push	ds	    ; preserve DS

; save the interrupt number for the second int 21h
	push	ax

; save the current interrupt vector
	push	si
	mov	ah, 35h     ; AH = 35h (get interrupt vector)
	int	21h
	pop	si

	mov	dx, [si]    ; DX = segment of ISR to install
	mov	ax, [si+2]  ; AX = offset of ISR to install

	mov	[si], bx    ; save the offset & segment of
	mov	[si+2], es  ;  the previous ISR

; update the interrupt vector
	mov	ds, ax	    ; DS:DX -> ISR to install
	pop	ax	    ; AL = interrupt number
	mov	ah, 25h     ; AH = 25h (set interrupt vector)
	int	21h

	pop	ds	    ; restore DS
	ret

HookISR 	ENDP

;
; ConsoleOut
;
; Caller:   BX = offset of string to display
;
; Returns:  nothing

ConsoleOut	PROC	NEAR

	push	ds

	mov	ax, seg TRANSIENT_DATA
	mov	ds, ax	    ; DS:DX -> string

	mov	cl, [bx]
	xor	ch, ch	    ; CX = length of string
	lea	dx, [bx+1]  ; DS:DX -> string data
	mov	bx, STDOUT
	mov	ah, 40h     ; AH = 40h (write file or device)
	int	21h

	pop	ds

	ret

ConsoleOut	ENDP

TRANSIENT_TEXT	ENDS


TRANSIENT_DATA	SEGMENT WORD PUBLIC 'TDATA'

; messages
Msg0	BYTE	19, "M256X uninstalled", CR, LF
Msg1	BYTE	32, "M256X not previously installed", CR, LF
Msg2	BYTE	57, "M256X is already installed", CR, LF
	BYTE	    "(Use M256X /U to uninstall)", CR, LF
Msg3	BYTE	30, "M256X successfully installed", CR, LF

; temporary VBE information block (used in LoadTSR)
TempVIB VBEINFOBLOCK	< >

TRANSIENT_DATA	ENDS

TRANSIENT_STACK SEGMENT STACK 'STACK'

	BYTE	512 dup(?)

TRANSIENT_STACK ENDS

		END	LoadTSR
