		TITLE	SAVEAREA.ASM
		NAME	SAVEAREA
		PAGE	55,132

.286

INCLUDE 	vs2.inc

DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

; EstablishPSA
;
;   void VS2API EstablishPSA( void );
;
; Notes:
;   This function establishes a Parameter Save Area for the EGA
;   or VGA video BIOS.	The save area reflects the current values
;   of the attribute controller Palette and Overscan registers.

		PUBLIC	ESTABLISHPSA
ESTABLISHPSA	PROC	FAR

	enter	0, 0

	push	si
	push	di
	cld

; preserve previous SAVE_PTR
	mov	di, offset _OldSavePtr
	push	ds
	pop	es		; ES:DI -> _OldSavePtr

	mov	ax, segVDDA	; AX = 40h
	mov	ds, ax
	mov	si, 0A8h	; DS:SI = 0040:00A8 (ptr to SAVE_PTR)

	mov	cx, 2
	rep movsw		; copy the previous value of SAVE_PTR

; copy the SAVE POINTER table from ROM to RAM
	lds	si, es:_OldSavePtr  ; DS:SI -> SAVE POINTER table
	mov	di, offset _NewSPTable
	mov	cx, 7*2
	rep movsw

; update SAVE_PTR with the address of the new SAVE POINTER table
	mov	ds, ax
	mov	si, 0A8h	; DS:SI -> SAVE_PTR
	mov	WORD PTR [si], offset _NewSPTable
	mov	[si+2], es

; update the new SAVE POINTER table with the address of a
;  Parameter Save Table
	push	es
	pop	ds		; DS = DGROUP
	mov	WORD PTR _NewSPTable[4], offset _PSA
	mov	WORD PTR _NewSPTable[6], ds

	pop	di
	pop	si
	leave
	ret

ESTABLISHPSA	ENDP

; EstablishUPP
;
;   void VS2API EstablishUPP( void );
;
; Notes:
;   This function establishes a User Palette Profile Save Area
;   for the VGA video BIOS.  This save area overrides the usual
;   default palette values for a specified list of video modes.

		PUBLIC	ESTABLISHUPP
ESTABLISHUPP	PROC	FAR

	enter	0, 0

	push	si
	push	di
	cld

; preserve previous SAVE_PTR
	mov	di, offset _OldSavePtr
	push	ds
	pop	es		; ES:DI -> _OldSavePtr

	mov	ax, segVDDA	; AX = 40h
	mov	ds, ax
	mov	si, 0A8h	; DS:SI = 0040:00A8 (ptr to SAVE_PTR)

	mov	cx, 2
	rep movsw		; copy the previous value of SAVE_PTR

; copy the SAVE POINTER table from ROM to RAM
	lds	si, es:_OldSavePtr  ; DS:SI -> SAVE POINTER table
	mov	di, offset _NewSPTable
	mov	cx, 7*2
	rep movsw

; update SAVE_PTR with the address of the new SAVE POINTER table
	mov	ds, ax
	mov	si, 0A8h	; DS:SI -> SAVE_PTR
	mov	WORD PTR [si], offset _NewSPTable
	mov	[si+2], es

; copy the SECONDARY SAVE POINTER table to RAM
	lds	si, es:_NewSPTable[16]
	mov	di, offset _NewSecSPTable
	mov	cx, [si]
	rep	movsb

; store the address of the new SECONDARY SAVE POINTER table
;  in the new SAVE POINTER table
	push	es
	pop	ds		; DS = DGROUP
	mov	WORD PTR _NewSPTable[16], offset _NewSecSPTable
	mov	WORD PTR _NewSPTable[18], ds

; store the address of the User Palette Profile in the new
;  SECONDARY SAVE POINTER table
	mov	_NewSecSPTable[10], offset _UPP
	mov	_NewSecSPTable[12], ds

	pop	di
	pop	si
	leave
	ret

ESTABLISHUPP	ENDP

VS2_TEXT	ENDS

_DATA		SEGMENT WORD PUBLIC 'DATA'

		PUBLIC	_OldSavePtr, _NewSPTable, _NewSecSPTable
_OldSavePtr	DWORD	?		; previous value of SAVE_PTR
_NewSPTable	DWORD	7 DUP(?)	; copy of the SAVE POINTER table
_NewSecSPTable	WORD	?
		DWORD	6 DUP(?)

		PUBLIC	_PSA, _UPP
_PSA		BYTE	256 DUP(0)	; Parameter Save Area

_UPP		BYTE	0	; underlining flag
		BYTE	0	; (reserved)
		WORD	0	; (reserved)
		WORD	17	; number of palette and overscan regs
		WORD	0	; first register specified in the table
		WORD	PalVals ; far pointer to list of ...
		WORD	DGROUP	;  ... palette register values
		WORD	0	; number of video DAC color registers
		WORD	0	; first video DAC color register
		DWORD	0	; far pointer to video DAC color table
		BYTE	3, 0FFh ; list of applicable video modes

; a table of "custom" 16-color palette and overscan register values
PalVals 	BYTE	00h, 09h, 12h, 1Bh, 24h, 2Dh, 36h, 3Fh
		BYTE	38h, 01h, 02h, 03h, 04h, 05h, 06h, 07h
		BYTE	08h

_DATA		ENDS

		END
