/***********************************************************************

  VBEINFO.C

  Displays information about the VESA BIOS extension (VBE)

***********************************************************************/

#include    <conio.h>	    /* _getch */
#include    <stdio.h>	    /* printf */
#include    <string.h>	    /* strcpy strcmp */
#include    <vs2.h>


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    main( int, char * * );

static void InitVMode( void );


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    USHORT  usMode, usRVal;
    int     n;
    static VBEINFOBLOCK     vib;
    static MODEINFOBLOCK    mib;


    InitVMode();

    /* issue VBE function 00h (return SVGA info) */
    printf( "\nFunction 4F00h ..." );

    strcpy( vib.VbeSignature, "VBE2" );     /* request VBE v2.0 data */

    __asm
    {
      mov   ax, 4F00h
      mov   di, offset vib
      push  ds
      pop   es
      int   10h
      mov   usRVal, ax
    }

    printf( " returns AX = %04X\n", usRVal );
    if( usRVal != 0x004F )
      return;

    printf( "VbeSignature = '%4.4s'\n", vib.VbeSignature );
    printf( "VbeVersion = %04X\n", vib.VbeVersion );
    printf( "OemStringPtr -> '%Fs'\n", vib.OemStringPtr );
    printf( "Capabilities = %08lX\n", vib.Capabilities );

    for( n=0; n<100; n++ )
    {
      printf( "VideoModePtr[%d] = %04X\n", n, vib.VideoModePtr[n] );
      if( 0xFFFF == vib.VideoModePtr[n] )
	break;
    }

    if( vib.VbeVersion >= 0x102 )
      printf( "TotalMemory = %u\n", vib.TotalMemory );

    if( vib.VbeVersion >= 0x200 )
    {
      printf( "OemSoftwareRev = %04X\n", vib.OemSoftwareRev );
      printf( "OemVendorNamePtr = %08lX ('%Fs')\n",
			    vib.OemVendorNamePtr,
			    vib.OemVendorNamePtr );
      printf( "OemProductNamePtr = %08lX ('%Fs')\n",
			    vib.OemProductNamePtr,
			    vib.OemProductNamePtr );
      printf( "OemProductRevPtr = %08lX ('%Fs')\n",
			    vib.OemProductRevPtr,
			    vib.OemProductRevPtr );
    }
    _getch();

    /* issue VBE function 01h (return SVGA mode info) */
    for( n=0; n<100; n++ )
    {
      InitVMode();

      usMode = vib.VideoModePtr[n];
      if( 0xFFFF == usMode )
	break;

      printf( "\nFunction 4F01h for mode %04X ...", usMode );
      __asm
      {
	mov	ax, 4F01h
	mov	cx, usMode
	mov	di, offset mib
	push	ds
	pop	es
	int	10h
	mov	usRVal, ax
      }

      printf( " returns AX = %04X\n", usRVal );
      if( usRVal != 0x004F )
      {
	_getch();
	continue;
      }

      printf( "ModeAttributes = %04X\n", mib.ModeAttributes );
      printf( "WinAAttributes = %02X\n", mib.WinAAttributes );
      printf( "WinBAttributes = %02X\n", mib.WinBAttributes );
      printf( "WinGranularity = %04X (%d Kb)\n",
		mib.WinGranularity, mib.WinGranularity );
      printf( "WinSize = %04X (%d Kb)\n", mib.WinSize, mib.WinSize );
      printf( "WinASegment = %04X\n", mib.WinASegment );
      printf( "WinBSegment = %04X\n", mib.WinBSegment );
      printf( "WinFuncPtr = %08lX\n", mib.WinFuncPtr );
      printf( "BytesPerScanLine = %u\n", mib.BytesPerScanLine );

      if( (vib.VbeVersion >= 0x102) || (2 & mib.ModeAttributes) )
      {
	printf( "XResolution = %u\n", mib.XResolution );
	printf( "YResolution = %u\n", mib.YResolution );
	printf( "XCharSize = %02X\n", mib.XCharSize );
	printf( "YCharSize = %02X\n", mib.YCharSize );
	printf( "NumberOfPlanes = %02X\n", mib.NumberOfPlanes );
	printf( "BitsPerPixel = %02X\n", mib.BitsPerPixel );
	printf( "NumberOfBanks = %02X\n", mib.NumberOfBanks );
	printf( "MemoryModel = %02X\n", mib.MemoryModel );
	printf( "BankSize = %02X\n", mib.BankSize );

	if( vib.VbeVersion >= 0x101 )
	  printf( "NumberOfImagePages = %02X\n", mib.NumberOfImagePages );

	if( vib.VbeVersion >= 0x102 )
	{
	  printf( "\nRedMaskSize = %02X\n", mib.RedMaskSize );
	  printf( "RedFieldPosition = %02X\n", mib.RedFieldPosition );
	  printf( "GreenMaskSize = %02X\n", mib.GreenMaskSize );
	  printf( "GreenFieldPosition = %02X\n", mib.GreenFieldPosition );
	  printf( "BlueMaskSize = %02X\n", mib.BlueMaskSize );
	  printf( "BlueFieldPosition = %02X\n", mib.BlueFieldPosition );
	  printf( "RsvdMaskSize = %02X\n", mib.RsvdMaskSize );
	  printf( "RsvdFieldPosition = %02X\n", mib.RsvdFieldPosition );
	  printf( "DirectColorModeInfo = %02X\n", mib.DirectColorModeInfo );
	}
      }

      _getch();
    }

    /* restore the default 80x25 alphanumeric mode */
    SetVMode( 3 );
}



/***********************************************************************

  InitVMode

***********************************************************************/

static void InitVMode()
{
    /* set 80x25 alphanumeric mode */
    SetVMode( 3 );

    /* display 8x8 characters */
    __asm
    {
      mov   ax, 1112h	; AH = 11h (int 10h character generator)
			; AL = 12h (use 8x8 characters)
      mov   bl, 0	; use alphanumeric glyph table 0
      int   10h
    }
}
