		TITLE	SVM256.ASM
		NAME	SVM256
		PAGE	55,132

.286

INCLUDE 	vs2.inc

DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

		EXTRN	FILLVBUF256:FAR

; SetVMode256
;
;   void VS2API SetVMode256( USHORT usMode, USHORT usClear );

usClear 	EQU	[bp+6]
usMode		EQU	[bp+8]

		PUBLIC	SETVMODE256
SETVMODE256	PROC	FAR

	enter	0, 0

; get the requested video mode number
	mov	bx, usMode

; look for one of the 256-color modes
	cmp	bx, VBEModeFirst
	jb	L001
	cmp	bx, VBEModeLast
	ja	L001

	jmp	L002

; pass other video-mode requests through INT 10H
L001:	mov	al, bl
	xor	ah, ah
	int	10h
	jmp	L005

; use the mode number to select the table of parameters
L002:	push	si	    ; preserve regs
	push	di

; get a pointer to video-mode parameters
	sub	bx, VBEModeFirst    ; BX = 0 through 5
	shl	bx, 1		    ; BX = offset into pRegs table
	mov	si, offset pRegs
	add	si, bx		    ; SI -> offset of parameter table
	mov	si, [si]	    ; DS:SI -> parameter table
	cld

; disable video data display
	mov	ah, 20h
	call	VideoOnOff

; set Miscellaneous Output register
	mov	dx, 3C4h	    ; DX = 3C4h (sequencer)
	mov	ax, 0100h	    ; synchronous reset
	out	dx, ax

	mov	dl, 0C2h	    ; DX = 3C2h (misc output)
	lodsb			    ; AL = value for Misc output
	out	dx, al

; set sequencer registers
	mov	dl, 0C4h	    ; DX = 3C4h
	mov	cx, 5		    ; CX = # of registers
	rep outsw

; set CRTC registers
	mov	dl, 0D4h	    ; DX = 3D4h
	mov	cl, 1Ah 	    ; CX = # of registers
	rep outsw

; set graphics controller registers
	mov	dl, 0CEh	    ; DX = 3CEh
	mov	cl, 9		    ; CX = # of registers
	rep outsw

; set attribute controller registers
	mov	dl, 0DAh	    ; DX = 3DAh (input status reg 1)
	in	al, dx		    ; reset the attribute-
				    ;  controller flip-flop

	mov	dl, 0C0h	    ; DX = 3C0h
	mov	cx, 2*15h+1	    ; all 21 regs and PAS bit
	cli
	rep outsb
	sti

; initialize the BIOS video state parameters
	mov	di, segVDDA
	mov	es, di
	mov	di, CRT_MODE
	mov	cx, 30
	rep	movsb

; save a pointer to 8x8 or 8x16 ROM BIOS character bitmap table
	lodsw
	mov	dx, ax		    ; DL = ROWS; DH = POINTS

	mov	ax, 1123h	    ; AH = 11h (BIOS Character Load)
				    ; AL = 23h (ROM BIOS 8x8 chars)
	cmp	dh, 8
	je	L003

	inc	al		    ; AL = 24h (ROM 8x16 characters)

L003:	xor	bl, bl		    ; BL = 0 (DL = # of rows)
	int	10h

; initialize the video DAC internal lookup table
	mov	dx, 3C8h
	xor	al, al
	out	dx, al		    ; set DAC address to 0

	inc	dl		    ; DX = 3C9h
	mov	si, offset DACLUT
	mov	cx, 3*256
	cli
	rep	outsb
	sti

; clear screen if necessary
	cmp	WORD PTR usClear, 0
	je	L004

	push	0
	call	FILLVBUF256

; enable video data display
L004:	xor	ah, ah
	call	VideoOnOff

; restore registers and exit
	pop	di
	pop	si

L005:	leave
	ret	4		    ; 2*sizeof(USHORT)

SETVMODE256	ENDP

; VideoOnOff
;
; Caller:   AH = 0 (video on) or 20h (video off)

VideoOnOff	PROC	 NEAR

	push	dx		    ; preserve regs
	push	ax

; get the current value of the sequencer clocking-mode register
	mov	dx, 3C4h	    ; sequencer
	mov	al, 1		    ; Clocking Mode reg #
	out	dx, al

	inc	dx		    ; DX = 3C5h
	in	al, dx		    ; AL = current reg value

; update bit 5
	and	al, 0DFh	    ; reset bit 5
	or	ah, al		    ; update bit 5

	dec	dx		    ; DX = 3C4h
	mov	al, 1		    ; reg #
	out	dx, ax

; restore regs and return
	pop	ax
	pop	dx
	ret

VideoOnOff	ENDP

VS2_TEXT	ENDS


_DATA		SEGMENT WORD PUBLIC 'DATA'

pRegs		DW  Regs320x200
		DW  Regs320x240
		DW  Regs320x400
		DW  Regs360x240
		DW  Regs360x400
		DW  Regs360x480

Regs320x200	DB  063h  ; miscellaneous output (port 3C2h)
			  ; sequencer (port 3C4h)
		DW  02101h  ; clocking mode
		DW  00F02h  ; map mask
		DW  00003h  ; character map select
		DW  00604h  ; memory mode
		DW  00300h  ; reset
			  ; CRTC (port 3D4h)
		DW  02C11h  ; (write-protect disabled)
		DW  05F00h  ; horizontal total
		DW  04F01h  ; horizontal display enable end
		DW  05002h  ; start horizontal blanking
		DW  08203h  ; end horizontal blanking
		DW  05404h  ; start horizontal retrace pulse
		DW  08005h  ; end horizontal retrace pulse
		DW  0BF06h  ; vertical total
		DW  01F07h  ; overflow
		DW  00008h  ; preset row scan
		DW  0C009h  ; max scan line
		DW  0000Ah  ; cursor start
		DW  0000Bh  ; cursor end
		DW  0000Ch  ; start address high
		DW  0000Dh  ; start address low
		DW  0000Eh  ; cursor location high
		DW  0000Fh  ; cursor location low
		DW  09C10h  ; vertical retrace start
		DW  0AE11h  ; vertical retrace end
		DW  08F12h  ; vertical display enable end
		DW  02813h  ; offset
		DW  00014h  ; underline location
		DW  09615h  ; vertical blank start
		DW  0B916h  ; vertical blank end
		DW  0E317h  ; CRT mode control
		DW  0FF18h  ; line compare
			 ; graphics controller (3CEh)
		DW  00000h  ; set/reset
		DW  00001h  ; enable set/reset
		DW  00002h  ; color compare
		DW  00003h  ; data rotate
		DW  00004h  ; read map select
		DW  04005h  ; graphics mode
		DW  00506h  ; miscellaneous
		DW  00F07h  ; color don't care
		DW  0FF08h  ; bit mask
			 ; attribute controller (3C0h)
		DW  0000h, 0101h, 0202h, 0303h	; internal palette
		DW  0404h, 0505h, 0606h, 0707h
		DW  0808h, 0909h, 0A0Ah, 0B0Bh
		DW  0C0Ch, 0D0Dh, 0E0Eh, 0F0Fh
		DW  4110h   ; attribute mode control
		DW  0011h   ; overscan color
		DW  0F12h   ; color plane enable
		DW  0013h   ; horizontal pel panning
		DW  0014h   ; color select
		DB  20h     ; PAS bit set
			  ; BIOS video display data area
		DB  M256Mode0 ; CRT_MODE
		DW  40	    ; CRT_COLS
		DW  80*200  ; CRT_LEN
		DW  0	    ; CRT_START
		DW  8 dup(0); CURSOR_POSN
		DW  0	    ; CURSOR_MODE
		DB  0	    ; ACTIVE_PAGE
		DW  3D4h    ; ADDR_6845
		DB  0	    ; CRT_MODE_SET
		DB  0	    ; CRT_PALETTE
		DB  25	    ; # of character rows
		DB  8	    ; POINTS

Regs320x240	DB  0E3h  ; miscellaneous output (port 3C2h)
			  ; sequencer (port 3C4h)
		DW  02101h  ; clocking mode
		DW  00F02h  ; map mask
		DW  00003h  ; character map select
		DW  00604h  ; memory mode
		DW  00300h  ; reset
			  ; CRTC (port 3D4h)
		DW  02C11h  ; (write-protect disabled)
		DW  05F00h  ; horizontal total
		DW  04F01h  ; horizontal display enable end
		DW  05002h  ; start horizontal blanking
		DW  08203h  ; end horizontal blanking
		DW  05404h  ; start horizontal retrace pulse
		DW  08005h  ; end horizontal retrace pulse
		DW  00B06h  ; vertical total
		DW  03E07h  ; overflow
		DW  00008h  ; preset row scan
		DW  0C009h  ; max scan line
		DW  0000Ah  ; cursor start
		DW  0000Bh  ; cursor end
		DW  0000Ch  ; start address high
		DW  0000Dh  ; start address low
		DW  0000Eh  ; cursor location high
		DW  0000Fh  ; cursor location low
		DW  0EA10h  ; vertical retrace start
		DW  0AC11h  ; vertical retrace end
		DW  0DF12h  ; vertical display enable end
		DW  02813h  ; offset
		DW  00014h  ; underline location
		DW  0E715h  ; vertical blank start
		DW  00516h  ; vertical blank end
		DW  0E317h  ; CRT mode control
		DW  0FF18h  ; line compare
			 ; graphics controller (3CEh)
		DW  00000h  ; set/reset
		DW  00001h  ; enable set/reset
		DW  00002h  ; color compare
		DW  00003h  ; data rotate
		DW  00004h  ; read map select
		DW  04005h  ; graphics mode
		DW  00506h  ; miscellaneous
		DW  00F07h  ; color don't care
		DW  0FF08h  ; bit mask
			 ; attribute controller (3C0h)
		DW  0000h, 0101h, 0202h, 0303h	; internal palette
		DW  0404h, 0505h, 0606h, 0707h
		DW  0808h, 0909h, 0A0Ah, 0B0Bh
		DW  0C0Ch, 0D0Dh, 0E0Eh, 0F0Fh
		DW  4110h   ; attribute mode control
		DW  0011h   ; overscan color
		DW  0F12h   ; color plane enable
		DW  0013h   ; horizontal pel panning
		DW  0014h   ; color select
		DB  20h     ; PAS bit set
			  ; BIOS video display data area
		DB  M256Mode1 ; CRT_MODE
		DW  40	    ; CRT_COLS
		DW  80*240  ; CRT_LEN
		DW  0	    ; CRT_START
		DW  8 dup(0); CURSOR_POSN
		DW  0	    ; CURSOR_MODE
		DB  0	    ; ACTIVE_PAGE
		DW  3D4h    ; ADDR_6845
		DB  0	    ; CRT_MODE_SET
		DB  0	    ; CRT_PALETTE
		DB  30	    ; # of character rows
		DB  8	    ; POINTS

Regs320x400	DB  063h  ; miscellaneous output (port 3C2h)
			  ; sequencer (port 3C4h)
		DW  02101h  ; clocking mode
		DW  00F02h  ; map mask
		DW  00003h  ; character map select
		DW  00604h  ; memory mode
		DW  00300h  ; reset
			  ; CRTC (port 3D4h)
		DW  02C11h  ; (write-protect disabled)
		DW  05F00h  ; horizontal total
		DW  04F01h  ; horizontal display enable end
		DW  05002h  ; start horizontal blanking
		DW  08203h  ; end horizontal blanking
		DW  05404h  ; start horizontal retrace pulse
		DW  08005h  ; end horizontal retrace pulse
		DW  0BF06h  ; vertical total
		DW  01F07h  ; overflow
		DW  00008h  ; preset row scan
		DW  04009h  ; max scan line
		DW  0000Ah  ; cursor start
		DW  0000Bh  ; cursor end
		DW  0000Ch  ; start address high
		DW  0000Dh  ; start address low
		DW  0000Eh  ; cursor location high
		DW  0000Fh  ; cursor location low
		DW  09C10h  ; vertical retrace start
		DW  0AE11h  ; vertical retrace end
		DW  08F12h  ; vertical display enable end
		DW  02813h  ; offset
		DW  00014h  ; underline location
		DW  09615h  ; vertical blank start
		DW  0B916h  ; vertical blank end
		DW  0E317h  ; CRT mode control
		DW  0FF18h  ; line compare
			 ; graphics controller (3CEh)
		DW  00000h  ; set/reset
		DW  00001h  ; enable set/reset
		DW  00002h  ; color compare
		DW  00003h  ; data rotate
		DW  00004h  ; read map select
		DW  04005h  ; graphics mode
		DW  00506h  ; miscellaneous
		DW  00F07h  ; color don't care
		DW  0FF08h  ; bit mask
			 ; attribute controller (3C0h)
		DW  0000h, 0101h, 0202h, 0303h	; internal palette
		DW  0404h, 0505h, 0606h, 0707h
		DW  0808h, 0909h, 0A0Ah, 0B0Bh
		DW  0C0Ch, 0D0Dh, 0E0Eh, 0F0Fh
		DW  4110h   ; attribute mode control
		DW  0011h   ; overscan color
		DW  0F12h   ; color plane enable
		DW  0013h   ; horizontal pel panning
		DW  0014h   ; color select
		DB  20h     ; PAS bit set
			  ; BIOS video display data area
		DB  M256Mode2 ; CRT_MODE
		DW  40	    ; CRT_COLS
		DW  80*400  ; CRT_LEN
		DW  0	    ; CRT_START
		DW  8 dup(0); CURSOR_POSN
		DW  0	    ; CURSOR_MODE
		DB  0	    ; ACTIVE_PAGE
		DW  3D4h    ; ADDR_6845
		DB  0	    ; CRT_MODE_SET
		DB  0	    ; CRT_PALETTE
		DB  25	    ; # of character rows
		DB  16	    ; POINTS

Regs360x240	DB  0E7h  ; miscellaneous output (port 3C2h)
			  ; sequencer (port 3C4h)
		DW  02101h  ; clocking mode
		DW  00F02h  ; map mask
		DW  00003h  ; character map select
		DW  00604h  ; memory mode
		DW  00300h  ; reset
			  ; CRTC (port 3D4h)
		DW  02C11h  ; (write-protect disabled)
		DW  06B00h  ; horizontal total
		DW  05901h  ; horizontal display enable end
		DW  05A02h  ; start horizontal blanking
		DW  08E03h  ; end horizontal blanking
		DW  05E04h  ; start horizontal retrace pulse
		DW  08A05h  ; end horizontal retrace pulse
		DW  00D06h  ; vertical total
		DW  03E07h  ; overflow
		DW  00008h  ; preset row scan
		DW  0C009h  ; max scan line
		DW  0000Ah  ; cursor start
		DW  0000Bh  ; cursor end
		DW  0000Ch  ; start address high
		DW  0000Dh  ; start address low
		DW  0000Eh  ; cursor location high
		DW  0000Fh  ; cursor location low
		DW  0EA10h  ; vertical retrace start
		DW  0AC11h  ; vertical retrace end
		DW  0DF12h  ; vertical display enable end
		DW  02D13h  ; offset
		DW  00014h  ; underline location
		DW  0E715h  ; vertical blank start
		DW  00616h  ; vertical blank end
		DW  0E317h  ; CRT mode control
		DW  0FF18h  ; line compare
			 ; graphics controller (3CEh)
		DW  00000h  ; set/reset
		DW  00001h  ; enable set/reset
		DW  00002h  ; color compare
		DW  00003h  ; data rotate
		DW  00004h  ; read map select
		DW  04005h  ; graphics mode
		DW  00506h  ; miscellaneous
		DW  00F07h  ; color don't care
		DW  0FF08h  ; bit mask
			 ; attribute controller (3C0h)
		DW  0000h, 0101h, 0202h, 0303h	; internal palette
		DW  0404h, 0505h, 0606h, 0707h
		DW  0808h, 0909h, 0A0Ah, 0B0Bh
		DW  0C0Ch, 0D0Dh, 0E0Eh, 0F0Fh
		DW  4110h   ; attribute mode control
		DW  0011h   ; overscan color
		DW  0F12h   ; color plane enable
		DW  0013h   ; horizontal pel panning
		DW  0014h   ; color select
		DB  20h     ; PAS bit set
			  ; BIOS video display data area
		DB  M256Mode3 ; CRT_MODE
		DW  45	    ; CRT_COLS
		DW  90*240  ; CRT_LEN
		DW  0	    ; CRT_START
		DW  8 dup(0); CURSOR_POSN
		DW  0	    ; CURSOR_MODE
		DB  0	    ; ACTIVE_PAGE
		DW  3D4h    ; ADDR_6845
		DB  0	    ; CRT_MODE_SET
		DB  0	    ; CRT_PALETTE
		DB  30	    ; # of character rows
		DB  8	    ; POINTS

Regs360x400	DB  067h  ; miscellaneous output (port 3C2h)
			  ; sequencer (port 3C4h)
		DW  02101h  ; clocking mode
		DW  00F02h  ; map mask
		DW  00003h  ; character map select
		DW  00604h  ; memory mode
		DW  00300h  ; reset
			  ; CRTC (port 3D4h)
		DW  02C11h  ; (write-protect disabled)
		DW  06B00h  ; horizontal total
		DW  05901h  ; horizontal display enable end
		DW  05A02h  ; start horizontal blanking
		DW  08E03h  ; end horizontal blanking
		DW  05E04h  ; start horizontal retrace pulse
		DW  08A05h  ; end horizontal retrace pulse
		DW  0C106h  ; vertical total
		DW  01F07h  ; overflow
		DW  00008h  ; preset row scan
		DW  04009h  ; max scan line
		DW  0000Ah  ; cursor start
		DW  0000Bh  ; cursor end
		DW  0000Ch  ; start address high
		DW  0000Dh  ; start address low
		DW  0000Eh  ; cursor location high
		DW  0000Fh  ; cursor location low
		DW  09C10h  ; vertical retrace start
		DW  0AE11h  ; vertical retrace end
		DW  08F12h  ; vertical display enable end
		DW  02D13h  ; offset
		DW  00014h  ; underline location
		DW  09615h  ; vertical blank start
		DW  0BB16h  ; vertical blank end
		DW  0E317h  ; CRT mode control
		DW  0FF18h  ; line compare
			 ; graphics controller (3CEh)
		DW  00000h  ; set/reset
		DW  00001h  ; enable set/reset
		DW  00002h  ; color compare
		DW  00003h  ; data rotate
		DW  00004h  ; read map select
		DW  04005h  ; graphics mode
		DW  00506h  ; miscellaneous
		DW  00F07h  ; color don't care
		DW  0FF08h  ; bit mask
			 ; attribute controller (3C0h)
		DW  0000h, 0101h, 0202h, 0303h	; internal palette
		DW  0404h, 0505h, 0606h, 0707h
		DW  0808h, 0909h, 0A0Ah, 0B0Bh
		DW  0C0Ch, 0D0Dh, 0E0Eh, 0F0Fh
		DW  4110h   ; attribute mode control
		DW  0011h   ; overscan color
		DW  0F12h   ; color plane enable
		DW  0013h   ; horizontal pel panning
		DW  0014h   ; color select
		DB  20h     ; PAS bit set
			  ; BIOS video display data area
		DB  M256Mode4 ; CRT_MODE
		DW  45	    ; CRT_COLS
		DW  90*400  ; CRT_LEN
		DW  0	    ; CRT_START
		DW  8 dup(0); CURSOR_POSN
		DW  0	    ; CURSOR_MODE
		DB  0	    ; ACTIVE_PAGE
		DW  3D4h    ; ADDR_6845
		DB  0	    ; CRT_MODE_SET
		DB  0	    ; CRT_PALETTE
		DB  25	    ; # of character rows
		DB  16	    ; POINTS

Regs360x480	DB  0E7h  ; miscellaneous output (port 3C2h)
			  ; sequencer (port 3C4h)
		DW  02101h  ; clocking mode
		DW  00F02h  ; map mask
		DW  00003h  ; character map select
		DW  00604h  ; memory mode
		DW  00300h  ; reset
			  ; CRTC (port 3D4h)
		DW  02C11h  ; (write-protect disabled)
		DW  06B00h  ; horizontal total
		DW  05901h  ; horizontal display enable end
		DW  05A02h  ; start horizontal blanking
		DW  08E03h  ; end horizontal blanking
		DW  05E04h  ; start horizontal retrace pulse
		DW  08A05h  ; end horizontal retrace pulse
		DW  00D06h  ; vertical total
		DW  03E07h  ; overflow
		DW  00008h  ; preset row scan
		DW  04009h  ; max scan line
		DW  0000Ah  ; cursor start
		DW  0000Bh  ; cursor end
		DW  0000Ch  ; start address high
		DW  0000Dh  ; start address low
		DW  0000Eh  ; cursor location high
		DW  0000Fh  ; cursor location low
		DW  0EA10h  ; vertical retrace start
		DW  0AC11h  ; vertical retrace end
		DW  0DF12h  ; vertical display enable end
		DW  02D13h  ; offset
		DW  00014h  ; underline location
		DW  0E715h  ; vertical blank start
		DW  00616h  ; vertical blank end
		DW  0E317h  ; CRT mode control
		DW  0FF18h  ; line compare
			 ; graphics controller (3CEh)
		DW  00000h  ; set/reset
		DW  00001h  ; enable set/reset
		DW  00002h  ; color compare
		DW  00003h  ; data rotate
		DW  00004h  ; read map select
		DW  04005h  ; graphics mode
		DW  00506h  ; miscellaneous
		DW  00F07h  ; color don't care
		DW  0FF08h  ; bit mask
			 ; attribute controller (3C0h)
		DW  0000h, 0101h, 0202h, 0303h	; internal palette
		DW  0404h, 0505h, 0606h, 0707h
		DW  0808h, 0909h, 0A0Ah, 0B0Bh
		DW  0C0Ch, 0D0Dh, 0E0Eh, 0F0Fh
		DW  4110h   ; attribute mode control
		DW  0011h   ; overscan color
		DW  0F12h   ; color plane enable
		DW  0013h   ; horizontal pel panning
		DW  0014h   ; color select
		DB  20h     ; PAS bit set
			  ; BIOS video display data area
		DB  M256Mode5 ; CRT_MODE
		DW  45	    ; CRT_COLS
		DW  90*480  ; CRT_LEN
		DW  0	    ; CRT_START
		DW  8 dup(0); CURSOR_POSN
		DW  0	    ; CURSOR_MODE
		DB  0	    ; ACTIVE_PAGE
		DW  3D4h    ; ADDR_6845
		DB  0	    ; CRT_MODE_SET
		DB  0	    ; CRT_PALETTE
		DB  30	    ; # of character rows
		DB  16	    ; POINTS

DACLUT		DB  00h, 00h, 00h   ; 00h
		DB  00h, 00h, 2Ah   ; 01h
		DB  00h, 2Ah, 00h   ; 02h
		DB  00h, 2Ah, 2Ah   ; 03h
		DB  2Ah, 00h, 00h   ; 04h
		DB  2Ah, 00h, 2Ah   ; 05h
		DB  2Ah, 15h, 00h   ; 06h
		DB  2Ah, 2Ah, 2Ah   ; 07h
		DB  15h, 15h, 15h   ; 08h
		DB  15h, 15h, 3Fh   ; 09h
		DB  15h, 3Fh, 15h   ; 0Ah
		DB  15h, 3Fh, 3Fh   ; 0Bh
		DB  3Fh, 15h, 15h   ; 0Ch
		DB  3Fh, 15h, 3Fh   ; 0Dh
		DB  3Fh, 3Fh, 15h   ; 0Eh
		DB  3Fh, 3Fh, 3Fh   ; 0Fh

		DB  00h, 00h, 00h   ; 10h
		DB  05h, 05h, 05h   ; 11h
		DB  08h, 08h, 08h   ; 12h
		DB  0Bh, 0Bh, 0Bh   ; 13h
		DB  0Eh, 0Eh, 0Eh   ; 14h
		DB  11h, 11h, 11h   ; 15h
		DB  14h, 14h, 14h   ; 16h
		DB  18h, 18h, 18h   ; 17h
		DB  1Ch, 1Ch, 1Ch   ; 18h
		DB  20h, 20h, 20h   ; 19h
		DB  24h, 24h, 24h   ; 1Ah
		DB  28h, 28h, 28h   ; 1Bh
		DB  2Dh, 2Dh, 2Dh   ; 1Ch
		DB  32h, 32h, 32h   ; 1Dh
		DB  38h, 38h, 38h   ; 1Eh
		DB  3Fh, 3Fh, 3Fh   ; 1Fh

		DB  00h, 00h, 3Fh   ; 20h
		DB  10h, 00h, 3Fh   ; 21h
		DB  1Fh, 00h, 3Fh   ; 22h
		DB  2Fh, 00h, 3Fh   ; 23h
		DB  3Fh, 00h, 3Fh   ; 24h
		DB  3Fh, 00h, 2Fh   ; 25h
		DB  3Fh, 00h, 1Fh   ; 26h
		DB  3Fh, 00h, 10h   ; 27h
		DB  3Fh, 00h, 00h   ; 28h
		DB  3Fh, 10h, 00h   ; 29h
		DB  3Fh, 1Fh, 00h   ; 2Ah
		DB  3Fh, 2Fh, 00h   ; 2Bh
		DB  3Fh, 3Fh, 00h   ; 2Ch
		DB  2Fh, 3Fh, 00h   ; 2Dh
		DB  1Fh, 3Fh, 00h   ; 2Eh
		DB  10h, 3Fh, 00h   ; 2Fh

		DB  00h, 3Fh, 00h   ; 30h
		DB  00h, 3Fh, 10h   ; 31h
		DB  00h, 3Fh, 1Fh   ; 32h
		DB  00h, 3Fh, 2Fh   ; 33h
		DB  00h, 3Fh, 3Fh   ; 34h
		DB  00h, 2Fh, 3Fh   ; 35h
		DB  00h, 1Fh, 3Fh   ; 36h
		DB  00h, 10h, 3Fh   ; 37h
		DB  1Fh, 1Fh, 3Fh   ; 38h
		DB  27h, 1Fh, 3Fh   ; 39h
		DB  2Fh, 1Fh, 3Fh   ; 3Ah
		DB  37h, 1Fh, 3Fh   ; 3Bh
		DB  3Fh, 1Fh, 3Fh   ; 3Ch
		DB  3Fh, 1Fh, 37h   ; 3Dh
		DB  3Fh, 1Fh, 2Fh   ; 3Eh
		DB  3Fh, 1Fh, 27h   ; 3Fh

		DB  3Fh, 1Fh, 1Fh   ; 40h
		DB  3Fh, 27h, 1Fh   ; 41h
		DB  3Fh, 2Fh, 1Fh   ; 42h
		DB  3Fh, 37h, 1Fh   ; 43h
		DB  3Fh, 3Fh, 1Fh   ; 44h
		DB  37h, 3Fh, 1Fh   ; 45h
		DB  2Fh, 3Fh, 1Fh   ; 46h
		DB  27h, 3Fh, 1Fh   ; 47h
		DB  1Fh, 3Fh, 1Fh   ; 48h
		DB  1Fh, 3Fh, 27h   ; 49h
		DB  1Fh, 3Fh, 2Fh   ; 4Ah
		DB  1Fh, 3Fh, 37h   ; 4Bh
		DB  1Fh, 3Fh, 3Fh   ; 4Ch
		DB  1Fh, 37h, 3Fh   ; 4Dh
		DB  1Fh, 2Fh, 3Fh   ; 4Eh
		DB  1Fh, 27h, 3Fh   ; 4Fh

		DB  2Dh, 2Dh, 3Fh   ; 50h
		DB  31h, 2Dh, 3Fh   ; 51h
		DB  36h, 2Dh, 3Fh   ; 52h
		DB  3Ah, 2Dh, 3Fh   ; 53h
		DB  3Fh, 2Dh, 3Fh   ; 54h
		DB  3Fh, 2Dh, 3Ah   ; 55h
		DB  3Fh, 2Dh, 36h   ; 56h
		DB  3Fh, 2Dh, 31h   ; 57h
		DB  3Fh, 2Dh, 2Dh   ; 58h
		DB  3Fh, 31h, 2Dh   ; 59h
		DB  3Fh, 36h, 2Dh   ; 5Ah
		DB  3Fh, 3Ah, 2Dh   ; 5Bh
		DB  3Fh, 3Fh, 2Dh   ; 5Ch
		DB  3Ah, 3Fh, 2Dh   ; 5Dh
		DB  36h, 3Fh, 2Dh   ; 5Eh
		DB  31h, 3Fh, 2Dh   ; 5Fh

		DB  2Dh, 3Fh, 2Dh   ; 60h
		DB  2Dh, 3Fh, 31h   ; 61h
		DB  2Dh, 3Fh, 36h   ; 62h
		DB  2Dh, 3Fh, 3Ah   ; 63h
		DB  2Dh, 3Fh, 3Fh   ; 64h
		DB  2Dh, 3Ah, 3Fh   ; 65h
		DB  2Dh, 36h, 3Fh   ; 66h
		DB  2Dh, 31h, 3Fh   ; 67h
		DB  00h, 00h, 1Ch   ; 68h
		DB  07h, 00h, 1Ch   ; 69h
		DB  0Eh, 00h, 1Ch   ; 6Ah
		DB  15h, 00h, 1Ch   ; 6Bh
		DB  1Ch, 00h, 1Ch   ; 6Ch
		DB  1Ch, 00h, 15h   ; 6Dh
		DB  1Ch, 00h, 0Eh   ; 6Eh
		DB  1Ch, 00h, 07h   ; 6Fh

		DB  1Ch, 00h, 00h   ; 70h
		DB  1Ch, 07h, 00h   ; 71h
		DB  1Ch, 0Eh, 00h   ; 72h
		DB  1Ch, 15h, 00h   ; 73h
		DB  1Ch, 1Ch, 00h   ; 74h
		DB  15h, 1Ch, 00h   ; 75h
		DB  0Eh, 1Ch, 00h   ; 76h
		DB  07h, 1Ch, 00h   ; 77h
		DB  00h, 1Ch, 00h   ; 78h
		DB  00h, 1Ch, 07h   ; 79h
		DB  00h, 1Ch, 0Eh   ; 7Ah
		DB  00h, 1Ch, 15h   ; 7Bh
		DB  00h, 1Ch, 1Ch   ; 7Ch
		DB  00h, 15h, 1Ch   ; 7Dh
		DB  00h, 0Eh, 1Ch   ; 7Eh
		DB  00h, 07h, 1Ch   ; 7Fh

		DB  0Eh, 0Eh, 1Ch   ; 80h
		DB  11h, 0Eh, 1Ch   ; 81h
		DB  15h, 0Eh, 1Ch   ; 82h
		DB  18h, 0Eh, 1Ch   ; 83h
		DB  1Ch, 0Eh, 1Ch   ; 84h
		DB  1Ch, 0Eh, 18h   ; 85h
		DB  1Ch, 0Eh, 15h   ; 86h
		DB  1Ch, 0Eh, 11h   ; 87h
		DB  1Ch, 0Eh, 0Eh   ; 88h
		DB  1Ch, 11h, 0Eh   ; 89h
		DB  1Ch, 15h, 0Eh   ; 8Ah
		DB  1Ch, 18h, 0Eh   ; 8Bh
		DB  1Ch, 1Ch, 0Eh   ; 8Ch
		DB  18h, 1Ch, 0Eh   ; 8Dh
		DB  15h, 1Ch, 0Eh   ; 8Eh
		DB  11h, 1Ch, 0Eh   ; 8Fh

		DB  0Eh, 1Ch, 0Eh   ; 90h
		DB  0Eh, 1Ch, 11h   ; 91h
		DB  0Eh, 1Ch, 15h   ; 92h
		DB  0Eh, 1Ch, 18h   ; 93h
		DB  0Eh, 1Ch, 1Ch   ; 94h
		DB  0Eh, 18h, 1Ch   ; 95h
		DB  0Eh, 15h, 1Ch   ; 96h
		DB  0Eh, 11h, 1Ch   ; 97h
		DB  14h, 14h, 1Ch   ; 98h
		DB  16h, 14h, 1Ch   ; 99h
		DB  18h, 14h, 1Ch   ; 9Ah
		DB  1Ah, 14h, 1Ch   ; 9Bh
		DB  1Ch, 14h, 1Ch   ; 9Ch
		DB  1Ch, 14h, 1Ah   ; 9Dh
		DB  1Ch, 14h, 18h   ; 9Eh
		DB  1Ch, 14h, 16h   ; 9Fh

		DB  1Ch, 14h, 14h   ; A0h
		DB  1Ch, 16h, 14h   ; A1h
		DB  1Ch, 18h, 14h   ; A2h
		DB  1Ch, 1Ah, 14h   ; A3h
		DB  1Ch, 1Ch, 14h   ; A4h
		DB  1Ah, 1Ch, 14h   ; A5h
		DB  18h, 1Ch, 14h   ; A6h
		DB  16h, 1Ch, 14h   ; A7h
		DB  14h, 1Ch, 14h   ; A8h
		DB  14h, 1Ch, 16h   ; A9h
		DB  14h, 1Ch, 18h   ; AAh
		DB  14h, 1Ch, 1Ah   ; ABh
		DB  14h, 1Ch, 1Ch   ; ACh
		DB  14h, 1Ah, 1Ch   ; ADh
		DB  14h, 18h, 1Ch   ; AEh
		DB  14h, 16h, 1Ch   ; AFh

		DB  00h, 00h, 10h   ; B0h
		DB  04h, 00h, 10h   ; B1h
		DB  08h, 00h, 10h   ; B2h
		DB  0Ch, 00h, 10h   ; B3h
		DB  10h, 00h, 10h   ; B4h
		DB  10h, 00h, 0Ch   ; B5h
		DB  10h, 00h, 08h   ; B6h
		DB  10h, 00h, 04h   ; B7h
		DB  10h, 00h, 00h   ; B8h
		DB  10h, 04h, 00h   ; B9h
		DB  10h, 08h, 00h   ; BAh
		DB  10h, 0Ch, 00h   ; BBh
		DB  10h, 10h, 00h   ; BCh
		DB  0Ch, 10h, 00h   ; BDh
		DB  08h, 10h, 00h   ; BEh
		DB  04h, 10h, 00h   ; BFh

		DB  00h, 10h, 00h   ; C0h
		DB  00h, 10h, 04h   ; C1h
		DB  00h, 10h, 08h   ; C2h
		DB  00h, 10h, 0Ch   ; C3h
		DB  00h, 10h, 10h   ; C4h
		DB  00h, 0Ch, 10h   ; C5h
		DB  00h, 08h, 10h   ; C6h
		DB  00h, 04h, 10h   ; C7h
		DB  08h, 08h, 10h   ; C8h
		DB  0Ah, 08h, 10h   ; C9h
		DB  0Ch, 08h, 10h   ; CAh
		DB  0Eh, 08h, 10h   ; CBh
		DB  10h, 08h, 10h   ; CCh
		DB  10h, 08h, 0Eh   ; CDh
		DB  10h, 08h, 0Ch   ; CEh
		DB  10h, 08h, 0Ah   ; CFh

		DB  10h, 08h, 08h   ; D0h
		DB  10h, 0Ah, 08h   ; D1h
		DB  10h, 0Ch, 08h   ; D2h
		DB  10h, 0Eh, 08h   ; D3h
		DB  10h, 10h, 08h   ; D4h
		DB  0Eh, 10h, 08h   ; D5h
		DB  0Ch, 10h, 08h   ; D6h
		DB  0Ah, 10h, 08h   ; D7h
		DB  08h, 10h, 08h   ; D8h
		DB  08h, 10h, 0Ah   ; D9h
		DB  08h, 10h, 0Ch   ; DAh
		DB  08h, 10h, 0Eh   ; DBh
		DB  08h, 10h, 10h   ; DCh
		DB  08h, 0Eh, 10h   ; DDh
		DB  08h, 0Ch, 10h   ; DEh
		DB  08h, 0Ah, 10h   ; DFh

		DB  0Bh, 0Bh, 10h   ; E0h
		DB  0Ch, 0Bh, 10h   ; E1h
		DB  0Dh, 0Bh, 10h   ; E2h
		DB  0Fh, 0Bh, 10h   ; E3h
		DB  10h, 0Bh, 10h   ; E4h
		DB  10h, 0Bh, 0Fh   ; E5h
		DB  10h, 0Bh, 0Dh   ; E6h
		DB  10h, 0Bh, 0Ch   ; E7h
		DB  10h, 0Bh, 0Bh   ; E8h
		DB  10h, 0Ch, 0Bh   ; E9h
		DB  10h, 0Dh, 0Bh   ; EAh
		DB  10h, 0Fh, 0Bh   ; EBh
		DB  10h, 10h, 0Bh   ; ECh
		DB  0Fh, 10h, 0Bh   ; EDh
		DB  0Dh, 10h, 0Bh   ; EEh
		DB  0Ch, 10h, 0Bh   ; EFh

		DB  0Bh, 10h, 0Bh   ; F0h
		DB  0Bh, 10h, 0Ch   ; F1h
		DB  0Bh, 10h, 0Dh   ; F2h
		DB  0Bh, 10h, 0Fh   ; F3h
		DB  0Bh, 10h, 10h   ; F4h
		DB  0Bh, 0Fh, 10h   ; F5h
		DB  0Bh, 0Dh, 10h   ; F6h
		DB  0Bh, 0Ch, 10h   ; F7h
		DB  3Fh, 3Fh, 3Fh   ; F8h
		DB  3Fh, 3Fh, 3Fh   ; F9h
		DB  3Fh, 3Fh, 3Fh   ; FAh
		DB  3Fh, 3Fh, 3Fh   ; FBh
		DB  3Fh, 3Fh, 3Fh   ; FCh
		DB  3Fh, 3Fh, 3Fh   ; FDh
		DB  3Fh, 3Fh, 3Fh   ; FEh
		DB  3Fh, 3Fh, 3Fh   ; FFh

_DATA		ENDS

		END
