		TITLE	VSP.ASM
		NAME	VSP
		PAGE	55,132

.286

INCLUDE 	vs2.inc

DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

		EXTRN	INITVNWAIT:FAR

; BufCopy
;
;   void VS2API BufCopy( LPVOID lpTo, LPVOID lpFrom, USHORT cb );

cb		EQU	[bp+6]
lpFrom		EQU	[bp+8]
lpTo		EQU	[bp+12]

		PUBLIC	BUFCOPY
BUFCOPY 	PROC	FAR

	enter	0, 0

; preserve registers
	push	ds
	push	es
	push	si
	push	di
	push	cx
	pushf

	mov	cx, cb		; CX = # bytes to copy
	jcxz	L002

; point to the source and destination buffers
	les	di, lpTo	; ES:DI -> destination buffer
	lds	si, lpFrom	; DS:SI -> source buffer

; copy the data
	cld
	shr	cx, 1		; cf=lo bit; CX=# words to copy
	rep movsw		; copy 2 bytes at a time
	jnc	L001		; jump if no odd byte
	movsb			; copy the odd byte

; restore registers
L001:	popf
	pop	cx
	pop	di
	pop	si
	pop	es
	pop	ds

L002:	leave
	ret	10

BUFCOPY 	ENDP

; InitVSP
;
;   void VS2API InitVSP( USHORT usNewMode );

usNewMode	EQU	[bp+6]

		PUBLIC	INITVSP
INITVSP 	PROC	FAR

	enter	0, 0
	push	ax
	push	bx
	push	cx

	mov	ax, usNewMode
	or	ah, ah
	jnz	L100

; come here for 8-bit video mode numbers
	and	ax, 7Fh 	; use bits 0-7
	cmp	ax, 13h
	ja	L101		; jump if unknown

	mov	cx, offset VSP00
	jmp	L102

; come here for 16-bit video mode numbers
L100:	and	ah, 3Fh 	; use AX bits 0-13
	cmp	ax, VBEModeFirst
	jb	L101
	cmp	ax, VBEModeLast
	ja	L101

; come here for 256-color modes
	sub	ax, VBEModeFirst
	mov	cx, offset VSPX0
	jmp	L102

; come here for unrecognized mode number
L101:	mov	ax, usNewMode
	mov	VSPXX.usMode, ax
	xor	ax, ax
	mov	cx, offset VSPXX

; copy the VSP data for the specified video mode
L102:	mov	bl, sizeof VIDEOSTATEPARAMS
	mul	bl
	add	ax, cx

	push	ds	    ; push lpTo
	push	offset VSP
	push	ds	    ; push lpFrom
	push	ax
	push	sizeof VSP  ; push cb
	call	BUFCOPY

; initialize a loop timeout value for detecting vertical nondisplay
	call	INITVNWAIT

	pop	cx
	pop	bx
	pop	ax
	leave
	ret	2

INITVSP 	ENDP

VS2_TEXT	ENDS

_DATA		SEGMENT WORD PUBLIC 'DATA'

; global VSP structure
		PUBLIC	VSP, _vsp
_vsp		LABEL	BYTE		    ; C-style alias
VSP		VIDEOSTATEPARAMS    < >

; video state params for BIOS video modes
VSP00 VIDEOSTATEPARAMS < 00h, 0B800h,, 0,  80,	40,  819,  40,	25 >
VSP01 VIDEOSTATEPARAMS < 01h, 0B800h,, 0,  80,	40,  819,  40,	25 >
VSP02 VIDEOSTATEPARAMS < 02h, 0B800h,, 0, 160,	80,  409,  80,	25 >
VSP03 VIDEOSTATEPARAMS < 03h, 0B800h,, 0, 160,	80,  409,  80,	25 >
VSP04 VIDEOSTATEPARAMS < 04h, 0B800h,, 4,  80, 320,  819, 320, 200 >
VSP05 VIDEOSTATEPARAMS < 05h, 0B800h,, 4,  80, 320,  819, 320, 200 >
VSP06 VIDEOSTATEPARAMS < 06h, 0B800h,, 8,  80, 640,  819, 640, 200 >
VSP07 VIDEOSTATEPARAMS < 07h, 0B000h,, 0, 160,	80,  200,  80,	25 >

VSP08 VIDEOSTATEPARAMS < 08h,	  0h,, 0,   0,	 0,    0,   0,	 0 >
VSP09 VIDEOSTATEPARAMS < 09h,	  0h,, 0,   0,	 0,    0,   0,	 0 >
VSP0A VIDEOSTATEPARAMS < 0Ah,	  0h,, 0,   0,	 0,    0,   0,	 0 >
VSP0B VIDEOSTATEPARAMS < 0Bh,	  0h,, 0,   0,	 0,    0,   0,	 0 >
VSP0C VIDEOSTATEPARAMS < 0Ch,	  0h,, 0,   0,	 0,    0,   0,	 0 >

VSP0D VIDEOSTATEPARAMS < 0Dh,,, 8,  40, 320, 1638, 320, 200,,,,, 4 >
VSP0E VIDEOSTATEPARAMS < 0Eh,,, 8,  80, 640,  819, 640, 200,,,,, 4 >
VSP0F VIDEOSTATEPARAMS < 0Fh,,, 8,  80, 640,  819, 640, 350,,,,, 1 >
VSP10 VIDEOSTATEPARAMS < 10h,,, 8,  80, 640,  819, 640, 350,,,,, 4 >
VSP11 VIDEOSTATEPARAMS < 11h,,, 8,  80, 640,  819, 640, 480,,,,, 1 >
VSP12 VIDEOSTATEPARAMS < 12h,,, 8,  80, 640,  819, 640, 480,,,,, 4 >
VSP13 VIDEOSTATEPARAMS < 13h,,, 1, 320, 320,  204, 320, 200,,,,, 8 >

; video state parameters for 256-color modes
VSPX0 VIDEOSTATEPARAMS < VBEMode0,,, 4, 80, 320, 819, 320, 200,,,,, 8 >
VSPX1 VIDEOSTATEPARAMS < VBEMode1,,, 4, 80, 320, 819, 320, 240,,,,, 8 >
VSPX2 VIDEOSTATEPARAMS < VBEMode2,,, 4, 80, 320, 819, 320, 400,,,,, 8 >
VSPX3 VIDEOSTATEPARAMS < VBEMode3,,, 4, 90, 360, 728, 360, 240,,,,, 8 >
VSPX4 VIDEOSTATEPARAMS < VBEMode4,,, 4, 90, 360, 728, 360, 400,,,,, 8 >
VSPX5 VIDEOSTATEPARAMS < VBEMode5,,, 4, 90, 360, 728, 360, 480,,,,, 8 >

; video state parameters for other modes
VSPXX	VIDEOSTATEPARAMS  < >

_DATA		ENDS

		END
