		TITLE	LINE.ASM
		NAME	LINE
		PAGE	55,132

.286

INCLUDE 	vs2.inc

DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

		EXTRN	PIXELADDR10:FAR
		EXTRN	PIXELADDR13:FAR
		EXTRN	PIXELADDR256:FAR

; parameters for all Line functions
ucPixel 	EQU	[bp+6]
y2		EQU	[bp+8]
x2		EQU	[bp+10]
y1		EQU	[bp+12]
x1		EQU	[bp+14]

; temporary variables
LineFn		EQU	WORD PTR [bp-2]
VertIncr	EQU	WORD PTR [bp-4]
Incr2		EQU	WORD PTR [bp-6]
Incr1		EQU	WORD PTR [bp-8]     ; (used only in Line256)
IncrY		EQU	WORD PTR [bp-10]    ; (used only in Line256)

; Line10
;
;   void VS2API
;     Line10( USHORT x1, USHORT y1, USHORT x2, USHORT y2,
;						    UCHAR ucPixel );

		PUBLIC	LINE10
LINE10		PROC	FAR

	enter	6, 0

	push	si
	push	di

	mov	si, VSP.cbRow

; check for horizontal line
	mov	bx, y2
	sub	bx, y1		; BX = y2 - y1
	jnz	L000

	call	HorzLine10
	jmp	L009

; check for vertical line
L000:	mov	cx, x2
	sub	cx, x1		; CX = x2 - x1
	jnz	L001

	call	VertLine10
	jmp	L009

; come here if not horizontal or vertical
L001:	jns	L002		; jump if x2 > x1

; at this point x2 < x1, so reverse the endpoints
	neg	cx		; CX = dX
	neg	bx		; BX = dY

	mov	ax, x2		; exchange x1 and x2
	xchg	ax, x1
	mov	x2, ax

	mov	ax, y2		; exchange y1 and y2
	xchg	ax, y1
	mov	y2, ax

; at this point, dX (in register CX) is positive
L002:	or	bx, bx		; test sign of dY
	jns	L004		; jump if slope is positive

	neg	bx		; make dY positive
	neg	si		; negate the y-increment

; select the appropriate routine for the slope of line
L004:	mov	LineFn, offset LoSlopeLine10
	cmp	bx, cx
	jl	L005		; jump if dY < dX (slope < 1)

	mov	LineFn, offset HiSlopeLine10
	xchg	bx, cx		; exchange dY and dX

; calculate the initial decision variable and increments
L005:	shl	bx, 1		; BX = Incr1 = 2*dY
	mov	dx, bx
	sub	dx, cx		; DX = d = 2*dY-dX
	mov	Incr2, dx
	sub	Incr2, cx	; Incr2 = 2*(dY-dX)

; calculate the first pixel address
	push	cx		; preserve dX and d
	push	dx

	push	x1
	push	y1
	call	PIXELADDR10	; DX:AX -> buffer
	mov	es, dx
	mov	di, ax		; ES:DI -> buffer

; set up the Graphics Controller
	call	SetGC10 	; (leaves DX = 3CEh)
	mov	ax, 0B05h	; AH = 0Bh (write mode 3, read mode 1)
	out	dx, ax

	mov	ax, 7		; AH = 0 (Color Don't Care value)
	out	dx, ax

; create a bit mask for the first pixel
	mov	ax, 8008h	; AH = 80h (bit mask)
				; AL = 8 (Bit Mask reg #)
	mov	cl, x1
	and	cl, 7		; CL = x1 MOD 8
	shr	ah, cl		; AH = initial bit mask

	pop	dx		; DX = d
	pop	cx		; CX = dX
	inc	cx		; CX = # of pixels to draw

				; at this point:
				;  ES:DI -> first pixel
				;   AH = bit mask for 1st pixel
				;   BX = Incr1
				;   CX = # pixels to draw
				;   DX = decision variable
				;   SI = y-increment

	call	LineFn		; draw the line

; restore the default Graphics Controller state
L009:	mov	dx, 3CEh
	xor	ax, ax		; AH = 0, AL = 0
	out	dx, ax		; Set/Reset

	inc	ax		; AH = 0, AL = 1
	out	dx, ax		; Enable Set/Reset

	mov	al, 3		; AH = 0, AL = 3
	out	dx, ax		; Data Rotate/Func Select

	mov	al, 5		; AH = 0, AL = 5
	out	dx, ax		; Mode

	mov	ax, 0FF08h	; AH = 11111111b, AL = 8
	out	dx, ax		; Bit Mask

; restore registers and return
	pop	di
	pop	si
	leave
	ret	10		; 4*sizeof(USHORT) + sizeof(UCHAR)

LINE10		ENDP

VertLine10	PROC	NEAR

	mov	ax, y1		; AX = y1
	mov	cx, y2
	sub	cx, ax		; CX = y2-y1
	jge	L011		; jump if dY >= 0

	add	ax, cx		; AX = y2
	neg	cx		; force dY >= 0

L011:	inc	cx		; CX = # of pixels to draw
	push	cx		; preserve CX

; point to the bottommost pixel
	push	x1
	push	ax
	call	PIXELADDR10
	mov	es, dx
	mov	di, ax		; ES:DI = byte address of pixel

	call	SetGC10 	; set up Graphics Controller

	mov	ax, 8008h	; AH = 80h (bit mask)
				; AL = 8 (Bit Mask register number)
	mov	cl, x1
	and	cl, 7		; CL = x1 MOD 8
	shr	ah, cl
	out	dx, ax

	pop	cx		; CX = # of pixels

; draw the line
L012:	and	es:[di], al	; set pixel
	add	di, si		; increment to next line
	loop	L012

	ret

VertLine10	ENDP

HorzLine10	PROC	NEAR

; ensure that x1 < x2 (move left to right)
	mov	di, x1		; DI = x1
	mov	si, x2		; SI = x2
	cmp	si, di
	jae	L020

	xchg	si, di		; DI = leftmost x-coordinate

; compute the bit masks for the leftmost and rightmost pixel(s)
L020:	mov	bx, 0FF80h	; BH = FFh (left mask)
				; BL = 80h (right mask)
	mov	cx, di
	and	cl, 7		; CL = x1 MOD 8
	shr	bh, cl		; BH = left mask
	mov	cx, si
	and	cl, 7		; CL = x2 MOD 8
	sar	bl, cl		; BL = right mask

; compute the offset of the first byte in the scan line
	mov	ax, y1
	mul	VSP.cbRow	; AX = y * (bytes per row)

; compute byte offsets for x1 and x2
	shr	di, 3		; DI = byte offset of x1
	shr	si, 3		; SI = byte offset of x2

	mov	cx, si
	sub	cx, di		; CX = difference of byte offsets

; point to the first byte in the line
	add	di, ax
	add	di, VSP.ofsVBuf
	mov	es, VSP.segVBuf ; ES:DI -> first byte

	call	SetGC10 	; set up Graphics Controller

	jcxz	L023		; jump if only 1 byte in the line

; set pixels in the leftmost byte of the line
	mov	ah, bh		; AH = bit mask for 1st byte
	mov	al, 8		; AL = 8 (Bit Mask register number)
	out	dx, ax		; update Graphics Controller

	and	es:[di], ah	; update the pixels

	dec	cx
	jz	L022		; jump if only 1 byte remains

; draw the middle of the line
	mov	ah, 0FFh	; AH = bit mask
	out	dx, ax		; update Bit Mask register

L021:	inc	di		; point to the next byte
	and	es:[di], ah	; update pixels
	loop	L021

; set pixels in the rightmost byte of the line
L022:	mov	ah, bl		; AH = bit mask for last byte
	out	dx, ax		; update Graphics Controller
	and	es:[di+1], ah	; update bit planes

	ret

; come here if only one byte in the line
L023:	mov	ah, bl
	and	ah, bh		; AH = bit mask for the byte
	mov	al, 8		; AL = 8 (Bit Mask register number)
	out	dx, ax

	and	es:[di], ah	; set the pixels
	ret

HorzLine10	ENDP

LoSlopeLine10	PROC	NEAR	; come here for dY < dX (slope < 1)

	mov	al, ah		; AL = initial bit mask

L031:	or	ah, al		; mask current pixel position
	ror	al, 1		; rotate pixel value
	jc	L034		; jump if bit mask rotated to
				;  leftmost pixel position

; bit mask not rotated around
	or	dx, dx		; test sign of d
	jns	L032		; jump if d >= 0

	add	dx, bx		; d = d + Incr1
	loop	L031

; at this point, all but the last byte of pixels have been set
	and	es:[di], ah	; set remaining pixel(s)
	ret

; come here when d >= 0
L032:	add	dx, Incr2	; d = d + Incr2
	and	es:[di], ah	; update bit planes
	mov	ah, al		; AH = bit mask for next pixel
	add	di, si		; increment y
	loop	L031
	ret

; bit mask rotated around
L034:	and	es:[di], ah	; update bit planes
	mov	ah, al		; AH = bit mask for next pixel
	inc	di		; increment x

	or	dx, dx		; test sign of d
	jns	L035		; jump if non-negative

	add	dx, bx		; d = d + Incr1
	loop	L031
	ret

L035:	add	dx, Incr2	; d = d + Incr2
	add	di, si		; increment y
	loop	L031
	ret

LoSlopeLine10	ENDP

HiSlopeLine10	PROC	NEAR	; come here for dY >= dX (slope >= 1)

L041:	and	es:[di], ah	; update bit planes
	add	di, si		; increment y

L042:	or	dx, dx		; test sign of d
	jns	L043		; jump if d >= 0

	add	dx, bx		; d = d + Incr1
	loop	L041		; loop until all pixels are drawn
	ret

L043:	add	dx, Incr2	; d = d + Incr2
	ror	ah, 1		; rotate bit mask
	adc	di, 0		; increment DI if mask has rotated to
				;  leftmost pixel position
	loop	L041
	ret

HiSlopeLine10	ENDP

; SetGC10
;
; Notes:
;   Leaves DX = 3CEh (Graphics Controller I/O address)

SetGC10 	PROC	NEAR

	push	ax

	mov	dx, 3CEh	; Graphics Controller I/O addr

	mov	ah, ucPixel	; AH = pixel value
	xor	al, al		; AL = Set/Reset reg #
	out	dx, ax

	mov	ax, 0F01h	; AH = 1111b (bit plane mask for
				;  Enable Set/Reset)
	out	dx, ax		; AL = Enable Set/Reset reg #

	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Func Select reg #
	out	dx, ax

	pop	ax
	ret

SetGC10 	ENDP

; Line13
;
;   void VS2API
;     Line13( USHORT x1, USHORT y1, USHORT x2, USHORT y2,
;						    UCHAR ucPixel );

		PUBLIC	LINE13
LINE13		PROC	FAR

	enter	6, 0

	push	si
	push	di

	mov	si, VSP.cbRow

; check for horizontal line
	mov	bx, y2
	sub	bx, y1		; BX = y2 - y1
	jnz	L100

	call	HorzLine13
	jmp	L109

; check for vertical line
L100:	mov	cx, x2
	sub	cx, x1		; CX = x2 - x1
	jnz	L101

	call	VertLine13
	jmp	L109

; come here if not horizontal or vertical
L101:	jns	L102		; jump if x2 > x1

; at this point x2 < x1, so reverse the endpoints
	neg	cx		; CX = dX
	neg	bx		; BX = dY

	mov	ax, x2		; exchange x1 and x2
	xchg	ax, x1
	mov	x2, ax

	mov	ax, y2		; exchange y1 and y2
	xchg	ax, y1
	mov	y2, ax

; at this point, dX (in register CX) is positive
L102:	or	bx, bx		; test sign of dY
	jns	L104		; jump if slope is positive

	neg	bx		; make dY positive
	neg	si		; negate the y-increment

; select the appropriate routine for the slope of line
L104:	mov	LineFn, offset LoSlopeLine13
	cmp	bx, cx
	jl	L105		; jump if dY < dX (slope < 1)

	mov	LineFn, offset HiSlopeLine13
	xchg	bx, cx		; exchange dY and dX

; calculate the initial decision variable and increments
L105:	shl	bx, 1		; BX = Incr1 = 2*dY
	mov	dx, bx
	sub	dx, cx		; DX = d = 2*dY-dX
	mov	Incr2, dx
	sub	Incr2, cx	; Incr2 = 2*(dY-dX)

; calculate the first pixel address
	push	cx		; preserve dX and d
	push	dx

	push	x1
	push	y1
	call	PIXELADDR13	; DX:AX -> buffer
	mov	es, dx
	mov	di, ax		; ES:DI -> buffer

; set up the Graphics Controller
	call	SetGC13 	; (leaves DX = 3CEh, AH = pixel value)

	pop	dx		; DX = d
	pop	cx		; CX = dX
	inc	cx		; CX = # of pixels to draw

				; at this point:
				;  ES:DI -> first pixel
				;   AH = pixel value
				;   BX = Incr1
				;   CX = # pixels to draw
				;   DX = decision variable
				;   SI = y-increment

	call	LineFn		; draw the line

; restore the default Graphics Controller state
L109:	mov	dx, 3CEh
	mov	ax, 3		; AH = 0, AL = 3
	out	dx, ax		; Data Rotate/Func Select

; restore registers and return
	pop	di
	pop	si
	leave
	ret	10		; 4*sizeof(USHORT) + sizeof(UCHAR)

LINE13		ENDP

VertLine13	PROC	NEAR

	mov	ax, y1		; AX = y1
	mov	cx, y2
	sub	cx, ax		; CX = y2-y1
	jge	L111		; jump if dY >= 0

	add	ax, cx		; AX = y2
	neg	cx		; force dY >= 0

L111:	inc	cx		; CX = # of pixels to draw

; point to the bottommost pixel
	push	x1
	push	ax
	call	PIXELADDR13
	mov	es, dx
	mov	di, ax		; ES:DI = byte address of pixel

	call	SetGC13

; draw the line
L112:	mov	al, ah		; AL = pixel value
	xchg	es:[di], al
	add	di, si		; increment y
	loop	L112

	ret

VertLine13	ENDP

HorzLine13	PROC	NEAR

; ensure that x1 < x2 (move left to right)
	mov	di, x1		; DI = x1
	mov	si, x2		; SI = x2
	cmp	si, di
	jae	L120

	xchg	si, di		; DI = leftmost x-coordinate

L120:	mov	cx, si
	sub	cx, di		; CX = dX
	inc	cx		; CX = # of pixels

; point to the leftmost pixel
	push	di
	push	y1
	call	PIXELADDR13
	mov	di, ax
	mov	es, dx		; ES:DI -> first byte

	call	SetGC13
	mov	al, ah		; AL = pixel value

; draw the line
L121:	mov	ah, es:[di]	; latch the current pixel value
	stosb			; update the pixel
	loop	L121

	ret

HorzLine13	ENDP

LoSlopeLine13	PROC	NEAR	; come here for dY < dX (slope < 1)

L131:	mov	al, ah
	xchg	al, es:[di]	; update the pixel
	inc	di		; point to next pixel

	or	dx, dx		; test sign of d
	jns	L132		; jump if d >= 0

	add	dx, bx		; d = d + Incr1
	loop	L131
	ret

; come here when d >= 0
L132:	add	dx, Incr2	; d = d + Incr2
	add	di, si		; increment y
	loop	L131
	ret

LoSlopeLine13	ENDP

HiSlopeLine13	PROC	NEAR	; come here for dY >= dX (slope >= 1)

L141:	mov	al, ah
	xchg	al, es:[di]	; update the pixel
	add	di, si		; increment y

	or	dx, dx		; test sign of d
	jns	L142		; jump if d >= 0

	add	dx, bx		; d = d + Incr1
	loop	L141		; loop until all pixels are drawn
	ret

L142:	add	dx, Incr2	; d = d + Incr2
	inc	di		; point to next pixel
	loop	L141
	ret

HiSlopeLine13	ENDP

; SetGC13
;
; Notes:
;   Leaves DX = 3CEh (Graphics Controller I/O addr), AH = pixel value

SetGC13 	PROC	NEAR

	mov	dx, 3CEh	; Graphics Controller I/O addr

	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Func Select reg #
	out	dx, ax

	mov	ah, ucPixel	; AH = pixel value
	ret

SetGC13 	ENDP

; Line256
;
;   void VS2API
;     Line256( USHORT x1, USHORT y1, USHORT x2, USHORT y2,
;						    UCHAR ucPixel );

		PUBLIC	LINE256
LINE256 	PROC	FAR

	enter	10, 0

	push	si
	push	di

	mov	si, VSP.cbRow

; check for horizontal line
	mov	bx, y2
	sub	bx, y1		; BX = y2 - y1
	jnz	L200

	call	HorzLine256
	jmp	L209

; check for vertical line
L200:	mov	cx, x2
	sub	cx, x1		; CX = x2 - x1
	jnz	L201

	call	VertLine256
	jmp	L209

; come here if not horizontal or vertical
L201:	jns	L202		; jump if x2 > x1

; at this point x2 < x1, so reverse the endpoints
	neg	cx		; CX = dX
	neg	bx		; BX = dY

	mov	ax, x2		; exchange x1 and x2
	xchg	ax, x1
	mov	x2, ax

	mov	ax, y2		; exchange y1 and y2
	xchg	ax, y1
	mov	y2, ax

; at this point, dX (in register CX) is positive
L202:	or	bx, bx		; test sign of dY
	jns	L204		; jump if slope is positive

	neg	bx		; make dY positive
	neg	si		; negate the y-increment

; select the appropriate routine for the slope of line
L204:	mov	LineFn, offset LoSlopeLine256
	cmp	bx, cx
	jl	L205		; jump if dY < dX (slope < 1)

	mov	LineFn, offset HiSlopeLine256
	xchg	bx, cx		; exchange dY and dX

; calculate the initial decision variable and increments
L205:	mov	IncrY, si
	shl	bx, 1		; BX = Incr1 = 2*dY
	mov	Incr1, bx
	sub	bx, cx		; BX = d = 2*dY - dX
	mov	si, bx
	sub	bx, cx		; BX = Incr2 = 2*(dY-dX)
	mov	Incr2, bx

; calculate the first pixel address
	push	cx		; preserve dX

	push	x1
	push	y1
	call	PIXELADDR256	; DX:AX -> buffer
	mov	es, dx
	mov	di, ax		; ES:DI -> buffer

; set up the Graphics Controller
	call	SetGC256	; (leaves DX = 3C4h)
	mov	bl, ucPixel

; create a map mask for the first pixel
	mov	ax, 1102h	; AH = 11h (map mask)
				; AL = 2 (Map Mask reg #)
	mov	cl, x1
	and	cl, 3		; CL = x1 MOD 4
	shl	ah, cl		; AH = initial map mask

	pop	cx		; CX = dX
	inc	cx		; CX = # of pixels to draw

				; at this point:
				;  ES:DI -> first pixel
				;   AH = map mask for 1st pixel
				;   AL = Map Mask register number
				;   BL = pixel value
				;   CX = number of pixels to draw
				;   DX = 3C4h
				;   SI = decision variable

	call	LineFn		; draw the line

; restore the default Sequencer and Graphics Controller states
L209:	mov	ax, 0F02h	; AH = 0, AL = 2
	out	dx, ax		; Map Mask

	mov	dl, 0CEh	; DX = 3CEh
	mov	ax, 3
	out	dx, ax		; Data Rotate/Function Select

; restore registers and return
	pop	di
	pop	si
	leave
	ret	10		; 4*sizeof(USHORT) + sizeof(UCHAR)

LINE256 	ENDP

VertLine256	PROC	NEAR

	mov	ax, y1		; AX = y1
	mov	cx, y2
	sub	cx, ax		; CX = y2-y1
	jge	L211		; jump if dY >= 0

	add	ax, cx		; AX = y2
	neg	cx		; force dY >= 0

L211:	inc	cx		; CX = # of pixels to draw
	push	cx		; preserve CX

; point to the bottommost pixel
	push	x1
	push	ax
	call	PIXELADDR256
	mov	es, dx
	mov	di, ax		; ES:DI = byte address of pixel

	call	SetGC256	; set up Graphics Controller

	mov	ax, 102h	; AH = 1 (map mask)
				; AL = 2 (Map Mask register number)
	mov	cl, x1
	and	cl, 3		; CL = x1 MOD 4
	shl	ah, cl
	out	dx, ax

	pop	cx		; CX = # of pixels
	mov	ah, ucPixel	; AH = pixel value

; draw the line
L212:	mov	al, ah
	xchg	es:[di], al	; set pixel
	add	di, si		; increment to next line
	loop	L212

	ret

VertLine256	ENDP

HorzLine256	PROC	NEAR

; ensure that x1 < x2 (move left to right)
	mov	di, x1		; DI = x1
	mov	si, x2		; SI = x2
	cmp	si, di
	jae	L220

	xchg	si, di		; DI = leftmost x-coordinate

; compute the bit masks for the leftmost and rightmost pixel(s)
L220:	mov	bx, 0FE1h	; BH = 0Fh (left mask)
				; BL = E1h (right mask)
	mov	cx, di
	and	cl, 3		; CL = x1 MOD 4
	shl	bh, cl		; BH = left mask
	mov	cx, si
	and	cl, 3		; CL = x2 MOD 4
	rol	bl, cl		; BL = right mask
	and	bx, 0F0Fh	; zero bits 4-7 in BH and BL

; compute the offset of the first byte in the scan line
	mov	ax, y1
	mul	VSP.cbRow	; AX = y * (bytes per row)

; compute byte offsets for x1 and x2
	shr	di, 2		; DI = byte offset of x1
	shr	si, 2		; SI = byte offset of x2
	mov	cx, si
	sub	cx, di		; CX = difference of byte offsets

; point to the first byte in the line
	add	di, ax
	add	di, VSP.ofsVBuf
	mov	es, VSP.segVBuf ; ES:DI -> first byte

	call	SetGC256	; set up Graphics Controller

	jcxz	L223		; jump if only 1 byte in the line

; set pixels in the leftmost byte of the line
	mov	ah, bh		; AH = map mask for 1st byte
	mov	al, 2		; AL = 2 (Map Mask register number)
	out	dx, ax		; update Sequencer

	mov	bh, ucPixel	; BH = pixel value
	mov	ah, bh
	xchg	es:[di], ah	; update the pixels

	dec	cx
	jz	L222		; jump if only 1 byte remains

; draw the middle of the line
	mov	ah, 0Fh 	; AH = map mask
	out	dx, ax		; update Map Mask register

L221:	inc	di		; point to the next byte
	mov	ah, bh		; AH = pixel value
	xchg	es:[di], ah	; update pixels
	loop	L221

; set pixels in the rightmost byte of the line
L222:	mov	ah, bl		; AH = bit mask for last byte
	out	dx, ax		; update Sequencer
	xchg	es:[di+1], bh	; update bit planes

	ret

; come here if only one byte in the line
L223:	mov	ah, bl
	and	ah, bh		; AH = bit mask for the byte
	mov	al, 2		; AL = 2 (Map Mask register number)
	out	dx, ax

	mov	ah, ucPixel
	xchg	es:[di], ah	; set the pixel(s)
	ret

HorzLine256	ENDP

LoSlopeLine256	PROC	NEAR	; come here for dY < dX (slope < 1)

	mov	bh, ah		; BH = initial map mask

L231:	or	ah, bh		; mask current pixel position
	rol	bh, 1		; rotate pixel value
	jc	L234		; jump if bit mask rotated to
				;  leftmost pixel position

; bit mask not rotated around
	or	si, si		; test sign of d
	jns	L232		; jump if d >= 0

	add	si, Incr1	; d = d + Incr1
	loop	L231

; at this point, all but the last byte of pixels have been set
	and	ah, 0Fh
	out	dx, ax
	xchg	es:[di], bl	; set remaining pixel(s)
	ret

; come here when d >= 0
L232:	add	si, Incr2	; d = d + Incr2
	and	ah, 0Fh
	out	dx, ax		; select bit planes
	mov	ah, bl		; AH = pixel value
	xchg	es:[di], ah	; update bit planes
	mov	ah, bh		; AH = map mask for next pixel
	add	di, IncrY	; increment y
	loop	L231
	ret

; bit mask rotated around
L234:	and	ah, 0Fh
	out	dx, ax		; select bit planes
	mov	ah, bl		; AH = pixel value
	xchg	es:[di], ah	; update bit planes
	mov	ah, bh		; AH = map mask for next pixel
	inc	di		; increment x

	or	si, si		; test sign of d
	jns	L235		; jump if non-negative

	add	si, Incr1	; d = d + Incr1
	loop	L231
	ret

L235:	add	si, Incr2	; d = d + Incr2
	add	di, IncrY	; increment y
	loop	L231
	ret

LoSlopeLine256	ENDP

HiSlopeLine256	PROC	NEAR	; come here for dY >= dX (slope >= 1)

	mov	bh, ah		; BH = initial map mask

L241:	mov	ah, bh		; AH = mask for current pixel
	and	ah, 0Fh
	out	dx, ax		; select bit planes
	mov	ah, bl		; AH = pixel value
	xchg	es:[di], ah	; update bit planes
	add	di, IncrY	; increment y

L242:	or	si, si		; test sign of d
	jns	L243		; jump if d >= 0

	add	si, Incr1	; d = d + Incr1
	loop	L241
	ret

L243:	add	si, Incr2	; d = d + Incr2
	rol	bh, 1		; rotate bit mask
	adc	di, 0		; increment DI if mask has rotated to
				;  leftmost pixel position
	loop	L241
	ret

HiSlopeLine256	ENDP

; SetGC256
;
; Notes:
;   Leaves DX = 3C4h (Sequencer I/O addr)

SetGC256	PROC	NEAR

	mov	dx, 3CEh	; Graphics Controller I/O addr

	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Func Select reg #
	out	dx, ax

	mov	dl, 0C4h	; DX = 3C4h
	ret

SetGC256	ENDP

VS2_TEXT	ENDS

_DATA		SEGMENT WORD PUBLIC 'DATA'

		EXTRN	VSP:VIDEOSTATEPARAMS

_DATA		ENDS

		END
