		TITLE	CIRCLE.ASM
		NAME	CIRCLE
		PAGE	55,132

.286

INCLUDE 	vs2.inc

DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

		EXTRN	PIXELADDR10:FAR
		EXTRN	PIXELADDR13:FAR
		EXTRN	PIXELADDR256:FAR

; parameters for all Circle functions
ucPixel 	EQU	[bp+6]
r		EQU	[bp+8]
yc		EQU	[bp+10]
xc		EQU	[bp+12]

; temporary variables
NNEAddr 	EQU	WORD PTR [bp-2]
NNWAddr 	EQU	WORD PTR [bp-4]
ESEAddr 	EQU	WORD PTR [bp-6]
ENEAddr 	EQU	WORD PTR [bp-8]
SSWAddr 	EQU	WORD PTR [bp-10]
SSEAddr 	EQU	WORD PTR [bp-12]
WNWAddr 	EQU	WORD PTR [bp-14]
WSWAddr 	EQU	WORD PTR [bp-16]
NNEMask 	EQU	BYTE PTR [bp-17]    ; same as SSE
NNWMask 	EQU	BYTE PTR [bp-18]    ; same as SSW
WNWMask 	EQU	BYTE PTR [bp-19]    ; same as WSW
ENEMask 	EQU	BYTE PTR [bp-20]    ; same as ESE
_d		EQU	WORD PTR [bp-22]
_x		EQU	WORD PTR [bp-24]
_y		EQU	WORD PTR [bp-26]

; Circle10
;
;   void VS2API Circle10( USHORT xc, USHORT yc,
;					USHORT r, UCHAR ucPixel );
; Notes:
;   Variable names follow the points of the compass, where the pixel
;   at (0,r) is north, (r,0) is east, etc.

		PUBLIC	CIRCLE10
CIRCLE10	PROC	FAR

	enter	26, 0
	push	si
	push	di

; set up the Graphics Controller
	mov	dx, 3CEh	; DX = Graphics Controller I/O addr
	mov	ax, 0B05h	; AL = Mode register number
				; AH = Write Mode 3 (bits 0,1)
	out	dx, ax		;	Read Mode 1 (bit 4)

	mov	ah, ucPixel	; AH = pixel value
	xor	al, al		; AL = Set/Reset reg number
	out	dx, ax

	mov	ax, 0F01h	; AH = value for Enable Set/Reset
	out	dx, ax		; AL = Enable Set/Reset reg number

	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Function Select reg
	out	dx, ax

	mov	ax, 7		; AH = 0; AL = Color Don't Care reg
	out	dx, ax

; initial buffer addresses
	push	xc
	mov	ax, yc
	add	ax, r
	push	ax
	call	PixelAddr10	; DX:AX -> pixel at (0,r)
	mov	NNEAddr, ax
	mov	NNWAddr, ax

	push	xc
	mov	ax, yc
	sub	ax, r
	push	ax
	call	PixelAddr10	; DX:AX -> pixel at (0,-r)
	mov	SSEAddr, ax
	mov	SSWAddr, ax

	mov	ax, xc
	add	ax, r
	push	ax
	push	yc
	call	PixelAddr10	; DX:AX -> pixel at (r,0)
	mov	ENEAddr, ax
	mov	ESEAddr, ax

	mov	ax, xc
	sub	ax, r
	push	ax
	push	yc
	call	PixelAddr10	; DX:AX -> pixel at (-r,0)
	mov	WNWAddr, ax
	mov	WSWAddr, ax
	mov	es, dx		; ES = video buffer segment

; initial bit masks
	mov	cl, xc
	and	cl, 7		; CL = xc MOD 8
	mov	ah, 80h
	shr	ah, cl		; AH = bit mask for first pixel
	mov	NNEMask, ah
	mov	NNWMask, ah

	mov	cl, xc
	add	cl, r
	and	cl, 7		; CL = (xc+r) MOD 8
	mov	ah, 80h
	shr	ah, cl		; AH = bit mask for first pixel
	mov	ENEMask, ah

	mov	cl, xc
	sub	cl, r
	and	cl, 7		; CL = (xc-r) MOD 8
	mov	ah, 80h
	shr	ah, cl		; AH = bit mask for first pixel
	mov	WNWMask, ah

; initial constants
	mov	bx, 0		; BX = x = 0
	mov	cx, r		; CX = y = r
	mov	si, 1
	sub	si, cx		; SI = d = 1-r

; loop from (0,r) until x > y
L010:	call	Set8Pixels10

	inc	bx		; x = x+1

	or	si, si
	jns	L011		; jump if d >= 0

; come here if d < 0
	add	si, bx
	add	si, bx		; d = d+2x
	inc	si		; d = d+2x+1

	cmp	bx, cx		; loop until x > y
	jle	L010
	jmp	L012

; come here if d >= 0
L011:	add	si, bx
	add	si, bx		; d = d+2x
	dec	cx		; y = y-1
	sub	si, cx
	sub	si, cx		; d = d+2x-2y
	inc	si		; d = d+2(x-y)+1

	cmp	bx, cx		; loop until x > y
	jle	L010

; restore the default Graphics Controller state
L012:	mov	dx, 3CEh
	xor	ax, ax		; AH = 0, AL = 0
	out	dx, ax		; Set/Reset

	inc	ax		; AH = 0, AL = 1
	out	dx, ax		; Enable Set/Reset

	mov	al, 3		; AH = 0, AL = 3
	out	dx, ax		; Data Rotate/Func Select

	mov	al, 5		; AH = 0, AL = 5
	out	dx, ax		; Mode

; restore registers and return
	pop	di
	pop	si
	leave			; 4*sizeof(USHORT) + sizeof(UCHAR)
	ret	8

CIRCLE10	ENDP

Set8Pixels10	PROC	NEAR

; set pixels at (x,y) and (x,-y)
	mov	ah, NNEMask
	mov	di, NNEAddr
	and	es:[di], ah
	mov	di, SSEAddr
	and	es:[di], ah

; set pixels at (-x,y) and (-x,-y)
	mov	ah, NNWMask
	mov	di, NNWAddr
	and	es:[di], ah
	mov	di, SSWAddr
	and	es:[di], ah

; set pixels at (y,x) and (y,-x)
	mov	ah, ENEMask
	mov	di, ENEAddr
	and	es:[di], ah
	mov	di, ESEAddr
	and	es:[di], ah

; set pixels at (-y,x) and (-y,-x)
	mov	ah, WNWMask
	mov	di, WNWAddr
	and	es:[di], ah
	mov	di, WSWAddr
	and	es:[di], ah

; update video-buffer offsets and bit masks
	mov	ax, VSP.cbRow
	or	si, si		; test sign of d
	js	L022		; jump if no y-decrement

; come here to decrement y
	sub	NNEAddr, ax
	sub	NNWAddr, ax
	add	SSEAddr, ax
	add	SSWAddr, ax

	rol	ENEMask, 1
	jnc	L021

	dec	ENEAddr 	; decrement addrs if mask rotated around
	dec	ESEAddr

L021:	ror	WNWMask, 1
	jnc	L022

	inc	WNWAddr 	; increment addrs if mask rotated around
	inc	WSWAddr

; come here to increment x
L022:	sub	WSWAddr, ax
	sub	ESEAddr, ax
	add	WNWAddr, ax
	add	ENEAddr, ax

	rol	NNWMask, 1
	jnc	L023

	dec	NNWAddr
	dec	SSWAddr

L023:	ror	NNEMask, 1
	jnc	L024

	inc	NNEAddr
	inc	SSEAddr

L024:	ret

Set8Pixels10	ENDP

; Circle13
;
;   void VS2API Circle13( USHORT xc, USHORT yc,
;					USHORT r, UCHAR ucPixel );

		PUBLIC	CIRCLE13
CIRCLE13	PROC	FAR

	enter	26, 0
	push	si
	push	di

; set Graphics Controller Data Rotate/Function Select register
	mov	dx, 3CEh	; DX = Graphics Controller I/O addr
	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Function Select reg
	out	dx, ax

; initial buffer addresses
	push	xc
	mov	ax, yc
	add	ax, r
	push	ax
	call	PixelAddr13	; DX:AX -> pixel at (0,r)
	mov	NNEAddr, ax
	mov	NNWAddr, ax

	push	xc
	mov	ax, yc
	sub	ax, r
	push	ax
	call	PixelAddr13	; DX:AX -> pixel at (0,-r)
	mov	SSEAddr, ax
	mov	SSWAddr, ax

	mov	ax, xc
	add	ax, r
	push	ax
	push	yc
	call	PixelAddr13	; DX:AX -> pixel at (r,0)
	mov	ENEAddr, ax
	mov	ESEAddr, ax

	mov	ax, xc
	sub	ax, r
	push	ax
	push	yc
	call	PixelAddr13	; DX:AX -> pixel at (-r,0)
	mov	WNWAddr, ax
	mov	WSWAddr, ax
	mov	es, dx		; ES = video buffer segment

; initial constants
	mov	bx, 0		; BX = x = 0
	mov	cx, r		; CX = y = r
	mov	si, 1
	sub	si, cx		; SI = d = 1-r

; loop from (0,r) until x > y
L110:	call	Set8Pixels13

	inc	bx		; x = x+1

	or	si, si
	jns	L111		; jump if d >= 0

; come here if d < 0
	add	si, bx
	add	si, bx		; d = d+2x
	inc	si		; d = d+2x+1

	cmp	bx, cx		; loop until x > y
	jle	L110
	jmp	L112

; come here if d >= 0
L111:	add	si, bx
	add	si, bx		; d = d+2x
	dec	cx		; y = y-1
	sub	si, cx
	sub	si, cx		; d = d+2x-2y
	inc	si		; d = d+2(x-y)+1

	cmp	bx, cx		; loop until x > y
	jle	L110

; restore the default Graphics Controller state
L112:	mov	dx, 3CEh
	mov	ax, 3		; AH = 0, AL = 3
	out	dx, ax		; Data Rotate/Func Select

; restore registers and return
	pop	di
	pop	si
	leave			; 4*sizeof(USHORT) + sizeof(UCHAR)
	ret	8

CIRCLE13	ENDP

Set8Pixels13	PROC	NEAR

	mov	ah, ucPixel	; AH = pixel value

; set pixels at (x,y) and (x,-y)
	mov	di, NNEAddr
	mov	al, ah
	xchg	es:[di], al
	mov	di, SSEAddr
	mov	al, ah
	xchg	es:[di], al

; set pixels at (-x,y) and (-x,-y)
	mov	di, NNWAddr
	mov	al, ah
	xchg	es:[di], al
	mov	di, SSWAddr
	mov	al, ah
	xchg	es:[di], al

; set pixels at (y,x) and (y,-x)
	mov	di, ENEAddr
	mov	al, ah
	xchg	es:[di], al
	mov	di, ESEAddr
	mov	al, ah
	xchg	es:[di], al

; set pixels at (-y,x) and (-y,-x)
	mov	di, WNWAddr
	mov	al, ah
	xchg	es:[di], al
	mov	di, WSWAddr
	xchg	es:[di], ah

; update video-buffer offsets
	mov	ax, VSP.cbRow
	or	si, si		; test sign of d
	js	L122		; jump if no y-decrement

; come here to decrement y
	sub	NNEAddr, ax
	sub	NNWAddr, ax
	add	SSEAddr, ax
	add	SSWAddr, ax

	dec	ENEAddr
	dec	ESEAddr
	inc	WNWAddr
	inc	WSWAddr

; come here to increment x
L122:	sub	WSWAddr, ax
	sub	ESEAddr, ax
	add	WNWAddr, ax
	add	ENEAddr, ax

	dec	NNWAddr
	dec	SSWAddr
	inc	NNEAddr
	inc	SSEAddr

	ret

Set8Pixels13	ENDP

; Circle256
;
;   void VS2API Circle256( USHORT xc, USHORT yc,
;					USHORT r, UCHAR ucPixel );

		PUBLIC	CIRCLE256
CIRCLE256	PROC	FAR

	enter	26, 0
	push	si
	push	di

; set up the Graphics Controller
	mov	dx, 3CEh	; DX = Graphics Controller I/O addr
	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Function Select reg
	out	dx, ax

; initial buffer addresses
	push	xc
	mov	ax, yc
	add	ax, r
	push	ax
	call	PixelAddr256	; DX:AX -> pixel at (0,r)
	mov	NNEAddr, ax
	mov	NNWAddr, ax

	push	xc
	mov	ax, yc
	sub	ax, r
	push	ax
	call	PixelAddr256	; DX:AX -> pixel at (0,-r)
	mov	SSEAddr, ax
	mov	SSWAddr, ax

	mov	ax, xc
	add	ax, r
	push	ax
	push	yc
	call	PixelAddr256	; DX:AX -> pixel at (r,0)
	mov	ENEAddr, ax
	mov	ESEAddr, ax

	mov	ax, xc
	sub	ax, r
	push	ax
	push	yc
	call	PixelAddr256	; DX:AX -> pixel at (-r,0)
	mov	WNWAddr, ax
	mov	WSWAddr, ax
	mov	es, dx		; ES = video buffer segment

; initial values for the Sequencer Map Mask register
	mov	cl, xc
	and	cl, 3		; CL = xc MOD 4
	mov	ah, 11h
	shl	ah, cl		; AH = initial map mask
	mov	NNEMask, ah
	mov	NNWMask, ah

	mov	cl, xc
	add	cl, r
	and	cl, 3		; CL = (xc+r) MOD 4
	mov	ah, 11h
	shl	ah, cl		; AH = initial map mask
	mov	ENEMask, ah

	mov	cl, xc
	sub	cl, r
	and	cl, 3		; CL = (xc-r) MOD 4
	mov	ah, 11h
	shl	ah, cl		; AH = bit mask for first pixel
	mov	WNWMask, ah

; initial constants
	mov	bx, 0		; BX = x = 0
	mov	cx, r		; CX = y = r
	mov	si, 1
	sub	si, cx		; SI = d = 1-r

; loop from (0,r) until x > y
L210:	call	Set8Pixels256

	inc	bx		; x = x+1

	or	si, si
	jns	L211		; jump if d >= 0

; come here if d < 0
	add	si, bx
	add	si, bx		; d = d+2x
	inc	si		; d = d+2x+1

	cmp	bx, cx		; loop until x > y
	jle	L210
	jmp	L212

; come here if d >= 0
L211:	add	si, bx
	add	si, bx		; d = d+2x
	dec	cx		; y = y-1
	sub	si, cx
	sub	si, cx		; d = d+2x-2y
	inc	si		; d = d+2(x-y)+1

	cmp	bx, cx		; loop until x > y
	jle	L210

; restore the default Sequencer and Graphics Controller states
L212:	mov	dx, 3C4h
	mov	ax, 0F02h	; AH = 0, AL = 2
	out	dx, ax		; Map Mask

	mov	dl, 0CEh	; DX = 3CEh
	mov	ax, 3
	out	dx, ax		; Data Rotate/Function Select

; restore registers and return
	pop	di
	pop	si
	leave			; 4*sizeof(USHORT) + sizeof(UCHAR)
	ret	8

CIRCLE256	ENDP

Set8Pixels256	PROC	NEAR

	push	bx

	mov	dx, 3C4h	; Sequencer I/O addr
	mov	al, 2		; Sequencer Map Mask reg #
	mov	bl, ucPixel	; BL = pixel value

; set pixels at (x,y) and (x,-y)
	mov	ah, NNEMask
	and	ah, 0Fh
	out	dx, ax		; select bit plane for current pixels
	mov	di, NNEAddr
	mov	ah, bl		; AH = pixel value
	xchg	es:[di], ah	; update NNE pixel
	mov	di, SSEAddr
	mov	ah, bl
	xchg	es:[di], ah	; update SSE pixel

; set pixels at (-x,y) and (-x,-y)
	mov	ah, NNWMask
	and	ah, 0Fh
	out	dx, ax
	mov	di, NNWAddr
	mov	ah, bl
	xchg	es:[di], ah
	mov	di, SSWAddr
	mov	ah, bl
	xchg	es:[di], ah

; set pixels at (y,x) and (y,-x)
	mov	ah, ENEMask
	and	ah, 0Fh
	out	dx, ax
	mov	di, ENEAddr
	mov	ah, bl
	xchg	es:[di], ah
	mov	di, ESEAddr
	mov	ah, bl
	xchg	es:[di], ah

; set pixels at (-y,x) and (-y,-x)
	mov	ah, WNWMask
	and	ah, 0Fh
	out	dx, ax
	mov	di, WNWAddr
	mov	ah, bl
	xchg	es:[di], ah
	mov	di, WSWAddr
	xchg	es:[di], bl

; update video-buffer offsets and bit masks
	mov	ax, VSP.cbRow
	or	si, si		; test sign of d
	js	L222		; jump if no y-decrement

; come here to decrement y
	sub	NNEAddr, ax
	sub	NNWAddr, ax
	add	SSEAddr, ax
	add	SSWAddr, ax

	ror	ENEMask, 1
	jnc	L221

	dec	ENEAddr 	; decrement addrs if mask rotated around
	dec	ESEAddr

L221:	rol	WNWMask, 1
	jnc	L222

	inc	WNWAddr 	; increment addrs if mask rotated around
	inc	WSWAddr

; come here to increment x
L222:	sub	WSWAddr, ax
	sub	ESEAddr, ax
	add	WNWAddr, ax
	add	ENEAddr, ax

	ror	NNWMask, 1
	jnc	L223

	dec	NNWAddr
	dec	SSWAddr

L223:	rol	NNEMask, 1
	jnc	L224

	inc	NNEAddr
	inc	SSEAddr

L224:	pop	bx
	ret

Set8Pixels256	ENDP

VS2_TEXT	ENDS

_DATA		SEGMENT WORD PUBLIC 'DATA'

		EXTRN	VSP:VIDEOSTATEPARAMS

_DATA		ENDS

		END
