/***********************************************************************

  FILL2.C

***********************************************************************/

#include    <stdio.h>	    /* printf sscanf */
#include    <conio.h>	    /* _getch */
#include    <vs2.h>


/***********************************************************************

  TYPEDEFS etc.

***********************************************************************/

#define QUEUESIZE   400

typedef struct tagQUEUE
{
  struct tagQUEUE * pQ;     /* pointer to opposite queue */
  short 	    d;	    /* direction (UP or DN) */
  short 	    n;	    /* number of unfilled runs in queue */
  short 	    h;	    /* index of head of queue */
  short 	    t;	    /* index of tail of queue */
  struct
  {
    USHORT	    xl;     /* leftmost pixel in run */
    USHORT	    xr;     /* rightmost pixel in run */
    short	    y;	    /* y-coordinate of run */
    short	    f;	    /* TRUE if run is filled (blocked) */
  }
    run[QUEUESIZE];
}
    QUEUE;

#define UP	    1
#define DN	   -1
#define FILLED	    TRUE
#define NOT_FILLED  FALSE
#define SCAN_UNTIL  0
#define SCAN_WHILE  1


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	      main( int, char * * );

void VS2API   RegionFill( USHORT, USHORT, UCHAR, UCHAR );
USHORT VS2API ScanLeft( USHORT, USHORT, UCHAR, int );
USHORT VS2API ScanRight( USHORT, USHORT, UCHAR, int );
USHORT	      NextBranch( USHORT, USHORT, short );
void	      AddToQueue( QUEUE *, USHORT, USHORT, short, short );
int	      InQueue( QUEUE *, USHORT, USHORT, short );
void	      ShowQueue( QUEUE *, char * );

/* defined in PATTERNS.C */
typedef void (VS2API *	FILLPROC)( USHORT, USHORT, UCHAR, UCHAR );
void	    Test1( FILLPROC );
void	    Test2( FILLPROC );
void	    Test3( FILLPROC );
void	    Test4( FILLPROC );
void	    Test5( FILLPROC );
void	    Test6( FILLPROC );
void	    Test7( FILLPROC );
void	    Test8( FILLPROC );
void	    AuxPrintf( char *, ... );
void	    InitVMode( USHORT );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

/* defined in PATTERNS.C */
extern UCHAR (VS2API * fnGetPixel)( USHORT, USHORT );
extern void (VS2API *  fnLine)( USHORT, USHORT, USHORT, USHORT, UCHAR );
extern USHORT (VS2API * fnScanLeft)( USHORT, USHORT, UCHAR, int );
extern USHORT (VS2API * fnScanRight)( USHORT, USHORT, UCHAR, int );

UCHAR	ucBorder, ucFill;
QUEUE	Qup;
QUEUE	Qdn;


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    USHORT  usMode;


    if( argc != 2 )
    {
      printf( "Syntax: FILL2 <mode #>\n" );
      return;
    }

    /* select the specified video mode */
    sscanf( argv[1], "%x", &usMode );
    InitVMode( usMode );

    Test1( RegionFill );
    _getch();
    InitVMode( usMode );

    Test2( RegionFill );
    _getch();
    InitVMode( usMode );

    Test3( RegionFill );
    _getch();
    InitVMode( usMode );

    Test4( RegionFill );
    _getch();
    InitVMode( usMode );

    Test5( RegionFill );
    _getch();
    InitVMode( usMode );

    Test6( RegionFill );
    _getch();
    InitVMode( usMode );

    Test7( RegionFill );
    _getch();
    InitVMode( usMode );

    Test8( RegionFill );
    _getch();
    SetVMode( 3 );	    /* restore 80x25 text mode */
}


/***********************************************************************

  RegionFill

***********************************************************************/

void VS2API
RegionFill( USHORT x0, USHORT y0, UCHAR ucPixelB, UCHAR ucPixelF )
{
    short   y, qp;
    short   bChangeDirection;
    USHORT  x, xl, xr, xln, xrn;
    QUEUE * Q;


    /* do nothing if the seed pixel is a border pixel */
    if( (*fnGetPixel)( x0, y0 ) == ucPixelB )
      return;

    /* save the border and fill values in global variables */
    ucBorder = ucPixelB;
    ucFill = ucPixelF;

    /* initialize the queues */
    Qup.pQ = &Qdn;		/* pointer to opposite queue */
    Qup.d = UP; 		/* direction for queue */
    Qup.h = -1;
    Qup.t = 0;

    Qdn.pQ = &Qup;
    Qdn.d = DN;
    Qdn.h = -1;
    Qdn.t = 0;

    /* put the seed run in the up queue */
    Q = &Qup;
    AddToQueue( Q,
		(*fnScanLeft)( x0, y0, ucBorder, SCAN_UNTIL ),
		(*fnScanRight)( x0, y0, ucBorder, SCAN_UNTIL ),
		y0, NOT_FILLED );

    /* main loop */
    for( ; ; )
    {
      /* if the queue is empty, switch queues */
      if( Q->n == 0 )
      {
	Q = Q->pQ;

	/* exit if the other queue is empty */
	if( Q->n == 0 )
	  break;
      }

      /* get the first non-filled run from the head of the queue */
      qp = Q->h;
      while( qp >= Q->t )
      {
	if( !Q->run[qp].f )
	{
	  y = Q->run[qp].y;
	  xl = Q->run[qp].xl;
	  xr = Q->run[qp].xr;

	  /* fill the run */
	  (*fnLine)( xl, y, xr, y, ucFill );
	  Q->run[qp].f = FILLED;
	  Q->n-- ;
	  break;
	}

	else
	  qp-- ;
      }

      /* remove previously-filled runs from the current queue */
      if( Q->d == UP )
	while( (Q->h > qp) && (Q->run[Q->h].y < (y-1)) )
	  Q->h-- ;

      else
	while( (Q->h > qp) && (Q->run[Q->h].y > (y+1)) )
	  Q->h-- ;

      /* branch in the current direction */
      xln = NextBranch( xl, xr, y+Q->d );

      while( xln != 0xFFFF )
      {
	/* determine the starting point for ScanRight */
	x = (xln > xl) ? xln : xl;

	/* add the new branch if it's not already in the queue */
	xrn = (*fnScanRight)( x, y+Q->d, ucBorder, SCAN_UNTIL );
	if( !InQueue( Q, xln, xrn, y+Q->d ) )
	  AddToQueue( Q, xln, xrn, y+Q->d, NOT_FILLED );

	/* if the new branch does not extend beyond the current run, */
	/*  look for another branch */
	if( xrn > (xr-2) )
	  break;
	else
	{
	  x = xrn+2;
	  xln = NextBranch( x, xr, y+Q->d );
	}
      }

      /* branch in the opposite direction */
      bChangeDirection = FALSE;
      xln = NextBranch( xl, xr, y-Q->d );
      while( xln != 0xFFFF )
      {
	/* determine the starting point for ScanRight */
	x = (xln > xl) ? xln : xl;

	/* add the new branch to the opposite queue if it is not */
	/*  already in the current queue */
	xrn = (*fnScanRight)( x, y-Q->d, ucBorder, SCAN_UNTIL );
	if( !InQueue( Q, xln, xrn, y-Q->d ) )
	{
	  AddToQueue( Q->pQ, xln, xrn, y-Q->d, NOT_FILLED );
	  bChangeDirection = TRUE;
	}

	/* if the new branch does not extend beyond the current run, */
	/*  look for another branch */
	if( xrn > (xr-2) )
	  break;
	else
	{
	  x = xrn+2;
	  xln = NextBranch( x, xr, y-Q->d );
	}
      }

      /* change direction if any turns were detected */
      if( bChangeDirection )
      {
	/* select the opposite queue */
	Q = Q->pQ;

	/* copy the current run to the opposite queue */
	AddToQueue( Q, xl, xr, y, FILLED );
      }
    }
}


/***********************************************************************

  ScanLeft

***********************************************************************/

USHORT VS2API ScanLeft( USHORT xl, USHORT y, UCHAR ucPixel, int f )
{
    UCHAR   v;


    if( f == SCAN_UNTIL )
    {
      /* return 0xFFFF if starting pixel is a border pixel */
      if( (*fnGetPixel)( xl, y ) == ucPixel )
	return 0xFFFF;

      do
      {
	--xl;

	/* clip */
	if( xl == 0xFFFF )
	  break;

	v = (*fnGetPixel)( xl, y );
      }
      while( v != ucPixel );
    }

    else    /* f == SCAN_WHILE */
    {
      /* return 0xFFFF if starting pixel is not a border pixel */
      if( (*fnGetPixel)( xl, y ) != ucPixel )
	return 0xFFFF;

      do
      {
	--xl;

	/* clip */
	if( xl == 0xFFFF )
	  break;

	v = (*fnGetPixel)( xl, y );
      }
      while( v == ucPixel );
    }

    return ++xl;
}


/***********************************************************************

  ScanRight

***********************************************************************/

USHORT VS2API ScanRight( USHORT xr, USHORT y, UCHAR ucPixel, int f )
{
    UCHAR   v;


    if( f == SCAN_UNTIL )
    {
      /* return 0xFFFF if starting pixel is a border pixel */
      if( (*fnGetPixel)( xr, y ) == ucPixel )
	return 0xFFFF;

      do
      {
	++xr;

	/* clip */
	if( xr == vsp.usCX )
	  break;

	v = (*fnGetPixel)( xr, y );
      }
      while( v != ucPixel );
    }

    else    /* f == SCAN_WHILE */
    {
      /* return 0xFFFF if starting pixel is not a border pixel */
      if( (*fnGetPixel)( xr, y ) != ucPixel )
	return 0xFFFF;

      do
      {
	++xr;

	/* clip */
	if( xr == vsp.usCX )
	  break;

	v = (*fnGetPixel)( xr, y );
      }
      while( v == ucPixel );
    }

    return --xr;
}


/***********************************************************************

  NextBranch

***********************************************************************/

USHORT NextBranch( USHORT xl, USHORT xr, short y )
{
    USHORT  xln;


    /* clip y */
    if( (y < 0) || (y == (short)vsp.usSY) )
      return 0xFFFF;

    /* look for a branch adjacent to the first pixel in the run */
    xln = (*fnScanLeft)( xl, y, ucBorder, SCAN_UNTIL );

    /* if the adjacent pixel is on the border, scan for the */
    /*	first non-border pixel */
    if( xln == 0xFFFF )
    {
      xln = (*fnScanRight)( xl, y, ucBorder, SCAN_WHILE );
      if( xln < xr )
	xln++ ;
      else
	xln = 0xFFFF;
    }

    return xln;
}


/***********************************************************************

  AddToQueue

***********************************************************************/

void AddToQueue( QUEUE * Q, USHORT xl, USHORT xr, short y, short f )
{
    short   qp;
    int     i;


    if( Q->d == UP )
    {
      /* find the last larger queue y-value */
      for( qp=Q->t; qp<=Q->h; qp++ )
	if( Q->run[qp].y <= y )
	  break;
    }

    else
    {
      /* find the last smaller queue y-value */
      for( qp=Q->t; qp<=Q->h; qp++ )
	if( Q->run[qp].y >= y )
	  break;
    }

    /* expand the queue if necessary */
    if( qp <= Q->h )
    {
      /* update the head pointer */
      Q->h++ ;

      /* expand up */
      for( i=Q->h; i>qp; --i )
      {
	Q->run[i].xl = Q->run[i-1].xl;
	Q->run[i].xr = Q->run[i-1].xr;
	Q->run[i].y  = Q->run[i-1].y;
	Q->run[i].f = Q->run[i-1].f;
      }
    }

    else    /* can't insert in queue; add to head */
      Q->h++ ;

    /* enter the data in the queue */
    Q->run[qp].xl = xl;
    Q->run[qp].xr = xr;
    Q->run[qp].y = y;
    Q->run[qp].f = f;

    /* update the number of unfilled runs in the queue */
    if( !f )
      Q->n++ ;
}


/***********************************************************************

  InQueue

***********************************************************************/

int InQueue( QUEUE * Q, USHORT xl, USHORT xr, short y )
{
    short   qp;
    int     bRVal = FALSE;


    if( Q->d == UP )
    {
      /* search from head to tail */
      for( qp=Q->h; qp>=0; --qp )
      {
	/* quit searching if y is not in the ordered list */
	if( Q->run[qp].y > y )
	  break;

	/* quit searching if the specified run is in the list */
	if( (Q->run[qp].y == y) &&
	    (Q->run[qp].xl == xl) &&
	    (Q->run[qp].xr == xr) )
	{
	  bRVal = TRUE;
	  break;
	}
      }
    }

    else
    {
      /* search from head to tail */
      for( qp=Q->h; qp>=0; --qp )
      {
	/* quit searching if y is not in the ordered list */
	if( Q->run[qp].y < y )
	  break;

	/* quit searching if the specified run is in the list */
	if( (Q->run[qp].y == y) &&
	    (Q->run[qp].xl == xl) &&
	    (Q->run[qp].xr == xr) )
	{
	  bRVal = TRUE;
	  break;
	}
      }
    }

    return bRVal;
}


/***********************************************************************

  ShowQueue

***********************************************************************/

void ShowQueue( QUEUE * Q, char * szComment )
{
    short   qp;


    if( szComment )
      AuxPrintf( szComment );

    for( qp=Q->t; qp<=Q->h; qp++ )
      AuxPrintf( "%c %3d: %2d %3d-%3d %d",
	(Q->d == UP) ? 'U' : 'D',
	qp, Q->run[qp].y, Q->run[qp].xl, Q->run[qp].xr, Q->run[qp].f );

    AuxPrintf( "%c: %d unfilled runs in queue ",
	(Q->d == UP) ? 'U' : 'D',
	Q->n );
}
