/***********************************************************************

  PATTERNS.C

***********************************************************************/

#include    <stdio.h>	    /* vsprintf */
#include    <conio.h>	    /* _getch */
#include    <string.h>	    /* strlen */
#include    <io.h>	    /* write */
#include    <stdarg.h>	    /* ANSI va_list */
#include    <vs2.h>


/***********************************************************************

  TYPEDEFS etc.

***********************************************************************/

typedef void (VS2API *	FILLPROC)( USHORT, USHORT, UCHAR, UCHAR );


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    Test1( FILLPROC );
void	    Test2( FILLPROC );
void	    Test3( FILLPROC );
void	    Test4( FILLPROC );
void	    Test5( FILLPROC );
void	    Test6( FILLPROC );
void	    Test7( FILLPROC );
void	    Test8( FILLPROC );
void	    AuxPrintf( char *, ... );
void	    InitVMode( USHORT );

static void HollowRectangle( USHORT, USHORT, USHORT, USHORT, UCHAR );
static void SolidRectangle( USHORT, USHORT, USHORT, USHORT, UCHAR );
static int  InitFns( void );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

UCHAR (VS2API * fnGetPixel)( USHORT, USHORT );
void (VS2API *	fnSetPixel)( USHORT, USHORT, UCHAR );
void (VS2API *	fnLine)( USHORT, USHORT, USHORT, USHORT, UCHAR );
void (VS2API *	fnEllipse)( USHORT, USHORT, USHORT, USHORT, UCHAR );
USHORT (VS2API * fnScanLeft)( USHORT, USHORT, UCHAR, short );
USHORT (VS2API * fnScanRight)( USHORT, USHORT, UCHAR, short );

static UCHAR	cRMWBits = 0x00;
static UCHAR	ucBorder, ucFill, ucFill2;


/***********************************************************************

  Test1 -- branches above a local maximum (below a local minimum)

***********************************************************************/

void Test1( FILLPROC fnFill )
{
    if( !InitFns() )
      return;

    HollowRectangle( 100, 191, 122, 199, ucBorder );
    SolidRectangle( 103, 191, 119,  194, ucBorder );
    SolidRectangle( 100, 196, 105,  197, ucBorder );
    SolidRectangle( 109, 196, 113,  197, ucBorder );
    SolidRectangle( 117, 196, 122,  197, ucBorder );

    (*fnFill)( 111, 198, ucBorder, ucFill );
}


/***********************************************************************

  Test2 -- fill includes a subsidiary cycle

***********************************************************************/

void Test2( FILLPROC fnFill )
{
    if( !InitFns() )
      return;

    HollowRectangle( 100, 100, 115, 111, ucBorder );
    (*fnLine)( 105, 100, 105, 102, ucBorder );
    (*fnLine)( 105, 104, 105, 108, ucBorder );
    (*fnLine)( 106, 106, 110, 106, ucBorder );
    (*fnLine)( 110, 105, 110, 102, ucBorder );
    (*fnFill)( 103, 101, ucBorder, ucFill );
}


/***********************************************************************

  Test3 -- a big ellipse with three holes

***********************************************************************/

void Test3( FILLPROC fnFill )
{
    USHORT  xc, yc;


    if( !InitFns() )
      return;

    xc = (USHORT)(vsp.usSX / 2);
    yc = (USHORT)(vsp.usSY / 2);

    (*fnEllipse)( xc, yc, (USHORT)(xc-16), (USHORT)(yc-16), ucBorder );
    (*fnEllipse)( 4*xc/5, yc, (USHORT)(xc/10), (USHORT)(yc/2), ucBorder );

    (*fnEllipse)( 6*xc/5, (USHORT)(yc/2),
			      (USHORT)(xc/4), (USHORT)(yc/10), ucBorder );
    (*fnEllipse)( 6*xc/5, (USHORT)(3*yc/2),
			      (USHORT)(xc/4), (USHORT)(yc/10), ucBorder );

    (*fnFill)( xc, yc, ucBorder, ucFill );
}


/***********************************************************************

  Test4 -- a big ellipse with three holes bounded by the fill color

***********************************************************************/

void Test4( FILLPROC fnFill )
{
    USHORT  xc, yc;


    if( !InitFns() )
      return;

    xc = (USHORT)(vsp.usSX / 2);
    yc = (USHORT)(vsp.usSY / 2);

    (*fnEllipse)( xc, yc, (USHORT)(xc-16), (USHORT)(yc-16), ucBorder );
    (*fnEllipse)( 4*xc/5, yc, (USHORT)(xc/10), (USHORT)(yc/2), ucFill );

    (*fnEllipse)( 6*xc/5, (USHORT)(yc/2),
			      (USHORT)(xc/4), (USHORT)(yc/10), ucFill );
    (*fnEllipse)( 6*xc/5, (USHORT)(3*yc/2),
			      (USHORT)(xc/4), (USHORT)(yc/10), ucFill );

    (*fnFill)( xc, yc, ucBorder, ucFill );
}


/***********************************************************************

  Test5 -- two spirals

***********************************************************************/

void Test5( FILLPROC fnFill )
{
    USHORT  x, y, xc, yc, d;


    if( !InitFns() )
      return;

    xc = (USHORT)(vsp.usSX / 2);
    yc = (USHORT)(vsp.usSY / 2);

    /* left-hand clockwise */
    x = xc/2;
    y = yc;
    d = 3;
    while( (x>vsp.usSX/10) && (y>vsp.usSY/10) )
    {
      (*fnLine)( x, y, x+d, y, ucBorder );
      (*fnLine)( x+d, y, x+d, y+d, ucBorder );
      x += d;
      y += d;
      d += 3;
      (*fnLine)( x, y, x-d, y, ucBorder );
      (*fnLine)( x-d, y, x-d, y-d, ucBorder );
      x -= d;
      y -= d;
      d += 3;
    }

    (*fnLine)( x, y, xc, y, ucBorder );
    (*fnLine)( x+3, y+3, xc, y+3, ucBorder );

    /* right-hand counterclockwise */
    x = 3*xc/2;
    y = yc;
    d = 3;
    while( (x<9*vsp.usSX/10) && (y>vsp.usSY/10) )
    {
      (*fnLine)( x, y, x-d, y, ucBorder );
      (*fnLine)( x-d, y, x-d, y+d, ucBorder );
      x -= d;
      y += d;
      d += 3;
      (*fnLine)( x, y, x+d, y, ucBorder );
      (*fnLine)( x+d, y, x+d, y-d, ucBorder );
      x += d;
      y -= d;
      d += 3;
    }

    (*fnLine)( x, y, xc, y, ucBorder );
    (*fnLine)( x-3, y+3, xc, y+3, ucBorder );

    /* fill from the left center outward */
    (*fnFill)( xc/2+1, yc+1, ucBorder, ucFill );
    _getch();

    /* fill from the right center outward */
    (*fnFill)( 3*xc/2+1, yc+1, ucBorder, ucFill2 );
    _getch();

    /* fill from the outer portion inward */
    (*fnFill)( xc, y+2, ucBorder, ucFill );
}


/***********************************************************************

  Test6 -- a grid with many short runs

***********************************************************************/

void Test6( FILLPROC fnFill )
{
    USHORT  x, y, xc, yc, d;


    if( !InitFns() )
      return;

    xc = (USHORT)(vsp.usSX / 2);
    yc = (USHORT)(vsp.usSY / 2);

    HollowRectangle( xc/2, yc/2, 3*xc/2, 3*yc/2, ucBorder );
    d = 0;
    for( x=xc/2+2; x<3*xc/2-1; x+=2 )
    {
      for( y=yc/2+d+1; y<3*yc/2-2; y+=4 )
	(*fnLine)( x, y, x, (USHORT)(y+2), ucBorder );
      d = (d+1) % 3;
    }

    (*fnFill)( xc+1, yc, ucBorder, ucFill );
}


/***********************************************************************

  Test7 -- a grid with many short runs adjacent to long runs

***********************************************************************/

void Test7( FILLPROC fnFill )
{
    USHORT  x, y, xc, yc;


    if( !InitFns() )
      return;

    xc = (USHORT)(vsp.usSX / 2);
    yc = (USHORT)(vsp.usSY / 2);

    SolidRectangle( xc/2, yc/2, 3*xc/2, 3*yc/2, ucBorder );

    for( x=xc/2+1; x<3*xc/2-1; x+=2 )
      (*fnLine)( x, (USHORT)(yc/2+1), x, (USHORT)(3*yc/2-1), 0 );

    for( y=yc/2+1; y<3*yc/2-1; y+=2 )
      (*fnLine)( xc/2+1, y, 3*xc/2-1, y, 0 );

    (*fnFill)( xc-1, yc-1, ucBorder, ucFill );
}


/***********************************************************************

  Test8 -- fills to the edge

***********************************************************************/

void Test8( FILLPROC fnFill )
{
    if( !InitFns() )
      return;

    (*fnLine)( 0, 32, 10, 32, ucBorder );
    (*fnLine)( 10, 32, vsp.usSX-1, vsp.usSY-10, ucBorder );
    (*fnLine)( 0, 64, vsp.usSX-10, vsp.usSY-1, ucBorder );

    (*fnFill)( 7, 48, ucBorder, ucFill );
}


/***********************************************************************

  AuxPrintf

***********************************************************************/

void AuxPrintf( char * szFmt, ... )
{
    int     hOut = 3;		/* 0=console; 3=COM1 */
    char    szBuf[128];
    va_list marker;


    if( hOut == 0 )
      SetCursorXY( 0, 0 );

    va_start( marker, szFmt );
    vsprintf( szBuf, szFmt, marker );
    va_end( marker );

    write( hOut, szBuf, strlen(szBuf) );
    write( hOut, "\r\n", 2 );
}


/***********************************************************************

  InitVMode

***********************************************************************/

void InitVMode( USHORT usMode )
{
    SetVMode( usMode );
    InitVSP( usMode );

    /* set up for Graphics Controller raster ops */
    vsp.cRMW = cRMWBits;
}


/***********************************************************************

  HollowRectangle

***********************************************************************/

static void HollowRectangle( USHORT x1, USHORT y1,
				USHORT x2, USHORT y2, UCHAR ucPixel )
{
    (*fnLine)( x1, y1, x1, y2, ucPixel );
    (*fnLine)( x1, y1, x2, y1, ucPixel );
    (*fnLine)( x2, y2, x1, y2, ucPixel );
    (*fnLine)( x2, y2, x2, y1, ucPixel );
}


/***********************************************************************

  SolidRectangle

***********************************************************************/

static void SolidRectangle( USHORT x1, USHORT y1,
				USHORT x2, USHORT y2, UCHAR ucPixel )
{
    USHORT   y;


    for( y=y1; y<=y2; y++ )
      (*fnLine)( x1, y, x2, y, ucPixel );
}


/***********************************************************************

  InitFns

***********************************************************************/

static int InitFns()
{
    switch( vsp.usMode )
    {
      case 0x0D:
      case 0x0E:
      case 0x0F:
      case 0x10:
      case 0x11:
      case 0x12:
	fnSetPixel = SetPixel10;
	fnGetPixel = GetPixel10;
	fnLine = Line10;
	fnEllipse = Ellipse10;
	fnScanLeft = ScanLeft10;
	fnScanRight = ScanRight10;
	ucBorder = 0x0A;
	ucFill = 0x09;
	ucFill2 = 0x0C;
	break;

      case 0x13:
	fnSetPixel = SetPixel13;
	fnGetPixel = GetPixel13;
	fnLine = Line13;
	fnEllipse = Ellipse13;
	fnScanLeft = ScanLeft13;
	fnScanRight = ScanRight13;
	ucBorder = 0x0A;
	ucFill = 0x09;
	ucFill2 = 0x0C;
	break;

      case 0x840:
      case 0x841:
      case 0x842:
      case 0x843:
      case 0x844:
      case 0x845:
	fnSetPixel = SetPixel256;
	fnGetPixel = GetPixel256;
	fnLine = Line256;
	fnEllipse = Ellipse256;
	fnScanLeft = ScanLeft256;
	fnScanRight = ScanRight256;
	ucBorder = 0x0A;
	ucFill = 0x09;
	ucFill2 = 0x0C;
	break;

      default:
	printf( "*** unsupported video mode ***\n" );
	return 0;
    }

    return 1;
}
