		TITLE	SCANPIXL.ASM
		NAME	SCANPIXL
		PAGE	55,132

.286

INCLUDE 	vs2.inc

DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

		EXTRN	PIXELADDR10:FAR
		EXTRN	PIXELADDR13:FAR
		EXTRN	PIXELADDR256:FAR

; parameters for all ScanRight and ScanLeft functions
flag		EQU	[bp+6]
ucPixel 	EQU	[bp+8]
y		EQU	[bp+10]
x		EQU	[bp+12]

; ScanRight10
;
;   USHORT VS2API
;   ScanRight10( USHORT x, USHORT y, UCHAR ucPixel, short flag );
;
; Notes:
;  Returns the x-coordinate of the rightmost pixel in the run that
;  contains (x,y).
;
;  If the specified flag is zero, this function scans until a matching
;  pixel is encountered; if the flag is nonzero, this function scans for
;  a non-matching pixel.

		PUBLIC	SCANRIGHT10
SCANRIGHT10	PROC	FAR

	enter	0, 0
	push	si
	push	di

; calculate the pixel address of (0,y)
	push	0
	push	y
	call	PixelAddr10
	mov	es, dx
	mov	di, ax		; ES:DI -> (0,y)
	mov	bx, ax		; BX = byte offset of (0,y)

; calculate the offset of x in row y
	mov	cx, x		; CX = x
	mov	si, cx
	shr	si, 3		; SI = byte offset of x in row y
	add	di, si		; DI = offset of x in buffer

; configure the Graphics Controller
	mov	dx, 3CEh	; DX = Graphics Controller I/O addr

	mov	ah, ucPixel	; AH = pixel value for Color Compare reg
	mov	al, 2		; AL = Color Compare reg number
	out	dx, ax

	mov	ax, 805h	; AH bit 3 = 1 (Read Mode 1)
	out	dx, ax		; AL = Mode reg number

	mov	ax, 0F07h	; AH = Color Don't Care reg value
	out	dx, ax		; AL = Color Compare reg number

; create a mask for the color-compare result
	mov	al, flag
	neg	al		; cf set if flag is nonzero
	rcr	al, 1
	cbw			; AH = mask (0 or FFh)

; calculate a bit mask for the seed pixel
	and	cl, 7		; CL = x MOD 8
	mov	ch, 80h
	shr	ch, cl		; CH = bit mask

; inspect the first pixel
	mov	al, es:[di]	; AL = nonzero bits corresponding to
				;  pixels that match ucPixel
	inc	di		; ES:DI -> next byte to scan
	xor	al, ah		; toggle bits if flag was nonzero
	test	al, ch		; apply bit mask
	jnz	L008		; jump if first pixel matched

; test the remaining pixels in the first byte
	mov	ch, 7Fh
	shr	ch, cl		; CH = bit mask
	and	al, ch
	jnz	L001		; jump if pixel(s) found

; scan the remainder of line for border pixels
	mov	cx, VSP.cbRow
	sub	cx, si
	dec	cx		; CX = # of bytes to scan
	jz	L001		; jump if only one byte to scan

	mov	al, ah		; AL = result mask
	repe	scasb		; scan for a non-matching byte
				;  (i.e., matching pixel(s) found)

	mov	al, es:[di-1]	; AL = non-matching byte
	xor	al, ah		; XOR with result mask

; compute the x-coordinate of the non-matching pixel
L001:	sub	di, bx		; DI = offset of the byte past the one
				;  that contains a non-matching pixel
	shl	di, 3		; DI = x-coordinate of the 1st pixel
				;  after the byte in question
	dec	di		; DI = x-coordinate of last pixel in
				;  the byte
	xor	cx, cx

	or	al, al
	jz	L009

L002:	shr	al, 1		; loop until the leftmost nonzero
	loopnz	L002		;  bit is shifted out

	add	di, cx
	jmp	L009

; come here if error
L008:	mov	di, 0FFFFh	; (error return value)

; restore the default Graphics Controller state
L009:	mov	ax, 2		; Color Compare register
	out	dx, ax

	mov	al, 5		; Mode register
	out	dx, ax

	mov	ax, di		; AX = return value

; restore registers and return
	pop	di
	pop	si
	leave
	ret	8

SCANRIGHT10	ENDP

; ScanLeft10
;
;   USHORT VS2API
;   ScanLeft10( USHORT x, USHORT y, UCHAR ucPixel, short flag );

		PUBLIC	SCANLEFT10
SCANLEFT10	PROC	FAR

	enter	0, 0
	push	si
	push	di

; calculate the pixel address of (0,y)
	push	0
	push	y
	call	PixelAddr10
	mov	es, dx
	mov	di, ax		; ES:DI -> (0,y)
	mov	bx, ax		; BX = byte offset of (0,y)

; calculate the offset of x in row y
	mov	cx, x		; CX = x
	mov	si, cx
	shr	si, 3		; SI = byte offset of x in row y
	add	di, si		; DI = offset of x in buffer

; configure the Graphics Controller
	mov	dx, 3CEh	; DX = Graphics Controller I/O addr

	mov	ah, ucPixel	; AH = pixel value for Color Compare reg
	mov	al, 2		; AL = Color Compare reg number
	out	dx, ax

	mov	ax, 805h	; AH bit 3 = 1 (Read Mode 1)
	out	dx, ax		; AL = Mode reg number

	mov	ax, 0F07h	; AH = Color Don't Care reg value
	out	dx, ax		; AL = Color Compare reg number

; create a mask for the color-compare result
	mov	al, flag
	neg	al		; cf set if flag is nonzero
	rcr	al, 1
	cbw			; AH = mask (0 or FFh)

; calculate a bit mask for the seed pixel
	and	cl, 7		; CL = x MOD 8
	mov	ch, 80h
	shr	ch, cl		; CH = bit mask

; inspect the first pixel
	mov	al, es:[di]	; AL = nonzero bits corresponding to
				;  pixels that match ucPixel
	dec	di		; ES:DI -> next byte to scan
	xor	al, ah		; toggle bits if flag was nonzero
	test	al, ch		; apply bit mask
	jnz	L108		; jump if first pixel matched

; test the remaining pixels in the first byte
	mov	ch, 0FEh
	xor	cl, 7		; CL = 7 - (x MOD 8)
	shl	ch, cl		; CH = bit mask
	and	al, ch

	mov	cx, si		; CX = # bytes to scan (zf unaffected)
	jnz	L101		; jump if pixel(s) found
	jcxz	L101		; jump if only one byte to scan

; scan the remainder of line for border pixels
	mov	al, ah		; AL = result mask
	std			; set df (autodecrement)
	repe	scasb		; scan for a non-matching byte
				;  (i.e., matching pixel(s) found)
	cld			; clear df

	mov	al, es:[di+1]	; AL = non-matching byte
	xor	al, ah		; XOR with result mask

; compute the x-coordinate of the non-matching pixel
L101:	mov	di, cx		; DI = offset of the byte
				;  that contains a non-matching pixel
	shl	di, 3		; DI = x-coordinate of the last pixel
				;  in the previous byte
	xor	cx, cx
	or	al, al
	jz	L109

L102:	shl	al, 1		; loop until the rightmost nonzero
	loopnz	L102		;  bit is shifted out

	sub	di, cx		; DI = x-coordinate of rightmost pixel
	jmp	L109

; come here if error
L108:	mov	di, 0FFFFh	; (error return value)

; restore the default Graphics Controller state
L109:	mov	ax, 2		; Color Compare register
	out	dx, ax

	mov	al, 5		; Mode register
	out	dx, ax

	mov	ax, di		; AX = return value

; restore registers and return
	pop	di
	pop	si
	leave
	ret	8

SCANLEFT10	ENDP

; ScanRight13
;
;   USHORT VS2API
;   ScanRight13( USHORT x, USHORT y, UCHAR ucPixel, short flag );

		PUBLIC	SCANRIGHT13
SCANRIGHT13	PROC	FAR

	enter	0, 0
	push	di

; calculate the pixel address of (0,y)
	push	x
	push	y
	call	PixelAddr13
	mov	es, dx
	mov	di, ax		; ES:DI -> (x,y)

; prepare to scan
	mov	al, ucPixel
	mov	bx, VSP.cbRow
	sub	bx, x		; BX = (cbRow-x) = # of bytes to scan
	mov	cx, bx

	cmp	WORD PTR flag, 0
	jz	L201		; jump if "scan until"

; come here to scan while the pixels match
	repe	scasb
	jz	L208		; jump if all pixels matched
	jmp	L202

; come here to scan until the pixels match
L201:	repne	scasb
	jnz	L208		; jump if no pixels matched

L202:	mov	ax, bx
	sub	ax, cx		; AX = (# matching pixels) + 1
	sub	ax, 2		; AX = (# matching pixels) - 1
	jc	L209		; jump if no matching pixels (AX=FFFFh)

	add	ax, x		; AX = x-coord of last matching byte
	jmp	L209

; come here if all pixels matched (or didn't match)
L208:	mov	ax, VSP.cbRow
	dec	ax		; AX = x-coordinate of rightmost pixel

; restore registers and return
L209:	pop	di
	leave
	ret	8

SCANRIGHT13	ENDP

; ScanLeft13
;
;   USHORT VS2API
;   ScanLeft13( USHORT x, USHORT y, UCHAR ucPixel, short flag );

		PUBLIC	SCANLEFT13
SCANLEFT13	PROC	FAR

	enter	0, 0
	push	di

; calculate the pixel address of (0,y)
	push	x
	push	y
	call	PixelAddr13
	mov	es, dx
	mov	di, ax		; ES:DI -> (x,y)

; prepare to scan
	mov	al, ucPixel
	mov	bx, x		; BX = x
	mov	cx, bx
	inc	cx		; CX = (x+1) = # of bytes to scan
	std			; set df (autodecrement)

	cmp	WORD PTR flag, 0
	jz	L301		; jump if "scan until"

; come here to scan while the pixels match
	repe	scasb
	jz	L308		; jump if all pixels matched
	jmp	L302

; come here to scan until the pixels match
L301:	repne	scasb
	jnz	L308		; jump if no pixels matched

L302:	mov	ax, cx
	inc	ax		; AX = x-coordinate of leftmost pixel
	cmp	ax, bx
	jbe	L309		; jump if no error

	mov	ax, 0FFFFh	; error (1st pixel matched)
	jmp	L309

; come here if all pixels matched (or didn't match)
L308:	xor	ax, ax		; AX = x-coordinate of leftmost pixel

L309:	cld			; clear df
	pop	di
	leave
	ret	8

SCANLEFT13	ENDP

; ScanRight256
;
;   USHORT VS2API
;   ScanRight256( USHORT x, USHORT y, UCHAR ucPixel, short flag );

		PUBLIC	SCANRIGHT256
SCANRIGHT256	PROC	FAR

	enter	0, 0
	push	si
	push	di

; calculate the pixel address of (0,y)
	push	0
	push	y
	call	PixelAddr256
	mov	es, dx
	mov	di, ax		; ES:DI -> (0,y)

; calculate the offset of x in row y
	mov	cx, x		; CX = x
	mov	si, cx
	shr	si, 2		; SI = byte offset of x in row y
	add	di, si		; DI = offset of x in buffer

; prepare to program the Graphics Controller
	mov	dx, 3CEh	; DX = Graphics Controller I/O addr
	mov	ah, cl
	and	ah, 3		; AH = (x MOD 8) = initial bit plane #
	mov	al, 4		; Read Map Select register number
	out	dx, ax

; prepare to scan the first pixel
	mov	cx, VSP.cbRow
	sub	cx, si		; CX = # of bytes to scan

	mov	bh, ucPixel	; BH = pixel value for scan

	cmp	WORD PTR flag, 0
	jz	L410		; jump if "scan until"

; come here to scan while the pixels match
	cmp	bh, es:[di]
	jne	L480		; jump if error

; select the bit plane for the next pixel
L401:	inc	ah
	and	ah, 3		; isolate bits 0-1
	jnz	L402		; jump if not plane 0

	inc	di		; ES:DI -> next byte to scan
	dec	cx
	jz	L485		; jump if scanned past the end

L402:	out	dx, ax		; select the next bit plane
	cmp	bh, es:[di]	; compare the pixel values
	je	L401		; jump if pixel matched
	jmp	L485

; come here to scan until the pixels match
L410:	cmp	bh, es:[di]
	je	L480		; jump if error

L411:	inc	ah
	and	ah, 3		; isolate bits 0-1
	jnz	L412		; jump if not plane 0

	inc	di		; ES:DI -> next byte to scan
	dec	cx
	jz	L485		; jump if scanned past the end

L412:	out	dx, ax		; select the next bit plane
	cmp	bh, es:[di]	; compare the pixel values
	jne	L411		; jump if pixel did not match
	jmp	L485

; come here if error
L480:	mov	ax, 0FFFFh	; (error return value)
	jmp	L490

; come here to compute the x-coordinate of the rightmost pixel
L485:	mov	bl, ah
	xor	bh, bh		; BX = bit plane #
	mov	ax, VSP.cbRow
	sub	ax, cx		; AX = cbRow - current x-offset
	shl	ax, 2
	add	ax, bx		; AX = x-coord of non-matching pixel
	dec	ax		; AX = x-coord of rightmost pixel

; restore registers and return
L490:	pop	di
	pop	si
	leave
	ret	8

SCANRIGHT256	ENDP

; ScanLeft256
;
;   USHORT VS2API
;   ScanLeft256( USHORT x, USHORT y, UCHAR ucPixel, short flag );

		PUBLIC	SCANLEFT256
SCANLEFT256	PROC	FAR

	enter	0, 0
	push	di

; calculate the pixel address of (0,y)
	push	0
	push	y
	call	PixelAddr256
	mov	es, dx
	mov	di, ax		; ES:DI -> (0,y)

; calculate the offset of x in row y
	mov	ax, x		; AX = x
	mov	cx, ax
	shr	cx, 2		; CX = byte offset of x in row y
	add	di, cx		; DI = offset of x in buffer

; prepare to program the Graphics Controller
	mov	dx, 3CEh	; DX = Graphics Controller I/O addr
	mov	ah, al
	and	ah, 3		; AH = (x MOD 8) = initial bit plane #
	mov	al, 4		; Read Map Select register number
	out	dx, ax

; prepare to scan the first pixel
	mov	bh, ucPixel	; BH = pixel value for scan

	cmp	WORD PTR flag, 0
	jz	L510		; jump if "scan until"

; come here to scan while the pixels match
	cmp	bh, es:[di]
	jne	L580		; jump if error

; select the bit plane for the next pixel
L501:	dec	ah
	jns	L502		; jump if not decremented below 0

	mov	ah, 3		; AH = bit plane 3
	dec	di		; ES:DI -> next byte to scan
	dec	cx
	js	L585		; jump if scanned past the end

L502:	out	dx, ax		; select the next bit plane
	cmp	bh, es:[di]	; compare the pixel values
	je	L501		; jump if pixel matched
	jmp	L585

; come here to scan until the pixels match
L510:	cmp	bh, es:[di]
	je	L580		; jump if error

L511:	dec	ah
	jns	L512		; jump if not decremented below 0

	mov	ah, 3		; AH = bit plane 3
	dec	di		; ES:DI -> next byte to scan
	dec	cx
	js	L585		; jump if scanned past the end

L512:	out	dx, ax		; select the next bit plane
	cmp	bh, es:[di]	; compare the pixel values
	jne	L511		; jump if pixel did not match
	jmp	L585

; come here if error
L580:	mov	ax, 0FFFFh	; (error return value)
	jmp	L590

; come here to compute the x-coordinate of the leftmost pixel
L585:	mov	al, ah
	xor	ah, ah		; AX = bit plane #
	shl	cx, 2
	add	ax, cx
	inc	ax		; AX = x-coord of leftmost pixel

; restore registers and return
L590:	pop	di
	leave
	ret	8

SCANLEFT256	ENDP

VS2_TEXT	ENDS

_DATA		SEGMENT WORD PUBLIC 'DATA'

		EXTRN	VSP:VIDEOSTATEPARAMS

_DATA		ENDS

		END
