		TITLE	BLT.ASM
		NAME	BLT
		PAGE	55,132

.286

INCLUDE 	vs2.inc

; defined for MS Windows 3.x ...
BITMAPINFOHEADER	STRUCT
biSize			DWORD	?
biWidth 		DWORD	?
biHeight		DWORD	?
biPlanes		WORD	?
biBitCount		WORD	?
biCompression		DWORD	?
biSizeImage		DWORD	?
biXPelsPerMeter 	DWORD	?
biYPelsPerMeter 	DWORD	?
biClrUsed		DWORD	?
biClrImportant		DWORD	?
BITMAPINFOHEADER	ENDS

BI_RGB		EQU	0
BI_RLE8 	EQU	1
BI_RLE4 	EQU	2
; end of Windows 3.x definitions

; PackPixels10
;
; At entry: BL = bits from plane 0
;	    BH = bits from plane 1
;	    DL = bits from plane 2
;	    DH = bits from plane 3
;	    CL = number of pixels to pack (1-4)
; Leaves:   AX = packed pixels
; Destroys: CH

PackPixels10	MACRO
		LOCAL	M000

	xor	ax, ax
	mov	ch, cl

M000:	rol	dh, 1		; cf = bit from plane 3
	rcl	ax, 1
	rol	dl, 1		; cf = bit from plane 2
	rcl	ax, 1
	rol	bh, 1		; cf = bit from plane 1
	rcl	ax, 1
	rol	bl, 1		; cf = bit from plane 0
	rcl	ax, 1
	dec	cl
	jnz	M000

; rotate packed pixels into position in AX
	mov	cl, ch
	add	cl, 2
	and	cl, 3
	shl	cl, 2
	ror	ax, cl

		ENDM

; UnpackPixels10
;
; At entry: DS:SI -> packed 4-bit pixels
; Leaves:   DS:SI -> next 8 pixels
;	    BL = bits for plane 0
;	    BH = bits for plane 1
;	    DL = bits for plane 2
;	    DH = bits for plane 3
; Destroys: AX, CX

UnpackPixels10	MACRO
		LOCAL	M100, M101

	mov	ch, 2	    ; outer-loop limit

M100:	mov	cl, 4	    ; inner-loop limit
	lodsw
	xchg	ah, al	    ; AX = 4 pixels

M101:	shl	ax, 1
	rcl	dh, 1	    ; update bits for plane 3
	shl	ax, 1
	rcl	dl, 1	    ; update bits for plane 2
	shl	ax, 1
	rcl	bh, 1	    ; update bits for plane 1
	shl	ax, 1
	rcl	bl, 1	    ; update bits for plane 0
	dec	cl
	jnz	M101

	dec	ch
	jnz	M100

		ENDM

DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

		EXTRN	PIXELADDR10:FAR
		EXTRN	PIXELADDR13:FAR
		EXTRN	PIXELADDR256:FAR
		EXTRN	VNWAIT:FAR

; parameters for all BltToRAM functions
h		EQU	WORD PTR [bp+6]
w		EQU	WORD PTR [bp+8]
y		EQU	WORD PTR [bp+10]
x		EQU	WORD PTR [bp+12]
pBMP		EQU	DWORD PTR [bp+14]

; BltToRAM10
;
;   ULONG VS2API
;   BltToRAM10( LPVOID pBMP, USHORT x, USHORT y, USHORT w, USHORT h );
;
; Notes:
;   Parameters x,y specify the upper left corner of the bitmap;
;   w,h specify the width and height in pixels.
;
;   No pixel data is copied if the specified buffer pointer is null.
;
;   Returns the actual size of the bitmap.

ofsRow		EQU	WORD PTR [bp-2]
RowIncr 	EQU	WORD PTR [bp-4]
PixRot		EQU	BYTE PTR [bp-5]
nPad		EQU	BYTE PTR [bp-6]
cbLow		EQU	WORD PTR [bp-8]
cbHigh		EQU	WORD PTR [bp-10]

		PUBLIC	BLTTORAM10
BLTTORAM10	PROC	FAR

	enter	10, 0
	push	ds
	push	si
	push	di

; compute the size of the pixel data in each row
	mov	ax, w	    ; AX = pixels per row
	inc	ax
	shr	ax, 1	    ; AX = bytes of pixel data per row
	mov	si, ax	    ; SI = bytes of pixel data per row
	add	ax, 3
	and	al, 0FCh    ; round up to nearest multiple of 4

	mov	bx, ax
	sub	bx, si	    ; BX = # of bytes of padding
	shr	bx, 1	    ; BX = # of words of padding
	mov	nPad, bl

; compute the size of the bitmap
	mul	h	    ; DX:AX = # bytes of bitmap data
	mov	bx, ax
	mov	cx, dx	    ; CX:BX = # bytes of bitmap data

	add	ax, sizeof BITMAPINFOHEADER
	adc	dx, 0
	add	ax, 16*4    ; size of 16-color table
	adc	dx, 0

	mov	cbLow, ax
	mov	cbHigh, dx

; verify that the specified buffer pointer is not null
	les	di, pBMP    ; ES:DI -> BMP in RAM
	mov	ax, es
	or	ax, di
	jnz	L000
	jmp	L090

; initialize the BITMAPINFOHEADER data structure
L000:	cld
	mov	ax, sizeof BITMAPINFOHEADER
	stosw
	xor	ax, ax
	stosw		    ; biSize

	mov	ax, w
	stosw
	xor	ax, ax
	stosw		    ; biWidth

	mov	ax, h
	stosw
	xor	ax, ax
	stosw		    ; biHeight

	mov	ax, 1	    ; biPlanes (always 1 according to
	stosw		    ;  Microsoft's documentation)

	mov	ax, 4	    ; biBitCount
	stosw

	mov	ax, BI_RGB  ; (no data compression)
	stosw
	xor	ax, ax
	stosw		    ; biCompression

	mov	ax, bx
	stosw
	mov	ax, cx
	stosw		    ; biSizeImage

	mov	cx, 8
	xor	ax, ax
	rep stosw	    ; biXPelsPerMeter, biYPelsPerMeter,
			    ;  biClrUsed, biClrImportant

; build bit masks for converting palette-register values to DAC indexes
	mov	al, 14h     ; Pixel Padding register number
	call	GetATCReg   ; AL = current Pixel Padding register value
	mov	bl, al
	shl	bl, 4	    ; move bits 2-3 into bits 6-7
	mov	bh, 0Fh     ; BH = AND mask for 4-bit palette-reg values

	mov	al, 10h     ; Mode Control register number
	call	GetATCReg   ; AL = current Mode Control reg value
	test	al, 80h     ; test bit 7
	jnz	L010	    ; jump if 4-bit palette-register values

; come here if using 6-bit palette-register values
	mov	bh, 3Fh     ; BH = AND mask for 6-bit palette-reg values
	and	bl, 0C0h    ; BL = OR mask for 6-bit palette-reg values

; copy the current color values from the video DAC
L010:	mov	cx, 16	    ; CX = # of palette registers

	call	VNWAIT	    ; wait for vertical nondisplay interval

L011:	mov	al, cl
	neg	al
	and	al, 0Fh     ; AL = register number (00h-0Fh)
	call	GetATCReg   ; AL = palette register value
	and	al, bh	    ; isolate low-order bits
	or	al, bl	    ; set high-order bits

	call	GetDACRGB   ; store RGB quad at ES:[DI]
	add	di, 4	    ; point to next RGB quad in color table
	loop	L011

; verify that the bitmap is not null
	mov	ax, h
	mul	w	    ; DX:AX = w*h
	or	ax, dx
	jnz	L015
	jmp	L090	    ; exit if empty bitmap

; compute the number of bits to shift the pixel data
L015:	mov	cx, x
	and	cl, 7	    ; CL = x MOD 8
	mov	PixRot, cl

; save the number of bytes per row in the video buffer
	mov	ax, VSP.cbRow
	mov	RowIncr, ax

; point to the first pixel
	push	x
	mov	ax, y
	add	ax, h
	dec	ax	    ; AX = y-coordinate of bottommost row
	push	ax
	call	PixelAddr10
	mov	si, ax
	mov	ds, dx	    ; DS:SI -> lower-left pixel
	mov	ofsRow, ax

; come here at the start of each row of pixels
L020:	push	w

; get pixels from the next byte
L025:	mov	cl, PixRot  ; CL = # of bits to rotate left

	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr
	mov	ax, 304h    ; select bit plane 3 for CPU reads
	out	dx, ax

	mov	bx, [si]    ; BX = bits from bit plane 3
	rol	bx, cl
	mov	ch, bl	    ; CH = rotated bits from plane 3

	dec	ah	    ; select bit plane 2
	out	dx, ax

	mov	bx, [si]    ; BX = bits from bit plane 2
	rol	bx, cl	    ; BL = rotated bits from plane 2
	mov	bh, ch	    ; BH = rotated bits from plane 3
	push	bx

	dec	ah	    ; select bit plane 1
	out	dx, ax

	mov	bx, [si]    ; BX = bits from bit plane 1
	rol	bx, cl
	mov	ch, bl	    ; CH = rotated bits from plane 1

	dec	ah	    ; select bit plane 0
	out	dx, ax

	mov	bx, [si]    ; BX = bits from bit plane 0
	rol	bx, cl	    ; BL = rotated bits from plane 0
	mov	bh, ch	    ; BH = rotated bits from plane 1
	pop	dx	    ; DL = rotated bits from plane 2
			    ; DH = rotated bits from plane 3

	inc	si	    ; DS:SI -> next byte

; do while 8 or more pixels remain in the row
	pop	cx
	sub	cx, 8
	js	L030
	push	cx

; serialize the 8 pixels
	mov	cl, 4
	PackPixels10
	stosw
	mov	cl, 4
	PackPixels10
	stosw
	jmp	L025

; come here when fewer than 8 pixels remain
L030:	add	cx, 8
	cmp	cx, 4
	jb	L031	    ; jump if fewer than 4 pixels remain

	push	cx
	mov	cl, 4
	PackPixels10
	stosw

	pop	cx
	sub	cx, 4

; come here when fewer than 4 pixels remain
L031:	jcxz	L032	    ; jump if no pixels remain

	PackPixels10
	stosw

L032:	cmp	nPad, 0
	jz	L033	    ; jump if no padding needed

	xor	ax, ax
	stosw		    ; add a word of padding

; come here when an entire row of pixels has been copied
L033:	dec	h
	jz	L090	    ; exit if no more rows to copy

; point to the first pixel in the next row
	mov	si, ofsRow
	sub	si, RowIncr
	mov	ofsRow, si
	jmp	L020

; restore registers and exit
L090:	pop	di
	pop	si
	pop	ds
	mov	ax, cbLow
	mov	dx, cbHigh  ; DX:AX = bitmap size
	leave
	ret	12	    ; sizeof(LPVOID)+4*sizeof(USHORT)

BLTTORAM10	ENDP

; BltToRAM13
;
;   ULONG VS2API
;   BltToRAM13( LPVOID pBMP, USHORT x, USHORT y, USHORT w, USHORT h );

RowIncr 	EQU	WORD PTR [bp-2]
nPad		EQU	BYTE PTR [bp-4]
cbLow		EQU	WORD PTR [bp-6]
cbHigh		EQU	WORD PTR [bp-8]

		PUBLIC	BLTTORAM13
BLTTORAM13	PROC	FAR

	enter	8, 0
	push	ds
	push	si
	push	di
	cld

; compute the size of the pixel data in each row
	mov	ax, w
	mov	si, ax	    ; SI = bytes of pixel data per row
	add	ax, 3
	and	al, 0FCh    ; round up to nearest multiple of 4

	mov	bx, ax
	sub	bx, si	    ; BX = # of bytes of padding
	mov	nPad, bl

; compute the size of the bitmap
	mul	h	    ; DX:AX = # bytes of bitmap data
	mov	bx, ax
	mov	cx, dx	    ; CX:BX = # bytes of bitmap data

	add	ax, sizeof BITMAPINFOHEADER
	adc	dx, 0
	add	ax, 256*4    ; size of 256-color table
	adc	dx, 0

	mov	cbLow, ax
	mov	cbHigh, dx

; verify that the specified buffer pointer is not null
	les	di, pBMP    ; ES:DI -> BMP in RAM
	mov	ax, es
	or	ax, di
	jnz	L100
	jmp	L190

; initialize the BITMAPINFOHEADER data structure
L100:	cld
	mov	ax, sizeof BITMAPINFOHEADER
	stosw
	xor	ax, ax
	stosw		    ; biSize

	mov	ax, w
	stosw
	xor	ax, ax
	stosw		    ; biWidth

	mov	ax, h
	stosw
	xor	ax, ax
	stosw		    ; biHeight

	mov	ax, 1	    ; biPlanes (always 1 according to
	stosw		    ;  Microsoft's documentation)

	mov	ax, 8	    ; biBitCount
	stosw

	mov	ax, BI_RGB  ; (no data compression)
	stosw
	xor	ax, ax
	stosw		    ; biCompression

	mov	ax, bx
	stosw
	mov	ax, cx
	stosw		    ; biSizeImage

	mov	cx, 8
	xor	ax, ax
	rep stosw	    ; biXPelsPerMeter, biYPelsPerMeter,
			    ;  biClrUsed, biClrImportant

; copy the current color values from the video DAC
L110:	mov	cx, 256     ; CX = # of DAC registers
	xor	al, al	    ; AL = 0 (first DAC register number)
	mov	dx, 3C7h    ; DX = 3C7h (DAC read I/O address)
	out	dx, al
	mov	dl, 0C9h    ; DX = 3C9h (DAC data register I/O addr)

	call	VNWAIT	    ; wait for vertical nondisplay interval

L112:	in	al, dx	    ; AL = red value
	xor	ah, ah	    ; AH = 0
	shl	ax, 2	    ; convert 6-bit values to 8 bits
	mov	es:[di+2], ax	; store red value and null
	in	al, dx
	mov	ah, al	    ; AH = green value
	in	al, dx	    ; AL = blue value
	shl	ax, 2	    ; convert 6-bit values to 8 bits
	mov	es:[di], ax
	add	di, 4	    ; ES:DI -> next RGBQUAD
	loop	L112

; verify that the bitmap is not null
	mov	ax, h
	mul	w	    ; DX:AX = w*h
	or	ax, dx
	jnz	L115
	jmp	L190	    ; exit if empty bitmap

; compute the increment between rows in the video buffer
L115:	mov	cx, VSP.cbRow
	add	cx, si	    ; CX = cbRow + bytes per row
	mov	RowIncr, cx

; point to the first pixel
	push	x
	mov	ax, y
	add	ax, h
	dec	ax	    ; AX = y-coordinate of bottommost row
	push	ax
	call	PixelAddr13
	mov	si, ax
	mov	ds, dx	    ; DS:SI -> lower-left pixel

	xor	al, al	    ; AL = 0

; come here at the start of each row of pixels
L120:	mov	cx, w	    ; CX = # of bytes (# of pixels) in row
	shr	cx, 1	    ; CX = # of words; cf set if w is odd
	rep	movsw	    ; copy word by word
	rcl	cx, 1	    ; CX = # of remaining bytes
	rep	movsb
	mov	cl, nPad    ; CX = # of bytes of padding
	rep	stosb	    ; pad to DWORD boundary

; point to the next row of pixels
	sub	si, RowIncr ; DS:SI -> start of next row
	dec	h
	jnz	L120

; restore registers and exit
L190:	pop	di
	pop	si
	pop	ds
	mov	ax, cbLow
	mov	dx, cbHigh  ; DX:AX = bitmap size
	leave
	ret	12	    ; sizeof(LPVOID)+4*sizeof(USHORT)

BLTTORAM13	ENDP

; BltToRAM256
;
;   ULONG VS2API
;   BltToRAM256( LPVOID pBMP, USHORT x, USHORT y, USHORT w, USHORT h );

ofsRow		EQU	WORD PTR [bp-2]
RowIncr 	EQU	WORD PTR [bp-4]
ofsSrc		EQU	WORD PTR [bp-6]
ofsDest 	EQU	WORD PTR [bp-8]
nPad		EQU	WORD PTR [bp-10]
ReadMap 	EQU	WORD PTR [bp-12]
cbLow		EQU	WORD PTR [bp-14]
cbHigh		EQU	WORD PTR [bp-16]

		PUBLIC	BLTTORAM256
BLTTORAM256	PROC	FAR

	enter	16, 0
	push	ds
	push	si
	push	di
	cld

; compute the size of the pixel data in each row
	mov	ax, w
	mov	si, ax	    ; SI = bytes of pixel data per row
	add	ax, 3
	and	al, 0FCh    ; round up to nearest multiple of 4

	mov	bx, ax
	sub	bx, si	    ; BX = # of bytes of padding
	mov	nPad, bx

; compute the size of the bitmap
	mul	h	    ; DX:AX = # bytes of bitmap data
	mov	bx, ax
	mov	cx, dx	    ; CX:BX = # bytes of bitmap data

	add	ax, sizeof BITMAPINFOHEADER
	adc	dx, 0
	add	ax, 256*4    ; size of 256-color table
	adc	dx, 0

	mov	cbLow, ax
	mov	cbHigh, dx

; verify that the specified buffer pointer is not null
	les	di, pBMP    ; ES:DI -> BMP in RAM
	mov	ax, es
	or	ax, di
	jnz	L200
	jmp	L290

; initialize the BITMAPINFOHEADER data structure
L200:	cld
	mov	ax, sizeof BITMAPINFOHEADER
	stosw
	xor	ax, ax
	stosw		    ; biSize

	mov	ax, w
	stosw
	xor	ax, ax
	stosw		    ; biWidth

	mov	ax, h
	stosw
	xor	ax, ax
	stosw		    ; biHeight

	mov	ax, 1	    ; biPlanes (always 1 according to
	stosw		    ;  Microsoft's documentation)

	mov	ax, 8	    ; biBitCount
	stosw

	mov	ax, BI_RGB  ; (no data compression)
	stosw
	xor	ax, ax
	stosw		    ; biCompression

	mov	ax, bx
	stosw
	mov	ax, cx
	stosw		    ; biSizeImage

	mov	cx, 8
	xor	ax, ax
	rep stosw	    ; biXPelsPerMeter, biYPelsPerMeter,
			    ;  biClrUsed, biClrImportant

; copy the current color values from the video DAC
L210:	mov	cx, 256     ; CX = # of DAC registers
	xor	al, al	    ; AL = 0 (first DAC register number)
	mov	dx, 3C7h    ; DX = 3C7h (DAC read I/O address)
	out	dx, al
	mov	dl, 0C9h    ; DX = 3C9h (DAC data register I/O addr)

	call	VNWAIT	    ; wait for vertical nondisplay interval

L212:	in	al, dx	    ; AL = red value
	xor	ah, ah	    ; AH = 0
	shl	ax, 2	    ; convert 6-bit values to 8 bits
	mov	es:[di+2], ax	; store red value and null
	in	al, dx
	mov	ah, al	    ; AH = green value
	in	al, dx	    ; AL = blue value
	shl	ax, 2	    ; convert 6-bit values to 8 bits
	mov	es:[di], ax
	add	di, 4	    ; ES:DI -> next RGBQUAD
	loop	L212

; verify that the bitmap is not null
	mov	ax, h
	mul	w	    ; DX:AX = w*h
	or	ax, dx
	jnz	L215
	jmp	L290	    ; exit if empty bitmap

; save the number of bytes per row in the video buffer
L215:	mov	ax, VSP.cbRow
	mov	RowIncr, ax

; determine the bit plane for the leftmost pixel
	mov	ah, BYTE PTR x
	and	ah, 3	    ; AH = x MOD 4
	mov	al, 4	    ; Read Map Select register number
	mov	ReadMap, ax

; point to the first pixel
	push	x
	mov	ax, y
	add	ax, h
	dec	ax	    ; AX = y-coordinate of bottommost row
	push	ax
	call	PixelAddr256
	mov	si, ax
	mov	ds, dx	    ; DS:SI -> lower-left pixel
	mov	ofsRow, ax

	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr

; come here at the start of each row of pixels
L220:	mov	ax, ReadMap ; AH = bit plane number
			    ; AL = 4 (Read Map Select register number)
	mov	ofsDest, di
	mov	ofsSrc, si  ; offset of first byte in video-buffer row
	mov	cx, w	    ; CX = # of bytes (# of pixels) in row
	mov	bx, 1	    ; BX = 1 (outer loop index)

; come here at the start of each bit plane
L221:	out	dx, ax	    ; select the next bit plane

; copy every 4th pixel (pixels in the row in the current bit plane)
L222:	movsb
	add	di, 3	    ; ES:DI -> next byte in the bitmap data
	sub	cx, 4
	ja	L222

; set up to copy pixels from the next bit plane
	mov	si, ofsSrc  ; DS:SI -> first byte in video-buffer row

	inc	ah
	and	ah, 3	    ; AH = next bit plane number
	jnz	L224	    ; jump if not plane 0

	inc	si	    ; start in plane 0 in next byte
	mov	ofsSrc, si

L224:	cmp	ax, ReadMap
	je	L225	    ; jump if all 4 planes have been copied

; prepare to loop in the next bit plane (every 4th pixel in the row)
	mov	cx, w
	sub	cx, bx
	jbe	L225	    ; jump if no remaining pixels

	mov	di, ofsDest
	add	di, bx	    ; ES:DI -> first byte in bitmap row

	inc	bx
	jmp	L221

; come here when the entire row of pixels has been copied
L225:	mov	cx, nPad
	xor	al, al	    ; AL = 0
	mov	di, ofsDest
	add	di, w	    ; ES:DI -> byte after last bitmap pixel
	rep	stosb	    ; store the padding

; point to the next row of pixels
	mov	si, ofsRow
	sub	si, RowIncr ; DS:SI -> start of next row
	mov	ofsRow, si

	dec	h
	jnz	L220

; restore registers and exit
L290:	pop	di
	pop	si
	pop	ds
	mov	ax, cbLow
	mov	dx, cbHigh  ; DX:AX = bitmap size
	leave
	ret	12	    ; sizeof(LPVOID)+4*sizeof(USHORT)

BLTTORAM256	ENDP


; parameters for all BltToVBuf functions
flag		EQU	WORD PTR [bp+6]
pBMP		EQU	DWORD PTR [bp+8]
y		EQU	WORD PTR [bp+12]
x		EQU	WORD PTR [bp+14]

; BltToVBuf10
;
;   void VS2API
;   BltToVBuf10( USHORT x, USHORT y, LPVOID pBMP, USHORT flag );
;
; Notes:
;   If the flag is nonzero, this routine updates the Attribute
;   Controller palette registers with the color-table values and zeroes
;   Mode Control register bit 7.  (Bits 2-3 of the Color Select register
;   are not modified.)

ofsRow		EQU	WORD PTR [bp-2]
RowIncr 	EQU	WORD PTR [bp-4]
PixShr		EQU	WORD PTR [bp-6]
w		EQU	WORD PTR [bp-8]
h		EQU	WORD PTR [bp-10]
LMask		EQU	WORD PTR [bp-12]
RMask		EQU	WORD PTR [bp-14]
Bits3		EQU	BYTE PTR [bp-15]
Bits2		EQU	BYTE PTR [bp-16]
Bits1		EQU	BYTE PTR [bp-17]
Bits0		EQU	BYTE PTR [bp-18]

		PUBLIC	BLTTOVBUF10
BLTTOVBUF10	PROC	FAR

	enter	18, 0
	push	ds
	push	si
	push	di
	cld

; configure the Graphics Controller for write mode 0, read mode 1
	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr
	mov	ax, 805h    ; AH = 8; AL = Mode register number
	out	dx, ax

	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Func Select reg #
	out	dx, ax

	mov	ax, 7	    ; AH = 0; AL = Color Don't Care reg number
	out	dx, ax

; save the number of bytes per row
	mov	ax, VSP.cbRow
	mov	RowIncr, ax

; compute the address of the first pixel
	push	x
	push	y
	call	PixelAddr10
	mov	di, ax
	mov	es, dx	    ; ES:DI -> upper-left pixel

; get the width, height, and number of colors in the color table
	lds	si, pBMP
	mov	cx, WORD PTR (BITMAPINFOHEADER PTR [si]).biWidth
	mov	w, cx
	mov	ax, WORD PTR (BITMAPINFOHEADER PTR [si]).biHeight
	mov	h, ax
	mov	cx, WORD PTR (BITMAPINFOHEADER PTR [si]).biClrUsed

; point to the last row of the destination
	dec	ax	    ; height - 1
	mul	RowIncr
	add	di, ax	    ; ES:DI -> lower-left pixel
	mov	ofsRow, di

; verify the number of entries in the color table
	or	cx, cx
	jnz	L300

; use the number of bits per pixel to compute the number of colors used
	mov	cx, (BITMAPINFOHEADER PTR [si]).biBitCount
	mov	ax, 1
	shl	ax, cl
	mov	cx, ax	    ; CX = number of colors

; point past the BITMAPINFOHEADER to the color table
L300:	add	si, sizeof BITMAPINFOHEADER

; if the specified flag is zero, don't update the palette registers
	cmp	flag, 0
	jnz	L301

; come here to skip over the color table
	shl	cx, 2	    ; CX = size of color table
	add	si, cx	    ; DS:SI -> bitmap data
	jmp	L310

; come here to update palette registers
L301:	mov	al, 10h     ; AL = 10h (Mode Control reg number)
	call	GetATCReg   ; AL = current value of Mode Control reg
	mov	ah, al
	and	ah, 7Fh     ; AH = new Mode Control reg value
			    ; (bit 7 = 0 for 6-bit palette values)

; set palette registers 0-0Fh with 6-bit values 0-0Fh
	mov	dx, 3C0h    ; DX = 3C0h (Attribute Controller I/O addr)
	mov	al, 0Fh
	call	VNWAIT	    ; wait for vertical nondisplay interval

L303:	cli
	out	dx, al	    ; select a palette register
	out	dx, al	    ; update the register
	sti
	dec	al
	jns	L303

; update the Mode Control register
	mov	al, 30h     ; AL bits 0-4 = 10h (Mode Control reg #)
	cli		    ; AL bit 5 = 1 (Palette Address Source bit)
	out	dx, al
	mov	al, ah	    ; AL = value for Mode Control register
	out	dx, al
	sti

; set DAC registers 0-15 with the values from the BMP color table
	mov	dl, 0C8h    ; DX = 3C8h (DAC write I/O address)
	xor	al, al

	cli
	out	dx, al
	inc	dx	    ; DX = 3C9h (DAC data register I/O addr)

L304:	lodsw
	mov	bx, ax	    ; BL = blue; BH = green
	lodsw		    ; AL = red
	shr	al, 2	    ; convert 8 bits to 6 bits
	out	dx, al
	mov	al, bh	    ; AL = green
	shr	al, 2	    ; convert 8 bits to 6 bits
	out	dx, al
	mov	al, bl	    ; AL = blue
	shr	al, 2	    ; convert 8 bits to 6 bits
	out	dx, al
	loop	L304
	sti

; compute a bit mask for the rightmost pixels
L310:	mov	ax, 0FFh
	mov	cx, x
	add	cx, w
	dec	cx	    ; CL = (x+w-1)
	and	cx, 7	    ; CL = (x+w-1) MOD 8
	inc	cx	    ; CL = (x+w) MOD 8, with 0 mapped to 8
	ror	ax, cl	    ; AH = mask
	mov	al, 8	    ; AL = Bit Mask register number
	mov	RMask, ax

; compute the number of bits to shift pixels rightward
	mov	cx, x
	and	cx, 7	    ; CX = x MOD 8
	mov	PixShr, cx

; compute a bit mask for the leftmost pixels
	mov	ah, 0FFh
	shr	ah, cl	    ; AH = mask
	mov	LMask, ax   ; store mask and Bit Map register number

	add	cx, w
	cmp	cx, 8
	ja	L330	    ; jump if at least 2 bytes of pixels per row

; come here if only one byte of pixels per row
	and	RMask, ax   ; RMask = RMask & LMask

L320:	mov	ax, RMask
	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr
	out	dx, ax

	UnpackPixels10
	call	WritePixels10

	dec	h
	jnz	L321
	jmp	L390	    ; exit if no more rows

; advance to the next row
L321:	mov	di, ofsRow
	sub	di, RowIncr
	mov	ofsRow, di
	jmp	L320

; come here if 2 or more bytes of pixels per row
L330:	push	w	    ; push w (number of pixels per row)

; set the leftmost pixels
	mov	ax, LMask
	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr
	out	dx, ax

	UnpackPixels10
	call	WritePixels10

; compute the number of pixels remaining in the row
	pop	cx
	add	cx, PixShr
	sub	cx, 8	    ; CX = w-(8-PixShr) = # of pixels remaining

; enable all 8 pixel positions
	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr
	mov	ax, 0FF08h  ; AH = 0FFh; AL = Bit Mask register number
	out	dx, ax

; set all but the rightmost pixels
L331:	cmp	cx, 8
	jb	L332	    ; jump if fewer than 8 pixels remain

	push	cx
	UnpackPixels10
	call	WritePixels10
	pop	cx

	sub	cx, 8
	jmp	L331

; set the rightmost pixels
L332:	jcxz	L334	    ; jump if no pixels remain

	mov	ax, RMask
	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr
	out	dx, ax

	cmp	cx, PixShr
	jbe	L333	    ; jump if all pixels are already unpacked

	UnpackPixels10

L333:	call	WritePixels10

L334:	dec	h
	jz	L390	    ; exit if no more rows

; advance to the next row
	mov	di, ofsRow
	sub	di, RowIncr
	mov	ofsRow, di
	jmp	L330

; restore the default Sequencer and Graphics Controller states
L390:	mov	dx, 3C4h
	mov	ax, 0F02h   ; Map Mask
	out	dx, ax

	mov	dl, 0CEh    ; DX = 3CEh
	mov	ax, 3	    ; Data Rotate/Function Select
	out	dx, ax

	mov	al, 5	    ; Mode register
	out	dx, ax

	mov	ax, 0FF08h  ; Bit Mask
	out	dx, ax

; restore registers and exit
	pop	di
	pop	si
	pop	ds
	leave
	ret	10	    ; 3*sizeof(USHORT)+sizeof(LPVOID)

BLTTOVBUF10	ENDP

; BltToVBuf13
;
;   void VS2API
;   BltToVBuf13( USHORT x, USHORT y, LPVOID pBMP, USHORT flag );
;
; Note:
;   If you do not want to support raster ops AND, OR, and XOR, you
;   can replace the loop at L421 with REP MOVSB.

nPad		EQU	WORD PTR [bp-2]
RowIncr 	EQU	WORD PTR [bp-4]
w		EQU	WORD PTR [bp-6]
h		EQU	WORD PTR [bp-8]

		PUBLIC	BLTTOVBUF13
BLTTOVBUF13	PROC	FAR

	enter	8, 0
	push	ds
	push	si
	push	di
	cld

; configure the Graphics Controller
	mov	dx, 3CEh	; DX = Graphics Controller I/O addr
	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Func Select reg #
	out	dx, ax

; save the number of bytes per video-buffer row
	mov	ax, VSP.cbRow
	mov	RowIncr, ax

; compute the address of the first pixel
	push	x
	push	y
	call	PixelAddr13
	mov	di, ax
	mov	es, dx	    ; ES:DI -> upper-left pixel

; get the width, height, and number of colors in the color table
	lds	si, pBMP
	mov	cx, WORD PTR (BITMAPINFOHEADER PTR [si]).biWidth
	mov	w, cx
	mov	ax, WORD PTR (BITMAPINFOHEADER PTR [si]).biHeight
	mov	h, ax
	mov	cx, WORD PTR (BITMAPINFOHEADER PTR [si]).biClrUsed

; point to the last row of the destination
	dec	ax	    ; height - 1
	mul	RowIncr
	add	di, ax	    ; ES:DI -> lower-left pixel

; verify the number of entries in the color table
	or	cx, cx
	jnz	L400

; use the number of bits per pixel to compute the number of colors used
	mov	cx, (BITMAPINFOHEADER PTR [si]).biBitCount
	mov	ax, 1
	shl	ax, cl
	mov	cx, ax	    ; CX = number of colors

; point past the BITMAPINFOHEADER to the color table
L400:	add	si, sizeof BITMAPINFOHEADER

; if the specified flag is zero, don't update the video DAC registers
	cmp	flag, 0
	jnz	L401

; come here to skip over the color table
	shl	cx, 2	    ; CX = size of color table
	add	si, cx	    ; DS:SI -> bitmap data
	jmp	L410

; update DAC registers with the values from the BMP color table
L401:	mov	dx, 3C8h    ; DX = 3C8h (DAC write I/O address)
	xor	al, al
	out	dx, al	    ; start with DAC register 0
	mov	dl, 0C9h    ; DX = 3C9h

	call	VNWAIT	    ; wait for vertical nondisplay interval

L404:	lodsw
	mov	bx, ax	    ; BL = blue; BH = green
	lodsw		    ; AL = red
	shr	al, 2	    ; convert 8 bits to 6 bits
	out	dx, al
	mov	al, bh	    ; AL = green
	shr	al, 2	    ; convert 8 bits to 6 bits
	out	dx, al
	mov	al, bl	    ; AL = blue
	shr	al, 2	    ; convert 8 bits to 6 bits
	out	dx, al
	loop	L404

; compute the increment between rows in the bitmap data
L410:	mov	cx, w
	add	RowIncr, cx ; CX = VSP.cbRow + w

	neg	cx
	and	cx, 3
	mov	nPad, cx    ; CX = # of bytes of padding

; copy each row of the bitmap
L420:	mov	cx, w	    ; CX = # of bytes to copy
L421:	lodsb		    ; AL = next byte
	xchg	es:[di], al ; update the video buffer
	inc	di
	loop	L421

	add	si, nPad    ; DS:SI -> next row of bitmap data

; advance to the next row
	sub	di, RowIncr

	dec	h
	jnz	L420

; restore the default Graphics Controller state
	mov	dx, 3CEh
	mov	ax, 3	    ; Data Rotate/Function Select
	out	dx, ax

; restore registers and exit
	pop	di
	pop	si
	pop	ds
	leave
	ret	10	    ; 3*sizeof(USHORT)+sizeof(LPVOID)

BLTTOVBUF13	ENDP

; BltToVBuf256
;
;   void VS2API
;   BltToVBuf256( USHORT x, USHORT y, LPVOID pBMP, USHORT flag );

ofsRow		EQU	WORD PTR [bp-2]
RowIncr 	EQU	WORD PTR [bp-4]
ofsSrc		EQU	WORD PTR [bp-6]
ofsDest 	EQU	WORD PTR [bp-8]
SrcIncr 	EQU	WORD PTR [bp-10]
MapMask 	EQU	WORD PTR [bp-12]
nPass		EQU	WORD PTR [bp-14]
w		EQU	WORD PTR [bp-16]
h		EQU	WORD PTR [bp-18]

		PUBLIC	BLTTOVBUF256
BLTTOVBUF256	PROC	FAR

	enter	18, 0
	push	ds
	push	si
	push	di
	cld

; configure the Graphics Controller
	mov	dx, 3CEh	; DX = Graphics Controller I/O addr
	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Func Select reg #
	out	dx, ax

; save the number of bytes per video-buffer row
	mov	ax, VSP.cbRow
	mov	RowIncr, ax

; compute the address of the first pixel
	push	x
	push	y
	call	PixelAddr256
	mov	di, ax
	mov	es, dx	    ; ES:DI -> upper-left pixel

; get the width, height, and number of colors in the color table
	lds	si, pBMP
	mov	cx, WORD PTR (BITMAPINFOHEADER PTR [si]).biWidth
	mov	w, cx
	mov	ax, WORD PTR (BITMAPINFOHEADER PTR [si]).biHeight
	mov	h, ax
	mov	cx, WORD PTR (BITMAPINFOHEADER PTR [si]).biClrUsed

; point to the last row of the destination
	dec	ax	    ; height - 1
	mul	RowIncr
	add	di, ax	    ; ES:DI -> lower-left pixel
	mov	ofsRow, di

; verify the number of entries in the color table
	or	cx, cx
	jnz	L500

; use the number of bits per pixel to compute the number of colors used
	mov	cx, (BITMAPINFOHEADER PTR [si]).biBitCount
	mov	ax, 1
	shl	ax, cl
	mov	cx, ax	    ; CX = number of colors

; point past the BITMAPINFOHEADER to the color table
L500:	add	si, sizeof BITMAPINFOHEADER

; if the specified flag is zero, don't update the video DAC registers
	cmp	flag, 0
	jnz	L501

; come here to skip over the color table
	shl	cx, 2	    ; CX = size of color table
	add	si, cx	    ; DS:SI -> bitmap data
	jmp	L510

; update DAC registers with the values from the BMP color table
L501:	mov	dx, 3C8h    ; DX = 3C8h (DAC write I/O address)
	xor	al, al
	out	dx, al	    ; start with DAC register 0
	mov	dl, 0C9h    ; DX = 3C9h

	call	VNWAIT	    ; wait for vertical nondisplay interval

L504:	lodsw
	mov	bx, ax	    ; BL = blue; BH = green
	lodsw		    ; AL = red
	shr	al, 2	    ; convert 8 bits to 6 bits
	out	dx, al
	mov	al, bh	    ; AL = green
	shr	al, 2	    ; convert 8 bits to 6 bits
	out	dx, al
	mov	al, bl	    ; AL = blue
	shr	al, 2	    ; convert 8 bits to 6 bits
	out	dx, al
	loop	L504

; compute the increment between rows in the bitmap data
L510:	mov	cx, w
	add	cx, 3
	and	cl, 0FCh    ; round up to next multiple of 4
	mov	SrcIncr, cx

; determine the bit plane mask for the leftmost pixel
	mov	cx, x
	and	cl, 3	    ; AH = x MOD 4
	mov	ax, 102h    ; AH = 1 (bit mask); AL = Map Mask reg #
	shl	ah, cl	    ; AH = bit mask
	mov	MapMask, ax

	mov	dx, 3C4h    ; DX = Sequencer I/O addr

; come here at the start of each row of pixels
L520:	mov	ax, MapMask ; AH = bit plane mask
			    ; AL = 2 (Map Mask register number)
	mov	ofsDest, di
	mov	ofsSrc, si  ; offset of first byte in video-buffer row
	mov	cx, w	    ; CX = # of bytes (# of pixels) in row
	mov	nPass, 1

; come here at the start of each bit plane
L521:	out	dx, ax	    ; select the next bit plane

; copy every 4th pixel (pixels in the row in the current bit plane)
L522:	mov	bl, [si]
	xchg	es:[di], bl ; update the video buffer
	add	si, 4
	inc	di

	sub	cx, 4
	ja	L522

; set up to copy pixels into the next bit plane
	mov	di, ofsDest ; ES:DI -> first byte in video-buffer row

	shl	ah, 1	    ; AH = next bit plane mask
	and	ah, 0Fh
	jnz	L524	    ; jump if not plane 0

	mov	ah, 1	    ; AH = mask for plane 0
	inc	di	    ; start in plane 0 in next byte
	mov	ofsDest, di

L524:	cmp	ax, MapMask
	je	L525	    ; jump if all 4 planes have been copied

; prepare to loop in the next bit plane (every 4th pixel in the row)
	mov	cx, w
	sub	cx, nPass
	jbe	L525	    ; jump if no remaining pixels

	mov	si, ofsSrc
	add	si, nPass   ; DS:SI -> first byte in bitmap row
	inc	nPass
	jmp	L521

; come here when the entire row of pixels has been copied
L525:	mov	si, ofsSrc
	add	si, SrcIncr ; DS:SI -> 1st bitmap pixel in next row

	mov	di, ofsRow
	sub	di, RowIncr ; ES:DI -> start of next row
	mov	ofsRow, di

	dec	h
	jnz	L520

; restore the default Sequencer and Graphics Controller states
L590:	mov	dx, 3C4h
	mov	ax, 0F02h   ; Map Mask
	out	dx, ax

	mov	dl, 0CEh    ; DX = 3CEh
	mov	ax, 3	    ; Data Rotate/Function Select
	out	dx, ax

; restore registers and exit
	pop	di
	pop	si
	pop	ds
	leave
	ret	10	    ; 3*sizeof(USHORT)+sizeof(LPVOID)

BLTTOVBUF256	ENDP


; parameters for all BltInVBuf functions
h		EQU	WORD PTR [bp+6]
w		EQU	WORD PTR [bp+8]
yFrom		EQU	WORD PTR [bp+10]
xFrom		EQU	WORD PTR [bp+12]
yTo		EQU	WORD PTR [bp+14]
xTo		EQU	WORD PTR [bp+16]

; temporary variables
RowIncr 	EQU	WORD PTR [bp-2]
RowTot		EQU	WORD PTR [bp-4]

; BltInVBuf10
;
;   void VS2API
;   BltInVBuf10( USHORT xTo, USHORT yTo,
;		    USHORT xFrom, USHORT yFrom, USHORT w, USHORT h );
; Notes:
;   The specified x-coordinates are rounded down to the nearest
;   multiple of 8.  The specified width is rounded up to the
;   nearest multiple of 8.

		PUBLIC	BLTINVBUF10
BLTINVBUF10	PROC	FAR

	enter	4, 0
	push	ds
	push	si
	push	di
	cld

; compute the number of bytes per row
	mov	ax, xFrom
	and	ax, 7	    ; AX = xFrom MOD 8
	add	ax, w
	add	ax, 7	    ; AX = (xFrom MOD 8)+w+7
	shr	ax, 3	    ; AX = ((xFrom MOD 8)+w+7)/8
	mov	RowTot, ax

; compute the increment between rows in the video buffer
	mov	bx, VSP.cbRow
	sub	bx, ax	    ; BX = VSP.cbRow - RowTot
	mov	RowIncr, bx

; point to the source and destination bitmaps
	mov	ax, xTo
	and	al, 0F8h
	push	ax
	push	yTo
	call	PixelAddr10
	mov	di, ax
	mov	es, dx	    ; ES:DI -> destination bitmap

	mov	ax, xFrom
	and	al, 0F8h    ; round down to nearest multiple of 8
	push	ax
	push	yFrom
	call	PixelAddr10
	mov	si, ax
	mov	ds, dx	    ; DS:SI -> source bitmap

; configure the Graphics Controller for write mode 1
	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr
	mov	ax, 105h    ; AH = 1; AL = Mode register number
	out	dx, ax

; copy the bitmap, row by row
	mov	dx, h	    ; DX = # of rows
	or	dx, dx
	jz	L690	    ; jump if no rows to copy

	mov	ax, RowIncr
	mov	bx, RowTot

L600:	mov	cx, bx	    ; CX = # of bytes per row
	rep	movsb

	add	si, ax	    ; DS:SI -> start of next source row
	add	di, ax	    ; ES:DI -> start of next destination row
	dec	dx
	jnz	L600

; restore the default Graphics Controller state
L690:	mov	dx, 3CEh
	mov	al, 5	    ; Mode register
	out	dx, ax

	pop	di
	pop	si
	pop	ds
	leave
	ret	12	    ; 6*sizeof(USHORT)

BLTINVBUF10	ENDP

; BltInVBuf13
;
;   void VS2API
;   BltInVBuf13( USHORT xTo, USHORT yTo,
;		    USHORT xFrom, USHORT yFrom, USHORT w, USHORT h );

		PUBLIC	BLTINVBUF13
BLTINVBUF13	PROC	FAR

	enter	4, 0
	push	ds
	push	si
	push	di
	cld

; compute the increment between rows in the video buffer
	mov	bx, VSP.cbRow
	sub	bx, w	    ; BX = VSP.cbRow - w
	mov	RowIncr, bx

; point to the source and destination bitmaps
	push	xTo
	push	yTo
	call	PixelAddr13
	mov	di, ax
	mov	es, dx	    ; ES:DI -> destination bitmap

	push	xFrom
	push	yFrom
	call	PixelAddr13
	mov	si, ax
	mov	ds, dx	    ; DS:SI -> source bitmap

; copy the bitmap, row by row
	mov	dx, h	    ; DX = # of rows
	or	dx, dx
	jz	L790	    ; jump if no rows to copy

	mov	ax, RowIncr
	mov	bx, w

L700:	mov	cx, bx	    ; CX = # of bytes per row
	shr	cx, 1	    ; CX = # of words per row; cf set if odd
	rep	movsw	    ; copy in 2-byte chunks
	rcl	cx, 1	    ; CX = 1 if w is odd
	rep	movsb	    ; copy the odd byte

	add	si, ax	    ; DS:SI -> start of next source row
	add	di, ax	    ; ES:DI -> start of next destination row
	dec	dx
	jnz	L700

; restore registers and return
L790:	pop	di
	pop	si
	pop	ds
	leave
	ret	12	    ; 6*sizeof(USHORT)

BLTINVBUF13	ENDP

; BltInVBuf256
;
;   void VS2API
;   BltInVBuf256( USHORT xTo, USHORT yTo,
;		    USHORT xFrom, USHORT yFrom, USHORT w, USHORT h );
; Notes:
;   The specified x-coordinates are rounded down to the nearest
;   multiple of 4.  The specified width is rounded up to the
;   nearest multiple of 4.

		PUBLIC	BLTINVBUF256
BLTINVBUF256	PROC	FAR

	enter	4, 0
	push	ds
	push	si
	push	di
	cld

; compute the number of bytes per row
	mov	ax, xFrom
	and	ax, 3	    ; AX = xFrom MOD 4
	add	ax, w
	add	ax, 3	    ; AX = (xFrom MOD 4)+w+3
	shr	ax, 2	    ; AX = ((xFrom MOD 4)+w+3)/4
	mov	RowTot, ax

; compute the increment between rows in the video buffer
	mov	bx, VSP.cbRow
	sub	bx, ax	    ; BX = VSP.cbRow - RowTot
	mov	RowIncr, bx

; point to the source and destination bitmaps
	mov	ax, xTo
	and	al, 0FCh    ; round down to nearest multiple of 4
	push	ax
	push	yTo
	call	PixelAddr256
	mov	di, ax
	mov	es, dx	    ; ES:DI -> destination bitmap

	mov	ax, xFrom
	and	al, 0FCh    ; round down to nearest multiple of 4
	push	ax
	push	yFrom
	call	PixelAddr256
	mov	si, ax
	mov	ds, dx	    ; DS:SI -> source bitmap

; configure the Graphics Controller for write mode 1
	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr
	mov	ax, 4105h   ; AH = 41h; AL = Mode register number
	out	dx, ax

; copy the bitmap, row by row
	mov	dx, h	    ; DX = # of rows
	or	dx, dx
	jz	L890	    ; jump if no rows to copy

	mov	ax, RowIncr
	mov	bx, RowTot

L800:	mov	cx, bx	    ; CX = # of bytes per row
	rep	movsb

	add	si, ax	    ; DS:SI -> start of next source row
	add	di, ax	    ; ES:DI -> start of next destination row
	dec	dx
	jnz	L800

; restore the default Graphics Controller state
L890:	mov	dx, 3CEh
	mov	ax, 4005h   ; Mode register
	out	dx, ax

	pop	di
	pop	si
	pop	ds
	leave
	ret	12	    ; 6*sizeof(USHORT)

BLTINVBUF256	ENDP

; GetATCReg
;
; Caller:   AL = register number
; Returns:  AL = current value of register

GetATCReg	PROC	NEAR

	push	dx

	mov	ah, al	    ; AH = register number
	mov	dx, 3DAh    ; CRT Status register I/O addr
	cli
	in	al, dx	    ; reset the Attribute Controller flip-flop
	mov	dl, 0C0h    ; DX = 3C0h (Attribute Controller I/O addr)
	mov	al, ah
	out	dx, al
	inc	dx	    ; DX = 3C1h
	in	al, dx	    ; AL = requested register value
	sti

	push	ax
	mov	dl, 0DAh
	in	al, dx	    ; reset the flip-flop again
	mov	dl, 0C0h
	mov	al, 20h
	out	dx, al	    ; set bit 5 (PAS bit) to keep
	pop	ax	    ;  attribute decoding enabled

	pop	dx
	ret

GetATCReg	ENDP

; GetDACRGB
;
; Caller:   AL = DAC register number
;	    ES:DI -> buffer

GetDACRGB	PROC	NEAR

	push	dx

	mov	ah, al	    ; AH = DAC register number
	mov	dx, 3C7h    ; DX = 3C7h (DAC read I/O address)
	mov	al, ah
	out	dx, al

	mov	dl, 0C9h    ; DX = 3C9h (DAC data register I/O addr)
	in	al, dx	    ; AL = red value
	xor	ah, ah	    ; AH = 0
	shl	ax, 2	    ; convert 6-bit values to 8 bits
	mov	es:[di+2], ax	; store red value and null
	in	al, dx
	mov	ah, al	    ; AH = green value
	in	al, dx	    ; AL = blue value
	shl	ax, 2	    ; convert 6-bit values to 8 bits
	mov	es:[di], ax

	pop	dx
	ret

GetDACRGB	ENDP

; WritePixels10
;
; At entry: DS:SI -> 8 packed 4-bit pixels
;	    ES:DI -> video buffer
; Leaves:   DS:SI -> next 8 packed pixels
;	    ES:DI -> next byte in video buffer
; Destroys: AX, CX

WritePixels10	PROC	NEAR

	mov	cx, PixShr  ; CL = # of bits to shift right

	push	dx
	mov	dx, 3C4h    ; DX = Sequencer I/O addr
	mov	ax, 802h    ; AH = mask for bit plane 3
	out	dx, ax
	pop	dx

	mov	ah, Bits3
	mov	al, dh
	mov	Bits3, dh   ; save new bits for plane 3
	shr	ax, cl
	and	es:[di], al ; update bit plane 3

	push	dx
	mov	dx, 3C4h
	mov	ax, 402h    ; AH = mask for bit plane 2
	out	dx, ax
	pop	dx

	mov	ah, Bits2
	mov	al, dl
	mov	Bits2, dl   ; save new bits for plane 2
	shr	ax, cl
	and	es:[di], al ; update bit plane 2

	mov	dx, 3C4h
	mov	ax, 202h    ; AH = mask for bit plane 1
	out	dx, ax

	mov	ah, Bits1
	mov	al, bh
	mov	Bits1, bh   ; save new bits for plane 1
	shr	ax, cl
	and	es:[di], al ; update bit plane 1

	mov	ax, 102h    ; AH = mask for bit plane 0
	out	dx, ax

	mov	ah, Bits0
	mov	al, bl
	mov	Bits0, bl   ; save new bits for plane 0
	shr	ax, cl
	and	es:[di], al ; update bit plane 0

	inc	di	    ; point to the next byte in video buffer
	ret

WritePixels10	ENDP

VS2_TEXT	ENDS

_DATA		SEGMENT WORD PUBLIC 'DATA'

		EXTRN	VSP:VIDEOSTATEPARAMS

_DATA		ENDS

		END
