		TITLE	GLYPHBLT.ASM
		NAME	GLYPHBLT
		PAGE	55,132

.286

INCLUDE 	vs2.inc

DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

		EXTRN	PIXELADDR10:FAR
		EXTRN	PIXELADDR13:FAR
		EXTRN	PIXELADDR256:FAR

; parameters for all GlyphBlt functions
flag		EQU	WORD PTR [bp+6]
h		EQU	WORD PTR [bp+8]
w		EQU	WORD PTR [bp+10]
pBits		EQU	DWORD PTR [bp+12]
bkgd		EQU	BYTE PTR [bp+16]
fgd		EQU	BYTE PTR [bp+18]
y		EQU	WORD PTR [bp+20]
x		EQU	WORD PTR [bp+22]

; GlyphBlt10
;
;   void VS2API
;   GlyphBlt10( USHORT x, USHORT y, UCHAR fgd, UCHAR bkgd,
;		LPVOID pBits, USHORT w, USHORT h, USHORT flag );
;
; Notes:
;   If the specified flag word is zero, this routine updates foreground
;   pixels only.  If the flag is nonzero, this routine updates both
;   foreground and background pixels.

; temporary variables
LMask		EQU	BYTE PTR [bp-1]
RMask		EQU	BYTE PTR [bp-2]
ofsUL		EQU	WORD PTR [bp-4]
cbPix		EQU	WORD PTR [bp-6]
PixRot		EQU	WORD PTR [bp-8]

		PUBLIC	GLYPHBLT10
GLYPHBLT10	PROC	FAR

	enter	8, 0
	push	ds
	push	si
	push	di
	cld

; configure the Graphics Controller for write mode 3
	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr
	mov	ax, 305h    ; AH = 3; AL = Mode register number
	out	dx, ax

	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Func Select reg #
	out	dx, ax

	mov	ax, 0F01h   ; AH = 0Fh; AL = Enable Set/Reset reg #
	out	dx, ax

; compute bit masks and bit-shift value
	mov	di, x	    ; DI = x-coordinate of leftmost pixel
	mov	si, di
	add	si, w	    ; SI = w + x
	dec	si	    ; SI = w + x - 1
			    ;	 = x-coordinate of rightmost pixel
	mov	cx, di
	and	cx, 7	    ; CX = x MOD 8
	mov	PixRot, cx

	mov	bx, 0FF80h  ; BH = 0FFh; BL = 80h
	shr	bh, cl	    ; BH = bit mask for left-hand pixels

	mov	cx, si
	and	cl, 7	    ; CL = (w+x-1) MOD 8
	sar	bl, cl	    ; BL = bit mask for right-hand pixels
	mov	WORD PTR RMask, bx  ; save RMask and LMask

; compute the pixel offsets in the video buffer
	shr	si, 3	    ; SI = byte offset of rightmost pixel
	shr	di, 3	    ; DI = byte offset of leftmost pixel
	sub	si, di	    ; SI = # of bytes in the video-buffer row
	mov	cbPix, si

	push	0
	push	y
	call	PixelAddr10
	mov	es, dx
	add	di, ax	    ; ES:DI -> leftmost pixel
	mov	ofsUL, di   ; save offset of upper-left pixel

; compute an increment between video-buffer rows
	mov	bx, VSP.cbRow
	sub	bx, si	    ; BX = increment

	cmp	flag, 0
	je	L010	    ; jump to skip the background

; put the background pixel value in the Set/Reset register
	mov	dx, 3CEh
	mov	ah, bkgd    ; AH = background pixel value
	xor	al, al	    ; AL = 0 (Set/Reset reg #)
	out	dx, ax

; fill the background pixels
	mov	dx, h	    ; DX = # of rows

	or	si, si
	jz	L005	    ; jump if only one byte of pixels in the row

; fill the leftmost pixel(s)
L000:	mov	cx, si	    ; CX = # of bytes to fill
	mov	al, LMask
	xchg	es:[di], al

	inc	di	    ; ES:DI -> next byte in the row
	dec	cx
	jz	L002	    ; jump if only 2 bytes of pixels in the row

; fill the middle pixels
L001:	mov	al, 0FFh
	xchg	es:[di], al
	inc	di
	loop	L001

; fill the rightmost pixel(s)
L002:	mov	al, RMask
	xchg	es:[di], al

; loop down
	add	di, bx	    ; ES:DI -> start of next row
	dec	dx
	jnz	L000	    ; jump if no more background rows
	jmp	L010

; come here if only one byte in each row
L005:	mov	ah, LMask
	and	ah, RMask   ; AH = mask for the byte

	mov	cx, dx	    ; CX = # of rows

L006:	mov	al, ah	    ; AL = pixel mask
	xchg	es:[di], al ; update the video buffer
	add	di, bx	    ; ES:DI -> start of next row
	loop	L006	    ; jump if no more background rows

; point to the source and destination addresses
L010:	mov	di, ofsUL   ; ES:DI -> upper left corner in video buffer
	lds	si, pBits   ; DS:SI -> upper left corner of glyphs

; put the foreground pixel value in the Set/Reset register
	mov	dx, 3CEh
	mov	ah, fgd     ; AH = foreground pixel value
	xor	al, al	    ; AL = 0 (Set/Reset reg #)
	out	dx, ax

	mov	cx, PixRot
	or	cx, cx
	jnz	L040	    ; jump if not byte-aligned

; come here if destination pixels are byte-aligned
	mov	dx, cbPix
	or	dx, dx
	jnz	L030	    ; jump if more than 1 byte per row

; come here if only 1 byte per row
	mov	ah, LMask
	and	ah, RMask   ; AH = mask for the byte
	mov	cx, h	    ; CX = # of rows

L020:	lodsb		    ; AL = glyph bits
	and	al, ah	    ; AL = mask for pixels
	xchg	es:[di], al ; update the video buffer
	add	di, bx	    ; ES:DI -> next row of pixels
	loop	L020
	jmp	L090

; come here if more than 1 byte per row
L030:	mov	cx, dx
	shr	cx, 1	    ; CX = # of words to copy
	jz	L033	    ; jump if only 1 byte to copy

; copy 2 bytes at a time
L032:	lodsw		    ; AL = bits for nth byte
			    ; AH = bits for (n+1)th byte
	xchg	es:[di], al ; update the video buffer
	xchg	es:[di+1], ah
	add	di, 2
	loop	L032

	test	dl, 1
	jz	L034	    ; jump if even number of bytes per row

; copy the odd byte
L033:	lodsb
	xchg	es:[di], al
	inc	di

; copy the rightmost byte
L034:	lodsb
	and	al, RMask
	xchg	es:[di], al

; increment to the next video-buffer row
L035:	add	di, bx	    ; ES:DI -> next video-buffer row
	dec	h
	jnz	L030	    ; jump if more rows to copy
	jmp	L090	    ; exit if done

; come here if destination pixels are not byte-aligned
L040:	mov	dx, cbPix
	or	dx, dx
	jnz	L050	    ; jump if more than 1 byte per row

; come here if only 1 byte per row
	mov	ah, LMask
	and	ah, RMask   ; AH = mask for the byte

L042:	lodsb		    ; AL = glyph bits
	shr	al, cl	    ; AL = bits in proper alignment
	and	al, ah	    ; AL = mask for pixels
	xchg	es:[di], al ; update the video buffer
	add	di, bx	    ; ES:DI -> next row of pixels
	dec	h
	jnz	L042
	jmp	L090

; come here if more than 1 byte per row
L050:	mov	al, [si]    ; AL = leftmost glyph bits
	shr	al, cl	    ; AL = bits in proper alignment
	and	al, LMask   ; AL = mask for leftmost pixels
	xchg	es:[di], al
	inc	di

	mov	dx, w
	sub	dx, 8
	add	dx, cx	    ; DX = w - (# of leftmost pixels)
	cmp	dx, 8
	jbe	L054	    ; jump if 8 or fewer pixels remain

; copy all but last byte
L052:	mov	ax, [si]
	inc	si
	ror	ax, cl	    ; AH = rotated bits
	xchg	es:[di], ah ; update the video buffer
	inc	di

	sub	dx, 8
	cmp	dx, 8
	ja	L052	    ; jump if more than 8 pixels remain

; rightmost pixels
L054:	lodsw		    ; AX = remaining glyph bits

	cmp	cx, dx	    ; decrement SI if CX > DX (that is, if an
	adc	si, -1	    ;  extra glyph-bit byte was read)

	ror	ax, cl	    ; AH = rotated bits
	and	ah, RMask
	xchg	es:[di], ah ; update the video buffer

	add	di, bx	    ; ES:DI -> next video-buffer row
	dec	h
	jnz	L050

; restore the default Graphics Controller state
L090:	mov	dx, 3CEh
	mov	ax, 1	    ; Enable Set/Reset
	out	dx, ax

	mov	al, 3	    ; Data Rotate/Function Select
	out	dx, ax

	mov	al, 5	    ; Mode
	out	dx, ax

; restore registers and exit
	pop	di
	pop	si
	pop	ds
	leave
	ret	18	    ; 7*sizeof(USHORT)+sizeof(LPVOID)

GLYPHBLT10	ENDP

; GlyphBlt13
;
;   void VS2API
;   GlyphBlt13( USHORT x, USHORT y, USHORT fgd, USHORT bkgd,
;		LPVOID pBits, USHORT w, USHORT h, USHORT flags );

; temporary variables
ofsUL		EQU	WORD PTR [bp-2]

		PUBLIC	GLYPHBLT13
GLYPHBLT13	PROC	FAR

	enter	2, 0
	push	ds
	push	si
	push	di
	cld

; configure the Graphics Controller
	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr
	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Func Select reg #
	out	dx, ax

; compute the starting pixel offset in the video buffer
	push	x
	push	y
	call	PixelAddr13
	mov	es, dx
	mov	di, ax	    ; ES:DI -> leftmost pixel
	mov	ofsUL, ax   ; save offset of upper-left pixel

; compute an increment between video-buffer rows
	mov	bx, VSP.cbRow
	sub	bx, w	    ; BX = increment

	cmp	flag, 0
	je	L110	    ; jump to skip the background

; fill the background pixels
	mov	ah, bkgd    ; AH = background pixel value
	mov	dx, h	    ; DX = # of rows
	mov	si, w	    ; SI = # of pixels per row

L100:	mov	cx, si	    ; CX = # of bytes to fill

L102:	mov	al, ah
	xchg	es:[di], al
	inc	di
	loop	L102

; loop down
	add	di, bx	    ; ES:DI -> start of next row
	dec	dx
	jnz	L100	    ; jump if no more background rows

; point to the source and destination addresses
L110:	mov	di, ofsUL   ; ES:DI -> upper left corner in video buffer
	lds	si, pBits   ; DS:SI -> upper left corner of glyphs

	mov	ah, fgd     ; AH = foreground pixel value

; come here at the start of each row of pixels
L130:	mov	cx, w	    ; number of pixels to copy

L132:	mov	dl, 8	    ; DL = # of bits to copy
	lodsb		    ; AL = 8 bits for 8 pixels
	or	al, al	    ; set flags
	jz	L135	    ; jump if no pixels to set

L133:	jns	L134
	mov	dh, ah	    ; DH = foreground pixel value
	xchg	es:[di], dh

L134:	inc	di
	dec	dx	    ; DL = # of bits remaining
	shl	al, 1
	loopnz	L133	    ; decrement CX; jump if AL <> 0

L135:	xor	dh, dh	    ; DX = # of pixels left over
	add	di, dx	    ; ES:DI -> next pixel to set
	sub	cx, dx	    ; CX = # of pixels left in row
	ja	L132

	add	di, cx	    ; back up if incremented past end of row

; loop down
	add	di, bx	    ; ES:DI -> start of next row
	dec	h
	jnz	L130

; restore the default Graphics Controller state
	mov	dx, 3CEh
	mov	ax, 3	    ; Data Rotate/Function Select
	out	dx, ax

; restore registers and exit
	pop	di
	pop	si
	pop	ds
	leave
	ret	18	    ; 7*sizeof(USHORT)+sizeof(LPVOID)

GLYPHBLT13	ENDP

; GlyphBlt256
;
;   void VS2API
;   GlyphBlt256( USHORT x, USHORT y, USHORT fgd, USHORT bkgd,
;		LPVOID pBits, USHORT w, USHORT h, USHORT flags );
; Notes:
;   This routine assumes DS == SS.  (The default data segment is
;   addressed through the SS register.)

; temporary variables
LMask		EQU	WORD PTR [bp-2]
RMask		EQU	WORD PTR [bp-4]
ofsUL		EQU	WORD PTR [bp-6]
cbPix		EQU	WORD PTR [bp-8]
RowIncr 	EQU	WORD PTR [bp-10]
nRows		EQU	WORD PTR [bp-12]
nInRow		EQU	WORD PTR [bp-14]
PixRot		EQU	WORD PTR [bp-16]

		PUBLIC	GLYPHBLT256
GLYPHBLT256	PROC	FAR

	enter	16, 0
	push	ds
	push	si
	push	di
	cld

; configure the Graphics Controller
	mov	dx, 3CEh    ; DX = Graphics Controller I/O addr
	mov	ah, VSP.cRMW	; AH = Read-Modify-Write bits
	mov	al, 3		; AL = Data Rotate/Func Select reg #
	out	dx, ax

; compute bit masks and bit-shift value
	mov	di, x	    ; DI = x-coordinate of leftmost pixel
	mov	si, di
	add	si, w	    ; SI = w + x
	dec	si	    ; SI = w + x - 1
			    ;	 = x-coordinate of rightmost pixel
	mov	cx, di
	and	cx, 3	    ; CX = x MOD 4
	mov	PixRot, cx

	mov	bx, 0F02h   ; BH = 0Fh; BL = 2 (Map Mask reg #)
	shl	bh, cl
	and	bh, 0Fh     ; BH = bit mask for left-hand pixels
	mov	LMask, bx

	mov	cx, si
	and	cl, 3	    ; CL = (w+x-1) MOD 4
	mov	bh, 0E1h
	rol	bh, cl
	and	bh, 0Fh     ; BH = bit mask for right-hand pixels
	mov	RMask, bx

; compute the pixel offsets in the video buffer
	shr	si, 2	    ; SI = byte offset of rightmost pixel
	shr	di, 2	    ; DI = byte offset of leftmost pixel
	sub	si, di	    ; SI = # of bytes in the video-buffer row
	mov	cbPix, si

	push	0
	push	y
	call	PixelAddr256
	mov	es, dx
	add	di, ax	    ; ES:DI -> leftmost pixel
	mov	ofsUL, di   ; save offset of upper-left pixel

; compute an increment between video-buffer rows
	mov	bx, VSP.cbRow
	sub	bx, si
	mov	RowIncr, bx

	mov	dx, 3C4h    ; DX = Sequencer I/O addr

	cmp	flag, 0
	je	L210	    ; jump to skip the background

; fill the background pixels
	mov	bh, bkgd    ; BH = background pixel value

	or	si, si
	jz	L205	    ; jump if only one byte of pixels in the row

	push	h
	pop	nRows	    ; save # of rows in a memory variable

; fill the leftmost pixel(s)
L200:	mov	cx, si	    ; CX = # of bytes to fill
	mov	ax, LMask
	out	dx, ax

	mov	bl, bh
	xchg	es:[di], bl ; update the video buffer

	inc	di	    ; ES:DI -> next byte in the row
	dec	cx
	jz	L202	    ; jump if only 2 bytes of pixels in the row

; fill the middle pixels
	mov	ax, 0F02h   ; enable all 4 bit planes
	out	dx, ax

L201:	mov	bl, bh
	xchg	es:[di], bl
	inc	di
	loop	L201

; fill the rightmost pixel(s)
L202:	mov	ax, RMask
	out	dx, ax
	mov	bl, bh
	xchg	es:[di], bl

; loop down
	add	di, RowIncr ; ES:DI -> start of next row
	dec	nRows
	jnz	L200	    ; jump if no more background rows
	jmp	L210

; come here if only one byte in each row
L205:	mov	ax, LMask
	and	ax, RMask   ; AH = mask for the byte
	mov	al, 2	    ; Map Mask register number
	out	dx, ax

	mov	cx, h	    ; CX = # of rows
	mov	dx, RowIncr

L206:	mov	bl, bh	    ; AL = foreground pixel value
	xchg	es:[di], bl ; update the video buffer
	add	di, dx	    ; ES:DI -> start of next row
	loop	L206	    ; jump if no more background rows

; point to the source and destination addresses
L210:	mov	di, ofsUL   ; ES:DI -> upper left corner in video buffer
	lds	si, pBits   ; DS:SI -> upper left corner of glyphs

	mov	bx, offset Reverse
	mov	dx, 3C4h    ; DX = Sequencer I/O addr

	mov	cx, PixRot
	or	cx, cx
	jz	L220
	jmp	L240	    ; jump if not byte-aligned

; come here if destination pixels are byte-aligned
L220:	cmp	cbPix, 0
	jne	L230	    ; jump if more than 1 byte per row

; come here if only 1 byte per row
	mov	ax, RMask
	and	LMask, ax

	mov	cx, h	    ; CX = # of rows

L221:	lodsb		    ; AL = glyph bits
	shr	al, 4
	jz	L222	    ; jump if no bits to set

	xlat	ss:[bx]     ; AL = reversed bits
	mov	ah, al
	and	ax, LMask   ; AH = bit-plane mask
	mov	al, 2	    ; Map Mask register number
	out	dx, ax
	mov	al, fgd
	xchg	es:[di], al ; update the video buffer

; loop down
L222:	add	di, RowIncr ; ES:DI -> next row of pixels
	loop	L221
	jmp	L290

; come here if more than 1 byte per row
L230:	mov	cx, cbPix   ; CX = (# of video-buffer bytes per row) - 1

L232:	mov	al, [si]    ; AL = bits for nth byte
	shr	al, 4
	jz	L233	    ; jump if left 4 pixels are 0

	xlat	ss:[bx]     ; AL = reversed bits
	mov	ah, al
	mov	al, 2	    ; Map Mask register number
	out	dx, ax
	mov	al, fgd
	xchg	es:[di], al ; update the video buffer

L233:	inc	di	    ; ES:DI -> next byte in video buffer
	dec	cx
	jz	L235	    ; jump if end of row

	lodsb		    ; get the bits again
	and	al, 0Fh
	jz	L234	    ; jump if right 4 pixels are 0

	xlat	ss:[bx]     ; AL = reversed bits
	mov	ah, al
	mov	al, 2	    ; Map Mask register number
	out	dx, ax
	mov	al, fgd
	xchg	es:[di], al ; update the video buffer

L234:	inc	di
	loop	L232

; copy the last byte
L235:	lodsb		    ; AL = last byte of glyph bits
	test	cbPix, 1
	jnz	L236	    ; jump to use right nibble

	shr	al, 4	    ; AL = left nibble

L236:	and	al, 0Fh
	jz	L238

	xlat	ss:[bx]     ; AL = reversed bits
	mov	ah, al
	and	ax, RMask
	mov	al, 2	    ; Map Mask register number
	out	dx, ax
	mov	al, fgd
	xchg	es:[di], al ; update the video buffer

; increment to the next video-buffer row
L238:	add	di, RowIncr ; ES:DI -> next video-buffer row
	dec	h
	jnz	L230	    ; jump if more rows to copy
	jmp	L290	    ; exit if done

; come here if destination pixels are not byte-aligned
L240:	cmp	cbPix, 0
	jne	L250	    ; jump if more than 1 byte per row

; come here if only 1 byte per row
	mov	ax, RMask
	and	LMask, ax

	mov	ch, fgd

L241:	lodsb		    ; AL = glyph bits
	shr	al, 4
	jz	L242	    ; jump if no bits to set

	xlat	ss:[bx]     ; AL = reversed bits
	mov	ah, al
	shl	ah, cl	    ; AH = shifted bits
	and	ax, LMask   ; AH = bit-plane mask
	mov	al, 2	    ; Map Mask register number
	out	dx, ax
	mov	al, ch
	xchg	es:[di], al ; update the video buffer

; loop down
L242:	add	di, RowIncr ; ES:DI -> next row of pixels
	dec	h
	jnz	L241
	jmp	L290

; come here if more than 1 byte per row
L250:	mov	al, [si]    ; AL = leftmost glyph bits
	shr	al, 4
	shr	al, cl	    ; AL bits 0-3 = shifted glyph bits
	jz	L251	    ; jump if leftmost bits are 0

	xlat	ss:[bx]     ; AL = reversed bits
	mov	ah, al
	and	ax, LMask   ; AH = bit-plane mask
	mov	al, 2	    ; Map Mask register number
	out	dx, ax
	mov	al, fgd
	xchg	es:[di], al ; update the video buffer

L251:	inc	di	    ; ES:DI -> next byte in video buffer

	mov	ax, w
	sub	ax, 4
	add	ax, cx	    ; AX = w - (# of leftmost pixels)
	mov	nInRow, ax
	cmp	ax, 4
	jbe	L255	    ; jump if 4 or fewer pixels remain

; copy all but the last byte of pixels
L252:	mov	al, [si]
	shr	al, cl	    ; AL bits 0-3 = shifted glyph bits
	and	al, 0Fh
	jz	L253	    ; jump if next 4 bits are 0

	xlat	ss:[bx]     ; AL = reversed bits
	mov	ah, al
	mov	al, 2	    ; Map Mask register number
	out	dx, ax
	mov	al, fgd
	xchg	es:[di], al ; update the video buffer

L253:	inc	di	    ; ES:DI -> next byte in video buffer
	sub	nInRow, 4
	cmp	nInRow, 4
	jbe	L255	    ; jump if 4 or fewer pixels remain

	mov	ax, [si]
	inc	si
	ror	ax, cl
	rol	ax, 4
	and	al, 0Fh     ; AL bits 0-3 = shifted glyph bits
	jz	L254	    ; jump if next 4 bits are 0

	xlat	ss:[bx]     ; AL = reversed bits
	mov	ah, al
	mov	al, 2	    ; Map Mask register number
	out	dx, ax
	mov	al, fgd
	xchg	es:[di], al ; update the video buffer

L254:	inc	di
	sub	nInRow, 4
	cmp	nInRow, 4
	ja	L252	    ; jump if more than 4 pixels remain

; rightmost pixels
L255:	lodsb		    ; AX = last glyph-bits byte

	test	cbPix, 1
	jnz	L257	    ; jump if odd # of bytes copied
			    ;  (AL bits 4-7 = remaining bits)

	mov	ah, al	    ; AH bits 0-3 = remaining bits

	cmp	cx, nInRow
	jae	L256	    ; jump if all glyph bits have been read

	lodsb

L256:	shr	ax, 4	    ; AL bits 4-7 = desired bits

L257:	shr	al, cl	    ; AL bits 0-3 = shifted glyph bits

	and	al, 0Fh
	jz	L258	    ; jump if remaining bits are 0

	xlat	ss:[bx]     ; AL = reversed bits
	mov	ah, al
	and	ax, RMask
	mov	al, 2	    ; Map Mask register number
	out	dx, ax
	mov	al, fgd
	xchg	es:[di], al ; update the video buffer

; increment to the next video-buffer row
L258:	add	di, RowIncr ; ES:DI -> next video-buffer row
	dec	h
	jz	L290
	jmp	L250	    ; jump if more rows to copy

; restore the default Sequencer and Graphics Controller states
L290:	mov	dx, 3C4h
	mov	ax, 0F02h   ; AH = 0, AL = 2
	out	dx, ax	    ; Map Mask

	mov	dl, 0CEh    ; DX = 3CEh
	mov	ax, 3	    ; Data Rotate/Function Select
	out	dx, ax

; restore registers and exit
	pop	di
	pop	si
	pop	ds
	leave
	ret	18	    ; 7*sizeof(USHORT)+sizeof(LPVOID)

GLYPHBLT256	ENDP

VS2_TEXT	ENDS

_DATA		SEGMENT WORD PUBLIC 'DATA'

		EXTRN	VSP:VIDEOSTATEPARAMS

; table of reversed bits for GlyphBlt256
Reverse 	BYTE	000h, 008h, 004h, 00Ch, 002h, 00Ah, 006h, 00Eh
		BYTE	001h, 009h, 005h, 00Dh, 003h, 00Bh, 007h, 00Fh

_DATA		ENDS

		END
