/***********************************************************************

  ALPHABLT.C

***********************************************************************/

#include    <stdio.h>	    /* printf _lseek */
#include    <conio.h>	    /* _getch */
#include    <malloc.h>	    /* malloc free */
#include    <fcntl.h>	    /* _open */
#include    <sys\types.h>   /* _open */
#include    <sys\stat.h>    /* _open */
#include    <io.h>	    /* _open _close */
#include    <string.h>	    /* strlen */
#include    <vs2.h>


/***********************************************************************

  TYPEDEFS etc.

***********************************************************************/

typedef struct
{
    USHORT  bfType;
    ULONG   bfSize;
    USHORT  bfReserved1;
    USHORT  bfReserved2;
    ULONG   bfOffBits;
}
  BITMAPFILEHEADER;
typedef BITMAPFILEHEADER *	PBITMAPFILEHEADER;

typedef struct
{
    ULONG   biSize;
    ULONG   biWidth;
    ULONG   biHeight;
    USHORT  biPlanes;
    USHORT  biBitCount;
    ULONG   biCompression;
    ULONG   biSizeImage;
    ULONG   biXPelsPerMeter;
    ULONG   biYPelsPerMeter;
    ULONG   biClrUsed;
    ULONG   biClrImportant;
}
  BITMAPINFOHEADER;
typedef BITMAPINFOHEADER *	PBITMAPINFOHEADER;


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void		    main( int, char * * );
PBITMAPFILEHEADER   GetBMP( char * );
void		    InitVMode( USHORT );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

static char	szName[] = "?.BMP";


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    USHORT		usMode, i, w, h, x, y, usNext;
    PBITMAPFILEHEADER	pbmfh;
    PBITMAPINFOHEADER	pbmih;
    LPVDDA		lpVDDA = ADDRVDDA;
    char		sz[80];


    /* verify the video mode */
    usMode = GetVMode();
    if( (usMode > 3) && (usMode != 7) )
    {
      printf( "This program runs in alphanumeric modes only\n" );
      return;
    }

    /* verify the command-line parameters */
    if( argc != 2 )
    {
      printf( "Syntax: ALPHABLT <filename>\n" );
      return;
    }

    /* display the specified bitmap in the center of the screen */
    InitVMode( usMode );
    pbmfh = GetBMP( argv[1] );
    if( NULL == pbmfh )
      return;

    /* validate the bitmap info */
    pbmih = (PBITMAPINFOHEADER)(pbmfh+1);
    if( pbmih->biBitCount != 1 )
      printf( "'%s' is not a monochrome bitmap\n", szName );

    /* draw the bitmap in the center of the screen */
    w = (USHORT)((pbmih->biWidth+7) / 8);
    h = (USHORT)((pbmih->biHeight+lpVDDA->Points-1) /
						lpVDDA->Points);
    x = (vsp.usSX - w) / 2;
    y = (vsp.usSY - h) / 2;
    BltToCGenRAM( x, y, pbmih, 0x07, 0 );

    /* display a caption */
    sprintf( sz, "%s: %ldx%ld pixels (%ux%u characters)",
		argv[1], pbmih->biWidth, pbmih->biHeight, w, h );
    TextAt( (vsp.usSX-strlen(sz))/2, y+h+1, sz, 0x07 );

    /* discard the memory used for the bitmap */
    free( pbmfh );
    _getch();

    /* display five smaller bitmaps */
    InitVMode( usMode );
    usNext = 0;
    for( i=0; i<5; i++ )
    {
      /* move the alphanumeric cursor */
      SetCursorXY( 5, i*4 );

      /* load a bitmap into memory */
      szName[0] = (char)(i+'A');	/* change the 1st letter */
      pbmfh = GetBMP( szName );
      if( NULL == pbmfh )
	return;

      /* validate the bitmap info */
      pbmih = (PBITMAPINFOHEADER)(pbmfh+1);
      if( pbmih->biBitCount != 1 )
	printf( "'%s' is not a monochrome bitmap\n", szName );

      else
      {
	/* compute the bitmap size in characters */
	w = (USHORT)((pbmih->biWidth+7) / 8);
	h = (USHORT)((pbmih->biHeight+lpVDDA->Points-1) /
						    lpVDDA->Points);

	/* display the bitmap */
	usNext = BltToCGenRAM( 0, i*4, pbmih, 0x16, usNext );
	sprintf( sz, "%s: %ldx%ld pixels (%ux%u characters)",
		    szName, pbmih->biWidth, pbmih->biHeight, w, h );
	TextAt( w+2, i*4+1, sz, 0x07 );

	/* free the memory used for the bitmap */
	free( pbmfh );
      }

      _getch();
    }

    /* that's all, folks */
    SetCursorXY( 0, 20 );
    SetCursorSize( 0x06, 0x07 );	/* default cursor */
}


/***********************************************************************

  GetBMP

***********************************************************************/

PBITMAPFILEHEADER GetBMP( char * szName )
{
    int 		hBMP;
    PBITMAPFILEHEADER	pbmfh;
    USHORT		usFileSize;


    /* open the file */
    hBMP = _open( szName, _O_BINARY | _O_RDONLY );
    if( -1 == hBMP )
    {
      printf( "Unable to open file '%s'\n", szName );
      return NULL;
    }

    /* determine the size of the file */
    usFileSize = (USHORT)_lseek( hBMP, 0L, SEEK_END );
    _lseek( hBMP, 0L, SEEK_SET );

    /* allocate memory for the bitmap */
    pbmfh = (PBITMAPFILEHEADER)malloc( usFileSize );
    if( NULL == pbmfh )
      printf( "Unable to allocate %u bytes for bitmap\n", usFileSize );

    else
    {
      /* load the bitmap into memory */
      if( usFileSize != (USHORT)_read( hBMP, pbmfh, usFileSize ) )
	printf( "Error reading '%s'\n", szName );
    }

    _close( hBMP );
    return pbmfh;
}


/***********************************************************************

  InitVMode

***********************************************************************/

void InitVMode( USHORT usMode )
{
    SetVMode( usMode );
    InitVSP( usMode );
    SetCursorSize( 0x20, 0x00 );	/* invisible cursor */

    /* use an 8-bit wide font */
    _asm
    {
      mov   ax, 1104h	    ; use ROM BIOS 8-by-16 characters
      mov   bl, 0	    ; use character-generator glyph table 0
      int   10h
    }
}
