		TITLE	CSETTSR.ASM
		NAME	CSETTSR
		PAGE	55,132

.286

MULTIPLEX_ID	EQU	0E9h
STDOUT		EQU	1
LF		EQU	0Ah
CR		EQU	0Dh


DGROUP		GROUP	_TEXT, _DATA

_TEXT		SEGMENT PARA PUBLIC 'CODE'
		ASSUME	ds:DGROUP

; ISR10 -- interrupt 10h service routine

		ALIGN	DWORD
ISR10		PROC	FAR

; trap int 10h function 0 for alphanumeric modes 0-3
	cmp	ax, 3
	jbe	L001

; jump to previous int 10h handler
L000::	jmp	FAR PTR L099	; (fixed up at load time)

; come here for alphanumeric modes
L001:	pushf			; set the specified mode
L002::	call	FAR PTR L099	; (fixed up at load time)

; reload the character-set defs
	mov	ax, ((MULTIPLEX_ID SHL 8) + 1)
	int	2Fh

L099:	iret

ISR10		ENDP

;
; ISR2F -- interrupt 2Fh service routine
;
; Caller:   AH = multiplex ID
;	    AL = function number

		ALIGN	DWORD
ISR2F		PROC	FAR

; look for this ISR's magic ID number
	cmp	ah, MULTIPLEX_ID
	je	L101

; jump to previous int 2F handler
L100::	jmp	FAR PTR L199	; (fixed up at load time)

; come here if it's our multiplex ID
L101:	or	al, al
	jnz	L110		; jump if AL <> 0

; Function 0: get installed state
;
; Notes:
;   Returns: AL = 0FFh
;	     BX = resident PSP segment
;	     CX = resident DGROUP

	dec	al		; AL = 0FFh
	mov	cx, cs		; CX = resident data seg
	mov	bx, cs:TSRPSP	; BX = PSP segment
	jmp	L199

L110:	cmp	al, 1
	jne	L199		; jump if AL <> 1

; Function 1: load character definitions
;
; Notes:
;   Loads a 4096-byte binary file containing character definitions
;   for all 256 ASCII characters (16 bytes per character)

; preserve regs
	pusha
	push	ds
	push	es

; open the file
	push	seg DGROUP
	pop	ds
	mov	dx, offset FileName

	mov	ax, 3D00h   ; AH = 3Dh (open file)
			    ; AL = 0 (read only)
	int	21h
	jc	L105	    ; jump if error
	mov	hFile, ax   ; save the file handle

; load the file into memory
	mov	bx, ax
	mov	cx, 256*16
	mov	dx, offset CSetDefs
	mov	ah, 3Fh     ; (read file)
	int	21h

; close the file
	mov	bx, hFile
	mov	ah, 3Eh     ; (close file)
	int	21h

; load the character set into video RAM
	push	bp
	mov	ax, 1100h   ; AH = 11h; AL = 0
	mov	bx, 1000h   ; BH = 16 (points)
			    ; BL = 0 (table)
	mov	cx, 256     ; # of characters
	mov	dx, 0	    ; first character
	push	seg DGROUP
	pop	es
	mov	bp, offset CSetDefs
	int	10h
	pop	bp

	jmp	L109

; come here if error opening file
L105:	mov	ax, 0E07h   ; beep
	int	10h

; restore regs and exit
L109:	pop	es
	pop	ds
	popa

L199:	iret

ISR2F		ENDP

_TEXT		ENDS


_DATA		SEGMENT WORD PUBLIC 'DATA'

hFile		WORD	?
FileName	BYTE	'(none)', 80 DUP(0)

TSRPSP		WORD	?

PrevISR10	LABEL	DWORD
		WORD	offset ISR10
		WORD	seg _TEXT

PrevISR2F	LABEL	DWORD
		WORD	offset ISR2F
		WORD	seg _TEXT

CSetDefs	BYTE	256*16 DUP(?)

_DATA		ENDS

; (end of resident portion)

TRANSIENT_TEXT	SEGMENT PARA PUBLIC 'TCODE'
		ASSUME	ds:DGROUP, ss:TRANSIENT_STACK

; LoadTSR
;
; Caller:   DS,ES -> program segment prefix
;	    SS:SP -> stack

LoadTSR 	PROC	NEAR

	cld

	call	GetUninstSwitch ; CF set if uninstall
	jnc	L501

; uninstall requested
	call	GetInstState	; CF set if installed
	jnc	L500

; come here to uninstall
; (warning: this is "bare bones" code without a check for
; subsequently-installed interrupt service routines)
	push	bx		; save previous PSP
	mov	ds, cx		; CX = resident DS

	mov	al, 10h 	; unhook int 10h
	mov	si, offset PrevISR10
	call	HookISR

	mov	al, 2Fh 	; unhook int 2Fh
	mov	si, offset PrevISR2F
	call	HookISR

; free memory used by the TSR
	pop	es		; ES = PSP segment
	mov	ah, 49h 	; (free allocated memory)
	int	21h

; reset the current video mode to get rid of
;  the current alphanumeric character set
	mov	ah, 0Fh
	int	10h		; AL = video mode
	xor	ah, ah		; AH = 0
	int	10h

	mov	bx, offset TRANSIENT_DATA:Msg1
	call	ConsoleOut
	jmp	L503

; uninstall requested but not previously installed
L500:	mov	bx, offset TRANSIENT_DATA:Msg2
	call	ConsoleOut
	jmp	L503

; uninstall not requested
L501:	call	GetFileName
	jnc	L502

; come here if not uninstalling and no filename
	mov	bx, offset TRANSIENT_DATA:Msg0
	call	ConsoleOut
	jmp	L503

; filename successfully specified
L502:	call	GetInstState
	jnc	L504

; come here if already installed to update the
;  alphanumeric-mode character set
	call	UpdateCSet

	mov	bx, offset TRANSIENT_DATA:Msg3
	call	ConsoleOut

L503:	mov	ax, 4C00h
	int	21h

; come here to install the TSR
L504:	push	seg DGROUP
	pop	ds
	mov	TSRPSP, es  ; save the PSP segment

; install ISR10
	mov	al, 10h
	mov	si, offset PrevISR10
	call	HookISR

; do the fixup at label L000 in ISR10
	mov	si, WORD PTR PrevISR10
	mov	WORD PTR L000+1, si
	mov	si, WORD PTR PrevISR10+2
	mov	WORD PTR L000+3, si

; do the fixup at label L002 in ISR10
	mov	si, WORD PTR PrevISR10
	mov	WORD PTR L002+1, si
	mov	si, WORD PTR PrevISR10+2
	mov	WORD PTR L002+3, si

; install ISR2F
	mov	al, 2Fh
	mov	si, offset PrevISR2F
	call	HookISR

; do the fixup at label L100 in ISR2F
	mov	si, WORD PTR PrevISR2F
	mov	WORD PTR L100+1, si
	mov	si, WORD PTR PrevISR2F+2
	mov	WORD PTR L100+3, si

; free memory allocated to the TSR's environment block
	mov	es, TSRPSP
	mov	es, es:[2Ch]	; ES = environment seg
	mov	ah, 49h 	; (free allocated memory)
	int	21h

; initialize the alphanumeric-mode character set
	call	UpdateCSet

; terminate and stay resident
	mov	bx, offset TRANSIENT_DATA:Msg4
	call	ConsoleOut

	mov	dx, cs	    ; DX = paragraph address of start
			    ;  of transient portion
	sub	dx, TSRPSP  ; DX = size of resident portion
	mov	ax, 3100h   ; AH = 31h (TSR)
			    ; AL = 0 (return code)
	int	21h

LoadTSR 	ENDP

; UpdateCSet

UpdateCSet	PROC	NEAR

	push	es

; compute the size of the memory used by the program
	mov	ah, 51h     ; (get PSP)
	int	21h
	mov	es, bx	    ; ES = current PSP

	sub	bx, seg TRANSIENT_STACK
	neg	bx
	add	bx, 32+2    ; ... plus stack size ...
			    ; ... plus some padding

; free unused memory
	mov	ah, 4Ah     ; (resize memory block)
	int	21h

; load the user-specified character set
	mov	ax, ((MULTIPLEX_ID SHL 8) + 1)
	int	2Fh

; do a video-mode set to display the new character set
	mov	ax, 3
	int	10h

	pop	es
	ret

UpdateCSet	ENDP

; GetFileName
;
; Returns:  cf clear if successful

GetFileName	PROC	NEAR

	push	ds

	mov	ah, 51h     ; function 51h: Get PSP
	int	21h
	mov	ds, bx
	mov	si, 81h     ; DS:SI -> command tail

; scan for first non-whitespace
L540:	lodsb		    ; AL = next byte in command tail

	or	al, al
	jz	L545	    ; jump if end of command tail

	cmp	al, CR
	je	L545	    ; jump if end of command tail

	cmp	al, 20h
	jbe	L540

	mov	dx, si	    ; DS:DX -> start of filename

; scan for next whitespace or null
L541:	lodsb

	cmp	al, 20h
	ja	L541

; copy the filename into the resident data segment
	push	es

	call	GetInstState
	jc	L542	    ; jump if installed

	mov	cx, seg DGROUP

L542:	mov	es, cx

	mov	di, offset DGROUP:FileName

	mov	cx, si
	sub	cx, dx	    ; CX = filename length
	mov	si, dx
	dec	si
	rep	movsb

	xor	al, al	    ; append a null
	stosb

	pop	es

; successful return
	clc
	pop	ds
	ret

; error return
L545:	stc		    ; set cf and return
	pop	ds
	ret

GetFileName	ENDP

; GetUninstSwitch
;
; Caller:   DS = PSP segment
;
; Returns:  CF set if /U switch is found in the command tail

GetUninstSwitch PROC NEAR

	mov	si, 81h     ; DS:SI -> command tail

; scan for '/', '-', CR, or null
L550:	lodsb		    ; AL = next byte in command tail

	or	al, al
	je	L551

	cmp	al, CR
	jne	L552	    ; jump if end of command tail

; CR encountered
L551:	clc		    ; clear CF
	jmp	L554	    ; exit

L552:	cmp	al, '/'     ; look for slash ...
	je	L553

	cmp	al, '-'     ; ... and dash
	jne	L550

; slash or dash encountered, so look at the next byte
L553:	lodsb
	or	al, 20h     ; force lowercase
	cmp	al, 'u'
	jne	L550	    ; jump (ignore) if not 'u'

	stc		    ; set CF

L554:	ret

GetUninstSwitch ENDP

; GetInstState
;
; Caller:   nothing
;
; Returns:  CF set if TSR already installed
;	    BX = previous PSP segment if TSR previously installed
;	    CX = resident DGROUP if TSR previously installed

GetInstState	PROC  NEAR

	push	es	    ; preserve ES

; look for an int 2Fh handler
	mov	ax, 352Fh
	int	21h

	mov	ax, es
	or	ax, bx
	jnz	L560

; no previous int 2Fh, so this TSR cannot have been installed
	clc
	jmp	L561

; determine whether the TSR is already installed
L560:	mov	ax, ((MULTIPLEX_ID SHL 8) + 0)
	int	2Fh
	add	al, 1	    ; CF set if AL = 0FFh

L561:	pop	es	    ; restore ES
	ret

GetInstState	ENDP

;
; HookISR
;
; Caller:   AL = interrupt number
;	    DS:SI -> storage for interrupt vector (in DGROUP)
;
; Returns:  nothing

HookISR 	PROC	NEAR

	push	ds	    ; preserve DS

; save the interrupt number for the second int 21h
	push	ax

; save the current interrupt vector
	push	si
	mov	ah, 35h     ; AH = 35h (get interrupt vector)
	int	21h
	pop	si

	mov	dx, [si]    ; DX = segment of ISR to install
	mov	ax, [si+2]  ; AX = offset of ISR to install

	mov	[si], bx    ; save the offset & segment of
	mov	[si+2], es  ;  the previous ISR

; update the interrupt vector
	mov	ds, ax	    ; DS:DX -> ISR to install
	pop	ax	    ; AL = interrupt number
	mov	ah, 25h     ; AH = 25h (set interrupt vector)
	int	21h

	pop	ds	    ; restore DS
	ret

HookISR 	ENDP

;
; ConsoleOut
;
; Caller:   BX = offset of string to display
;
; Returns:  nothing

ConsoleOut	PROC	NEAR

	push	ds

	mov	ax, seg TRANSIENT_DATA
	mov	ds, ax	    ; DS:DX -> string

	mov	cl, [bx]
	xor	ch, ch	    ; CX = length of string
	lea	dx, [bx+1]  ; DS:DX -> string data
	mov	bx, STDOUT
	mov	ah, 40h     ; AH = 40h (write file or device)
	int	21h

	pop	ds

	ret

ConsoleOut	ENDP

TRANSIENT_TEXT	ENDS


TRANSIENT_DATA	SEGMENT WORD PUBLIC 'TDATA'

TransientPSP	WORD	?
Msg0		BYTE	48, "Syntax: CSETTSR <filename>", CR, LF
		BYTE	    "    or: CSETTSR /U", CR, LF
Msg1		BYTE	21, "CSETTSR uninstalled", CR, LF
Msg2		BYTE	34, "CSETTSR not previously installed", CR, LF
Msg3		BYTE	61, "CSETTSR is already installed", CR, LF
		BYTE	    "(Use CSETTSR /U to uninstall)", CR, LF
Msg4		BYTE	32, "CSETTSR successfully installed", CR, LF

TRANSIENT_DATA	ENDS

TRANSIENT_STACK SEGMENT STACK 'STACK'

		BYTE	32*16 DUP(?)	; 512 bytes (32 paragraphs)

TRANSIENT_STACK ENDS

		END	LoadTSR
