		TITLE	VBUF.ASM
		NAME	VBUF
		PAGE	55,132

.286

INCLUDE 	vs2.inc

DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

; SetVBufStart
;
;   USHORT VS2API SetVBufStart( USHORT ofs );
;
; Notes:
;   Returns the previous offset.
;
;   This function does *not* reprogram the CRTC.

; parameter
ofs		EQU	WORD PTR [bp+6]

		PUBLIC	SETVBUFSTART
SETVBUFSTART	PROC	FAR

	enter	0, 0

	mov	ax, ofs
	xchg	ax, VSP.ofsVBuf

	leave
	ret	2	    ; sizeof(USHORT)

SETVBUFSTART	ENDP


; SetScreenStart
;
;   void VS2API SetScreenStart( USHORT ofs );
;
; Notes:
;   This function reprograms the CRTC.

; parameter
ofs		EQU	WORD PTR [bp+6]

		PUBLIC	SETSCREENSTART
SETSCREENSTART	PROC	FAR

	enter	0, 0

; save the new offset
	mov	bx, ofs
	mov	VSP.ofsVBuf, bx

; reprogram the CRTC
	add	bx, VSP.ofsXY	; adjust for origin (x,y)
	call	IsGraphicsMode
	call	ComputeStartAddress

; update the CRTC Start Address registers
	push	segVDDA     ; get the CRTC I/O address
	pop	es
	mov	dx, es:[ADDR6845]

	mov	ah, bh	    ; AH = high-order byte of start offset
	mov	al, 0Ch     ; AL = 0Ch (Start Address High reg #)
	out	dx, ax

	mov	ah, bl	    ; AH = low-order byte of start offset
	inc	ax	    ; AL = 0Dh (Start Address Low reg #)
	out	dx, ax

	leave
	ret	2	    ; sizeof(USHORT)

SETSCREENSTART	ENDP

; SetVBufDims
;
;   USHORT VS2API SetVBufDims( USHORT x, USHORT y, USHORT flag );
;
; Notes:
;   Returns the logical video-buffer size in bytes.
;
;   If the flag is nonzero, this function converts (x,y) from
;   character to pixel coordinates.
;
;   This routine computes the row length by rounding up (if necessary)
;   to the next highest number of pixels that can be displayed using the
;   value stored in the CRTC Offset register.
;
;   This routine always sets the requested row length, even if the
;   actual number of rows of data available in the video buffer is less
;   than requested.

; parameters
flag		EQU	WORD PTR [bp+6]
y		EQU	WORD PTR [bp+8]
x		EQU	WORD PTR [bp+10]

; temporary variables
bGraphicsMode	EQU	BYTE PTR [bp-1]
Chain4Factor	EQU	WORD PTR [bp-4]

		PUBLIC	SETVBUFDIMS
SETVBUFDIMS	PROC	FAR

	enter	4, 0

; save a flag indicating graphics or alphanumeric mode
	call	IsGraphicsMode
	mov	bGraphicsMode, al

	push	segVDDA
	pop	es

; look for "chain-4" video mode (BIOS mode 13h)
	mov	dx, 3C4h	; Sequencer I/O addr
	mov	al, 4		; Memory Mode register number
	out	dx, al
	inc	dx
	in	al, dx		; AL = current Memory Mode value
	dec	dx		; DX = 3C4h

	and	ax, 8		; isolate AX bit 3
	jnz	L200		; jump if AL = 8

	mov	al, 2		; AX = 2

L200:	mov	Chain4Factor, ax

; get character width (8 or 9 dots)
	mov	al, 1		; Clocking Mode register number
	out	dx, al
	inc	dx		; DX = 3C5h
	in	al, dx		; AL = current Clocking Mode value
	and	al, 1		; isolate bit 0
	mov	bx, 9		; BX = 9
	sub	bl, al		; BX = 8 or 9 = Dots per character

; convert character to pixel coordinates if necessary
	mov	ax, x		; AX = x
	cmp	flag, 0
	je	L201		; jump if (x,y) are pixel coordinates

	mul	bx

; convert pixels to a value for the CRTC Offset register
L201:	xor	dx, dx		; DX:AX = pixels
	cmp	bGraphicsMode, 0
	jne	L202		; jump if graphics mode

	div	bx		; AX = characters per row
	neg	dx		; cf set if DX <> 0
	adc	ax, 0		; increment AX if necessary

	shr	ax, 1		; AL = words per row; cf set if odd
	adc	al, 0		; increment AL if necessary

	jmp	L205

L202:	div	VSP.usPPB	; AX = bytes per row
	neg	dx		; cf set if DX <> 0
	adc	ax, 0		; increment AX if DX <> 0

	xor	dx, dx
	div	Chain4Factor	; divide AX by either 8 or 2
	neg	dx
	adc	ax, 0		; AL = Offset register value

; program the CRTC
L205:	mov	cl, al		; CL = Offset register value
	mov	ah, al
	mov	al, 13h 	; AL = 13h (Offset register number)
	mov	dx, es:[ADDR6845]
	out	dx, ax

; compute the number of bytes per video-buffer row
	xor	ch, ch		; CX = Offset register value
	cmp	bGraphicsMode, 0
	jne	L207		; jump if graphics mode

	shl	cx, 2		; CX = bytes per row
	jmp	L210

L207:	mov	ax, Chain4Factor
	mul	cx
	mov	cx, ax		; CX = bytes per row

; compute the desired vertical height in rows
L210:	mov	ax, y
	cmp	flag, 0
	je	L212		; jump if y is specified as pixels

; come here if y specifies a height in characters
	cmp	bGraphicsMode, 0
	je	L215		; jump if alphanumeric mode

	mul	WORD PTR es:[POINTS]	; AX = y in pixels (rows)
	jmp	L215

; come here if y is specified as pixels
L212:	cmp	bGraphicsMode, 0
	jne	L215		; jump if graphics mode

	xor	dx, dx
	div	WORD PTR es:[POINTS]	; AX = # of character rows
	neg	dx
	adc	ax, 0		; round up

; save the smaller of the requested and available rows
L215:	mov	bx, ax		; BX = rows
	xor	ax, ax
	mov	dx, 1		; DX:AX = 10000h
	div	cx		; AX = maximum rows in video buffer
	cmp	ax, bx
	jb	L217		; jump if requested # of rows is smaller

	mov	ax, bx

L217:	mov	VSP.usCY, ax	; save the number of rows
	mov	VSP.cbRow, cx	; save the number of bytes per row

; compute the number of character and pixel columns
	mov	ax, cx		; AX = bytes per row
	cmp	bGraphicsMode, 0
	jne	L222		; jump if graphics mode

	shr	ax, 1		; AX = # of character columns
	mov	es:[CRT_COLS], ax
	mov	VSP.usCX, ax
	jmp	L225

L222:	mul	VSP.usPPB	; AX = (bytes/row)*(pixels/byte)
	mov	VSP.usCX, ax
	shr	ax, 3		; divide by (8 pixels/char)
	mov	es:[CRT_COLS], ax

; leave the virtual buffer size in AX
L225:	mov	ax, VSP.cbRow
	mul	VSP.usCY	; AX = size of logical buffer
	mov	es:[CRT_LEN], ax

	leave
	ret	6	    ; 3*sizeof(USHORT)

SETVBUFDIMS	ENDP

; SetScreenOrg
;
;   void VS2API SetScreenOrg( USHORT x, USHORT y, USHORT flag );
;
; Notes:
;   If the flag is nonzero, this function converts (x,y) from
;   character to pixel coordinates.
;
;   This function should be called during the vertical nondisplay
;   interval.

; parameters
flag		EQU	WORD PTR [bp+6]
y		EQU	WORD PTR [bp+8]
x		EQU	WORD PTR [bp+10]

; temporary variables
bGraphicsMode	EQU	BYTE PTR [bp-1]
PRSReg		EQU	BYTE PTR [bp-2]
Dots		EQU	WORD PTR [bp-4]
Pts		EQU	WORD PTR [bp-6]

		PUBLIC	SETSCREENORG
SETSCREENORG	PROC	FAR

	enter	6, 0

; save a flag indicating graphics or alphanumeric mode
	call	IsGraphicsMode
	mov	bGraphicsMode, al

	push	segVDDA
	pop	es		; ES -> BIOS video display data area

; get character width (8 or 9 dots)
	mov	dx, 3C4h	; Sequencer I/O addr
	mov	al, 1		; AL = Clocking Mode register number
	out	dx, al
	inc	dx		; DX = 3C5h
	in	al, dx		; AL = current Clocking Mode value
	and	ax, 1		; isolate AX bit 0
	sub	al, 9		; AL = -9 or -8
	neg	al		; AX = 9 or 8
	mov	Dots, ax

; get character height
	mov	ax, es:[POINTS] ; AX = character height
	mov	Pts, ax

; compute the video-buffer offset of the upper-left corner
	mov	ax, y		; AX = y
	mov	bx, x		; BX = x
	mov	PRSReg, 0	; zero PRSReg

	cmp	bGraphicsMode, 0
	jne	L305		; jump if graphics mode

; convert alphanumeric mode pixel coords to character coords
	cmp	flag, 0
	jne	L300		; jump if character coordinates

	xor	dx, dx		; DX:AX = y
	div	Pts		; AX = y in character coordinates
	mov	PRSReg, dl	; save (y MOD Pts)
	xchg	ax, bx		; AX = x, BX = y
	mov	cx, ax		; CX = x (pixel coordinates)

	xor	dx, dx
	div	Dots		; AX = x in character coordinates
	xchg	ax, bx		; AX = y, BX = x
	jmp	L302

; compute x as pixel coordinates
L300:	xchg	ax, bx		; AX = x (character coords)
				; BX = y
	mov	cx, ax		; CX = x (character coords)
	mul	Dots		; AX = x (pixel coords)
	xchg	ax, cx		; AX = x (character coords)
				; CX = x (pixel coords)
	xchg	ax, bx		; AX = y, BX = x

L302:	mov	VSP.usX, cx	; save origin coords
	mov	VSP.usY, ax

; convert alphanumeric character coordinates to a buffer offset
	mul	VSP.cbRow	; AX = y*(bytes per row)
	add	bx, bx		; BX = 2*x
	add	bx, ax		; BX = byte offset of (x,y)
	jmp	L310

; convert graphics-mode character coords to pixel coords
L305:	cmp	flag, 0
	je	L307		; jump if pixel coordinates

	mov	ax, x
	mul	Dots
	mov	bx, ax		; BX = x

	mov	ax, y
	mul	Pts		; AX = y

L307:	mov	VSP.usX, bx	; save origin coords
	mov	VSP.usY, ax

; compute the byte offset into the video buffer
	mov	cx, bx		; CX = x

	mul	VSP.cbRow	; AX = y * (bytes per row)
	xchg	ax, bx		; AX = x, BX = y*(bytes per row)
	xor	dx, dx
	div	VSP.usPPB	; AX = x/(pixels per byte)
	add	bx, ax		; BX = byte offset of (x,y)

; at this point:
;   BX = byte offset of (x,y)
;   CX = x (in pixel coordinates)

L310:	mov	VSP.ofsXY, bx	; save byte offset of (x,y)
	add	bx, VSP.ofsVBuf ; BX = video-buffer offset
	cmp	bGraphicsMode, 0
	call	ComputeStartAddress

; compute a value for the Horizontal Pel Pan register
	mov	ax, cx
	add	ax, 8
	xor	dx, dx
	div	Dots		; DL = (x + 8) MOD Dots = desired value
	mov	ah, dl		; AH = desired value

	mov	dx, 3CEh	; Graphics Controller I/O addr
	mov	al, 5		; Mode register number
	out	dx, al
	inc	dx
	in	al, dx
	test	al, 40h 	; test bit 6 (set if 256-color mode)
	jz	L312		; jump if not 256-color mode

	and	ah, 3		; in 256-color modes ...
	add	ah, ah		; ... adjust to 0, 2, 4, or 6

; set the Horizontal Pel Pan register
L312:	mov	dx, es:[ADDR6845]
	add	dx, 6		; DX = Input Status Reg 1 I/O addr
	in	al, dx		; reset the Attr Controller flip-flop

	mov	dl, 0C0h	; DX = 3C0h (Attr Controller I/O port)
	mov	al, 33h 	; bit 0-4: 13h (Horiz Pel Pan reg #)
				; bit 5 = 1 (palette address source)

	cli			; disable interrupts
	out	dx, al
	mov	al, ah		; Horiz Pel Pan reg value
	out	dx, al

; program the CRTC Preset Row Scan register
	mov	dx, es:[ADDR6845]
	mov	ah, PRSReg
	mov	al, 8		; Preset Row Scan reg number
	out	dx, ax

; program the CRTC Start Address registers
	mov	ah, bh		; AH = high-order byte of start offset
	mov	al, 0Ch 	; AL = 0Ch (Start Address High reg #)
	out	dx, ax
	mov	ah, bl		; AH = low-order byte of start offset
	inc	ax		; AL = 0Dh (Start Address Low reg #)
	out	dx, ax
	sti			; enable interrupts

	leave
	ret	6		; 3*sizeof(USHORT)

SETSCREENORG	ENDP

; SetScreenSplit
;
;   void VS2API SetScreenSplit( USHORT y, USHORT ofs, USHORT flag );
;
;   If the flag is nonzero, this function converts (x,y) from
;   character to pixel coordinates.
;
;   The Line Compare value is a 10-bit value stored in 3 VGA registers:
;	Bits 0-7:   Line Compare register (18h)
;	Bit 8:	    Overflow register (07h) bit 4
;	Bit 9:	    Max Scan Line register (09h) bit 6
;
;   This function should be called during the vertical nondisplay
;   interval.

; parameters
flag		EQU	WORD PTR [bp+6]
ofs		EQU	WORD PTR [bp+8]
y		EQU	WORD PTR [bp+10]

; temporary variables
bGraphicsMode	EQU	BYTE PTR [bp-1]

		PUBLIC	SETSCREENSPLIT
SETSCREENSPLIT	PROC	FAR

	enter	2, 0

; save a flag indicating graphics or alphanumeric mode
	call	IsGraphicsMode
	mov	bGraphicsMode, al

	push	segVDDA
	pop	es		; ES -> BIOS video display data area

; convert y into a pixel value if necessary
	mov	ax, y
	cmp	ax, -1
	je	L400		; jump if y specifies no split

	cmp	flag, 0
	je	L400		; jump if y is specified as pixels

	mul	WORD PTR es:[POINTS]

L400:	mov	bx, ax		; BX = scan line to split at

; program the CRTC in a uniform manner for 256-color modes in which
;  each line of pixels is displayed in two scan lines
	mov	dx, es:[ADDR6845]
	mov	al, 9		; Max Scan Line register number
	out	dx, al
	inc	dx
	in	al, dx		; AL = current Max Scan Line value
	dec	dx

	mov	ah, al		; AH = current Max Scan Line value
	and	al, 9Fh 	; isolate bit 7 and bits 0-4
	cmp	al, 1
	jne	L401		; jump if bit 7 = 1 or bits 0-4 <> 1

	and	ah, 60h 	; zero bit 7 and bits 0-4
	or	ah, 80h 	; bit 7 = 1 (set double-scanline mode)
				; bits 0-4 = 0 (1 scan line per char)
	mov	al, 9
	out	dx, ax

; in modes where scan lines are doubled, double the Line Compare value
L401:	test	ah, 80h 	; test Max Scan Line bit 7
	jz	L402		; jump if not a double-scanline mode

	add	bx, bx		; BX = doubled Line Compare value
	inc	bx		; make BX an odd number

; update the CRTC Line Compare value
L402:	mov	ah, bl		; AH = bits 0-7
	mov	al, 18h 	; Line Compare register number
	out	dx, ax

	mov	al, 7		; Overflow register
	out	dx, al
	inc	dx
	in	al, dx		; AL = current Overflow value
	dec	dx

	mov	bl, bh
	shl	bl, 4
	and	bl, 10h 	; BL bit 4 = Line Compare bit 8

	and	al, 0EFh	; zero bit 4
	or	al, bl		; bit 4 = Line Compare bit 8
	mov	ah, al
	mov	al, 7
	out	dx, ax

	mov	al, 9		; Max Scan Line register number
	out	dx, al
	inc	dx
	in	al, dx		; AL = current Max Scan Line value
	dec	dx

	shl	bh, 5
	and	bh, 40h 	; BL bit 6 = Line Compare bit 9

	and	al, 0BFh	; zero bit 6
	or	al, bh		; bit 6 = Line Compare bit 9
	mov	ah, al
	mov	al, 9
	out	dx, ax

; compute a value for the CRTC Start Address registers
	mov	bx, ofs
	add	bx, VSP.ofsXY	; adjust for origin (x,y)
	cmp	bGraphicsMode, 0
	call	ComputeStartAddress

; update the CRTC Start Address registers
	mov	dx, es:[ADDR6845]
	mov	ah, bh		; Start Addr High value
	mov	al, 0Ch 	; Start Addr High reg #
	out	dx, ax
	mov	ah, bl		; Start Addr Low value
	inc	al		; Start Addr Low reg #
	out	dx, ax

	cmp	bGraphicsMode, 0
	je	L409		; jump if alphanumeric mode

; update the Attribute Controller Mode Control register
	add	dx, 6		; DX = Input Status Reg 1 I/O addr
	mov	cx, dx		; CX = Input Status Reg 1 I/O addr
	in	al, dx		; reset ATC flip-flop

	mov	dl, 0C0h	; DX = 3C0h (ATC I/O addr)
	mov	al, 30h 	; bits 0-4: 10h (Mode Control reg number)
				; bit 5:    1 (PAS bit)
	out	dx, al
	inc	dx
	in	al, dx		; AL = current Mode Control value

	or	al, 20h 	; set bit 5 to allow split-screen
	mov	ah, al		;  horizontal panning

	mov	dx, cx		; DX = Input Status Reg 1 I/O addr
	in	al, dx		; reset ATC flip-flop

	mov	dl, 0C0h	; DX = 3C0h (ATC I/O addr)
	mov	al, 30h 	; bits 0-4: 10h (Mode Control reg number)
				; bit 5:    1 (PAS bit)
	out	dx, al
	mov	al, ah
	out	dx, al		; update the Mode Control register

L409:	leave
	ret	6		; 3*sizeof(USHORT)

SETSCREENSPLIT	ENDP

; IsGraphicsMode
;
; Returns:  AL = 1 if graphics mode, 0 if alphanumeric mode
;	    zf set if alphanumeric mode

IsGraphicsMode	PROC	NEAR

	push	dx

	mov	dx, 3CEh	; Graphics Controller I/O addr
	mov	al, 6		; Miscellaneous register number
	out	dx, al
	inc	dx
	in	al, dx		; AL = current Miscellaneous reg value
	and	al, 1		; isolate bit 0 and update zf

	pop	dx
	ret

IsGraphicsMode	ENDP

; ComputeStartAddress
;
; Caller:   BX = byte offset in video buffer
;	    zf set if alphanumeric mode
; Returns:  BX = CRTC Start Address

ComputeStartAddress PROC    NEAR

	push	ax
	push	dx

	jnz	L500		; jump if graphics mode

; alphanumeric-mode computation
	shr	bx, 1		; BX = adjusted Start Address
	jmp	L502

; graphics-mode computation
L500:	mov	dx, 3C4h	; Sequencer I/O address
	mov	al, 4		; Memory Mode register number
	out	dx, al
	inc	dx
	in	al, dx		; AL = current Memory Mode value
	test	al, 8		; test bit 3
	jz	L502		; jump if bit 3 = 0

	shr	bx, 2		; BX = BX/4 for chain-4 addressing

L502:	pop	dx
	pop	ax
	ret

ComputeStartAddress ENDP

VS2_TEXT	ENDS

_DATA		SEGMENT WORD PUBLIC 'DATA'

		EXTRN	VSP:VIDEOSTATEPARAMS

_DATA		ENDS

		END
