		TITLE	VGADAC.ASM
		NAME	VGADAC
		PAGE	55,132

.286

INCLUDE 	vs2.inc

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'

; RotateDACTable
;
;   void VS2API RotateDACTable( LPVOID lpTable, short nSize, short n );
;
; Notes:
;   Abs(n) specifies the number of positions to rotate.  The sign of n
;   specifies the direction to rotate.

; parameters
n		EQU	WORD PTR [bp+6]
nSize		EQU	WORD PTR [bp+8]
lpTable 	EQU	DWORD PTR [bp+10]

; temporary variable
OldSP		EQU	WORD PTR [bp-2]

		PUBLIC	ROTATEDACTABLE
ROTATEDACTABLE	PROC	FAR

	enter	2, 0
	push	si
	push	di

	mov	OldSP, sp   ; save current SP

	mov	ax, nSize   ; AX = nSize
	mov	bx, n	    ; BX = n

; force n to be positive
	or	bx, bx
	jns	L001	    ; jump if n >= 0

	add	bx, ax	    ; BX = nSize-n = +n

L001:	sub	ax, bx	    ; AX = nSize-n
	cmp	ax, bx
	ja	L050	    ; jump if (nSize-n) > +n

; come here to rotate toward the beginning of the table
; (AX = number of positions to rotate)
	mov	cx, ax	    ; CX = distance
	add	cx, ax
	add	cx, ax	    ; CX = 3*n = distance in bytes
	mov	ax, cx	    ; AX = distance in bytes

; copy overlapped data to temporary space on the stack
	cld
	inc	cx	    ; round up to next even number
	and	cl, 0FEh
	sub	sp, cx	    ; reserve space on the stack

	mov	di, sp
	push	ss
	pop	es	    ; ES:DI -> space on stack
	lds	si, lpTable ; DS:SI -> start of table
	mov	cx, ax	    ; CX = # of bytes to copy
	rep movsb

; rotate data in table toward the beginning of table
	mov	cx, nSize
	add	cx, cx
	add	cx, nSize   ; CX = size of table in bytes
	sub	cx, ax	    ; CX = # of bytes to copy

	les	di, lpTable ; ES:DI -> start of table
	rep movsb

; copy remaining data to the end of the table (assumes DS = SS)
	mov	si, sp	    ; DS:SI -> overlapped data
	mov	cx, ax
	rep movsb
	jmp	L090

; come here to rotate toward the end of the table
; (BX = number of positions to rotate)
L050:	mov	cx, bx	    ; CX = distance
	add	cx, bx
	add	cx, bx	    ; CX = 3*n = distance in bytes
	mov	ax, cx	    ; AX = distance in bytes

; copy overlapped data to temporary space on the stack
	mov	bx, sp
	dec	bx	    ; SS:BX -> end of temporary space

	std
	inc	cx	    ; round up to next even number
	and	cl, 0FEh
	sub	sp, cx	    ; reserve space on the stack

	mov	di, bx
	push	ss
	pop	es	    ; ES:DI -> space on stack
	lds	si, lpTable ; DS:SI -> start of table

	mov	dx, nSize
	add	dx, dx
	add	dx, nSize   ; DX = size of table in bytes

	add	si, dx
	dec	si	    ; DS:SI -> last byte in table
	mov	cx, ax	    ; CX = # of bytes to copy
	rep movsb

; rotate data in table toward the end of table
	mov	cx, dx	    ; CX = size of table in bytes
	sub	cx, ax	    ; CX = # of bytes to copy

	les	di, lpTable
	add	di, dx
	dec	di	    ; ES:DI -> last byte in table
	rep movsb

; copy remaining data to the beginning of the table (assumes DS = SS)
	mov	si, bx	    ; DS:SI -> overlapped data
	mov	cx, ax
	rep movsb
	cld

L090:	mov	sp, OldSP   ; restore previous SP
	pop	di
	pop	si
	leave
	ret	8	    ; sizeof(LPVOID)+2*sizeof(short)

ROTATEDACTABLE	ENDP

; GetDACRegs
;
;   void VS2API GetDACRegs( LPVOID lpTable, short nStart, short n );
;
; Notes:
;   The table is stored as an array of 6-bit RGB triples.

; parameters
n		EQU	WORD PTR [bp+6]
nStart		EQU	WORD PTR [bp+8]
lpTable 	EQU	DWORD PTR [bp+10]

		PUBLIC	GETDACREGS
GETDACREGS	PROC	FAR

	enter	0, 0
	push	di

	cld
	les	di, lpTable ; ES:DI -> RGB table
	mov	cx, n	    ; CX = # of table entries
	add	cx, cx
	add	cx, n	    ; CX = 3*n = size of table

	mov	dx, 3C7h    ; DX = 3C7h (DAC read I/O address)
	mov	ax, nStart

	out	dx, al	    ; start with the specified DAC register
	mov	dl, 0C9h    ; DX = 3C9h (DAC data I/O address)
	rep insb	    ; load the DAC registers

	pop	di
	leave
	ret	8	    ; sizeof(LPVOID)+2*sizeof(short)

GETDACREGS	ENDP

; SetDACRegs
;
;   void VS2API SetDACRegs( LPVOID lpTable, short nStart, short n );
;
; Notes:
;   The table must be formatted as an array of 6-bit RGB triples.

; parameters
n		EQU	WORD PTR [bp+6]
nStart		EQU	WORD PTR [bp+8]
lpTable 	EQU	DWORD PTR [bp+10]

		PUBLIC	SETDACREGS
SETDACREGS	PROC	FAR

	enter	0, 0
	push	ds
	push	si

	cld
	lds	si, lpTable ; DS:SI -> RGB table
	mov	cx, n	    ; CX = # of table entries
	add	cx, cx
	add	cx, n	    ; CX = 3*n = size of table

	mov	dx, 3C8h    ; DX = 3C8h (DAC write I/O address)
	mov	ax, nStart
	out	dx, al	    ; start with the specified DAC register

	mov	dl, 0C9h    ; DX = 3C9h (DAC data I/O address)
	rep outsb	    ; load the DAC registers

	pop	si
	pop	ds
	leave
	ret	8	    ; sizeof(LPVOID)+2*sizeof(short)

SETDACREGS	ENDP

VS2_TEXT	ENDS

		END
