		TITLE	VNI.ASM
		NAME	VNI
		PAGE	55,132

.286

INCLUDE 	vs2.inc

DGROUP		GROUP	_DATA

cbStack 	EQU	2048

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

		EXTRN	VRWAIT:FAR

; EnableVNI -- enables vertical nondisplay interrupts
;
;   USHORT VS2API EnableVNI( LPVOID lpfn );
;
; Notes:
;   Returns a nonzero value if successful.

; parameter
lpfn		EQU	DWORD PTR [bp+6]

		PUBLIC	ENABLEVNI
ENABLEVNI	PROC	FAR

	enter	0, 0

; determine whether the hardware supports vertical nondisplay interrupts
	mov	nVNI, 0 	; initialize a counter

	push	cs
	pop	es
	mov	bx, offset CountInt
	call	DoEnable	; enable VNIs

	call	VRWAIT		; wait for vertical retrace
	call	VRWAIT

	call	DISABLEVNI	; disable VNIs

	cmp	nVNI, 0 	; (nonzero if a VNI occurred)
	je	L090

; enable vertical nondisplay interrupts
	les	bx, lpfn	; ES:BX = function pointer
	call	DoEnable

L090:	mov	ax, nVNI	; return nonzero if successful

	leave
	ret	4		; sizeof(LPVOID)

ENABLEVNI	ENDP

; DoEnable
;
; Caller:   ES:BX -> function to be called by ISR0A

DoEnable	PROC	NEAR

; save the function pointer
	mov	WORD PTR fnVNI, bx
	mov	WORD PTR fnVNI+2, es

; save a pointer to the previous interrupt service routine
	mov	ax, 350Ah   ; AH = int 21h function number
			    ; AL = interrupt number
	int	21h	    ; Get Interrupt Vector
	mov	WORD PTR PrevISR0A, bx
	mov	WORD PTR PrevISR0A+2, es

; update the interrupt 0Ah vector
	push	ds
	mov	ax, 250Ah   ; AH = int 21h function number
			    ; AL = interrupt number
	mov	dx, seg ISR0A
	mov	ds, dx
	mov	dx, offset ISR0A
	int	21h	    ; Set Interrupt Vector
	pop	ds

; enable IRQ2 by clearing bit 2 of the 1st PIC mask register
	cli		    ; disable interrupts
	in	al, 21h
	mov	PIC21, al   ; save current value
	and	al, 0FBh    ; bit 2 = 0
	out	21h, al

; get the current CRTC Vertical Retrace End register value
	mov	dx, 3D4h
	mov	al, 11h     ; AL = Vert Retr End reg #
	out	dx, al
	inc	dx	    ; DX = 3D5h
	in	al, dx	    ; AL = current reg value
	dec	dx	    ; DX = 3D4h

; save the current register value
	mov	ah, al	    ; AH = current reg value
	mov	al, 11h     ; AL = register number
	mov	VREReg, ax

; save a copy of the register value with bit 4 clear and bit 5 set
	and	ah, 0EFh    ; bit 4 = 0 (clear interrupt latch)
	or	ah, 20h     ; bit 5 = 1 (disable VNIs)
	out	dx, ax	    ; write to CRTC Vert Retr End reg
	mov	VNIClear, ax

; save a copy of the register value with bit 4 set and bit 5 clear
	and	ah, 0DFh    ; bit 5 = 0 (enable VNIs)
	or	ah, 10h     ; bit 4 = 1 (enable interrupt latch)
	out	dx, ax	    ; write to CRTC Vert Retr End reg
	mov	VNIReEnable, ax

	sti		    ; enable interrupts
	ret

DoEnable	ENDP

; CountInt

CountInt	PROC	FAR

	inc	nVNI	    ; increment a counter
	ret		    ; far return

CountInt	ENDP

; DisableVNI -- disables vertical nondisplay interrupts
;
;   void VS2API DisableVNI( void );

		PUBLIC	DISABLEVNI
DISABLEVNI	PROC	FAR

	enter	0, 0

	cli

; restore the Vertical Retrace End register to its original state
	mov	dx, 3D4h
	mov	ax, VREReg
	out	dx, ax

; restore the previous value of the 1st PIC mask register
	mov	al, PIC21
	out	21h, al
	sti

; restore the interrupt 0Ah vector
	push	ds
	mov	ax, 250Ah   ; AH = int 21h function number
			    ; AL = interrupt number
	lds	dx, PrevISR0A
	int	21h	    ; Set Interrupt Vector
	pop	ds

	leave
	ret

DISABLEVNI	ENDP

; ISR0A -- interrupt service routine for vertical nondisplay interrupt
;
; Notes:
;   This routine makes a far call (without parameters) to the function
;   whose address was supplied to EnableVNI.
;
;   This routine won't work in monochrome graphics modes because the
;   value of the CRTC I/O address is hard-coded as 3D4h.

		PUBLIC	ISR0A
ISR0A		PROC	FAR

; preserve regs
	push	ds
	push	ax
	push	dx

; address the default data segment through DS
	mov	ax, seg DGROUP
	mov	ds, ax

; determine whether the interrupt was generated by the VGA
	mov	dx, 3C2h    ; Input Status Reg 0
	in	al, dx
	test	al, 80h     ; test bit 7
	jz	L295	    ; jump if non-VNI int 0Ah has occurred

; preserve the current CRTC Address register value
	mov	dl, 0D4h    ; DX = 3D4h
	in	al, dx
	mov	CRTCAddrReg, al

; disable VNIs and clear the VGA interrupt latch
	mov	ax, VNIClear
	out	dx, ax

; allow subsequent interrupts to occur on IRQ2
	mov	al, 20h     ; bit 5 = 1 (nonspecific EOI)
	out	20h, al     ; write to PIC

; issue a "global rearm" for ISA adapters that share IRQ2
	mov	dx, 2F2h
	out	dx, al

; avoid recursion
	cmp	InISR, 0
	jne	L210	    ; jump if already in this ISR

	inc	InISR	    ; set status flag

; switch stacks so that DS = SS
	mov	OldSS, ss
	mov	OldSP, sp

	mov	ax, ds
	mov	ss, ax
	mov	sp, offset StackBottom

; call the user-supplied function
	sti		    ; enable interrupts
	push	es	    ; preserve registers
	push	bx
	push	cx
	push	si
	push	di
	push	bp

	cld		    ; clear the direction flag
	call	fnVNI	    ; call the user-supplied function

	pop	bp	    ; restore registers
	pop	di
	pop	si
	pop	cx
	pop	bx
	pop	es

; switch the stack back
	mov	ss, OldSS
	mov	sp, OldSP

; clear the recursion flag
	dec	InISR

; re-enable VNIs
L210:	mov	dx, 3D4h
	mov	ax, VNIReEnable
	out	dx, ax

; restore previous CRTC Address register value
	mov	al, CRTCAddrReg
	out	dx, al

L290:	pop	dx
	pop	ax
	pop	ds
	iret

; come here if the VGA has not generated a VNI
L295:	pushf
	call	PrevISR0A   ; chain to the previous ISR
	jmp	L290

ISR0A		ENDP

VS2_TEXT	ENDS


_DATA		SEGMENT WORD PUBLIC 'DATA'

nVNI		WORD	?   ; interrupt counter
PrevISR0A	DWORD	?   ; pointer to previous ISR
InISR		WORD	0   ; nonzero while ISR is executing

; saved register values
OldSS		WORD	?
OldSP		WORD	?

VNIClear	WORD	?
VNIReEnable	WORD	?

VREReg		WORD	?
PIC21		BYTE	?
CRTCAddrReg	BYTE	?

; far pointer to user-supplied function
fnVNI		DWORD	?

; temporary stack in default data segment
		BYTE	cbStack DUP(?)
StackBottom	LABEL	BYTE

_DATA		ENDS

		END
