		TITLE	VNTIMER.ASM
		NAME	VNTIMER
		PAGE	55,132

.286

INCLUDE 	vs2.inc

; Timer0Cmd
;
; Destroys: AL
;
; Notes:
;   The command byte for the Intel 8254 looks like this:
;	bits 6-7: counter number
;	bits 4-5: read/write command
;	bits 1-3: timer mode
;	bit 0:	  count method

Timer0Cmd	MACRO	    ControlByte

	mov	al, ControlByte
	out	43h, al     ; write to control byte register

		ENDM

; SetTimer0
;
; Destroys: AX

SetTimer0	MACRO	Count

	mov	ax, Count
	out	40h, al     ; write low-order byte to count register
	mov	al, ah
	out	40h, al     ; write high-order byte to control register

		ENDM

; LatchTimer0
;
; Leaves:   AX = current 16-bit count

LatchTimer0	MACRO

	mov	al, 0	    ; bits 4-5 = 0 (read 16 bits)
	out	43h, al

	in	al, 40h     ; AL = low byte
	mov	ah, al
	in	al, 40h     ; AL = high byte
	xchg	ah, al	    ; AX = result

		ENDM

DGROUP		GROUP	_DATA

cbStack 	EQU	2048

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

		EXTRN	VNWAIT:FAR
		EXTRN	VRWAIT:FAR

; EnableVNTimer -- enables vertical nondisplay interval timer
;
;   void VS2API EnableVNTimer( LPVOID lpfn );
;
; Notes:
;   Returns nonzero if successful
;
;   Call InitVSP prior to executing this function.

; parameter
lpfn		EQU	DWORD PTR [bp+6]

		PUBLIC	ENABLEVNTIMER
ENABLEVNTIMER	PROC	FAR

	enter	0, 0

; save the function pointer
	les	bx, lpfn
	mov	WORD PTR fnVNTimer, bx
	mov	WORD PTR fnVNTimer+2, es

; initialize the optional skip count
	mov	ax, VSP.usVNSkip
	mov	SkipCount, ax

; use timer mode 2 (divide-by-n) instead of default mode 3 (square wave)
;  so that the timer counts by 1
	cli
	Timer0Cmd   00110100b	; timer mode 2
	SetTimer0   0

; count the number of timer ticks between vertical retrace pulses
	call	VRWAIT
	LatchTimer0
	mov	bx, ax		; BX = initial timer count

	call	VRWAIT
	LatchTimer0		; AX = final timer count
	sti

	sub	bx, ax		; BX = ticks per refresh cycle
	mov	Ticks, bx

; compute the initial timer count value
	sub	bx, 2*40	; subtract 2 scan lines' worth of
				;  timer ticks (about 40 ticks/line)
				;  (for very slow PCs, increase this
				;  value)
	mov	InitCount, bx

; establish timer mode 0
	cli
	call	VNWAIT		; wait for vertical nondisplay
	Timer0Cmd   00110000b	; timer mode 0
	SetTimer0   InitCount
	sti

; save a pointer to the previous interrupt service routine
	mov	ax, 3508h   ; AH = int 21h function number
			    ; AL = interrupt number
	int	21h	    ; Get Interrupt Vector
	mov	WORD PTR PrevISR08, bx
	mov	WORD PTR PrevISR08+2, es

; update the interrupt 08h vector
	push	ds
	mov	ax, 2508h   ; AH = int 21h function number
			    ; AL = interrupt number
	mov	dx, seg ISR08
	mov	ds, dx
	mov	dx, offset ISR08
	int	21h	    ; Set Interrupt Vector
	pop	ds

	leave
	ret	4	    ; sizeof(LPVOID)

ENABLEVNTIMER	ENDP

; DisableVNTimer -- disables vertical nondisplay interval timer
;
;   void VS2API DisableVNTimer( void );

		PUBLIC	DISABLEVNTIMER
DISABLEVNTIMER	PROC	FAR

	enter	0, 0

; restore the interrupt 08h vector
	push	ds
	mov	ax, 2508h   ; AH = int 21h function number
			    ; AL = interrupt number
	lds	dx, PrevISR08
	int	21h	    ; Set Interrupt Vector
	pop	ds

; restore the default timer 0 mode and initial count
	cli
	Timer0Cmd   00110110b	; mode 3 (square wave)
	SetTimer0   0
	sti

	leave
	ret

DISABLEVNTIMER	ENDP

; ISR08 -- interrupt service routine for vertical nondisplay
;	    interval timer
;
; Notes:
;   This routine makes a far call (without parameters) to the function
;   whose address was supplied to EnableVNTimer.

		PUBLIC	ISR08
ISR08		PROC	FAR

; preserve regs
	push	ds
	push	ax
	push	dx

; address the default data segment through DS
	mov	ax, seg DGROUP
	mov	ds, ax

; wait for the start of vertical nondisplay
	call	VNWAIT

; reinitialize timer 0
	Timer0Cmd   00110000b
	SetTimer0   InitCount

; optionally skip the user-supplied function
	dec	SkipCount
	jns	L210

	mov	ax, VSP.usVNSkip
	mov	SkipCount, ax

; avoid recursion
	cmp	InISR, 0
	jne	L210	    ; jump if already in this ISR

	inc	InISR	    ; set status flag

; switch stacks so that DS = SS
	mov	OldSS, ss
	mov	OldSP, sp

	mov	ax, ds
	mov	ss, ax
	mov	sp, offset StackBottom

; call the user-supplied function
	sti		    ; enable interrupts
	push	es	    ; preserve registers
	push	bx
	push	cx
	push	si
	push	di
	push	bp

	cld		    ; clear the direction flag
	call	fnVNTimer   ; call the user-supplied function

	pop	bp	    ; restore registers
	pop	di
	pop	si
	pop	cx
	pop	bx
	pop	es

; switch the stack back
	mov	ss, OldSS
	mov	sp, OldSP

; clear the recursion flag
	dec	InISR

; increment the system timer count
L210:	mov	ax, Ticks
	add	CumCount, ax
	jc	L295		; jump to chain to previous ISR 8

; allow subsequent interrupts to occur on IRQ0
	mov	al, 20h     ; bit 5 = 1 (nonspecific EOI)
	out	20h, al     ; write to PIC

L290:	pop	dx
	pop	ax
	pop	ds
	iret

; come here to chain to the previous timer ISR
L295:	pushf
	call	PrevISR08
	jmp	L290

ISR08		ENDP

VS2_TEXT	ENDS

_DATA		SEGMENT WORD PUBLIC 'DATA'

		EXTRN	VSP:VIDEOSTATEPARAMS

Ticks		WORD	?   ; number of timer ticks per refresh cycle
InitCount	WORD	?   ; initial timer count
CumCount	WORD	0   ; cumulative timer count
SkipCount	WORD	?   ; optional skip count

PrevISR08	DWORD	?   ; pointer to previous ISR
InISR		WORD	0   ; nonzero while ISR is executing

; saved register values
OldSS		WORD	?
OldSP		WORD	?

; far pointer to user-supplied function
fnVNTimer	DWORD	?

; temporary stack in default data segment
		BYTE	cbStack DUP(?)
StackBottom	LABEL	BYTE

_DATA		ENDS

		END
