		TITLE	S3.ASM
		NAME	S3
		PAGE	55,132

.286

INCLUDE 	vs2.inc

DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

; SetPixel110
;
;   void VS2API SetPixel110( USHORT usX, USHORT usY, USHORT usPixel );

; parameters
usPixel 	EQU	[bp+6]
usY		EQU	[bp+8]
usX		EQU	[bp+10]

		PUBLIC	SETPIXEL110
SETPIXEL110	PROC	FAR

	enter	0, 0

; unlock S3 CRTC registers 30h-3Fh
	mov	dx, 3D4h    ; DX = CRTC I/O addr
	mov	ax, 4838h   ; AH = 01xx10xxb (S3 "unlocking code")
			    ; AL = 38h (S3R8 register number)
	out	dx, ax

; enable hardware-window selection
	mov	al, 31h     ; S3R1 register number
	out	dx, al
	inc	dx
	in	al, dx	    ; AL = current S3R1 register value
	dec	dx
	mov	ah, al
	or	ah, 1	    ; set bit 0 to enable hardware windowing
	mov	al, 31h
	out	dx, ax

; position the 64 Kb hardware window
	mov	ax, usY
	mul	VSP.cbRow   ; DX:AX = byte offset of start of row
	mov	bx, usX     ; BX = x
	add	bx, bx	    ; BX = byte offset of x within row
	add	ax, bx
	adc	dx, 0	    ; DX:AX = byte offset of the pixel

	mov	bx, ax	    ; BX = byte offset of pixel within the
			    ;	    64 Kb hardware window
	mov	es, VSP.segVBuf ; ES:BX -> low-order byte of pixel

	mov	ah, dl	    ; AH = 64 Kb boundary number
	mov	al, 35h     ; S3R5 register number
	mov	dx, 3D4h
	out	dx, al
	inc	dx
	in	al, dx	    ; AL = current S3R5 register value
	dec	dx

	and	ax, 0FF0h   ; zero AH bits 4-7, AL bits 0-3
	or	ah, al	    ; AH bits 0-3 = video memory bank number
	mov	al, 35h
	out	dx, ax

; update the pixel value
	mov	ax, usPixel
	mov	es:[bx], ax

; lock S3 registers 30h-3Fh
	mov	ax, 38h 	; AH = 0 (value for S3R8)
				; AL = 38h (S3R8 register number)
	out	dx, ax

	leave
	ret	6		; 2*sizeof(int) + sizeof(char)

SETPIXEL110	ENDP

; Line110
;
;   void VS2API Line110( USHORT x1, USHORT y1,
;			    USHORT x2, USHORT y2, USHORT usPixel );
; Notes:
;   This function follows S3's technical documentation ("86C911 GUI
;   Accelerator", March 1993).
;
;   The S3 registers must be unlocked (i.e., not write-protected) prior
;   to calling this function.

; parameters
usPixel 	EQU	[bp+6]
y2		EQU	[bp+8]
x2		EQU	[bp+10]
y1		EQU	[bp+12]
x1		EQU	[bp+14]

		PUBLIC	LINE110
LINE110 	PROC	FAR

	enter	0, 0

; wait for 3 empty entries in the FIFO queue
	mov	dx, 9AE8h   ; Graphics Processor Status reg read addr

L000:	in	ax, dx	    ; AX = current GP-STAT register value
	test	al, 20h     ; test the status bit for queue entry 5
	jnz	L000	    ; loop while queue entry is in use

; program the Foreground Mix register
	mov	dh, 0BAh    ; DX = BAE8h (FRGD-MIX reg I/O addr)
	mov	al, VSP.cRMW
	and	ax, 1Fh     ; AX bits 0-4 = raster op
	or	ax, 20h     ; AX bits 5-6: 2 (color from FRGD-COLOR reg)
	out	dx, ax

; store the pixel value in the Foreground Color register
	mov	dh, 0A6h    ; DX = A6E8h (FRGD-COLOR reg I/O addr)
	mov	ax, usPixel
	out	dx, ax

; program the Data Manipulation Control register
	mov	dh, 0BEh    ; DX = BEE8h (MULTIFUNC-CNTL reg I/O addr)
	mov	ax, 0A000h  ; bits 12-15: 0Ah (select DT-MP-CNTL reg)
			    ; bits 6-7:   00h (no data extension)
	out	dx, ax

; wait for 7 empty entries in the FIFO queue
	mov	dh, 9Ah     ; DX = 9AE8h (GP-STAT reg read addr)

L001:	in	ax, dx	    ; AX = current GP-STAT register value
	test	al, 2	    ; test the status bit for queue entry 1
	jnz	L001	    ; loop while queue entry is in use

; store the line-draw parameters
	mov	ax, x1
	mov	dh, 86h     ; DX = 86E8h (Current X reg I/O addr)
	out	dx, ax

	mov	ax, y1
	mov	dh, 82h     ; DX = 82E8h (Current Y reg I/O addr)
	out	dx, ax

	mov	cx, 2013h   ; initial value for Drawing Command reg
			    ; bits 13-15: 1 (draw line)
			    ; bit 9:	  0 (assume 8-bit bus)
			    ; bit 8:	  0 (no wait)
			    ; bit 7:	  0 (Y-direction is negative)
			    ; bit 6:	  0 (|slope| <= 1)
			    ; bit 5:	  0 (X-direction is negative)
			    ; bit 4:	  1 (execute the command)
			    ; bit 3:	  0 (x-y direction)
			    ; bit 2:	  0 (draw the last pixel)
			    ; bit 1:	  1 (draw multiple pixels)
			    ; bit 0:	  1 (write)

	mov	ax, x1
	sub	ax, x2	    ; AX = (x1-x2) = dX
	jnc	L002	    ; jump if dX < 0

	neg	ax	    ; AX = |dX|
	or	cl, 20h     ; CX bit 5 = 1 (X-direction is positive)

L002:	mov	bx, y1
	sub	bx, y2	    ; BX = (y1-y2) = dY
	jnc	L003	    ; jump if dY < 0

	neg	bx	    ; BX = |dY|
	or	cl, 80h     ; CX bit 7 = 1 (Y-direction is positive)

L003:	cmp	ax, bx
	jae	L004	    ; jump if |dX| >= |dY| (i.e., |slope| <= 1)

	xchg	ax, bx
	or	cl, 40h     ; CX bit 6 = 1 (Y is the major axis)

; at this point: AX=max( |dY|, |dX| ), BX=min( |dY|, |dX| )
L004:	mov	dh, 96h     ; DX = 96E8h (MAJ_AXIS_PCNT reg I/O addr)
	out	dx, ax	    ; (major-axis pixel count)

	xchg	ax, bx	    ; AX = min, BX = max
	sub	ax, bx	    ; AX = min - max
	add	ax, ax	    ; AX = 2*(min - max)
	mov	dh, 8Eh     ; DX = 8EE8h (DESTX-DIASTP reg I/O addr)
	out	dx, ax	    ; (diagonal step constant)

	add	ax, bx
	add	ax, bx	    ; AX = 2*min
	mov	dh, 8Ah     ; DX = 8AE8h (DESTX-AXSTP reg I/O addr)
	out	dx, ax	    ; (axial step constant)

	sub	ax, bx	    ; AX = 2*min - max
	test	cl, 20h
	jnz	L005	    ; jump if X-direction is positive

	dec	ax	    ; AX = 2*min - max - 1

L005:	mov	dh, 92h     ; DX = 92E8h (ERR-TERM register I/O addr)
	out	dx, ax	    ; error term

	mov	ax, cx	    ; AX = value for Command register
	mov	dh, 9Ah     ; DX = 9AE8h (CMD register write addr)
	out	dx, ax	    ; draw the line

	leave
	ret	10	    ; 5*sizeof(USHORT)

LINE110 	ENDP

; SetClip110
;
;   void VS2API
;     SetClip110( USHORT left, USHORT top, USHORT right, USHORT bottom );
;
; Notes:
;   The S3 registers must be unlocked (i.e., not write-protected) prior
;   to calling this function.

; parameters
bottom		EQU	[bp+6]
right		EQU	[bp+8]
top		EQU	[bp+10]
left		EQU	[bp+12]

		PUBLIC	SETCLIP110
SETCLIP110	PROC	FAR

	enter	0, 0

; wait for 4 empty entries in the FIFO queue
	mov	dx, 9AE8h   ; GP-STAT reg read addr

L100:	in	ax, dx	    ; AX = current GP-STAT register value
	test	al, 10h     ; test the status bit for queue entry 4
	jnz	L100	    ; loop while queue entry is in use

; load the multifunction control register with the clipping boundaries
	mov	dh, 0BEh    ; DX = BEE8h (MULTIFUNC-CNTL reg I/O addr)

	mov	ax, top
	and	ah, 7	    ; zero AX bits 11-15
	or	ah, 10h     ; AX bits 12-15 = 1 (SCISSORS-T index value)
	out	dx, ax

	mov	ax, left
	and	ah, 7	    ; zero AX bits 11-15
	or	ah, 20h     ; AX bits 12-15 = 2 (SCISSORS-L index value)
	out	dx, ax

	mov	ax, bottom
	and	ah, 7	    ; zero AX bits 11-15
	or	ah, 30h     ; AX bits 12-15 = 3 (SCISSORS-B index value)
	out	dx, ax

	mov	ax, right
	and	ah, 7	    ; zero AX bits 11-15
	or	ah, 40h     ; AX bits 12-15 = 4 (SCISSORS-R index value)
	out	dx, ax

	leave
	ret	8	    ; 4*sizeof(USHORT)

SETCLIP110	ENDP

; S3UnlockRegs
;
;   void VS2API S3UnlockRegs( void );

		PUBLIC	S3UNLOCKREGS
S3UNLOCKREGS	PROC	FAR

	enter	0, 0

; unlock S3 CRTC registers 30h-3Fh
	mov	dx, 3D4h    ; DX = CRTC I/O addr
	mov	ax, 4838h   ; AH = 01xx10xxb (S3 "unlocking code")
			    ; AL = 38h (S3R8 register number)
	out	dx, ax

; unlock S3 CRTC registers 40h-4Fh
	mov	ax, 0A039h  ; AH = 101xxxxxb (S3 "unlocking code")
			    ; AL = 39h (S3R9 register number)
	out	dx, ax

; unlock S3 "enhanced" registers (x2E8h, x6E8h xAE8h, xEE8h)
	mov	al, 40h     ; (SC0 register number)
	out	dx, al
	inc	dx
	in	al, dx	    ; AL = current SC0 register value
	dec	dx
	mov	ah, al
	mov	al, 40h
	mov	PrevSC0, ax ; preserve SC0 value and reg number

	or	ah, 1	    ; AH bit 0 = 1
	out	dx, ax

	leave
	ret

S3UNLOCKREGS	ENDP

; S3LockRegs
;
;   void VS2API S3LockRegs( void );

		PUBLIC	S3LOCKREGS
S3LOCKREGS	PROC	FAR

	enter	0, 0

	mov	dx, 3D4h    ; DX = CRTC I/O addr
	mov	ax, PrevSC0
	out	dx, ax	    ; restore register SC0

	mov	ax, 39h     ; AH = 0 (value for S3R9)
	out	dx, ax	    ; AL = 39h (S3R9 register number)

	mov	ax, 38h     ; AH = 0 (value for S3R8)
	out	dx, ax	    ; AL = 38h (S3R8 register number)

	leave
	ret

S3LOCKREGS	ENDP

VS2_TEXT	ENDS

_DATA		SEGMENT WORD PUBLIC 'DATA'

		EXTRN	VSP:VIDEOSTATEPARAMS

PrevSC0 	WORD	?	    ; saved value of S3 register SC0

_DATA		ENDS

		END
