/***********************************************************************

  S3TEST.C -- test routines for S3 86C911 SVGA chip

***********************************************************************/

#include    <stdio.h>	    /* printf sscanf */
#include    <stdlib.h>	    /* exit rand */
#include    <conio.h>	    /* _kbhit _getch */
#include    <string.h>	    /* memcpy */
#include    <vs2.h>


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

typedef void (* TESTPROC)( void );

void	    main( int, char * * );
void	    Test0( void );
void	    Test1( void );
void	    Test2( void );
void	    Test3( void );
void	    XRGB( USHORT, USHORT *, USHORT *, USHORT * );
void	    Line110X( USHORT, USHORT, USHORT, USHORT, USHORT );
void	    InitVMode( void );
USHORT	    MulDiv( USHORT, USHORT, USHORT );

/* defined in S3.ASM */
void VS2API SetPixel110( USHORT, USHORT, USHORT );
void VS2API Line110( USHORT, USHORT, USHORT, USHORT, USHORT );
void VS2API SetClip110( USHORT, USHORT, USHORT, USHORT );
void VS2API S3UnlockRegs( void );
void VS2API S3LockRegs( void );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS     vsp;

static VIDEOSTATEPARAMS     vsp110 = { 0x110, 0xA000, 0, 0, 2048,
					1024, 512, 640, 480,
					0, 0, 0, 0, 15, 0 };
static TESTPROC Test[] = { Test0, Test1, Test2, Test3 };


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    int     t;


    /* verify command-line syntax */
    if( argc != 2 )
    {
      printf( "Syntax: S3TEST <test #>\n" );
      printf( "  0 = SetPixel110\n" );
      printf( "  1 = Line110: slopes\n" );
      printf( "  2 = Line110: colors\n" );
      printf( "  3 = SetClip110\n" );
      return;
    }

    /* get the requested test number */
    sscanf( argv[1], "%d", &t );
    if( (t < 0) || (t > 3) )
    {
      printf( "The test number must be between 0 and 3" );
      return;
    }

    InitVMode();
    (Test[t])();

    /* restore 80x25 alphanumeric mode */
    SetVMode( 3 );
}


/***********************************************************************

  Test0 -- set all pixels to create a color gamut with varying intensity

***********************************************************************/

void Test0()
{
    USHORT  x, y, yy, p;
    USHORT  r, g, b, ry, gy, by;


    for( x=0; x<640; x++ )
    {
      /* vary the RGB components across x */
      XRGB( x, &r, &g, &b );
      for( y=0; y<vsp.usSY; y+=15 )
      {
	/* decreasing brightness with increasing y */
	ry = (r * (480-y)) / 480;
	gy = (g * (480-y)) / 480;
	by = (b * (480-y)) / 480;

	/* pack rgb values into 3 5-bit fields */
	p = (ry << 10) + (gy << 5) + by;
	for( yy=y; yy<y+14; yy++ )
	  SetPixel110( x, yy, p );

	/* separate vertically with a gray pixel (r=g=b=0x08) */
	SetPixel110( x, yy, 0x2108 );
      }
    }

    _getch();
}


/***********************************************************************

  Test1 -- lines in all directions

***********************************************************************/

void Test1()
{
    USHORT  xc, yc;
    USHORT  r = (0x1F << 10);
    USHORT  g = (0x1F << 5);
    USHORT  b = 0x1F;


    xc = vsp.usSX / 2;
    yc = vsp.usSY / 2;

    S3UnlockRegs();

    /* just one pixel */
    Line110X( xc, yc, xc, yc, r+g+b );

    /* horizontal lines */
    Line110X( xc+1, yc, xc+200, yc, b );
    Line110X( xc-1, yc, xc-200, yc, b );

    /* vertical lines */
    Line110X( xc, yc+1, xc, yc+200, b );
    Line110X( xc, yc-1, xc, yc-200, b );

    /* diagonal lines */
    Line110X( xc-1, yc-1, xc-200, yc-200, g );
    Line110X( xc+1, yc+1, xc+200, yc+200, g );
    Line110X( xc-1, yc+1, xc-200, yc+200, g );
    Line110X( xc+1, yc-1, xc+200, yc-200, g );

    /* lines with |slope| > 1 */
    Line110X( xc-1, yc-1, xc-100, yc-200, r );
    Line110X( xc+1, yc+1, xc+100, yc+200, r );
    Line110X( xc-1, yc+1, xc-100, yc+200, r );
    Line110X( xc+1, yc-1, xc+100, yc-200, r );

    /* lines with |slope| < 1 */
    Line110X( xc-1, yc-1, xc-200, yc-100, r );
    Line110X( xc+1, yc+1, xc+200, yc+100, r );
    Line110X( xc-1, yc+1, xc-200, yc+100, r );
    Line110X( xc+1, yc-1, xc+200, yc-100, r );

    S3LockRegs();

    _getch();
}


/***********************************************************************

  Test2 -- a color gamut with constant intensity

***********************************************************************/

void Test2()
{
    USHORT  x, r, g, b, p;


    S3UnlockRegs();

    for( x=0; x<vsp.usSX; x++ )
    {
      XRGB( x, &r, &g, &b );
      p = (r<<10) + (g<<5) + b;
      Line110X( x, 0, x, vsp.usSY-1, p );
    }

    S3LockRegs();

    _getch();
}


/***********************************************************************

  Test3 -- random lines

  Notes:
    See the note to Line110X below for an explanation of the goofy
    technique used to clip and draw lines in this routine.

***********************************************************************/

void Test3()
{
    USHORT  x1, y1, x2, y2;
    USHORT  g, p;
    USHORT  Xulc, Yulc, Xlrc, Ylrc;


    S3UnlockRegs();

    /* use a clipping rectangle that's smaller than the screen */
    Xulc = (USHORT)(vsp.usSX / 5);
    Xlrc = (USHORT)(vsp.usSX - Xulc );
    Yulc = (USHORT)(vsp.usSY / 5);
    Ylrc = (USHORT)(vsp.usSY - Yulc );

    /* generate lots of random lines */
    while( !_kbhit() )
    {
      /* pick some random coordinates */
      x1 = MulDiv( (USHORT)rand(), vsp.usSX-1, RAND_MAX );
      x2 = MulDiv( (USHORT)rand(), vsp.usSX-1, RAND_MAX );
      y1 = MulDiv( (USHORT)rand(), vsp.usSY-1, RAND_MAX );
      y2 = MulDiv( (USHORT)rand(), vsp.usSY-1, RAND_MAX );

      /* pick a random shade of gray */
      g = MulDiv( (USHORT)rand(), 16, RAND_MAX ) + 4;
      p = (g<<10) + (g<<5) + g;

      /* draw the entire line in gray */
      SetClip110( 0, 0, vsp.usSX+vsp.usCX, vsp.usSY+vsp.usCY );
      Line110X( x1, y1, x2, y2, p );

      /* draw the clipped line in a random color */
      p = (USHORT)rand();
      SetClip110( Xulc, Yulc, Xlrc, Ylrc );
      Line110( x1, y1, x2, y2, p&0xFF );
      SetClip110( Xulc+vsp.usCX, Yulc, Xlrc+vsp.usCX, Ylrc );
      Line110( x1+vsp.usCX, y1, x2+vsp.usCX, y2, p );
    }

    S3LockRegs();

    /* throw away the key that was pressed */
    _getch();
}


/***********************************************************************

  XRGB -- red, green, and blue values for a color gamut varying with x

***********************************************************************/

void XRGB( USHORT x, USHORT * pr, USHORT * pg, USHORT * pb )
{
    USHORT  CMax = 32;


    switch( (x/106) % 6 )
    {
      case 0:			  /* 0 <= x < 106 */
	*pr = CMax - 1;
	*pg = (CMax * (x%636)) / 106;
	*pb = 0;
	break;

      case 1:			  /* 106 <= x < 212 */
	*pr = (CMax * (211-x)) / 106;
	*pg = CMax - 1;
	*pb = 0;
	break;

      case 2:			  /* 212 <= x < 318 */
	*pr = 0;
	*pg = CMax - 1;
	*pb = (CMax * (x-212)) / 106;
	break;

      case 3:			  /* 318 <= x < 424 */
	*pr = 0;
	*pg = (CMax * (423-x)) / 106;
	*pb = CMax - 1;
	break;

      case 4:			  /* 424 <= x < 530 */
	*pr = (CMax * (x-424)) / 106;
	*pg = 0;
	*pb = CMax - 1;
	break;

      case 5:			  /* 530 <= x < 636 */
	*pr = CMax -1;
	*pg = 0;
	*pb = (CMax * (635-x)) / 106;
	break;
    }
}


/***********************************************************************

  Line110X -- line-drawing magic for mode 110h on the S3 86C911

  Notes:
    This video mode is implemented by S3 as a chained, 2-plane mode
    with 16 bits per pixel (bits 0-7 in bit plane 0, 8-15 in plane 1).
    However, the Graphics Processor can only draw lines in one plane
    at a time, hence the need to draw the same line twice.  (See S3's
    technical documentation on the 86C911 for more details.)

***********************************************************************/

void Line110X( USHORT x1, USHORT y1, USHORT x2, USHORT y2, USHORT p )
{
    /* draw the line in bit plane 0 (pixel bits 0-7) */
    Line110( x1, y1, x2, y2, p&0xFF );

    /* draw the line in bit plane 1 (pixel bits 8-15) */
    Line110( x1+vsp.usCX, y1, x2+vsp.usCX, y2, p>>8 );
}


/***********************************************************************

  InitVMode

  Notes:
    Sets up 640-by-480 15bpp graphics mode (VESA mode 110h)

***********************************************************************/

void InitVMode()
{
    SetVMode( 0x110 );

#ifdef NEVER
    if( 0x110 != GetVMode() )
    {
      printf( "Can't set video mode." );
      exit( 3 );
    }

    /* initialize global variables for VBE mode 110h */
    _asm
    {
      mov   ax, 4F06h		; VBE function 06h
      mov   bl, 1		;  (get logical scan line length)
      int   10h

      mov   vsp110.cbRow, bx	; bytes per row
      mov   vsp110.usCX, cx	; pixels per row
      mov   vsp110.usCY, dx	; number of rows in video buffer
    }
#endif

    memcpy( &vsp, &vsp110, sizeof(VIDEOSTATEPARAMS) );

    /* select a raster op (see S3 documentation) */
    vsp.cRMW = 0x07;		/* replace: 0x07 */
				/* XOR:     0x05 */
				/* AND:     0x0C */
				/* OR:	    0x0B */
}


/***********************************************************************

  MulDiv

  Notes:
    Leaves its result in register AX.  (The MS C compiler generates
    a warning message anyway.)

***********************************************************************/

USHORT MulDiv( USHORT n1, USHORT n2, USHORT n3 )
{
    __asm
    {
      mov   ax, n1
      mul   n2
      div   n3
    }
}
