		TITLE	HGC.ASM
		NAME	HGC
		PAGE	55,132

.286

INCLUDE 	vs2.inc

DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

		EXTRN	BUFCOPY:FAR

; ModeVGAHGC (VGA emulation of Hercules 720x348 monochrome graphics)
;
;   void VS2API ModeVGAHGC( void );

		PUBLIC	MODEVGAHGC
MODEVGAHGC	PROC	FAR

	enter	0, 0

	push	si	    ; preserve regs
	push	di

; get a pointer to video-mode parameters
	mov	si, offset RegsVGAHGC
	cld

; disable video data display
	mov	ah, 20h
	call	VideoOnOff

; set Miscellaneous Output register
	mov	dx, 3C4h	; DX = 3C4h (sequencer)
	mov	ax, 0100h	; synchronous reset
	out	dx, ax

	mov	dl, 0C2h	; DX = 3C2h (misc output)
	lodsb			; AL = value for Misc output
	out	dx, al

; set sequencer registers
	mov	dl, 0C4h	; DX = 3C4h
	mov	cx, 5		; CX = # of registers
	rep outsw

; set CRTC registers
	mov	dl, 0B4h	; DX = 3B4h
	mov	cl, 1Ah 	; CX = # of registers
	rep outsw

; set graphics controller registers
	mov	dl, 0CEh	; DX = 3CEh
	mov	cl, 9		; CX = # of registers
	rep outsw

; set attribute controller registers
	mov	dl, 0BAh	; DX = 3BAh (input status reg 1)
	in	al, dx		; reset the attribute-
				;  controller flip-flop

	mov	dl, 0C0h	; DX = 3C0h
	mov	cx, 2*15h+1	; all 21 regs and PAS bit
	rep outsb

; initialize the BIOS video state parameters
	mov	di, segVDDA
	mov	es, di
	mov	di, CRT_MODE
	mov	cx, 30
	rep	movsb

; save a pointer to the 8x8 ROM BIOS character bitmap table
	lodsw
	mov	dx, ax	    ; DL = ROWS; DH = POINTS

	mov	ax, 1123h   ; AH = 11h (BIOS Character Load)
			    ; AL = 23h (ROM BIOS 8x8 chars)
	xor	bl, bl	    ; BL = 0 (DL = # of rows)
	int	10h

; initialize the video DAC with a reasonable palette
	mov	dx, 3C8h
	xor	al, al
	out	dx, al		; set DAC address to 0

	inc	dx		; DX = 3C9h
	mov	si, offset DACPalette
	mov	cx, 3*16
	rep	outsb

; fill the video buffer with nulls
	mov	cx, 0B000h
	mov	es, cx
	xor	di, di		; ES:DI -> B000:0000
	xor	ax, ax
	mov	cx, 8000h
	rep	stosw

; enable video data display
	xor	ah, ah
	call	VideoOnOff

; copy initial video state params
	push	ds	    ; push lpTo
	push	offset VSP
	push	ds	    ; push lpFrom
	push	offset VSPHGC
	push	sizeof VSP  ; push cb
	call	BUFCOPY

; restore registers and exit
	pop	di
	pop	si

	leave
	ret

MODEVGAHGC	ENDP

;
; VideoOnOff (for VGA, not HGC!)
;
; Caller:   AH = 0 (video on) or 20h (video off)
;
; Returns:  nothing
;
; Registers modified:	none
;

VideoOnOff	PROC	 NEAR

	push	dx	    ; preserve regs
	push	ax

; get the current value of the sequencer clocking-mode register
	mov	dx, 3C4h    ; sequencer
	mov	al, 1	    ; clocking-mode reg #
	out	dx, al

	inc	dx	    ; DX = 3C5h
	in	al, dx	    ; AL = current reg value

; update bit 5
	and	al, 0DFh    ; reset bit 5
	or	ah, al	    ; update bit 5

	dec	dx	    ; DX = 3C4h
	mov	al, 1	    ; reg #
	out	dx, ax

; restore regs and return
	pop	ax
	pop	dx
	ret

VideoOnOff	ENDP

; ModeHGC (Hercules 720x348 monochrome graphics)
;
;   void VS2API ModeHGC( void );

		PUBLIC	MODEHGC
MODEHGC 	PROC	FAR

	enter	0, 0
	push	si	    ; preserve regs
	push	di

; update the BIOS Video Display Data Area with reasonable values
	mov	ax, segVDDA
	mov	es, ax
	mov	di, CRT_MODE	; ES:DI ->VDDA

	mov	si, offset HGCBIOSData
	mov	cx, HGCBIOSDataLen
	rep	movsb

; set the HGC Configuration Switch
	mov	dx, 3BFh    ; DX = Configuration Switch port
	mov	al, 1	    ; AL bit 1 = 0 (exclude 2nd 32K of
			    ;		      video buffer)
			    ; AL bit 0 = 1 (allow graphics mode
	out	dx, al	    ;		     setting via 3B8h)

; blank the screen to avoid interference during CRTC programming
	mov	dx, 3B8h    ; DX = CRTC Mode Control Register port
	xor	al, al	    ; AL bit 3 = 0 (disable video signal)
	out	dx, al	    ; blank the screen

; program the CRTC
	sub	dl, 4	    ; DX = CRTC Address Reg port 3B4h
	mov	si, offset RegsHGC
	mov	cx, 9
	rep outsw

; set graphics mode
	add	dl, 4	    ; DX = 3B8h (CRTC Mode Control Reg)
	mov	al, CRTMode ; AL bit 1 = 1 (enable graphics mode)
			    ;	 bit 3 = 1 (enable video)
	out	dx, al

; copy initial video state params
	push	ds	    ; push lpTo
	push	offset VSP
	push	ds	    ; push lpFrom
	push	offset VSPHGC
	push	sizeof VSP  ; push cb
	call	BUFCOPY

; restore registers and exit
	pop	di
	pop	si
	leave
	ret

MODEHGC 	ENDP

; parameters for all PixelAddr functions
usY		EQU	[bp+6]
usX		EQU	[bp+8]

; PixelAddrHGC
;
;   LPCHAR VS2API PixelAddrHGC( USHORT usX, USHORT usY );

		PUBLIC	PIXELADDRHGC
PIXELADDRHGC	PROC	FAR

	enter	0, 0

	mov	ax, usX 	; AX = x
	mov	cl, al		; CL = low-order byte of x
	mov	bx, usY 	; BX = y

	shr	bx, 1		; BX = y/2
	rcr	ax, 1		; AX = 8000h*(y&1) + x/2
	shr	bx, 1		; BX = y/4
	rcr	ax, 1		; AX = 4000h*(y&3) + x/4
	shr	ax, 1		; AX = 2000h*(y&3) + x/8
	xchg	ax, bx		; BX = 2000h*(y&3) + x/8
	mul	VSP.cbRow	; AX = cbRow*(y/4)
	add	ax, bx
	add	ax, VSP.ofsVBuf ; add origin offset
	mov	dx, VSP.segVBuf ; DX:AX -> pixel

	leave
	ret	4		; 2*sizeof(USHORT)

PIXELADDRHGC	 ENDP

; parameters for all SetPixel functions
ucPixel 	EQU	[bp+6]
usY		EQU	[bp+8]
usX		EQU	[bp+10]

; SetPixelHGC
;
;   void VS2API SetPixelHGC( USHORT usX, USHORT usY, UCHAR ucPixel );

		PUBLIC	SETPIXELHGC
SETPIXELHGC	PROC	FAR

	enter	0, 0

; point to the specified pixel
	push	usX
	push	usY
	call	PixelAddrHGC
	mov	bx, ax
	mov	es, dx	    ; ES:BX -> pixel

; compute a bit mask for the 1-bit packed pixel
	mov	ax, 101h    ; AH = 1 (bit mask)
			    ; AL = 1 (mask for pixel value)
	not	cl
	and	cl, 7	    ; CL = # bits to shift left

; shift the bit mask and new pixel value into position
	and	al, ucPixel
	shl	ax, cl

; jump to the Replace, AND, OR or XOR routine
	jmp	SetPixelOpHGC

; come here to Replace the pixel value
ReplPixelHGC::
	not	ah		; AH = inverse bit mask
	and	es:[bx], ah	; zero the pixel value
	or	es:[bx], al	; set the pixel value
	jmp	L001

; come here to AND the pixel value
ANDPixelHGC::
	not	ah		; AH = inverse bit mask
	or	al, ah		; AL = all 1's except pixel value
	and	es:[bx], al
	jmp	L001
				
; come here to OR the pixel value
ORPixelHGC::
	or	es:[bx], al
	jmp	L001

; come here to XOR the pixel value
XORPixelHGC::
	xor	es:[bx], al

L001:	leave
	ret	6

SETPIXELHGC	ENDP

; LineHGC
;
;   void VS2API
;     LineHGC( USHORT x1, USHORT y1, USHORT x2, USHORT y2, UCHAR ucPixel );

; parameters for all Line functions
ucPixel 	EQU	[bp+6]
y2		EQU	[bp+8]
x2		EQU	[bp+10]
y1		EQU	[bp+12]
x1		EQU	[bp+14]

; temporary variables
LineFn		EQU	WORD PTR [bp-2]
VertIncr	EQU	WORD PTR [bp-4]
Incr2		EQU	WORD PTR [bp-6]
Incr1		EQU	WORD PTR [bp-8]
IncrY		EQU	WORD PTR [bp-10]

		PUBLIC	LINEHGC
LINEHGC 	PROC	FAR

	enter	10, 0

	push	si
	push	di

	and	BYTE PTR ucPixel, 1	; force a valid pixel value
	mov	si, VSP.cbRow

; check for horizontal line
	mov	bx, y2
	sub	bx, y1		; BX = y2 - y1
	jnz	L100

	call	HorzLineHGC
	jmp	L109

; check for vertical line
L100:	mov	cx, x2
	sub	cx, x1		; CX = x2 - x1
	jnz	L101

	call	VertLineHGC
	jmp	L109

; come here if not horizontal or vertical
L101:	jns	L102		; jump if x2 > x1

; at this point x2 < x1, so reverse the endpoints
	neg	cx		; CX = dX
	neg	bx		; BX = dY

	mov	ax, x2		; exchange x1 and x2
	xchg	ax, x1
	mov	x2, ax

	mov	ax, y2		; exchange y1 and y2
	xchg	ax, y1
	mov	y2, ax

; at this point, dX (in register CX) is positive
L102:	sub	si, 8000h
	mov	IncrY, si	; IncrY = cbRow-8000h
	mov	si, 2000h

	or	bx, bx		; test sign of dY
	jns	L104		; jump if slope is positive

	neg	bx		; make dY positive
	neg	si		; negate interleave increments
	neg	IncrY

; select the appropriate routine for the slope of line
L104:	mov	LineFn, offset LoSlopeLineHGC
	cmp	bx, cx
	jl	L105		; jump if dY < dX (slope < 1)

	mov	LineFn, offset HiSlopeLineHGC
	xchg	bx, cx		; exchange dY and dX

; calculate the initial decision variable and increments
L105:	shl	bx, 1		; BX = Incr1 = 2*dY
	mov	Incr1, bx
	mov	dx, bx
	sub	dx, cx		; DX = d = 2*dY-dX
	mov	Incr2, dx
	sub	Incr2, cx	; Incr2 = 2*(dY-dX)

; calculate the first pixel address
	push	cx		; preserve dX and d
	push	dx

	push	x1
	push	y1
	call	PixelAddrHGC	; DX:AX -> buffer
	mov	es, dx
	mov	di, ax		; ES:DI -> buffer

; create a bit mask for the first pixel
	mov	ah, 80h 	; AH = 80h (bit mask)
	mov	cl, x1
	and	cl, 7		; CL = x1 MOD 8
	shr	ah, cl		; AH = bit mask

	pop	dx		; DX = d
	pop	cx		; CX = dX
	inc	cx		; CX = # of pixels to draw

; mask the pixel value
	mov	bx, offset PackedPixelHGC
	mov	al, ucPixel
	xlat
	mov	bl, al
	and	bl, ah		; BL = masked pixel
	mov	bh, ah
	not	bh		; BH = inverse mask

				; at this point:
				;  ES:DI -> first pixel
				;   BH = inverse bit mask for 1st pixel
				;   BL = pixel value
				;   CX = # pixels to draw
				;   DX = decision variable
				;   SI = y-increment

	call	LineFn		; draw the line

; restore registers and return
L109:	pop	di
	pop	si
	leave
	ret	10		; 4*sizeof(USHORT) + sizeof(UCHAR)

LINEHGC 	ENDP

VertLineHGC	PROC	NEAR

	mov	ax, y1		; AX = y1
	mov	cx, y2
	sub	cx, ax		; CX = y2-y1
	jge	L110		; jump if dY >= 0

	add	ax, cx		; AX = y2
	neg	cx		; force dY >= 0

L110:	inc	cx		; CX = # of pixels to draw
	push	cx		; preserve CX

; point to the bottommost pixel
	push	x1
	push	ax
	call	PixelAddrHGC
	mov	es, dx
	mov	di, ax		; ES:DI = byte address of pixel

L111:	mov	al, ucPixel
	mov	bx, offset PackedPixelHGC
	xlat
	mov	bl, al		; AL = packed pixel value

	mov	ax, 807Fh	; AH = 80h (bit mask)
				; AL = 7Fh (inverse bit mask)
	mov	cl, x1
	and	cl, 7		; CL = x1 MOD 8
	shr	ah, cl		; AH = bit mask
	ror	al, cl		; AL = inverse bit mask

	pop	cx		; CX = # of pixels

; create a y-increment for moving through the 4-way interleave
	sub	si, 8000h	; SI = cbRow - 8000h

; draw the line
	and	ah, bl		; AH = pixel value
L112:	and	es:[di], al	; zero previous pixel
	or	es:[di], ah	; update pixel

	add	di, 2000h	; increment through interleave
	jns	L113

	add	di, si
L113:	loop	L112

	ret

VertLineHGC	ENDP

HorzLineHGC	PROC	NEAR

; ensure that x1 < x2 (move left to right)
	mov	di, x1		; DI = x1
	mov	si, x2		; SI = x2
	cmp	si, di
	jae	L120

	xchg	si, di		; DI = leftmost x-coordinate

; determine the bit masks for the leftmost and rightmost pixel(s)
L120:	mov	dx, 0FF80h	; DH = FFh (left mask)
				; DL = 80h (right mask)
	mov	cx, di
	and	cl, 7		; CL = x1 MOD 8
	shr	dh, cl		; DH = left mask
	mov	cx, si
	and	cl, 7		; CL = x2 MOD 8
	sar	dl, cl		; DL = right mask

; compute the byte offset of the first byte in the row
	push	dx		; preserve bit masks
	mov	ax, y1
	xor	bx, bx		; BX = 0
	shr	ax, 1		; AX = y/2; cf = y MOD 2
	rcr	bx, 1		; BX = 8000h * (y MOD 2)
	shr	ax, 1		; AX = y/4
	rcr	bx, 1		; BX = 4000h * (y MOD 4)
	shr	bx, 1		; BX = 2000h * (y MOD 4)
	mul	VSP.cbRow	; AX = y * (bytes per row)
	add	ax, bx		; AX = offset of first byte in row
	pop	dx		; restore bit masks

; compute byte offsets for x1 and x2
	shr	di, 3		; DI = byte offset of x1
	shr	si, 3		; DX = byte offset of x2

	mov	cx, si
	sub	cx, di		; CX = difference of byte offsets

; point to the first byte in the line
	add	di, ax
	add	di, VSP.ofsVBuf
	mov	es, VSP.segVBuf ; ES:DI -> first byte

	jcxz	L123		; jump if only 1 byte in the line

; set pixels in the leftmost byte of the line
	mov	al, ucPixel
	mov	bx, offset PackedPixelHGC
	xlat			; AL = packed pixel value

	mov	ah, dh		; AH = bit mask for 1st byte
	and	al, ah		; AL = masked pixel(s)
	not	ah		; AH = inverse bit mask

	and	ah, es:[di]	; AH = masked pixel data
	or	al, ah		; AL = updated pixel data
	stosb			; store into video buffer

	mov	al, ucPixel
	xlat			; AL = packed pixel value

	dec	cx
	jz	L122		; jump if only 1 byte remains

; draw the middle of the line
	mov	ah, al		; AH:AL = packed pixels
	shr	cx, 1		; CX = # of words; cf set if odd
	rep	stosw		; update 2 bytes at a time
	jnc	L122		; jump if even # of bytes
	stosb			; update the odd byte

; set pixels in the rightmost byte of the line
L122:	mov	ah, dl		; AH = bit mask for last byte
	and	al, ah		; AL = masked pixel(s)
	not	ah		; AH = inverse bit mask

	and	ah, es:[di]	; AH = masked pixel data
	or	ah, al		; AH = updated pixel data
	mov	es:[di], ah	; store into video buffer

	ret

; come here if only one byte in the line
L123:	mov	al, ucPixel
	mov	bx, offset PackedPixelHGC
	xlat			; AL = packed pixel value

	mov	ah, dl
	and	ah, dh		; AH = bit mask for the byte
	and	al, ah		; AL = masked pixel(s)
	not	ah		; AH = inverse bit mask

	and	ah, es:[di]	; AH = masked pixel data
	or	ah, al		; AH = updated pixel data
	mov	es:[di], ah	; store into video buffer

	ret

HorzLineHGC	ENDP

LoSlopeLineHGC	PROC	NEAR	; come here for dY < dX (slope < 1)

L130:	mov	al, es:[di]	; AL = next byte from video buffer

L131:	and	al, bh		; zero current pixel
	or	al, bl		; set current pixel
	ror	bl, 1		; rotate pixel value
	ror	bh, 1		; rotate inverse mask
	jnc	L134		; jump if bit mask rotated to
				;  leftmost pixel position

; bit mask not rotated around
	or	dx, dx		; test sign of d
	jns	L132		; jump if d >= 0

	add	dx, Incr1	; d = d + Incr1
	loop	L131

; at this point, all but the last byte of pixels have been set
	mov	es:[di], al	; set remaining pixel(s)
	ret

; come here when d>=0
L132:	add	dx, Incr2	; d = d + Incr2
	mov	es:[di], al	; update bit planes
	add	di, si		; increment y
	jns	L133
	add	di, IncrY
L133:	loop	L130
	ret

; bit mask rotated around
L134:	stosb			; update bit planes; autoincrement DI

	or	dx, dx		; test sign of d
	jns	L135		; jump if non-negative

	add	dx, Incr1	; d = d + Incr1
	loop	L130
	ret

L135:	add	dx, Incr2	; d = d + Incr2
	add	di, si		; increment y
	jns	L136
	add	di, IncrY
L136:	loop	L130
	ret

LoSlopeLineHGC	ENDP

HiSlopeLineHGC	PROC	NEAR	; come here for dY >= dX (slope >= 1)

L141:	mov	al, es:[di]	; AL = next byte from video buffer
	and	al, bh		; zero current pixel
	or	al, bl		; update current pixel
	mov	es:[di], al	; store updated pixel
	add	di, si		; increment y
	jns	L142
	add	di, IncrY

L142:	or	dx, dx		; test sign of d
	jns	L143		; jump if d >= 0

	add	dx, Incr1	; d = d + Incr1
	loop	L141		; loop until all pixels are drawn
	ret

L143:	add	dx, Incr2	; d = d + Incr2
	ror	bl, 1		; rotate pixel value
	ror	bh, 1		; rotate bit mask
	cmc			; cf set if mask has rotated to
				;  leftmost pixel position
	adc	di, 0		; conditionally increment DI
	loop	L141
	ret

HiSlopeLineHGC	ENDP

VS2_TEXT	ENDS

_DATA		SEGMENT WORD PUBLIC 'DATA'

		EXTRN	VSP:VIDEOSTATEPARAMS
VSPHGC	VIDEOSTATEPARAMS  < 07h, 0B000h,, 8,  90, 720,	348, 720, 348 >

; ModeVGAHGC ...
RegsVGAHGC	DB  086h  ; miscellaneous output (port 3C2h)
			  ; sequencer (port 3C4h)
		DW  02101h  ; clocking mode
		DW  00F02h  ; map mask
		DW  00003h  ; character map select
		DW  00604h  ; memory mode
		DW  00600h  ; reset
			  ; CRTC (port 3B4h)
		DW  02511h  ; (write-protect disabled)
		DW  06900h  ; horizontal total
		DW  05901h  ; horizontal display enable end
		DW  05A02h  ; start horizontal blanking
		DW  00C03h  ; end horizontal blanking
		DW  05D04h  ; start horizontal retrace pulse
		DW  08B05h  ; end horizontal retrace pulse
		DW  08706h  ; vertical total
		DW  01F07h  ; overflow
		DW  00008h  ; preset row scan
		DW  04309h  ; max scan line
		DW  0000Ah  ; cursor start
		DW  0000Bh  ; cursor end
		DW  0000Ch  ; start address high
		DW  0000Dh  ; start address low
		DW  0000Eh  ; cursor location high
		DW  0000Fh  ; cursor location low
		DW  07110h  ; vertical retrace start
		DW  08311h  ; vertical retrace end
		DW  05B12h  ; vertical display enable end
		DW  02D13h  ; offset
		DW  00F14h  ; underline location
		DW  06315h  ; vertical blank start
		DW  08116h  ; vertical blank end
		DW  0C017h  ; CRT mode control
		DW  0FF18h  ; line compare
			 ; graphics controller (3CEh)
		DW  00000h  ; set/reset
		DW  00001h  ; enable set/reset
		DW  00002h  ; color compare
		DW  00003h  ; data rotate
		DW  00004h  ; read map select
		DW  00005h  ; graphics mode
		DW  00906h  ; miscellaneous
		DW  00007h  ; color don't care
		DW  0FF08h  ; bit mask
			 ; attribute controller (3C0h)
		DW  0000h, 0101h, 0202h, 0303h	; internal palette
		DW  0404h, 0505h, 0606h, 0707h
		DW  0808h, 0909h, 0A0Ah, 0B0Bh
		DW  0C0Ch, 0D0Dh, 0E0Eh, 0F0Fh
		DW  4110h   ; attribute mode control
		DW  0011h   ; overscan color
		DW  0F12h   ; color plane enable
		DW  0013h   ; horizontal pel panning
		DW  0014h   ; color select
		DB  20h     ; PAS bit set
			  ; BIOS video display data area
		DB  27h     ; CRT_MODE
		DW  90	    ; CRT_COLS
		DW  90*350  ; CRT_LEN
		DW  0	    ; CRT_START
		DW  8 dup(0); CURSOR_POSN
		DW  0	    ; CURSOR_MODE
		DB  0	    ; ACTIVE_PAGE
		DW  3B4h    ; ADDR_6845
		DB  0	    ; CRT_MODE_SET
		DB  0	    ; CRT_PALETTE
		DB  43	    ; # of character rows
		DB  8	    ; POINTS

DACPalette	DB  00h, 00h, 00h   ; 00h
		DB  00h, 00h, 2Ah   ; 01h
		DB  00h, 2Ah, 00h   ; 02h
		DB  00h, 2Ah, 2Ah   ; 03h
		DB  2Ah, 00h, 00h   ; 04h
		DB  2Ah, 00h, 2Ah   ; 05h
		DB  2Ah, 15h, 00h   ; 06h
		DB  2Ah, 2Ah, 2Ah   ; 07h
		DB  15h, 15h, 15h   ; 08h
		DB  15h, 15h, 3Fh   ; 09h
		DB  15h, 3Fh, 15h   ; 0Ah
		DB  15h, 3Fh, 3Fh   ; 0Bh
		DB  3Fh, 15h, 15h   ; 0Ch
		DB  3Fh, 15h, 3Fh   ; 0Dh
		DB  3Fh, 3Fh, 15h   ; 0Eh
		DB  3Fh, 3Fh, 3Fh   ; 0Fh

; ModeHGC ...
HGCBIOSData	DB  7		; CRT_MODE
		DW  80		; CRT_COLS
		DW  8000h	; CRT_LEN
		DW  0		; CRT_START
		DW  8 dup(0)	; CURSOR_POSN
		DW  0		; CURSOR_MODE
		DB  0		; ACTIVE_PAGE
CRTCAddr	DW  3B4h	; ADDR_6845
CRTMode 	DB  0Ah 	; CRT_MODE_SET (value for port 3B8h)
		DB  0		; CRT_PALETTE (unused)

HGCBIOSDataLen	EQU $-HGCBIOSData

; CRTC parameters recommended by Hercules
RegsHGC 	DW  3500h   ; Horizontal Total
		DW  2D01h   ; Horizontal Displayed
		DW  2E02h   ; Horizontal Sync Position
		DW  0703h   ; Horizontal Sync Width
		DW  5B04h   ; Vertical Total
		DW  0205h   ; Vertical Adjust
		DW  5706h   ; Vertical Displayed
		DW  5707h   ; Vertical Sync Position
		DW  0309h   ; Max Scan Line

; SetPixelHGC ...
SetPixelOpHGC	WORD	ReplPixelHGC

; LineHGC ...
PackedPixelHGC	BYTE	00000000b, 11111111b

_DATA		ENDS

		END
