		TITLE	VGA.ASM
		NAME	VGA
		PAGE	55,132

; Setting VGA pixels using Graphics Controller write modes 0 and 3

.286

INCLUDE 	vs2.inc

; parameters for all SetPixel functions
ucPixel 	EQU	[bp+6]
usY		EQU	[bp+8]
usX		EQU	[bp+10]


DGROUP		GROUP	_DATA

VS2_TEXT	SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	ds:DGROUP

; SP100a (write mode 0, Set/Reset)
;
;   void VS2API SP100a( USHORT usX, USHORT usY, UCHAR ucPixel );

		PUBLIC	SP100a
SP100a		PROC	FAR

	enter	0, 0

; point to the specified pixel
	mov	ax, usY
	mul	VSP.cbRow   ; AX = y * (bytes per row)

	mov	dx, usX
	mov	cx, dx	    ; CX = x
	shr	dx, 3	    ; DX = x/8
	add	ax, dx	    ; AX = y*(bytes per row) + x/8

	add	ax, VSP.ofsVBuf ; add origin offset
	mov	bx, ax
	mov	es, VSP.segVBuf ; ES:BX -> pixel

; compute a mask for the pixel's bit position
	and	cl, 7	    ; CL = # of bits to shift
	mov	ax, 8008h   ; AH = 80h (bit mask)
			    ; AL = 8 (Bit Mask reg #)
	shr	ah, cl	    ; AH = bit mask for pixel

; set the Graphics Controller Bit Mask register
	mov	dx, 3CEh    ; GC I/O address
	out	dx, ax

; set the Graphics Controller Mode register
	mov	ax, 5	    ; AL =  Mode register number
			    ; AH =  Write Mode 0 (bits 0,1)
			    ;	     Read Mode 0 (bit 3)
	out	dx, ax

; set the Data Rotate/Function Select register
	mov	ah, VSP.cRMW  ; AH = Read-Modify-Write bits
	mov	al, 3	    ; AL = Data Rotate/Function Select
	out	dx, ax

; set the Set/Reset and Enable Set/Reset registers
	mov	ah, ucPixel ; AH = pixel value
	mov	al, 0	    ; AL = Set/Reset reg number
	out	dx, ax

	mov	ax, 0F01h   ; AH = value for Enable Set/Reset
			    ;  (all bit planes enabled)
			    ; AL = Enable Set/Reset reg number
	out	dx, ax

; set the pixel value
	or	es:[bx], al ; load latches during CPU read;
			    ; update latches and bit planes
			    ;  during CPU write

; restore default Graphics Controller registers
	mov	ax, 0FF08h  ; Bit Mask register
	out	dx, ax

	mov	ax, 5	    ; Mode register
	out	dx, ax

	mov	ax, 3	    ; Function Select
	out	dx, ax

	mov	ax, 1	    ; Enable Set/Reset
	out	dx, ax

	leave
	ret	6	    ; 2*sizeof(int) + sizeof(char)

SP100a		ENDP

; SP100b (write mode 0, Sequencer Map Mask)
;
;   void VS2API SP100b( USHORT usX, USHORT usY, UCHAR ucPixel );

		PUBLIC	SP100b
SP100b		PROC	FAR

	enter	0, 0

; point to the specified pixel
	mov	ax, usY
	mul	VSP.cbRow   ; AX = y * (bytes per row)

	mov	dx, usX
	mov	cx, dx	    ; CX = x
	shr	dx, 3	    ; DX = x/8
	add	ax, dx	    ; AX = y*(bytes per row) + x/8

	add	ax, VSP.ofsVBuf ; add origin offset
	mov	bx, ax
	mov	es, VSP.segVBuf ; ES:BX -> pixel

; compute a mask for the pixel's bit position
	and	cl, 7	    ; CL = # of bits to shift
	mov	ax, 8008h   ; AH = 80h (bit mask)
			    ; AL = 8 (Bit Mask reg #)
	shr	ah, cl	    ; AH = bit mask for pixel

; set the Graphics Controller Bit Mask register
	mov	dx, 3CEh    ; GC I/O address
	out	dx, ax

; zero the masked bits in all bit planes
	and	BYTE PTR es:[bx], 0

; set the Sequencer Map Mask register
	mov	dl, 0C4h    ; DX = 3C4h (Sequencer addr reg port)
	mov	ah, ucPixel ; AH = value for Map Mask register
			    ;  (nonzero bits in pixel value
			    ;	 enable bit planes)
	mov	al, 2	    ; AL = Map Mask register number
	out	dx, ax

; set the nonzero bits in the enabled bit planes
	mov	BYTE PTR es:[bx], 0FFh

; restore default Sequencer registers
	mov	ah, 0Fh     ; AH = value for Map Mask reg
			    ;  (all bit planes enabled)
	out	dx, ax

; restore default Graphics Controller registers
	mov	dl, 0CEh    ; DX = 3CEh (Graphics Controller port)
	mov	ax, 0FF08h  ; default Bit Mask
	out	dx, ax

	leave
	ret	6	    ; 2*sizeof(int) + sizeof(char)

SP100b		ENDP

; SP103 (write mode 3)
;
;   void VS2API SP103( USHORT usX, USHORT usY, UCHAR ucPixel );

		PUBLIC	SP103
SP103		PROC	FAR

	enter	0, 0

; point to the specified pixel
	mov	ax, usY
	mul	VSP.cbRow   ; AX = y * (bytes per row)

	mov	dx, usX
	mov	cx, dx	    ; CX = x
	shr	dx, 3	    ; DX = x/8
	add	ax, dx	    ; AX = y*(bytes per row) + x/8

	add	ax, VSP.ofsVBuf ; add origin offset
	mov	bx, ax
	mov	es, VSP.segVBuf ; ES:BX -> pixel

; compute a mask for the pixel's bit position
	and	cl, 7	    ; CL = # of bits to shift
	mov	ch, 80h     ; CH = 1 (bit mask)
	shr	ch, cl	    ; CH = shifted bit mask

; set the Graphics Controller Mode register
	mov	dx, 3CEh    ; GC I/O address
	mov	ax, 305h    ; AL =  Mode register number
			    ; AH =  Write Mode 3 (bits 0,1)
			    ;	     Read Mode 0 (bit 3)
	out	dx, ax

; set the Data Rotate/Function Select register
	mov	ah, VSP.cRMW  ; AH = Read-Modify-Write bits
	mov	al, 3	    ; AL = Data Rotate/Function Select
	out	dx, ax

; set the Set/Reset and Enable Set/Reset registers
	mov	ah, ucPixel ; AH = pixel value
	mov	al, 0	    ; AL = Set/Reset reg number
	out	dx, ax

	mov	ax, 0F01h   ; AH = value for Enable Set/Reset
			    ;  (all bit planes enabled)
			    ; AL = Enable Set/Reset reg number
	out	dx, ax

; set the pixel value
	xchg	es:[bx], ch ; load latches during CPU read;
			    ; update latches and bit planes
			    ;  during CPU write

; restore default Graphics Controller registers
	mov	ax, 5	    ; Mode register
	out	dx, ax

	mov	ax, 3	    ; Function Select
	out	dx, ax

	mov	ax, 1	    ; Enable Set/Reset
	out	dx, ax

	leave
	ret	6	    ; 2*sizeof(int) + sizeof(char)

SP103		ENDP

VS2_TEXT	ENDS


_DATA		SEGMENT WORD PUBLIC 'DATA'

		EXTRN	VSP:VIDEOSTATEPARAMS

_DATA		ENDS

		END
