/***********************************************************************

  BLTTEST.C

***********************************************************************/

#include    <stdio.h>	    /* printf sscanf */
#include    <conio.h>	    /* _getch */
#include    <malloc.h>	    /* _fmalloc */
#include    <string.h>	    /* _fmemset */
#include    <vs2.h>


/***********************************************************************

  TYPEDEFS etc.

***********************************************************************/

typedef struct
{
    ULONG   biSize;
    ULONG   biWidth;
    ULONG   biHeight;
    USHORT  biPlanes;
    USHORT  biBitCount;
    ULONG   biCompression;
    ULONG   biSizeImage;
    ULONG   biXPelsPerMeter;
    ULONG   biYPelsPerMeter;
    ULONG   biClrUsed;
    ULONG   biClrImportant;
}
  BITMAPINFOHEADER;
typedef BITMAPINFOHEADER __far * LPBITMAPINFOHEADER;

typedef struct
{
    UCHAR   rgbBlue;
    UCHAR   rgbGreen;
    UCHAR   rgbRed;
    UCHAR   rgbReserved;
}
  RGBQUAD;
typedef RGBQUAD __far * LPRGBQUAD;


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    main( int, char * * );
void	    InitVMode( USHORT );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

void (VS2API *	fnSetPixel)( USHORT, USHORT, UCHAR );
ULONG (VS2API * fnBltToRAM)( LPVOID, USHORT, USHORT, USHORT, USHORT );
void (VS2API *	fnBltToVBuf)( USHORT, USHORT, LPVOID, USHORT );

static UCHAR	cRMWBits = 0x00;


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    USHORT		usMode;
    ULONG		lRVal = 0;
    LPBITMAPINFOHEADER	lpbmih;
    LPRGBQUAD		ColorTable;
    LPCHAR		lpp;
    USHORT		n, q, r, w, h, nColors;


    if( argc != 4 )
    {
      printf( "Syntax: BLTTEST <mode #> <width> <height>\n" );
      return;
    }

    /* get the specified video mode, width, and height */
    sscanf( argv[1], "%x", &usMode );
    sscanf( argv[2], "%u", &w );
    sscanf( argv[3], "%u", &h );

    /* test BltToRAM functions */
    for( q=0; q<16; q++ )
    {
      /* clear the screen */
      InitVMode( usMode );

      /* compute the size of the memory block required */
      lRVal = (*fnBltToRAM)( NULL, q, q, w, h );
      printf( "BltToRAM( NULL, %d, %d, %u, %u ) returns %ld\n",
		q, q, w, h, lRVal );

      nColors = (USHORT)((usMode < 0x13) ? 16 : 256);
      printf( "(Size should be %u bytes.)\n",
		    sizeof(BITMAPINFOHEADER) + nColors*sizeof(RGBQUAD) +
		    h*(( ((w+1)/2)+3 ) & 0xFFFC) );

      /* allocate a block of memory for the bitmap */
      lpbmih = (LPBITMAPINFOHEADER)_fmalloc( (short)lRVal );

      /* set some pixels */
      for( r=0; r<(h+16); r++ )
	for( n=0; n<(w+16); n++ )
	  (*fnSetPixel)( (USHORT)n, (USHORT)r, (UCHAR)n );

      /* grab the bitmap */
      _fmemset( lpbmih, 0xFF, (int)lRVal );
      lRVal = (*fnBltToRAM)( lpbmih, q, q,  w, h );
      printf( "BltToRAM( %08lX, %d, %d, %u, %u ) returns %ld\n",
		lpbmih, q, q, w, h, lRVal );

      ColorTable = (LPRGBQUAD)(lpbmih + 1);
      nColors = (USHORT)(1 << lpbmih->biBitCount);
      lpp = (LPCHAR)(ColorTable+nColors);

      if( q == 0 )
      {
	/* show BITMAPINFOHEADER */
	printf( "BMP at %08lX:\n", lpbmih );
	printf( "biSize          = %ld\n", lpbmih->biSize );
	printf( "biWidth         = %ld\n", lpbmih->biWidth );
	printf( "biHeight        = %ld\n", lpbmih->biHeight );
	printf( "biPlanes        = %d\n",  lpbmih->biPlanes );
	printf( "biBitCount      = %d\n",  lpbmih->biBitCount );
	printf( "biCompression   = %ld\n", lpbmih->biCompression );
	printf( "biSizeImage     = %ld\n", lpbmih->biSizeImage );
	printf( "biXPelsPerMeter = %ld\n", lpbmih->biXPelsPerMeter );
	printf( "biYPelsPerMeter = %ld\n", lpbmih->biYPelsPerMeter );
	printf( "biClrUsed       = %ld\n", lpbmih->biClrUsed );
	printf( "biClrImportant  = %ld\n", lpbmih->biClrImportant );
	_getch();

#ifdef NEVER
	/* show color table */
	printf( "\nColorTable at %08lX\n", ColorTable );
	for( n=0; n<nColors; n++ )
	  printf( "%02X: R=%02X G=%02X B=%02X rsvd=%02X\n",
		    n, ColorTable[n].rgbRed,
		    ColorTable[n].rgbGreen,
		    ColorTable[n].rgbBlue,
		    ColorTable[n].rgbReserved );
	_getch();
#endif
      }

      /* show the bitmap data */
      lRVal -= sizeof(BITMAPINFOHEADER) + nColors*sizeof(RGBQUAD);
      printf( "Pixel data at %08lX (%ld bytes):\n", lpp, lRVal );
      for( r=0; r<(lRVal/16+1); r++ )
      {
	for( n=0; (n<16)&&((r*16+n)<(USHORT)lRVal); n++ )
	  printf( "%02X ", lpp[r*16+n] );
	printf( "\n" );
      }

      /* discard memory used for the bitmap */
      _ffree( lpbmih );

      _getch();
    }

    /* test BltToVBuf functions */
    for( q=32; q<48; q++ )
    {
      /* copy a bit block from RAM to the video buffer */
      InitVMode( usMode );
      printf( "\t\tUpper left: (%u,%u) ...\n", q, q );

      /* draw some pixels */
      for( r=0; r<(h+16); r++ )
	for( n=0; n<(w+16); n++ )
	  (*fnSetPixel)( (USHORT)n, (USHORT)r, (UCHAR)n );

      /* determine the size of the bitmap in RAM */
      lRVal = (*fnBltToRAM)( NULL, 0, 0, w, h );

      /* create a bitmap in RAM */
      lpbmih = (LPBITMAPINFOHEADER)_fmalloc( (short)lRVal );
      (*fnBltToRAM)( lpbmih, 0, 0, w, h );

      /* show the bitmap data */
      ColorTable = (LPRGBQUAD)(lpbmih + 1);
      nColors = (USHORT)(1 << lpbmih->biBitCount);
      lpp = (LPCHAR)(ColorTable+nColors);

      lRVal -= sizeof(BITMAPINFOHEADER) + nColors*sizeof(RGBQUAD);
      printf( "\n\n\n\n\n\n\nPixel data at %08lX (%ld bytes):\n", lpp, lRVal );
      for( r=0; r<(USHORT)(lRVal/16+1); r++ )
      {
	for( n=0; (n<16)&&((r*16+n)<(USHORT)lRVal); n++ )
	  printf( "%02X ", lpp[r*16+n] );
	printf( "\n" );
      }

      /* draw white pixels just outside the destination bitmap */
      (*fnSetPixel)( q-1, q-1, 0x0F );
      (*fnSetPixel)( q+w, q+h, 0x0F );

      /* draw red pixels just inside the destination bitmap */
      (*fnSetPixel)( q, q, 0x0C );
      (*fnSetPixel)( q+w-1, q+h-1, 0x0C );

      /* copy the bitmap into the video buffer */
      (*fnBltToVBuf)( q, q, lpbmih, TRUE );
      _getch();

      _ffree( lpbmih );
    }

    /* restore 80x25 text mode */
    SetVMode( 3 );
}


/***********************************************************************

  InitVMode

***********************************************************************/

void InitVMode( USHORT usMode )
{
    SetVMode( usMode );
    InitVSP( usMode );

    /* set up for Graphics Controller raster ops */
    vsp.cRMW = cRMWBits;

    switch( vsp.usMode )
    {
      case 0x0D:
      case 0x0E:
      case 0x0F:
      case 0x10:
      case 0x11:
      case 0x12:
	fnSetPixel = SetPixel10;
	fnBltToRAM = BltToRAM10;
	fnBltToVBuf = BltToVBuf10;
	break;

      case 0x13:
	fnSetPixel = SetPixel13;
	fnBltToRAM = BltToRAM13;
	fnBltToVBuf = BltToVBuf13;
	break;

      case 0x840:
      case 0x841:
      case 0x842:
      case 0x843:
      case 0x844:
      case 0x845:
	fnSetPixel = SetPixel256;
	fnBltToRAM = BltToRAM256;
	fnBltToVBuf = BltToVBuf256;
	break;

      default:
	printf( "*** unsupported video mode ***\n" );
	return;
    }
}
