/***********************************************************************

  BOUNCE1.C -- animation demo

***********************************************************************/

#include    <stdio.h>	    /* sprintf sscanf */
#include    <conio.h>	    /* _kbhit _getch */
#include    <stdlib.h>	    /* exit */
#include    <stdarg.h>	    /* ANSI va_list */
#include    <malloc.h>	    /* malloc free */
#include    <fcntl.h>	    /* _open */
#include    <sys\types.h>   /* _open */
#include    <sys\stat.h>    /* _open */
#include    <io.h>	    /* _open _close */
#include    <vs2.h>


/***********************************************************************

  TYPEDEFS etc.

***********************************************************************/

typedef struct
{
    USHORT  bfType;
    ULONG   bfSize;
    USHORT  bfReserved1;
    USHORT  bfReserved2;
    ULONG   bfOffBits;
}
  BITMAPFILEHEADER;
typedef BITMAPFILEHEADER *	PBITMAPFILEHEADER;

typedef struct
{
    ULONG   biSize;
    ULONG   biWidth;
    ULONG   biHeight;
    USHORT  biPlanes;
    USHORT  biBitCount;
    ULONG   biCompression;
    ULONG   biSizeImage;
    ULONG   biXPelsPerMeter;
    ULONG   biYPelsPerMeter;
    ULONG   biClrUsed;
    ULONG   biClrImportant;
}
  BITMAPINFOHEADER;
typedef BITMAPINFOHEADER *	PBITMAPINFOHEADER;


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void			    main( int, char * * );
static void		    InitVMode( void );
static void		    InitBitmaps( void );
static void		    InitPosition( short );
static void		    Animate( void );
static void		    UpdateBall( int );
static void		    UpdateBallPos( int );
static void		    UpdateScreenPos( int );
static PBITMAPFILEHEADER    GetBMP( char * );
static void		    ShowText( char *, ... );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS     vsp;

static USHORT		    ofsPage[3];
static USHORT		    usBkgdX;

/* pointers to bitmaps in RAM */
static PBITMAPFILEHEADER    pbmfhXOR;
static PBITMAPFILEHEADER    pbmfhAND;
static PBITMAPINFOHEADER    pbmihXOR;
static PBITMAPINFOHEADER    pbmihAND;

static struct
{
  short x, y;
  short ix, iy;
}
			    Pos[3];


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    short   i;


    /* verify command-line syntax */
    if( argc != 2 )
    {
      printf( "Syntax: BOUNCE1 <increment>\n" );
      return;
    }

    sscanf( argv[1], "%d", &i );

    InitVMode();
    InitBitmaps();
    InitPosition( i );
    Animate();

    /* discard memory used for bitmaps */
    free( pbmfhXOR );
    free( pbmfhAND );

    /* restore 80x25 alphanumeric mode */
    SetVMode( 3 );
}


/***********************************************************************

  InitVMode

  Notes:
    Sets up the video buffer as follows:
      Offset 0x0000:	Page 0 (480x32 pixels, 0F00h bytes)
      Offset 0x0F00:	Page 1 (480x256 pixels, 7800h bytes)
      Offset 0x8700:	Page 2 (480x256 pixels, 7800h bytes)

    Page 1 and 2 are displayed in alternation for smooth animation.
    The left-hand 320 pixels in Page 0 are used to display the (x,y)
    coordinates of the bouncing ball in the lower portion of the
    split screen.  The right-hand 120 pixels are used for temporary
    bitmap storage.

    This routine calls VBE function 6 to set the logical scan-line
    length so that subsequent text output can be done through BIOS
    function 13h.

***********************************************************************/

static void InitVMode()
{
    USHORT  y;


    /* establish 320x240 256-color mode */
    SetVMode( 0x841 );
    if( 0x841 != GetVMode() )
    {
      printf( "Can't set video mode. "
	      "Be sure that M256X.EXE is loaded." );
      exit( 3 );
    }

    SetBorder( 0x18 );

    /* set up the video buffer */
    InitVSP( 0x841 );
    ofsPage[2] = SetVBufDims( 480, 256, FALSE );
    ofsPage[1] = vsp.cbRow * 32;
    ofsPage[2] += ofsPage[1];
    ofsPage[0] = 0;

    /* inform the VBE of the new video-buffer dimensions */
    __asm
    {
      mov   ax, 4F06h	; Get/Set Logical Scan Line Length
      mov   bl, 0	; subfunction number
      mov   cx, 480
      int   10h
    }

    /* split the screen so that there are 16 lines at the bottom */
    VNWait();
    SetScreenSplit( vsp.usSY-17, ofsPage[1], FALSE );

    /* draw a shaded border below the split */
    SetVBufStart( ofsPage[0] );
    for( y=0; y<7; y++ )
      Line256( 0, y, vsp.usCX-1, y, (UCHAR)(0x1F-2*y) );
}


/***********************************************************************

  InitBitmaps

  Notes:
    The DAC is loaded using the color table in BKGDTILE.BMP.

***********************************************************************/

static void InitBitmaps()
{
    PBITMAPFILEHEADER	pbmfh;
    PBITMAPINFOHEADER	pbmih;
    short		p, x, y;


    /* load the background tile bitmap into pages 1 and 2 */
    pbmfh = GetBMP( "BKGDTILE.BMP" );
    pbmih = (PBITMAPINFOHEADER)(pbmfh+1);

    for( p=1; p<=2; p++ )
    {
      SetVBufStart( ofsPage[p] );
      BltToVBuf256( 0, 0, pbmih, TRUE );
      for( x=0; x<30; x++ )
	for( y=0; y<16; y++ )
	  BltInVBuf256( (USHORT)(x*16), (USHORT)(y*16), 0, 0, 16, 16 );
    }

    free( pbmfh );

    /* each page's background bitmap will be stored at the far right */
    usBkgdX = vsp.usCX-24;

    /* load the XOR mask ("foreground") for the bouncing ball */
    pbmfhXOR = GetBMP( "XORBALL.BMP" );
    pbmihXOR = (PBITMAPINFOHEADER)(pbmfhXOR+1);

    /* load the AND mask ("background") for the bouncing ball */
    pbmfhAND = GetBMP( "ANDBALL.BMP" );
    pbmihAND = (PBITMAPINFOHEADER)(pbmfhAND+1);
}


/***********************************************************************

  InitPosition

***********************************************************************/

static void InitPosition( short i )
{
    short   p;


    for( p=1; p<=2; p++ )
    {
      /* set the initial ball position and direction */
      Pos[p].x = 17;
      Pos[p].y = 13;
      Pos[p].ix = i;		    /* x-increment */
      Pos[p].iy = i;		    /* y-increment */

      /* save the background at the initial position */
      SetVBufStart( ofsPage[p] );
      BltInVBuf256( usBkgdX, 0, Pos[p].x, Pos[p].y, 20, 16 );
    }
}


/***********************************************************************

  Animate

***********************************************************************/

static void Animate()
{
    /* do until the user hits a key */
    do
    {
      UpdateBall( 1 );
      UpdateBall( 2 );
    }
    while( !_kbhit() );
    _getch();	    /* discard the keypress */
}


/***********************************************************************

  UpdateBall

***********************************************************************/

static void UpdateBall( int p )
{
    /* blot the previous image of the ball */
    SetVBufStart( ofsPage[p] );
    BltInVBuf256( Pos[p].x, Pos[p].y, usBkgdX, 0, 20, 16 );

    /* compute a new position for the ball */
    UpdateBallPos( p );

    /* save the background at the new position */
    BltInVBuf256( usBkgdX, 0, Pos[p].x, Pos[p].y, 20, 16 );

    /* display the ball in its new position */
    vsp.cRMW = 0x08;	    /* Graphics Controller raster op = AND */
    BltToVBuf256( Pos[p].x, Pos[p].y, pbmihAND, FALSE );
    vsp.cRMW = 0x18;	    /* Graphics Controller raster op = XOR */
    BltToVBuf256( Pos[p].x, Pos[p].y, pbmihXOR, FALSE );

    /* display the updated portion of the video buffer */
    UpdateScreenPos( p );
    SetScreenStart( ofsPage[p] );
}


/***********************************************************************

  UpdateBallPos

***********************************************************************/

static void UpdateBallPos( int p )
{
    /* increment the x-position */
    Pos[p].x = Pos[3-p].x + Pos[3-p].ix;

    /* change direction if at the edge of the video buffer */
    if( Pos[3-p].ix > 0 )
    {
      if( Pos[p].x >= (short)(usBkgdX-16) )
      {
	Pos[p].x = usBkgdX-16;
	Pos[p].ix = -Pos[3-p].ix;
      }
      else
	Pos[p].ix = Pos[3-p].ix;
    }
    else
    {
      if( Pos[p].x <= 0 )
      {
	Pos[p].x = 0;
	Pos[p].ix = -Pos[3-p].ix;
      }
      else
	Pos[p].ix = Pos[3-p].ix;
    }

    /* increment the y-position */
    Pos[p].y = Pos[3-p].y + Pos[3-p].iy;

    /* change direction if at the edge of the video buffer */
    if( Pos[3-p].iy > 0 )
    {
      if( Pos[p].y >= (short)(vsp.usCY-16) )
      {
	Pos[p].y = vsp.usCY-16;
	Pos[p].iy = -Pos[3-p].iy;
      }
      else
	Pos[p].iy = Pos[3-p].iy;
    }
    else
    {
      if( Pos[p].y <= 0 )
      {
	Pos[p].y = 0;
	Pos[p].iy = -Pos[3-p].iy;
      }
      else
	Pos[p].iy = Pos[3-p].iy;
    }

    /* show the x,y coordinates of the ball and the screen origin */
    ShowText( "%3d,%-3d   %3d,%-3d",
		Pos[p].x, Pos[p].y, vsp.usX, vsp.usY );
}


/***********************************************************************

  UpdateScreenPos

***********************************************************************/

static void UpdateScreenPos( int p )
{
    short	    OrgX, OrgY;


    if( Pos[p].ix > 0 )
    {
      /* pan to the right if the ball is within 52 pixels of the
	 right-hand edge (36 plus the width of the ball bitmap) */
      if( Pos[p].x > (short)(vsp.usX + vsp.usSX - 36) )
      {
	OrgX = Pos[p].x - vsp.usSX + 36;
	if( OrgX > (short)(usBkgdX - vsp.usSX) )
	  OrgX = usBkgdX - vsp.usSX;
      }

      else
	OrgX = vsp.usX;
    }

    else
    {
      /* pan to the left if the ball is within 36 pixels of the
	 left-hand edge */
      if( Pos[p].x < (short)(vsp.usX + 36) )
      {
	OrgX = (short)(Pos[p].x - 36);
	if( OrgX < 0 )
	  OrgX = 0;
      }

      else
	OrgX = vsp.usX;
    }

    if( Pos[p].iy > 0 )
    {
      /* pan downward if the ball is within 68 pixels of the bottom
	 edge (36 plus the height of the ball bitmap plus the height
	 of the lower part of the split screen) */
      if( Pos[p].y > (short)(vsp.usY + vsp.usSY - 68) )
      {
	OrgY = Pos[p].y - vsp.usSY + 68;
	if( OrgY > (short)(vsp.usCY - vsp.usSY + 16) )
	  OrgY = vsp.usCY - vsp.usSY + 16;
      }

      else
	OrgY = vsp.usY;
    }

    else
    {
      /* pan upward if the ball is within 36 pixels of the top edge */
      if( Pos[p].y < (short)(vsp.usY + 36) )
      {
	OrgY = (short)(Pos[p].y - 36);
	if( OrgY < 0 )
	  OrgY = 0;
      }

      else
	OrgY = vsp.usY;
    }

    VNWait();
    SetScreenOrg( OrgX, OrgY, FALSE );
}


/***********************************************************************

  GetBMP

***********************************************************************/

static PBITMAPFILEHEADER GetBMP( char * szName )
{
    int 		hBMP;
    PBITMAPFILEHEADER	pbmfh;
    USHORT		usFileSize;


    /* open the file */
    hBMP = _open( szName, _O_BINARY | _O_RDONLY );
    if( -1 == hBMP )
    {
      printf( "Unable to open file '%s'\n", szName );
      exit( 3 );
    }

    /* determine the size of the file */
    usFileSize = (USHORT)_lseek( hBMP, 0L, SEEK_END );
    _lseek( hBMP, 0L, SEEK_SET );

    /* allocate memory for the bitmap */
    pbmfh = (PBITMAPFILEHEADER)malloc( usFileSize );
    if( NULL == pbmfh )
    {
      printf( "Unable to allocate %u bytes for bitmap\n", usFileSize );
      exit( 3 );
    }

    /* load the bitmap into memory */
    if( usFileSize != (USHORT)_read( hBMP, pbmfh, usFileSize ) )
      printf( "Error reading '%s'\n", szName );

    _close( hBMP );
    return pbmfh;
}


/***********************************************************************

  ShowText

  Notes:
    The text is written at (11,1) in character coordinates, which
    corresponds to (88,8) in pixel coordinates because the default
    BIOS character dimensions are 8x8 pixels.

***********************************************************************/

static void ShowText( char * szFmt, ... )
{
    char    szBuf[64];
    va_list marker;


    /* create a formatted output string */
    va_start( marker, szFmt );
    vsprintf( szBuf, szFmt, marker );
    va_end( marker );

    /* display the string at the start of the video buffer */
    TextAt( 11, 1, szBuf, 0x67 );
}
