/***********************************************************************

  CIRCTEST.C

***********************************************************************/

#include    <stdio.h>	    /* printf sscanf */
#include    <conio.h>	    /* _getch _kbhit */
#include    <math.h>	    /* sqrt */
#include    <stdlib.h>	    /* rand */
#include    <vs2.h>


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    main( int, char * * );
static void InitVMode( USHORT );
int	    MulDiv( int, int, int );
void VS2API CircleIncr( USHORT, USHORT, USHORT, UCHAR );
void	    Set8Pixels( USHORT, USHORT, USHORT, USHORT, UCHAR );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

void (VS2API * fnCircle)( USHORT, USHORT, USHORT, UCHAR );
void (VS2API * fnPixel)( USHORT, USHORT, UCHAR );

static UCHAR cRMWBits = 0x00;


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    USHORT  usMode, usColors, u, usSQ;
    USHORT  xc, yc, r, ix, iy, i;
    UCHAR   ucPixel;


    if( argc != 2 )
    {
      printf( "Syntax: CIRCTEST <mode #>\n" );
      return;
    }

    /* select the specified video mode */
    sscanf( argv[1], "%x", &usMode );
    InitVMode( usMode );

    /* set up according to mode */
    switch( usMode )
    {
      case 0x0D:
      case 0x0E:
      case 0x0F:
      case 0x10:
      case 0x11:
      case 0x12:
	fnPixel = SetPixel10;
	fnCircle = Circle10;
	usColors = 16;
	ucPixel = 0x0E;
	break;

      case 0x13:
	fnPixel = SetPixel13;
	fnCircle = Circle13;
	usColors = 256;
	ucPixel = 0x0A;
	break;

      case 0x840:
      case 0x841:
      case 0x842:
      case 0x843:
      case 0x844:
      case 0x845:
	fnPixel = SetPixel256;
	fnCircle = Circle256;
	usColors = 256;
	ucPixel = 0x0A;
	break;

      default:
	printf( "*** unsupported video mode ***" );
	_getch();
	return;
    }

    usSQ = (USHORT)((vsp.usSX < vsp.usSY) ? vsp.usSX : vsp.usSY);

    /* increasing radii */
    for( u=0; u<usSQ/2; u+=13 )
      (*fnCircle)( (USHORT)(vsp.usSX/2), (USHORT)(vsp.usSY/2), u, ucPixel );

    _getch();
    InitVMode( usMode );

    /* generate lots of random Circles */
    while( !_kbhit() )
    {
      /* random center */
      xc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSX-14), RAND_MAX ));
      yc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSY-14), RAND_MAX ));

      /* random radii */
      ix = (USHORT)((xc < vsp.usSX/2) ? xc : vsp.usSX-xc-1);
      iy = (USHORT)((yc < vsp.usSY/2) ? yc : vsp.usSY-yc-1);
      i = (USHORT)((ix < iy) ? ix : iy);
      r = (USHORT)(MulDiv( rand(), (int)i, RAND_MAX ));

      /* draw the Circle in a random color */
      ucPixel = (UCHAR)MulDiv( rand(), (short)usColors, RAND_MAX );
      (*fnCircle)( xc, yc, r, ucPixel );
    }

    /* throw away the key that was pressed */
    _getch();

    /* restore 80x25 text mode */
    _getch();
    SetVMode( 3 );
}


/***********************************************************************

  CircleIncr

  Notes:
    Uses the midpoint algorithm

***********************************************************************/

void VS2API CircleIncr( USHORT xc, USHORT yc, USHORT r, UCHAR ucPixel )
{
    short   x = 0;
    short   y = r;
    short   d = (short)(1-r);


    do
    {
      Set8Pixels( xc, yc, x, y, ucPixel );

      x++ ;

      if( d < 0 )
	d += (short)(2*x + 1);

      else
      {
	y-- ;
	d += (short)(2*(x-y) + 1);
      }
    }
    while( x <= y );
}


/***********************************************************************

  Set8Pixels

***********************************************************************/

void
Set8Pixels( USHORT xc, USHORT yc, USHORT x, USHORT y, UCHAR ucPixel )
{
    (*fnPixel)( (USHORT)(xc+x), (USHORT)(yc+y), ucPixel );
    (*fnPixel)( (USHORT)(xc-x), (USHORT)(yc+y), ucPixel );
    (*fnPixel)( (USHORT)(xc+x), (USHORT)(yc-y), ucPixel );
    (*fnPixel)( (USHORT)(xc-x), (USHORT)(yc-y), ucPixel );

    (*fnPixel)( (USHORT)(xc+y), (USHORT)(yc+x), ucPixel );
    (*fnPixel)( (USHORT)(xc-y), (USHORT)(yc+x), ucPixel );
    (*fnPixel)( (USHORT)(xc+y), (USHORT)(yc-x), ucPixel );
    (*fnPixel)( (USHORT)(xc-y), (USHORT)(yc-x), ucPixel );
}


#ifdef NEVER
/***********************************************************************

  Set4Pixels

***********************************************************************/

void
Set4Pixels( USHORT xc, USHORT yc, USHORT x, USHORT y, UCHAR ucPixel )
{
    if( x != 0 )
    {
      (*fnPixel)( (USHORT)(xc+x), (USHORT)(yc+y), ucPixel );
      (*fnPixel)( (USHORT)(xc-x), (USHORT)(yc+y), ucPixel );

      if( y != 0 )
      {
	(*fnPixel)( (USHORT)(xc+x), (USHORT)(yc-y), ucPixel );
	(*fnPixel)( (USHORT)(xc-x), (USHORT)(yc-y), ucPixel );
      }
    }

    else
    {
      (*fnPixel)( xc, (USHORT)(yc+y), ucPixel );

      if( y != 0 )
	(*fnPixel)( xc, (USHORT)(yc-y), ucPixel );
    }
}
#endif

/***********************************************************************

  InitVMode

***********************************************************************/

static void InitVMode( USHORT usMode )
{
    SetVMode( usMode );
    InitVSP( usMode );

    /* set up for Graphics Controller raster ops */
    vsp.cRMW = cRMWBits;
}


/***********************************************************************

  MulDiv

  Notes:
    Leaves its result in register AX.  (The MS C compiler generates
    a warning message anyway.)

***********************************************************************/

int MulDiv( int n1, int n2, int n3 )
{
    __asm
    {
      mov   ax, n1
      mul   n2
      div   n3
    }
}
