/***********************************************************************

  CLIPTEST.C

***********************************************************************/

#include    <stdio.h>	    /* printf sscanf */
#include    <conio.h>	    /* _kbhit _getch */
#include    <stdlib.h>	    /* rand */
#include    <vs2.h>


/***********************************************************************

  TYPEDEFS

***********************************************************************/

typedef union
{
    struct
    {
      int   code0 : 1;
      int   code1 : 1;
      int   code2 : 1;
      int   code3 : 1;
    }
	ocs;
    short   oci;
}
    OUTCODES;

typedef struct
{
  short     x1;
  short     y1;
  short     x2;
  short     y2;
}
    RECTANGLE;

typedef struct
{
  short     x1;
  short     y1;
  short     x2;
  short     y2;
}
    ENDPOINTS;

/* the following defines make ClipLine more readable */
#define XUL r->x1
#define YUL r->y1
#define XLR r->x2
#define YLR r->y2
#define X1  e->x1
#define Y1  e->y1
#define X2  e->x2
#define Y2  e->y2


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	 main( int, char * * );
short	 MulDiv( int, int, int );

int	 ClipLine( RECTANGLE *, ENDPOINTS * );
void	 SetOutcodes( OUTCODES *, RECTANGLE *, short, short );
void	 Swap( short *, short * );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

void (VS2API * fnLine)( USHORT, USHORT, USHORT, USHORT, UCHAR );


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * * argv )
{
    USHORT	usMode, usColors;
    UCHAR	ucPixel;
    RECTANGLE	r;
    ENDPOINTS	e;


    /* verify command line */
    if( argc != 2 )
    {
      printf( "Syntax: CLIPTEST <mode>\n" );
      return;
    }

    /* establish the specified video mode */
    sscanf( argv[1], "%x", &usMode );
    SetVMode( usMode );
    InitVSP( usMode );

    /* select the appropriate line routine */
    switch( usMode )
    {
      case 0x0D:
      case 0x0E:
      case 0x0F:
      case 0x10:
      case 0x11:
      case 0x12:
	fnLine = Line10;
	usColors = 16;
	break;

      case 0x13:
	fnLine = Line13;
	usColors = 256;
	break;

      case 0x840:
      case 0x841:
      case 0x842:
      case 0x843:
      case 0x844:
      case 0x845:
	fnLine = Line256;
	usColors = 256;
	break;

      default:
	printf( "*** unsupported video mode ***" );
	_getch();
	return;
    }

    /* use a clipping rectangle that's smaller than the screen */
    r.x1 = (short)(vsp.usSX / 5);
    r.x2 = (short)(vsp.usSX - r.x1);
    r.y1 = (short)(vsp.usSY / 5);
    r.y2 = (short)(vsp.usSY - r.y1);

    /* display the clipping rectangle */
    (*fnLine)( r.x1, r.y1, r.x1, r.y2, (UCHAR)7 );
    (*fnLine)( r.x1, r.y1, r.x2, r.y1, (UCHAR)7 );
    (*fnLine)( r.x2, r.y2, r.x1, r.y2, (UCHAR)7 );
    (*fnLine)( r.x2, r.y2, r.x2, r.y1, (UCHAR)7 );

    /* generate lots of random lines and clip them into the rectangle */
    while( !_kbhit() )
    {
      /* pick some random coordinates */
      e.x1 = MulDiv( rand(), vsp.usSX-1, RAND_MAX );
      e.x2 = MulDiv( rand(), vsp.usSX-1, RAND_MAX );
      e.y1 = MulDiv( rand(), vsp.usSY-1, RAND_MAX );
      e.y2 = MulDiv( rand(), vsp.usSY-1, RAND_MAX );

      /* pick a random shade of gray */
      if( 16 == usColors )
	ucPixel = (UCHAR)((rand() & 1) + 7);	/* 7 or 8 */

      else
	ucPixel =				/* 12h-1Ah */
	    (UCHAR)(MulDiv( rand(), 8, RAND_MAX ) + 0x12);

      /* draw the entire line in gray */
      (*fnLine)( e.x1, e.y1, e.x2, e.y2, ucPixel );
      if( ClipLine( &r, &e ) )
      {
	/* draw the clipped line in a random color */
	ucPixel = (UCHAR)MulDiv( rand(), (int)usColors, RAND_MAX );
	(*fnLine)( e.x1, e.y1, e.x2, e.y2, ucPixel );
      }
    }

    /* throw away the key that was pressed */
    _getch();
}


/***********************************************************************

  MulDiv

  Notes:
    Leaves its result in register AX.  (The MS C compiler generates
    a warning message anyway.)

***********************************************************************/

short MulDiv( int n1, int n2, int n3 )
{
    __asm
    {
      mov   ax, n1
      mul   n2
      div   n3
    }
}


/***********************************************************************

  ClipLine

***********************************************************************/

int ClipLine( RECTANGLE * r, ENDPOINTS * e )
{
    OUTCODES	o1, o2;
    int 	bInside, bOutside;


    /* initialize outcodes */
    SetOutcodes( &o1, r, X1, Y1 );
    SetOutcodes( &o2, r, X2, Y2 );

    /* initialize result flags */
    bInside  = ((o1.oci | o2.oci) == 0);
    bOutside = ((o1.oci & o2.oci) != 0);

    while( !bOutside && !bInside )
    {
      /* ensure that (x1,y1) needs to be clipped */
      if( o1.oci == 0 )
      {
	Swap( &X1, &X2 );
	Swap( &Y1, &Y2 );
	Swap( &o1.oci, &o2.oci );
      }

      if( o1.ocs.code0 )	    /* clip left */
      {
	Y1 += (short)((long)(Y2-Y1) * (XUL-X1) / (X2-X1));
	X1 = XUL;
      }

      else
      if( o1.ocs.code1 )	    /* clip above */
      {
	X1 += (short)((long)(X2-X1) * (YUL-Y1) / (Y2-Y1));
	Y1 = YUL;
      }

      else
      if( o1.ocs.code2 )	    /* clip right */
      {
	Y1 += (short)((long)(Y2-Y1) * (XLR-X1) / (X2-X1));
	X1 = XLR;
      }

      else
      if( o1.ocs.code3 )	    /* clip below */
      {
	X1 += (short)((long)(X2-X1) * (YLR-Y1) / (Y2-Y1));
	Y1 = YLR;
      }

      /* update outcodes */
      SetOutcodes( &o1, r, X1, Y1 );
      bInside  = ((o1.oci | o2.oci) == 0);
      bOutside = ((o1.oci & o2.oci) != 0);
    }

    return bInside;
}

	
/***********************************************************************

  SetOutcodes

***********************************************************************/

void SetOutcodes( OUTCODES * o, RECTANGLE * r, short x, short y )
{
    o->oci = 0;
    o->ocs.code0 = (x < XUL);
    o->ocs.code1 = (y < YUL);
    o->ocs.code2 = (x > XLR);
    o->ocs.code3 = (y > YLR);
}

	
/***********************************************************************

  Swap

***********************************************************************/

void Swap( short * pa, short * pb )
{
    short   t;


    t = *pa;
    *pa = *pb;
    *pb = t;
}
