/***********************************************************************

  C_LINES.C

***********************************************************************/

#include    <stdio.h>	    /* printf sscanf */
#include    <conio.h>	    /* _getch */
#include    <math.h>	    /* abs fabs */
#include    <vs2.h>

#define TEST0

/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

int	    main( int, char * * );
static void Test( USHORT );
void VS2API LineEq( USHORT, USHORT, USHORT, USHORT, UCHAR );
void VS2API LineBres( USHORT, USHORT, USHORT, USHORT, UCHAR );
static void usSwap( USHORT *, USHORT * );
static void nSwap( int *, int * );

/* stuff not defined in VS2.H ... */
void VS2API ModeVGAHGC( void );
void VS2API SetPixel04( USHORT, USHORT, UCHAR );
void VS2API SetPixel06( USHORT, USHORT, UCHAR );
void VS2API SetPixelHGC( USHORT, USHORT, UCHAR );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

void (VS2API * fnLine)( USHORT, USHORT, USHORT, USHORT, UCHAR );
void (VS2API * SetPixel)( USHORT, USHORT, UCHAR );


/***********************************************************************

  main

***********************************************************************/

int main( int argc, char * argv[] )
{
    USHORT  usMode;


    if( argc != 2 )
    {
      printf( "Syntax: C_LINES <mode #>\n" );
      return 3;
    }

    /* select the specified video mode */
    sscanf( argv[1], "%x", &usMode );

    if( 7 != usMode )
    {
      SetVMode( usMode );
      InitVSP( usMode );
    }
    else
      ModeVGAHGC();

#ifdef TEST0
    fnLine = LineEq;
#endif

#ifdef TEST1
    fnLine = LineBres;
#endif

    Test( usMode );

    /* restore 80x25 text mode */
    SetVMode( 3 );
}


/***********************************************************************

  Test -- lines using the equation of the line

***********************************************************************/

static void Test( USHORT usMode )
{
    USHORT  usX, usY;


    switch( usMode )
    {
      case 4:
      case 5:
	SetPixel = SetPixel04;
	break;

      case 6:
	SetPixel = SetPixel06;
	break;

      case 7:
	SetPixel = SetPixelHGC;
	break;

      case 0x0D:
      case 0x0E:
      case 0x0F:
      case 0x10:
      case 0x11:
      case 0x12:
	SetPixel = SetPixel10;
	break;

      case 0x13:
	SetPixel = SetPixel13;
	break;

      case 0x840:
      case 0x841:
      case 0x842:
      case 0x843:
      case 0x844:
      case 0x845:
	SetPixel = SetPixel256;
	break;

      default:
	printf( "*** unsupported video mode ***" );
	_getch();
	return;
    }

#ifdef NEVER
    /* vertical lines */
    for( usX=0; usX<vsp.usSX; usX++ )
      (*fnLine)( usX, 0, usX, vsp.usSY, (UCHAR)usX );
#endif

    for( usY=0; usY<vsp.usSY; usY++ )
      (*fnLine)( 0, 0, (USHORT)(vsp.usSX-1), usY, (UCHAR)  7 );

    for( usX=0; usX<vsp.usSX; usX++ )
      (*fnLine)( 0, 0, usX, (USHORT)(vsp.usSY-1), (UCHAR)  6 );

    _getch();
}


/***********************************************************************

  LineEq

  Notes:
    Uses the equation of the line ... runs about as slow as possible

***********************************************************************/

void VS2API
LineEq( USHORT x1, USHORT y1, USHORT x2, USHORT y2, UCHAR ucPixel )
{
    USHORT  x, y;
    double  m;		/* slope */
    double  b;		/* y-intercept */


    if( x2 == x1 )			     /* vertical line */
    {
      if( y1 > y2 )
	usSwap( &y1, &y2 );			  /* force y1 < y2 */

      for( y=y1; y<=y2; y++ )		     /* draw from y1 to y2 */
	(*SetPixel)( x1, y, ucPixel );

      return;
    }

    if( x1 > x2 )			       /* force x1 < x2 */
    {
      usSwap( &x1, &x2 );
      usSwap( &y1, &y2 );
    }

    m = (double)(y2-y1) / (double)(x2-x1);    /* compute m and b */
    b = y1 - (m*x1);

    if( fabs(m) < 1 )
    {
      for( x=x1; x<=x2; x++ )		       /* draw from x1 to x2 */
      {
	y = (USHORT)(m*x + b);
	(*SetPixel)( x, y, ucPixel );
      }
    }

    else
    {
      for( y=y1; y<=y2; y++ )
      {
	x = (USHORT)((y-b) / m);
	(*SetPixel)( x, y, ucPixel );
      }
    }
}


/***********************************************************************

  LineBres

  Notes:
    Uses Bresenham's algorithm

***********************************************************************/

void VS2API
LineBres( USHORT x1, USHORT y1, USHORT x2, USHORT y2, UCHAR ucPixel )
{
    USHORT *	px;
    USHORT *	py;
    int 	y, a, b, a1, b1, a2, b2;
    int 	Aincr, Bincr, bincr;
    int 	d, dx, dy, da, db;


    if( x2 == x1 )
    {
      /* force y1 < y2 */
      if( y1 > y2 )
	usSwap( &y1, &y2 );

      /* draw a vertical line */
      for( y=(int)y1; y<=(int)y2; y++ )
	(*SetPixel)( x1, (USHORT)y, ucPixel );

      return;
    }

    dy = y2 - y1;
    dx = x2 - x1;

    if( abs(dy) <= abs(dx) )
    {
      /* slope is between -1 and 1: step horizontally */
      b2 = y2;
      b1 = y1;
      a2 = x2;
      a1 = x1;
      px = (USHORT *)&a;
      py = (USHORT *)&b;
    }

    else
    {
      /* step vertically */
      b2 = x2;
      b1 = x1;
      a2 = y2;
      a1 = y1;
      px = (USHORT *)&b;
      py = (USHORT *)&a;
    }

    /* always move in the positive direction */
    if( a1 > a2 )
    {
      nSwap( &a1, &a2 );
      nSwap( &b1, &b2 );
    }

    /* determine the increment for b */
    if( b2 > b1 )
      bincr = 1;
    else
      bincr = -1;

    /* initialize the decision variables */
    da = a2 - a1;
    db = abs( b2-b1 );
    d = 2*db - da;
    Aincr = 2 * (db - da);
    Bincr = 2 * db;

    /* set the first pixel */
    a = a1;
    b = b1;
    SetPixel( *px, *py, ucPixel );

    /* set the remaining pixels */
    for( a=a1+1; a<=a2; a++ )
    {
      if( d >= 0 )
      {
	/* move diagonally */
	b = b + bincr;
	d += Aincr;
      }

      else
	d += Bincr;

      SetPixel( *px, *py, ucPixel );
    }

}


/***********************************************************************

  usSwap

***********************************************************************/

static void usSwap( USHORT * a, USHORT * b )
{
    USHORT  t;


    t = *a;
    *a = *b;
    *b = t;
}



/***********************************************************************

  nSwap

***********************************************************************/

static void nSwap( int * a, int * b )
{
    int     t;


    t = *a;
    *a = *b;
    *b = t;
}
