/***********************************************************************

  ELTEST.C

***********************************************************************/

#include    <stdio.h>	    /* printf sscanf */
#include    <conio.h>	    /* _getch _kbhit */
#include    <math.h>	    /* sqrt */
#include    <stdlib.h>	    /* rand */
#include    <vs2.h>


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    main( int, char * * );
static void InitVMode( USHORT );
int	    MulDiv( int, int, int );
void VS2API EllipseEq( USHORT, USHORT, USHORT, USHORT, UCHAR );
void VS2API EllipseIncr( USHORT, USHORT, USHORT, USHORT, UCHAR );
void	    Set4Pixels( USHORT, USHORT, USHORT, USHORT, UCHAR );

/* stuff not defined in VS2.H ... */
void VS2API Ellipse10EGA( USHORT, USHORT, USHORT, USHORT, UCHAR );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

void (VS2API * fnEllipse)( USHORT, USHORT, USHORT, USHORT, UCHAR );
void (VS2API * fnPixel)( USHORT, USHORT, UCHAR );

static UCHAR cRMWBits = 0x00;


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    USHORT  usMode, usColors, u;
    USHORT  xc, yc, a0, b0;
    UCHAR   ucPixel;


    if( argc != 2 )
    {
      printf( "Syntax: ELTEST <mode #>\n" );
      return;
    }

    /* select the specified video mode */
    sscanf( argv[1], "%x", &usMode );
    InitVMode( usMode );

    /* set up according to mode */
    switch( usMode )
    {
      case 0x0D:
      case 0x0E:
      case 0x0F:
      case 0x10:
      case 0x11:
      case 0x12:
	fnPixel = SetPixel10;
	fnEllipse = Ellipse10;
	usColors = 16;
	ucPixel = 0x0E;
	break;

      case 0x13:
	fnPixel = SetPixel13;
	fnEllipse = Ellipse13;
	usColors = 256;
	ucPixel = 0x0A;
	break;

      case 0x840:
      case 0x841:
      case 0x842:
      case 0x843:
      case 0x844:
      case 0x845:
	fnPixel = SetPixel256;
	fnEllipse = Ellipse256;
	usColors = 256;
	ucPixel = 0x0A;
	break;

      default:
	printf( "*** unsupported video mode ***" );
	_getch();
	return;
    }

    xc = (USHORT)(vsp.usSX / 2);
    yc = (USHORT)(vsp.usSY / 2);

    /* increasing semiaxes */
    for ( u=0; u<xc; u+=13 )
      (*fnEllipse)( xc, yc, u, (USHORT)(u/3), ucPixel );

    _getch();
    InitVMode( usMode );

    for( u=0; u<yc; u+=13 )
      (*fnEllipse)( xc, yc, (USHORT)(u/3), u, ucPixel );

    _getch();
    InitVMode( usMode );

    /* tiny ellipses */
    for( u=0; u<9; u+=3)
      (*fnEllipse)( xc, yc, (USHORT)(xc-8), u, ucPixel );

    _getch();
    InitVMode( usMode );

    for( u=0; u<9; u+=3)
      (*fnEllipse)( xc, yc, u, (USHORT)(yc-4), ucPixel );

    _getch();
    InitVMode( usMode );

    /* generate lots of random ellipses */
    while( !_kbhit() )
    {
      /* random center */
      xc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSX-14), RAND_MAX ));
      yc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSY-14), RAND_MAX ));

      /* random semiaxes */
      if( xc < vsp.usSX/2 )
	a0 = (USHORT)(MulDiv( rand(), (int)(xc-1), RAND_MAX ));
      else
	a0 = (USHORT)(MulDiv( rand(), (int)(vsp.usSX-xc-1), RAND_MAX ));

      if( yc < vsp.usSY/2 )
	b0 = (USHORT)(MulDiv( rand(), (int)(yc-1), RAND_MAX ));
      else
	b0 = (USHORT)(MulDiv( rand(), (int)(vsp.usSY-yc-1), RAND_MAX ));

      /* draw the ellipse in a random color */
      ucPixel = (UCHAR)MulDiv( rand(), (short)usColors, RAND_MAX );
      (*fnEllipse)( xc, yc, a0, b0, ucPixel );
    }

    /* throw away the key that was pressed */
    _getch();

    /* restore 80x25 text mode */
    _getch();
    SetVMode( 3 );
}


/***********************************************************************

  EllipseEq

  Notes:
    Uses the equation of the ellipse ... runs about as slow as possible

***********************************************************************/

void VS2API
EllipseEq( USHORT xc, USHORT yc, USHORT a0, USHORT b0, UCHAR ucPixel )
{
    double  x = 0;
    double  y = b0;
    double  Bsquared = (double)b0 * (double)b0;
    double  Asquared = (double)a0 * (double)a0;


    /* avoid degenerate cases */
    if( (0 == a0) || (0 == b0) )
      return;

    /* do while dy/dx >= -1 */
    do
    {
      y = sqrt( Bsquared - ((Bsquared/Asquared) * x*x) );
      Set4Pixels( xc, yc, (USHORT)x, (USHORT)y, ucPixel );
      ++x;
    }
    while( (x <= a0) && (Bsquared*x < Asquared*y) );

    --y;

    /* do while dy/dx < -1 */
    while( y >= 0 )
    {
      x = sqrt( Asquared - ((Asquared/Bsquared) * y*y) );
      Set4Pixels( xc, yc, (USHORT)x, (USHORT)y, ucPixel );
      --y;
    }
}


/***********************************************************************

  EllipseIncr

  Notes:
    Uses the midpoint algorithm

***********************************************************************/

void VS2API
EllipseIncr( USHORT xc, USHORT yc, USHORT a0, USHORT b0, UCHAR ucPixel )
{
    short   x = 0;
    short   y = b0;

    long    a = a0;
    long    b = b0;

    long    Asquared = a * a;
    long    TwoAsquared = 2 * Asquared;
    long    Bsquared = b * b;
    long    TwoBsquared = 2 * Bsquared;

    long    d;
    long    dx, dy;


    /* initialize decision variable and increments */
    d = Bsquared - Asquared*b + Asquared/4;
    dx = 0;
    dy = TwoAsquared * b;

    /* do while slope > -1 */
    while( dx<dy )
    {
      Set4Pixels( xc, yc, x, y, ucPixel );

      if( d > 0 )
      {
	--y;
	dy -= TwoAsquared;
	d -= dy;
      }

      ++x;
      dx += TwoBsquared;
      d += Bsquared + dx;
    }

    /* at this point slope <= -1 */
    d += (3L*(Asquared-Bsquared)/2L - (dx+dy)) / 2L;

    /* do until the x-axis is reached */
    while ( y>=0 )
    {
      Set4Pixels( xc, yc, x, y, ucPixel );

      if( d < 0 )
      {
	++x;
	dx += TwoBsquared;
	d += dx;
      }

      --y;
      dy -= TwoAsquared;
      d += Asquared - dy;
    }
}


#ifdef NEVER
/***********************************************************************

  Set4Pixels

***********************************************************************/

void
Set4Pixels( USHORT xc, USHORT yc, USHORT x, USHORT y, UCHAR ucPixel )
{
    (*fnPixel)( (USHORT)(xc+x), (USHORT)(yc+y), ucPixel );
    (*fnPixel)( (USHORT)(xc-x), (USHORT)(yc+y), ucPixel );
    (*fnPixel)( (USHORT)(xc+x), (USHORT)(yc-y), ucPixel );
    (*fnPixel)( (USHORT)(xc-x), (USHORT)(yc-y), ucPixel );
}
#endif


/***********************************************************************

  Set4Pixels

***********************************************************************/

void
Set4Pixels( USHORT xc, USHORT yc, USHORT x, USHORT y, UCHAR ucPixel )
{
    if( x != 0 )
    {
      (*fnPixel)( (USHORT)(xc+x), (USHORT)(yc+y), ucPixel );
      (*fnPixel)( (USHORT)(xc-x), (USHORT)(yc+y), ucPixel );

      if( y != 0 )
      {
	(*fnPixel)( (USHORT)(xc+x), (USHORT)(yc-y), ucPixel );
	(*fnPixel)( (USHORT)(xc-x), (USHORT)(yc-y), ucPixel );
      }
    }

    else
    {
      (*fnPixel)( xc, (USHORT)(yc+y), ucPixel );

      if( y != 0 )
	(*fnPixel)( xc, (USHORT)(yc-y), ucPixel );
    }
}


/***********************************************************************

  InitVMode

***********************************************************************/

static void InitVMode( USHORT usMode )
{
    SetVMode( usMode );
    InitVSP( usMode );

    /* set up for Graphics Controller raster ops */
    vsp.cRMW = cRMWBits;
}


/***********************************************************************

  MulDiv

  Notes:
    Leaves its result in register AX.  (The MS C compiler generates
    a warning message anyway.)

***********************************************************************/

int MulDiv( int n1, int n2, int n3 )
{
    __asm
    {
      mov   ax, n1
      mul   n2
      div   n3
    }
}
