/***********************************************************************

  ELTIME.C -- for BIOS mode 12h (640-480 16-color)

***********************************************************************/

#include    <stdio.h>	    /* printf sscanf */
#include    <conio.h>	    /* _getch */
#include    <stdlib.h>	    /* rand srand */
#include    <vs2.h>


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    main( int, char * * );
int	    MulDiv( int, int, int );

/* defined in 32BIT.ASM */
void VS2API Ellipse10X( USHORT, USHORT, USHORT, USHORT, UCHAR );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    int     n, nFn, nIter, nColors = 16;
    USHORT  xc, yc, a0, b0, r;
    USHORT  ix, iy, i;
    UCHAR   ucPixel;
    short   h0, m0, s0, c0;
    short   h1, m1, s1, c1;
    double  Duration;


    if( argc != 3 )
    {
      printf( "Syntax: ELTIME <function #> <iterations>\n" );
      printf( "  0 = Empty Circle loop\n" );
      printf( "  1 = Circle10\n" );
      printf( "  2 = Ellipse10\n" );
      printf( "  3 = Circles using Ellipse10\n" );
      printf( "  4 = Ellipse10X\n" );
      return;
    }

    /* get the function number */
    sscanf( argv[1], "%d", &nFn );

    /* get the number of iterations */
    sscanf( argv[2], "%d", &nIter );

    /* use the number of iterations as a seed for the random number generator */
    srand( nIter );

    /* set 640x480 16-color mode */
    SetVMode( 0x12 );
    InitVSP( 0x12 );

    /* get starting time */
    __asm
    {
      mov   ax, 2C00h
      int   21h
      mov   al, ch
      xor   ah, ah
      mov   h0, ax
      xor   ch, ch
      mov   m0, cx
      mov   al, dh
      mov   s0, ax
      xor   dh, dh
      mov   c0, dx
    }

    if( 0 == nFn )
    {
      /* generate lots of random Circles */
      for( n=0; n<nIter; n++ )
      {
	/* random center */
	xc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSX-14), RAND_MAX ));
	yc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSY-14), RAND_MAX ));

	/* random radii */
	ix = (USHORT)((xc < vsp.usSX/2) ? xc : vsp.usSX-xc-1);
	iy = (USHORT)((yc < vsp.usSY/2) ? yc : vsp.usSY-yc-1);
	i = (USHORT)((ix < iy) ? ix : iy);
	r = (USHORT)(MulDiv( rand(), (int)i, RAND_MAX ));

	/* don't draw the Circle in a random color */
	ucPixel = (UCHAR)MulDiv( rand(), nColors, RAND_MAX );
      }
    }

    if( 1 == nFn )
    {
      /* generate lots of random Circles */
      for( n=0; n<nIter; n++ )
      {
	/* random center */
	xc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSX-14), RAND_MAX ));
	yc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSY-14), RAND_MAX ));

	/* random radii */
	ix = (USHORT)((xc < vsp.usSX/2) ? xc : vsp.usSX-xc-1);
	iy = (USHORT)((yc < vsp.usSY/2) ? yc : vsp.usSY-yc-1);
	i = (USHORT)((ix < iy) ? ix : iy);
	r = (USHORT)(MulDiv( rand(), (int)i, RAND_MAX ));

	/* draw the Circle in a random color */
	ucPixel = (UCHAR)MulDiv( rand(), nColors, RAND_MAX );
	Circle10( xc, yc, r, ucPixel );
      }
    }

    if( 2 == nFn )
    {
      /* generate lots of random ellipses */
      for( n=0; n<nIter; n++ )
      {
	/* random center */
	xc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSX-14), RAND_MAX ));
	yc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSY-14), RAND_MAX ));

	/* random semiaxes */
	if( xc < vsp.usSX/2 )
	  a0 = (USHORT)(MulDiv( rand(), (int)(xc-1), RAND_MAX ));
	else
	  a0 = (USHORT)(MulDiv( rand(), (int)(vsp.usSX-xc-1), RAND_MAX ));

	if( yc < vsp.usSY/2 )
	  b0 = (USHORT)(MulDiv( rand(), (int)(yc-1), RAND_MAX ));
	else
	  b0 = (USHORT)(MulDiv( rand(), (int)(vsp.usSY-yc-1), RAND_MAX ));

	/* draw the ellipse in a random color */
	ucPixel = (UCHAR)MulDiv( rand(), nColors, RAND_MAX );
	Ellipse10( xc, yc, a0, b0, ucPixel );
      }
    }

    if( 3 == nFn )
    {
      /* generate lots of random circles using the ellipse function */
      for( n=0; n<nIter; n++ )
      {
	/* random center */
	xc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSX-14), RAND_MAX ));
	yc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSY-14), RAND_MAX ));

	/* random radii */
	ix = (USHORT)((xc < vsp.usSX/2) ? xc : vsp.usSX-xc-1);
	iy = (USHORT)((yc < vsp.usSY/2) ? yc : vsp.usSY-yc-1);
	i = (USHORT)((ix < iy) ? ix : iy);
	r = (USHORT)(MulDiv( rand(), (int)i, RAND_MAX ));

	/* draw the Circle in a random color */
	ucPixel = (UCHAR)MulDiv( rand(), nColors, RAND_MAX );
	Ellipse10( xc, yc, r, r, ucPixel );
      }
    }

    if( 4 == nFn )
    {
      /* generate lots of random ellipses */
      for( n=0; n<nIter; n++ )
      {
	/* random center */
	xc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSX-14), RAND_MAX ));
	yc = (USHORT)(7 + MulDiv( rand(), (int)(vsp.usSY-14), RAND_MAX ));

	/* random semiaxes */
	if( xc < vsp.usSX/2 )
	  a0 = (USHORT)(MulDiv( rand(), (int)(xc-1), RAND_MAX ));
	else
	  a0 = (USHORT)(MulDiv( rand(), (int)(vsp.usSX-xc-1), RAND_MAX ));

	if( yc < vsp.usSY/2 )
	  b0 = (USHORT)(MulDiv( rand(), (int)(yc-1), RAND_MAX ));
	else
	  b0 = (USHORT)(MulDiv( rand(), (int)(vsp.usSY-yc-1), RAND_MAX ));

	/* draw the ellipse in a random color */
	ucPixel = (UCHAR)MulDiv( rand(), nColors, RAND_MAX );
	Ellipse10X( xc, yc, a0, b0, ucPixel );
      }
    }

    /* get ending time */
    __asm
    {
      mov   ax, 2C00h
      int   21h
      mov   al, ch
      xor   ah, ah
      mov   h1, ax
      xor   ch, ch
      mov   m1, cx
      mov   al, dh
      mov   s1, ax
      xor   dh, dh
      mov   c1, dx
    }

    /* restore 80x25 text mode */
    _getch();
    SetVMode( 3 );

    /* compute duration */
    c1 -= c0;		/* centiseconds */
    if( c1 < 0 )
    {
      c1 += 100;
      --s1;
    }

    s1 -= s0;		/* seconds */
    if( s1 < 0 )
    {
      s1 += 60;
      --m1;
    }

    m1 -= m0;		/* minutes */
    if( m1 < 0 )
    {
      m1 += 60;
      --h1;
    }

    h1 -= h0;		/* hours */
    if( h1 < 0 )
      h1 += 24;

    Duration = 60.0*(double)m1 + (double)s1 + (double)c1/100.0;
    printf( "Time = %5.2f seconds (%d iterations)\n", Duration, nIter );
}


/***********************************************************************

  MulDiv

  Notes:
    Leaves its result in register AX.  (The MS C compiler generates
    a warning message anyway.)

***********************************************************************/

int MulDiv( int n1, int n2, int n3 )
{
    __asm
    {
      mov   ax, n1
      mul   n2
      div   n3
    }
}
