/***********************************************************************

  GPTEST.C

***********************************************************************/

#include    <conio.h>	    /* _getch */
#include    <stdio.h>	    /* printf sscanf */
#include    <string.h>	    /* strlen */
#include    <vs2.h>


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    main( int, char * * );
static void Test0( USHORT );

/* stuff not defined in VS2.H ... */
void VS2API SetPixel04( USHORT, USHORT, UCHAR );
void VS2API SetPixel06( USHORT, USHORT, UCHAR );

UCHAR VS2API GetPixel04( USHORT, USHORT );
UCHAR VS2API GetPixel06( USHORT, USHORT );


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    USHORT  usMode;


    if( argc != 2 )
    {
      printf( "Syntax: GPTEST <mode #>\n" );
      return;
    }

    /* select the specified video mode */
    sscanf( argv[1], "%x", &usMode );
    SetVMode( usMode );

    /* initialize global variables */
    InitVSP( usMode );

    Test0( usMode );

    /* restore 80x25 text mode */
    SetVMode( 3 );
}


/***********************************************************************

  Test0 -- set and get every pixel on the screen

***********************************************************************/

static void Test0( USHORT usMode )
{
    void (VS2API * SetPixel)( USHORT, USHORT, UCHAR );
    UCHAR (VS2API * GetPixel)( USHORT, USHORT );
    USHORT  usX, usY;
    UCHAR   c, ucMask;


    switch( usMode )
    {
      case 4:
      case 5:
	SetPixel = SetPixel04;
	GetPixel = GetPixel04;
	ucMask = 3;
	break;

      case 6:
	SetPixel = SetPixel06;
	GetPixel = GetPixel06;
	ucMask = 1;
	break;

      case 0x0D:
      case 0x0E:
      case 0x0F:
      case 0x10:
      case 0x11:
      case 0x12:
	SetPixel = SetPixel10;
	GetPixel = GetPixel10;
	ucMask = 0x0F;
	FillVBuf10( 0x06 );
	break;

      case 0x13:
	SetPixel = SetPixel13;
	GetPixel = GetPixel13;
	ucMask = 0xFF;
	FillVBuf13( 0x04 );
	break;

      case 0x840:
      case 0x841:
      case 0x842:
      case 0x843:
      case 0x844:
      case 0x845:
	SetPixel = SetPixel256;
	GetPixel = GetPixel256;
	ucMask = 0xFF;
	FillVBuf256( 0x0F );
	break;

      default:
	printf( "*** unsupported video mode ***" );
	getch();
	return;
    }

    for( usX=0; usX<vsp.usSX; usX++ )
      for( usY=0; usY<vsp.usSY; usY++ )
      {
	c = (UCHAR)(ucMask & (usX+usY));
	(*SetPixel)( usX, usY, c );
	if( c != (*GetPixel)( usX, usY ) )
	  printf( "Error at (%u, %u) ?= %02X\n",
		    usX, usY, (*GetPixel)(usX, usY) );
      }

    getch();
}
