/***********************************************************************

  LATCH.C

  Notes:
    This function looks for undocumented VGA CRTC register 22h,
    whose value represents the current contents of the Graphics
    Controller latch specified by bits 0-1 of the GC's Read Plane
    Select register (register 4).

    Original IBM VGA hardware supports this register but there is no
    IBM documentation for its functionality.  Some other manufacturers
    of VGA/SVGA subsystems (S3, for example) also support -- and
    document -- this register.

***********************************************************************/

#include    <stdio.h>	    /* printf */
#include    <conio.h>	    /* _getch */
#include    <vs2.h>


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void		    main( int, char * * );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    UCHAR   p;
    USHORT  usRVal;
    USHORT  usPair[4];


    /* establish 640x480 16-color graphics mode */
    SetVMode( 0x12 );
    InitVSP( 0x12 );

    /* store the following in the bit planes at A000:0000 ...
	plane 3: 11110000b (F0h)
	plane 2: 11001100b (CCh)
	plane 1: 10101010b (AAh)
	plane 0: 01010101b (55h)    */

    SetPixel10( 0, 0, 0x07 );
    SetPixel10( 1, 0, 0x0B );
    SetPixel10( 2, 0, 0x05 );
    SetPixel10( 3, 0, 0x09 );
    SetPixel10( 4, 0, 0x06 );
    SetPixel10( 5, 0, 0x0A );
    SetPixel10( 6, 0, 0x04 );
    SetPixel10( 7, 0, 0x08 );

    /* read the bit planes the usual way */
    printf( "\nBit plane read ...\n" );
    for( p=0; p<4; p++ )
    {
      /* program the read map select register */
      __asm
      {
	mov bx, 0A000h
	mov es, bx
	xor bx, bx	    ; ES:BX = A000:0000

	mov dx, 3CEh	    ; Graphics Controller I/O address
	mov ah, p	    ; bit plane number (0-3)
	mov al, 4	    ; Read Map Select register number
	out dx, ax

	mov al, es:[bx]
	xor ah, ah	    ; AX = byte from bit plane
	mov usRVal, ax
      }

      printf( "p=%d  usRVal=%02X\n", p, usRVal );
    }

    /* read the bit planes through CRTC register 22h */
    printf( "\nCRTC register 22h ...\n" );
    for( p=0; p<4; p++ )
    {
      /* program the read map select register */
      __asm
      {
	mov bx, 0A000h
	mov es, bx
	xor bx, bx	    ; ES:BX = A000:0000

	mov dx, 3CEh	    ; Graphics Controller I/O address
	mov ah, p	    ; bit plane number (0-3)
	mov al, 4	    ; Read Map Select register number
	out dx, ax

	mov bl, es:[bx]     ; latch the bit planes
	mov dx, 3D4h	    ; DX = CRTC I/O addr
	mov al, 22h	    ; CRTC register number
	out dx, al
	inc dx
	in  al, dx	    ; AL = current value of CRTC reg 22h
	xor ah, ah
	mov usRVal, ax
      }

      printf( "p=%d  usRVal=%02X\n", p, usRVal );
    }

    /* update the bit planes using XOR, assuming default values for
	Bit Mask (11111111b) and Enable Set/Reset (0) */
    __asm
    {
      mov bx, 0A000h
      mov es, bx
      xor bx, bx	; ES:BX = A000:0000

      mov dx, 3CEh	; Graphics Controller I/O address
      mov ax, 1803h	; AH = 18h (XOR raster op)
			; AL = 3 (Data Rotate/Function Select reg #)
      out dx, ax

      mov al, -1
      xchg al, es:[bx]	; all 4 bit planes XOR'd

#ifdef NEVER
			; program the GC so that a CPU read always
			;  returns FFh ...
      mov ax, 7 	; AH = 0 (Color Don't Care value)
			; AL = 7 (Color Don't Care reg #)
      out dx, ax

      mov ax, 805h	; AH = 8 (read mode 1, write mode 0)
			; AL = 5 (Mode reg #)
      out dx, ax

      mov al, -1
      and al, es:[bx]	; all 4 bit planes XOR'd
#endif
    }

    /* read the latches via CRTC register 22h */
    for( p=0; p<4; p++ )
    {
      __asm
      {
	mov bx, 0A000h
	mov es, bx
	xor bx, bx	    ; ES:BX = A000:0000

	mov dx, 3CEh	    ; Graphics Controller I/O address
	mov ah, p	    ; bit plane number (0-3)
	mov al, 4	    ; Read Map Select register number
	out dx, ax

	mov dx, 3D4h	    ; DX = CRTC I/O addr
	mov al, 22h	    ; CRTC register number
	out dx, al
	inc dx
	in  al, dx	    ; AL = current value of CRTC reg 22h
	xor ah, ah
	mov usRVal, ax
      }

      usPair[p] = usRVal;
    }

    /* read the bit planes in the usual way (the MOV instruction updates
	the latches) */
    for( p=0; p<4; p++ )
    {
      __asm
      {
	mov bx, 0A000h
	mov es, bx
	xor bx, bx	    ; ES:BX = A000:0000

	mov dx, 3CEh	    ; Graphics Controller I/O address
	mov ah, p	    ; bit plane number (0-3)
	mov al, 4	    ; Read Map Select register number
	out dx, ax

	mov ah, es:[bx]     ; AH = value of byte in pth bit plane
	xor al, al
	mov usRVal, ax
      }

      usPair[p] |= usRVal;  /* update the high-order byte */
    }

    printf( "\nHigh-order byte: MOV instruction\n" );
    printf( "Low-order byte:  CRTC register 22h ...\n" );
    for( p=0; p<4; p++ )
      printf( "usPair[%d]=%04X\n", p, usPair[p] );

    /* wait for a keypress */
    _getch();

    /* restore 80x25 alphanumeric mode */
    SetVMode( 3 );
}
