/***********************************************************************

  LINETEST.C

***********************************************************************/

#include    <stdio.h>	    /* printf sscanf */
#include    <conio.h>	    /* _getch */
#include    <math.h>	    /* abs fabs */
#include    <vs2.h>


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    main( int, char * * );
static void DoLine( USHORT, USHORT, USHORT, USHORT, UCHAR );
void VS2API LineEq( USHORT, USHORT, USHORT, USHORT, UCHAR );
void VS2API LineBres( USHORT, USHORT, USHORT, USHORT, UCHAR );
static void usSwap( USHORT *, USHORT * );
static void nSwap( int *, int * );
static void InitVMode( USHORT );

/* stuff not defined in VS2.H ... */
void VS2API ModeVGAHGC( void );
void VS2API SetPixel04( USHORT, USHORT, UCHAR );
void VS2API SetPixel06( USHORT, USHORT, UCHAR );
void VS2API SetPixelHGC( USHORT, USHORT, UCHAR );
void VS2API Line04( USHORT, USHORT, USHORT, USHORT, UCHAR );
void VS2API Line06( USHORT, USHORT, USHORT, USHORT, UCHAR );
void VS2API LineHGC( USHORT, USHORT, USHORT, USHORT, UCHAR );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

void (VS2API * fnLine)( USHORT, USHORT, USHORT, USHORT, UCHAR );
void (VS2API * fnPixel)( USHORT, USHORT, UCHAR );

static UCHAR cRMWBits = 0x18;


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    USHORT  usMode, u, usX, usY, usSQ;
    UCHAR   ucPixel;


    if( argc != 2 )
    {
      printf( "Syntax: LINETEST <mode #>\n" );
      return;
    }

    /* select the specified video mode */
    sscanf( argv[1], "%x", &usMode );
    InitVMode( usMode );

    /* set up according to mode */
    switch( usMode )
    {
      case 4:
      case 5:
	fnPixel = SetPixel04;
	fnLine = Line04;
	ucPixel = 0x0D;
	break;

      case 6:
	fnPixel = SetPixel06;
	fnLine = Line06;
	ucPixel = 0x05;
	break;

      case 7:
	fnPixel = SetPixelHGC;
	fnLine = LineHGC;
	ucPixel = 0x05;
	break;

      case 0x0D:
      case 0x0E:
      case 0x0F:
      case 0x10:
      case 0x11:
      case 0x12:
	fnPixel = SetPixel10;
	fnLine = Line10;
	ucPixel = 0x0E;
	break;

      case 0x13:
	fnPixel = SetPixel13;
	fnLine = Line13;
	ucPixel = 0x0A;
	break;

      case 0x840:
      case 0x841:
      case 0x842:
      case 0x843:
      case 0x844:
      case 0x845:
	fnPixel = SetPixel256;
	fnLine = Line256;
	ucPixel = 0x0A;
	break;

      default:
	printf( "*** unsupported video mode ***" );
	_getch();
	return;
    }

    usSQ = (USHORT)((vsp.usSX < vsp.usSY) ? vsp.usSX : vsp.usSY);

    /* vertical lines */
    for( u=0; u<100; u+=2 )
      DoLine( u, (USHORT)(100-u), u, (USHORT)(100+u), ucPixel );
    for( u=0; u<100; u+=2 )
      DoLine( u, (USHORT)(100-u), u, (USHORT)(100+u), ucPixel );

    /* vertical lines with endpoints reversed */
    for( u=1; u<100; u+=2 )
      DoLine( u, (USHORT)(100+u), u, (USHORT)(100-u), ucPixel );

    /* tiny vertical lines */
    for( u=0; u<32; u++ )
      DoLine( u, u, u, (USHORT)(u+2), ucPixel );

    _getch();
    InitVMode( usMode );

    /* horizontal lines */
    for( u=0; u<150; u+=2 )
      DoLine( (USHORT)(150-u), u, (USHORT)(150+u), u, ucPixel );

    /* horizontal lines with endpoints reversed */
    for( u=1; u<150; u+=2 )
      DoLine( (USHORT)(150+u), u, (USHORT)(150-u), u, ucPixel );

    for( u=1; u<150; u+=2 )
      DoLine( (USHORT)(150+u), u, (USHORT)(150-u), u, ucPixel );

    /* tiny horizontal lines */
    for( u=0; u<32; u++ )
      DoLine( u, u ,(USHORT)(u+2), u, ucPixel );

    _getch();
    InitVMode( usMode );

    /* lines from upper left corner */
    for( usX=0; usX<vsp.usSX; usX++ )
      DoLine( 0, 0, usX, (USHORT)(vsp.usSY-1), ucPixel );

    for( usY=0; usY<vsp.usSY; usY++ )
      DoLine( 0, 0, (USHORT)(vsp.usSX-1), usY, ucPixel );

    _getch();
    InitVMode( usMode );

    /* lines with slope <= 1 */
    for( u=0; u<usSQ; u++ )
      DoLine( (USHORT)(usSQ/2), (USHORT)(usSQ/2), 0, u, ucPixel );

    for( u=0; u<usSQ; u++ )
      DoLine( (USHORT)(usSQ/2), (USHORT)(usSQ/2), (USHORT)(usSQ-1), u, ucPixel );

    /* lines with slope >= 1 */
    for( u=0; u<usSQ; u++ )
      DoLine( (USHORT)(usSQ/2), (USHORT)(usSQ/2), u, 0, ucPixel );

    for( u=0; u<usSQ; u++ )
      DoLine( (USHORT)(usSQ/2), (USHORT)(usSQ/2), u, (USHORT)(usSQ-1), ucPixel );

    _getch();
    InitVMode( usMode );

    /* lines with non-aligned endpoints */
    for( u=0; u<vsp.usSY; u++ )
      DoLine( 0, u, (USHORT)(vsp.usSX-1), u, 9 );   /* colored background */
    ucPixel = 2;

    for( u=0; u<usSQ; u++ )
      DoLine( (USHORT)(usSQ-1-u/3), (USHORT)(usSQ-1-u/2),
		(USHORT)(2*u/3), (USHORT)(3*u/4), ucPixel );

    /* tiny non-orthogonal lines */
    for( u=100; u<vsp.usSX; u+=7 )
      DoLine( u, 7, (USHORT)(u-4), 8, ucPixel );

    for( u=110; u<vsp.usSX; u+=7 )
      DoLine( u, 20, (USHORT)(u-1), 24, ucPixel );

    /* restore 80x25 text mode */
    _getch();
    SetVMode( 3 );
}


/***********************************************************************

  DoLine

***********************************************************************/

static void DoLine( USHORT x1, USHORT y1, USHORT x2,USHORT y2, UCHAR p )
{
    /* order the coordinates according to the parity of x1+y1 */
    if( (x1+y1)&1 )
      (*fnLine)( x1, y1, x2, y2, p );

    else
      (*fnLine)( x2, y2, x1, y1, p );
}


/***********************************************************************

  LineEq

  Notes:
    Uses the equation of the line ... runs about as slow as possible

***********************************************************************/

void VS2API
LineEq( USHORT x1, USHORT y1, USHORT x2, USHORT y2, UCHAR ucPixel )
{
    USHORT  x, y;
    double  m;		/* slope */
    double  b;		/* y-intercept */


    if( x2 == x1 )			    /* vertical line */
    {
      /* force y1 < y2 */
      if( y1 > y2 )
	usSwap( &y1, &y2 );

      /* draw from y1 to y2 */
      for( y=y1; y<=y2; y++ )
	(*fnPixel)( x1, y, ucPixel );

      return;
    }

    /* force x1 < x2 */
    if( x1 > x2 )
    {
      usSwap( &x1, &x2 );
      usSwap( &y1, &y2 );
    }

    /* compute m and b (slope and y-intercept) */
    m = (double)(y2-y1) / (double)(x2-x1);
    b = y1 - (m*x1);

    /* slope is between -1 and 1: draw from x1 to x2 */
    if( fabs(m) < 1 )
      for( x=x1; x<=x2; x++ )
      {
	y = (USHORT)(m*x + b);
	(*fnPixel)( x, y, ucPixel );
      }

    /* slope is less than -1 or greater than 1: draw from y1 to y2 */
    else
      for( y=y1; y<=y2; y++ )
      {
	x = (USHORT)((y-b) / m);
	(*fnPixel)( x, y, ucPixel );
      }
}


/***********************************************************************

  LineBres

  Notes:
    Uses Bresenham's algorithm

***********************************************************************/

void VS2API
LineBres( USHORT x1, USHORT y1, USHORT x2, USHORT y2, UCHAR ucPixel )
{
    USHORT *	px;
    USHORT *	py;
    int 	y, a, b, a1, b1, a2, b2;
    int 	Aincr, Bincr, bincr;
    int 	d, dx, dy, da, db;


    if( x2 == x1 )
    {
      /* force y1 < y2 */
      if( y1 > y2 )
	usSwap( &y1, &y2 );

      /* draw a vertical line */
      for( y=(int)y1; y<=(int)y2; y++ )
	(*fnPixel)( x1, (USHORT)y, ucPixel );

      return;
    }

    dy = y2 - y1;
    dx = x2 - x1;

    if( abs(dy) <= abs(dx) )
    {
      /* slope is between -1 and 1: step horizontally */
      b2 = y2;
      b1 = y1;
      a2 = x2;
      a1 = x1;
      px = (USHORT *)&a;
      py = (USHORT *)&b;
    }

    else
    {
      /* step vertically */
      b2 = x2;
      b1 = x1;
      a2 = y2;
      a1 = y1;
      px = (USHORT *)&b;
      py = (USHORT *)&a;
    }

    /* always move in the positive direction */
    if( a1 > a2 )
    {
      nSwap( &a1, &a2 );
      nSwap( &b1, &b2 );
    }

    /* determine the increment for b */
    if( b2 > b1 )
      bincr = 1;
    else
      bincr = -1;

    /* initialize the decision variables */
    da = a2 - a1;
    db = abs( b2-b1 );
    d = 2*db - da;
    Aincr = 2 * (db - da);
    Bincr = 2 * db;

    /* set the first pixel */
    a = a1;
    b = b1;
    (*fnPixel)( *px, *py, ucPixel );

    /* set the remaining pixels */
    for( a=a1+1; a<=a2; a++ )
    {
      if( d >= 0 )
      {
	/* move diagonally */
	b += bincr;
	d += Aincr;
      }

      else
	d += Bincr;

      (*fnPixel)( *px, *py, ucPixel );
    }
}


/***********************************************************************

  usSwap

***********************************************************************/

static void usSwap( USHORT * a, USHORT * b )
{
    USHORT  t;


    t = *a;
    *a = *b;
    *b = t;
}


/***********************************************************************

  nSwap

***********************************************************************/

static void nSwap( int * a, int * b )
{
    int     t;


    t = *a;
    *a = *b;
    *b = t;
}


/***********************************************************************

  InitVMode

***********************************************************************/

static void InitVMode( USHORT usMode )
{
    if( 7 != usMode )
    {
      SetVMode( usMode );
      InitVSP( usMode );
    }

    else
      ModeVGAHGC();

    /* set up for Graphics Controller raster ops */
    vsp.cRMW = cRMWBits;
}
