/***********************************************************************

  LINETIME.C -- for BIOS mode 12h (640-480 16-color)

***********************************************************************/

#include    <stdio.h>	    /* printf sscanf */
#include    <conio.h>	    /* _getch */
#include    <math.h>	    /* abs fabs */
#include    <vs2.h>


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    main( int, char * * );
void VS2API LineEq( USHORT, USHORT, USHORT, USHORT, UCHAR );
void VS2API LineBres( USHORT, USHORT, USHORT, USHORT, UCHAR );
static void usSwap( USHORT *, USHORT * );
static void nSwap( int *, int * );

typedef void (VS2API * FNLINE)( USHORT, USHORT, USHORT, USHORT, UCHAR );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

FNLINE	fnLine;


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    int     n, nIter;
    USHORT  usX, usY, usMode;
    char    h0, m0, s0, c0;
    char    h1, m1, s1, c1;
    double  Duration, TotalPixels;


    if( argc != 4 )
    {
      printf( "Syntax: LINETIME <mode> <function #> <iterations>\n" );
      printf( "  0 = LineEq (mode 12h only)\n" );
      printf( "  1 = LineBres (mode 12h only)\n" );
      printf( "  2 = Line10, Line13, or Line256\n" );
      return;
    }

    /* get the video mode */
    sscanf( argv[1], "%x", &usMode );

    /* get the line function */
    sscanf( argv[2], "%d", &n );
    switch( n )
    {
      case 0:
	fnLine = LineEq;
	usMode = 0x12;		/* force 640x480 16-color mode */
	break;

      case 1:
	fnLine = LineBres;
	usMode = 0x12;		/* force 640x480 16-color mode */
	break;

      case 2:
	switch( usMode )
	{
	  case 0x0D:
	  case 0x0E:
	  case 0x0F:
	  case 0x10:
	  case 0x11:
	  case 0x12:
	    fnLine = Line10;
	    break;

	  case 0x13:
	    fnLine = Line13;
	    break;

	  case 0x840:
	  case 0x841:
	  case 0x842:
	  case 0x843:
	  case 0x844:
	  case 0x845:
	    fnLine = Line256;
	    break;

	}
    }

    /* get the number of iterations */
    sscanf( argv[3], "%d", &nIter );

    /* set the requested video mode */
    SetVMode( usMode );
    InitVSP( usMode );

    /* get starting time */
    __asm
    {
      mov   ax, 2C00h
      int   21h
      mov   h0, ch
      mov   m0, cl
      mov   s0, dh
      mov   c0, dl
    }

    for( n=0; n<nIter; n++ )
    {
      /* all lines from upper left corner */
      for( usY=0; usY<vsp.usSY; usY++ )
	(*fnLine)( 0, 0, (USHORT)(vsp.usSX-1), usY, (UCHAR)(n+2) );

      for( usX=0; usX<vsp.usSX; usX++ )
	(*fnLine)( 0, 0, usX, (USHORT)(vsp.usSY-1), (UCHAR)(n+6) );
    }

    /* get ending time */
    __asm
    {
      mov   ax, 2C00h
      int   21h
      mov   h1, ch
      mov   m1, cl
      mov   s1, dh
      mov   c1, dl
    }

    /* restore 80x25 text mode */
    _getch();
    SetVMode( 3 );

    /* compute duration */
    c1 = (char)(c1 - c0);   /* centiseconds */
    if( c1 < 0 )
    {
      c1 += 100;
      --s1;
    }

    s1 = (char)(s1 - s0);   /* seconds */
    if( s1 < 0 )
    {
      s1 += 60;
      --m1;
    }

    m1 = (char)(m1 - m0);   /* minutes */
    if( m1 < 0 )
    {
      m1 += 60;
      --h1;
    }

    h1 = (char)(h1 - h0);   /* hours */
    if( h1 < 0 )
      h1 += 24;

    Duration = (60.0*(double)m1 + (double)s1 + (double)c1/100.0) / (double)n;
    TotalPixels =  (double)vsp.usSX * (double)vsp.usSY * 2.0;

    printf( "Time       = %5.2f seconds (%6.0f pixels)\n",
		Duration, TotalPixels );
    printf( "Pixel Rate = %5.0f pixels/second\n",
		TotalPixels/Duration );
}


/***********************************************************************

  LineEq

  Notes:
    Uses the equation of the line ... runs about as slow as possible

***********************************************************************/

void VS2API
LineEq( USHORT x1, USHORT y1, USHORT x2, USHORT y2, UCHAR ucPixel )
{
    USHORT  x, y;
    double  m;		/* slope */
    double  b;		/* y-intercept */


    if( x2 == x1 )			     /* vertical line */
    {
      if( y1 > y2 )
	usSwap( &y1, &y2 );			  /* force y1 < y2 */

      for( y=y1; y<=y2; y++ )		     /* draw from y1 to y2 */
	SetPixel10( x1, y, ucPixel );

      return;
    }

    if( x1 > x2 )			       /* force x1 < x2 */
    {
      usSwap( &x1, &x2 );
      usSwap( &y1, &y2 );
    }

    m = (double)(y2-y1) / (double)(x2-x1);    /* compute m and b */
    b = y1 - (m*x1);

    if( fabs(m) < 1 )
    {
      for( x=x1; x<=x2; x++ )		       /* draw from x1 to x2 */
      {
	y = (USHORT)(m*x + b);
	SetPixel10( x, y, ucPixel );
      }
    }

    else
    {
      for( y=y1; y<=y2; y++ )
      {
	x = (USHORT)((y-b) / m);
	SetPixel10( x, y, ucPixel );
      }
    }
}


/***********************************************************************

  LineBres

  Notes:
    Uses Bresenham's algorithm

***********************************************************************/

void VS2API
LineBres( USHORT x1, USHORT y1, USHORT x2, USHORT y2, UCHAR ucPixel )
{
    USHORT *	px;
    USHORT *	py;
    int 	y, a, b, a1, b1, a2, b2;
    int 	Aincr, Bincr, bincr;
    int 	d, dx, dy, da, db;


    if( x2 == x1 )
    {
      /* force y1 < y2 */
      if( y1 > y2 )
	usSwap( &y1, &y2 );

      /* draw a vertical line */
      for( y=(int)y1; y<=(int)y2; y++ )
	SetPixel10( x1, (USHORT)y, ucPixel );

      return;
    }

    dy = y2 - y1;
    dx = x2 - x1;

    if( abs(dy) <= abs(dx) )
    {
      /* slope is between -1 and 1: step horizontally */
      b2 = y2;
      b1 = y1;
      a2 = x2;
      a1 = x1;
      px = (USHORT *)&a;
      py = (USHORT *)&b;
    }

    else
    {
      /* step vertically */
      b2 = x2;
      b1 = x1;
      a2 = y2;
      a1 = y1;
      px = (USHORT *)&b;
      py = (USHORT *)&a;
    }

    /* always move in the positive direction */
    if( a1 > a2 )
    {
      nSwap( &a1, &a2 );
      nSwap( &b1, &b2 );
    }

    /* determine the increment for b */
    if( b2 > b1 )
      bincr = 1;
    else
      bincr = -1;

    /* initialize the decision variables */
    da = a2 - a1;
    db = abs( b2-b1 );
    d = 2*db - da;
    Aincr = 2 * (db - da);
    Bincr = 2 * db;

    /* set the first pixel */
    a = a1;
    b = b1;
    SetPixel10( *px, *py, ucPixel );

    /* set the remaining pixels */
    for( a=a1+1; a<=a2; a++ )
    {
      if( d >= 0 )
      {
	/* move diagonally */
	b = b + bincr;
	d += Aincr;
      }

      else
	d += Bincr;

      SetPixel10( *px, *py, ucPixel );
    }

}


/***********************************************************************

  usSwap

***********************************************************************/

static void usSwap( USHORT * a, USHORT * b )
{
    USHORT  t;


    t = *a;
    *a = *b;
    *b = t;
}



/***********************************************************************

  nSwap

***********************************************************************/

static void nSwap( int * a, int * b )
{
    int     t;


    t = *a;
    *a = *b;
    *b = t;
}
