/***********************************************************************

  RECTTEST.C

***********************************************************************/

#include    <stdio.h>	    /* printf sscanf */
#include    <conio.h>	    /* _kbhit _getch */
#include    <stdlib.h>	    /* rand */
#include    <vs2.h>


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    main( int, char * * );
void	    SolidRectangle( USHORT, USHORT, USHORT, USHORT, UCHAR );
static int  MulDiv( int, int, int );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

void (VS2API * fnLine)( USHORT, USHORT, USHORT, USHORT, UCHAR );


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * * argv )
{
    USHORT  usMode, usX1, usY1, usX2, usY2, usColors;
    UCHAR   ucPixel;


    if( argc != 2 )
    {
      printf( "Syntax: RECTTEST <mode #>\n" );
      return;
    }

    /* set up the requested video mode */
    sscanf( argv[1], "%x", &usMode );
    SetVMode( usMode );
    InitVSP( usMode );

    /* select the appropriate line routine */
    switch( usMode )
    {
      case 0x0D:
      case 0x0E:
      case 0x0F:
      case 0x10:
      case 0x11:
      case 0x12:
	fnLine = Line10;
	usColors = 16;
	break;

      case 0x13:
	fnLine = Line13;
	usColors = 256;
	break;

      case 0x840:
      case 0x841:
      case 0x842:
      case 0x843:
      case 0x844:
      case 0x845:
	fnLine = Line256;
	usColors = 256;
	break;

      default:
	printf( "*** unsupported video mode ***" );
	_getch();
	return;
    }

    while( !_kbhit() )
    {
      /* pick some random coordinates */
      usX1 = (USHORT)MulDiv( rand(), vsp.usSX-1, RAND_MAX );
      usX2 = (USHORT)MulDiv( rand(), vsp.usSX-1, RAND_MAX );
      usY1 = (USHORT)MulDiv( rand(), vsp.usSY-1, RAND_MAX );
      usY2 = (USHORT)MulDiv( rand(), vsp.usSY-1, RAND_MAX );
      ucPixel = (UCHAR)MulDiv( rand(), (int)usColors, RAND_MAX );

      /* call with the y-coordinates in ascending order */
      if( usY1 < usY2 )
	SolidRectangle( usX1, usY1, usX2, usY2, ucPixel );
      else
	SolidRectangle( usX2, usY2, usX1, usY1, ucPixel );
    }

    /* throw away the key that was pressed */
    _getch();
}


/***********************************************************************

  SolidRectangle

***********************************************************************/

void SolidRectangle( USHORT x1, USHORT y1,
			USHORT x2, USHORT y2, UCHAR ucPixel )
{
    USHORT   y;

#ifdef NEVER
    printf( "(%u,%u)-(%u,%u) [%u]\n", x1, y1, x2, y2, ucPixel );
#endif

    for( y=y1; y<=y2; y++ )
      (*fnLine)( x1, y, x2, y, ucPixel );
}


/***********************************************************************

  MulDiv

  Notes:
    Leaves its result in register AX.  (The MS C compiler generates
    a warning message anyway.)

***********************************************************************/

static int MulDiv( int n1, int n2, int n3 )
{
    __asm
    {
      mov   ax, n1
      mul   n2
      div   n3
    }
}
