/***********************************************************************

  TILE.C

***********************************************************************/

#include    <stdio.h>	    /* printf _lseek */
#include    <conio.h>	    /* _getch */
#include    <malloc.h>	    /* _fmalloc _ffree */
#include    <fcntl.h>	    /* _open */
#include    <sys\types.h>   /* _open */
#include    <sys\stat.h>    /* _open */
#include    <io.h>	    /* _open _close */
#include    <string.h>	    /* _fmemmove */
#include    <vs2.h>


/***********************************************************************

  TYPEDEFS etc.

***********************************************************************/

typedef struct
{
    USHORT  bfType;
    ULONG   bfSize;
    USHORT  bfReserved1;
    USHORT  bfReserved2;
    ULONG   bfOffBits;
}
  BITMAPFILEHEADER;
typedef BITMAPFILEHEADER __far * LPBITMAPFILEHEADER;

typedef struct
{
    ULONG   biSize;
    ULONG   biWidth;
    ULONG   biHeight;
    USHORT  biPlanes;
    USHORT  biBitCount;
    ULONG   biCompression;
    ULONG   biSizeImage;
    ULONG   biXPelsPerMeter;
    ULONG   biYPelsPerMeter;
    ULONG   biClrUsed;
    ULONG   biClrImportant;
}
  BITMAPINFOHEADER;
typedef BITMAPINFOHEADER __far * LPBITMAPINFOHEADER;


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	main( int, char * * );
void	Tile( LPBITMAPINFOHEADER );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

void (VS2API *	fnBltToVBuf)( USHORT, USHORT, LPVOID, USHORT );
void (VS2API *	fnBltInVBuf)( USHORT, USHORT, USHORT,
					    USHORT, USHORT, USHORT );
char		Src[1024];


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    int 		hBMP, cb;
    LPBITMAPFILEHEADER	lpbmfh;
    LPBITMAPINFOHEADER	lpbmih;
    long		lFileSize;
    LPCHAR		Dest;
    USHORT		usMode, x, y;


    if( argc != 2 )
    {
      printf( "Syntax: TILE <filename>\n" );
      return;
    }

    /* open the file */
    hBMP = _open( argv[1], _O_BINARY | _O_RDONLY );
    if( -1 == hBMP )
    {
      printf( "Unable to open file '%s'\n", argv[1] );
      return;
    }

    /* determine the size of the file */
    lFileSize = _lseek( hBMP, 0L, SEEK_END );
    _lseek( hBMP, 0L, SEEK_SET );

    if( lFileSize >= 65530 )
    {
      printf( "Sorry, file size exceeds 64 Kb.\n" );
      return;
    }

    /* allocate memory for the bitmap */
    lpbmfh = (LPBITMAPFILEHEADER)_fmalloc( (short)lFileSize );
    if( NULL == lpbmfh )
    {
      printf( "Unable to allocate %ld bytes for bitmap\n", lFileSize );
      return;
    }

    /* load the bitmap into memory */
    Dest = (LPCHAR)lpbmfh;
    for( ; ; )
    {
      cb = _read( hBMP, Src, 1024 );
      if( 0 == cb )
	break;

      _fmemmove( Dest, Src, cb );
      Dest += cb;
    }

    /* close the file */
    _close( hBMP );

    /* select the appropriate video mode */
    lpbmih = (LPBITMAPINFOHEADER)(lpbmfh+1);
    switch( lpbmih->biBitCount )
    {
      case 4:	    /* 16 colors */
	usMode = 0x12;
	fnBltToVBuf = BltToVBuf10;
	fnBltInVBuf = BltInVBuf10;
	break;

      case 8:	    /* 256 colors */
	SetVMode( 0x844 );		/* look for 360-by-400 mode */
	usMode = GetVMode();
	if( usMode == 0x844 )
	{
	  fnBltToVBuf = BltToVBuf256;
	  fnBltInVBuf = BltInVBuf256;
	}

	else				/* settle for 320-by-200 mode */
	{
	  usMode = 0x13;
	  fnBltToVBuf = BltToVBuf13;
	  fnBltInVBuf = BltInVBuf13;
	}
	break;

      default:
	printf( "Can't display %d bit-per-pixel bitmap\n",
					lpbmih->biBitCount );
	return;
    }

    /* center the bitmap */
    SetVMode( usMode );
    InitVSP( usMode );
    x = (vsp.usSX - (USHORT)lpbmih->biWidth) / 2;
    y = (vsp.usSY - (USHORT)lpbmih->biHeight) / 2;
    (*fnBltToVBuf)( x, y, lpbmih, TRUE );
    _getch();

    /* tile the bitmap */
    SetVMode( usMode );
    InitVSP( usMode );
    Tile( lpbmih );
    _getch();

    /* free the memory block */
    _ffree( lpbmfh );

    /* set 80-by-25 text mode */
    SetVMode( 3 );

    return;
}


/***********************************************************************

  Tile

***********************************************************************/

void Tile( LPBITMAPINFOHEADER lpbmih )
{
    USHORT  x, y, x0, y0;
    short   nx, ny, r, c;


    /* determine the number of copies that fit on the screen */
    nx = vsp.usSX / (USHORT)lpbmih->biWidth;
    ny = vsp.usSY / (USHORT)lpbmih->biHeight;

    /* determine the coordinates of the upper left tile */
    x0 = (vsp.usSX - nx*(USHORT)lpbmih->biWidth) / 2;
    y0 = (vsp.usSY - ny*(USHORT)lpbmih->biHeight) / 2;

    /* draw the upper left tile */
    (*fnBltToVBuf)( x0, y0, lpbmih, TRUE );

    /* draw the remaining tiles in the top row */
    x = x0;
    for( c=1; c<nx; c++ )
    {
      x += (USHORT)lpbmih->biWidth;
      (*fnBltToVBuf)( x, y0, lpbmih, FALSE );
    }

    /* copy the topmost row of tiles to the remaining rows */
    y = y0;
    for( r=1; r<ny; r++ )
    {
      y += (USHORT)lpbmih->biHeight;
      (*fnBltInVBuf)( x0, y, x0, y0,
			nx*(USHORT)lpbmih->biWidth,
			(USHORT)lpbmih->biHeight );
    }
}
