/***********************************************************************

  VBETEST.C

***********************************************************************/

#include    <dos.h>	    /* _int86 _FP_SEG _FP_OFF */
#include    <conio.h>	    /* _getch */
#include    <stdio.h>	    /* printf sscanf */
#include    <string.h>	    /* strlen */
#include    <vs2.h>

#define TMAX	6


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    main( int, char * * );
static void ShowText( char *, UCHAR );
static void Test0( void );
static void Test1( void );
static void Test2( void );
static void Test3( void );
static void Test4( void );
static void Test5( void );
static void Test6( void );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

union _REGS	regs;				/* used by _int86 */
struct _SREGS	sregs;				/* used by _int86x */

static USHORT	usMode;
typedef void (* TESTPROC)( void );
static TESTPROC Test[TMAX+1] = { Test0, Test1, Test2, Test3,
				 Test4, Test5, Test6 };


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    int     n;


    if( argc != 3 )
    {
      printf( "Syntax: VBETEST <mode #> <test #>\n" );
      printf( " 0: VS2.H data structures\n" );
      printf( " 1: VBE character output\n" );
      printf( " 2: VBE-supported pixel operations\n" );
      printf( " 3: VBE info functions\n" );
      printf( " 4: VBE set/get logical scan line length\n" );
      printf( " 5: VBE set/get display start\n" );
      printf( " 6: VBE-supported character string display\n" );
      return;
    }

    sscanf( argv[2], "%d", &n );
    if( (n < 0) || (n > TMAX) )
    {
      printf( "Specify a value between 0 and %d\n", TMAX );
      return;
    }

    /* select the specified 256-color mode */
    sscanf( argv[1], "%x", &usMode );
    SetVMode( usMode );

    /* do the specified test */
    (Test[n])();

    /* restore 80x25 text mode */
    regs.x.ax = 0x0003;
    _int86( 0x10, &regs, &regs );
}


/***********************************************************************

  ShowText

***********************************************************************/

static void ShowText( char * sz, UCHAR c )
{
    int     i;


    for( i=0; i<(int)strlen(sz); i++ )
    {
      regs.h.ah = 0x0E;
      regs.h.al = sz[i];
      regs.x.bx = c;
      _int86( 0x10, &regs, &regs );
    }
}


/***********************************************************************

  Test0 -- VS2.H data structures

***********************************************************************/

static void Test0()
{
    int     n;
    LPVDDA  vdda = ADDRVDDA;


    printf( "vdda at %08lX ...\n", vdda );
    printf( "CRTMode         = %02X\n", vdda->CRTMode );
    printf( "CRTCols         = %04X\n", vdda->CRTCols );
    printf( "CRTLen          = %04X\n", vdda->CRTLen );
    printf( "CRTStart        = %04X\n", vdda->CRTStart );
    for( n=0; n<8; n++ )
      printf( "CursorPosn[%d]  = %04X\n", n, vdda->CursorPosn[n] );
    printf( "CursorMode      = %04X\n", vdda->CursorMode );
    getch();

    printf( "ActivePage      = %02X\n", vdda->ActivePage );
    printf( "Addr6845        = %04X\n", vdda->Addr6845 );
    printf( "CRTModeSet      = %02X\n", vdda->CRTModeSet );
    printf( "CRTPalette      = %02X\n", vdda->CRTPalette );
    printf( "Rows            = %02X\n", vdda->Rows );
    printf( "Points          = %04X\n", vdda->Points );
    printf( "Info            = %02X\n", vdda->Info );
    printf( "Info3           = %02X\n", vdda->Info3 );
    printf( "Flags           = %02X\n", vdda->Flags );
    printf( "DCC             = %02X\n", vdda->DCC );
    printf( "lpSaveArea      = %08lX\n", vdda->lpSaveArea );
    getch();
}


/***********************************************************************

  Test1 -- Character output

***********************************************************************/

static void Test1()
{
    int     n;
    char    szBuf[80];


    for( n=0; n<25; n++ )
    {
      /* move to the start of the nth character row */
      regs.h.ah = 2;
      regs.h.dh = (UCHAR)n;
      regs.h.dl = 0;
      _int86( 0x10, &regs, &regs );

      /* display a string in the current row */
      sprintf( szBuf, "This is row %02d  This is row %02d", n, n );
      ShowText( szBuf, (UCHAR)((n%15)+1) );
      if( n < 24 )
	ShowText( "\r\n", 0 );
    }

    /* test M256X.EXE BIOS function 6 */
    regs.h.ah = 6;
    regs.h.al = 0;
    regs.h.bh = 4;
    regs.h.ch = 5;
    regs.h.cl = 2;
    regs.h.dh = 16;
    regs.h.dl = 20;
    _int86( 0x10, &regs, &regs );

    /* pause */
    _getch();

    /* test M256X function 0E */
    /* emit CR */
    regs.h.ah = 0x0E;
    regs.h.al = 0x0D;
    _int86( 0x10, &regs, &regs );

    /* emit some LFs */
    for( n=0; n<8; n++ )
    {
      /* does this screw things up somehow? */
      regs.x.ax = 0x0102;
      regs.x.cx = 0;
      _int86( 0x10, &regs, &regs );

      /* emit CR */
      regs.h.ah = 0x0E;
      regs.h.al = 0x0D;
      _int86( 0x10, &regs, &regs );

      /* emit LF */
      regs.h.ah = 0x0E;
      regs.x.bx = 0x000F;
      regs.h.al = 0x0A;
      _int86( 0x10, &regs, &regs );

      /* show new cursor position */
      regs.h.ah = 3;
      regs.x.bx = 0x000F;
      regs.h.bh = 0;
      _int86( 0x10, &regs, &regs );
      sprintf( szBuf, "%d,%d*", regs.h.dl, regs.h.dh );
      ShowText( szBuf, 0x0F );

      _getch();
    }
}


/***********************************************************************

  Test2 -- pixel operations

***********************************************************************/

static void Test2()
{
    LPVDDA  vdda = ADDRVDDA;
    USHORT  usCX, usCY;
    USHORT  x, y;
char szMsg[80];


    /* determine pixel dimensions */
    usCX = (USHORT)(8 * vdda->CRTCols);
    usCY = (USHORT)((vdda->Rows+1) * vdda->Points);

sprintf( szMsg, "usCX=%u usCY=%u\r\n", usCX, usCY );
ShowText( szMsg, 0x0E );


    for( x=0; x<usCX; x++ )
      for( y=0; y<usCY; y++ )
      {
	regs.h.ah = 0x0C;
	regs.h.al = (UCHAR)x;
	regs.h.bh = 0;
	regs.x.cx = x;
	regs.x.dx = y;
	_int86( 0x10, &regs, &regs );
      }

    /* pause */
    _getch();
}


/***********************************************************************

  Test3 -- VBE info functions

***********************************************************************/

static void Test3()
{
    VBEINFOBLOCK    vib;
    LPVOID	    lpVIB = (LPVOID)&vib;
    MODEINFOBLOCK   mib;
    LPVOID	    lpMIB = (LPVOID)&mib;
    int 	    n;


    printf( "\nFunction 4F00h ..." );
    regs.x.ax = 0x4F00;
    regs.x.bx = 0x000F;
    sregs.es = _FP_SEG(lpVIB);
    regs.x.di = _FP_OFF(lpVIB);
    _int86x( 0x10, &regs, &regs, &sregs );
    printf( " returns AX = %04X:\n", regs.x.ax );

    printf( "VbeSignature = '%4.4s'\n", vib.VbeSignature );
    printf( "VbeVersion = %04X\n", vib.VbeVersion );
    printf( "OemStringPtr -> '%Fs'\n", vib.OemStringPtr );
    printf( "Capabilities = %08lX\n", vib.Capabilities );

    for( n=0; n<100; n++ )
    {
      printf( "VideoModePtr[%d] = %04X\n", n, vib.VideoModePtr[n] );
      if( 0xFFFF == vib.VideoModePtr[n] )
	break;
    }

    printf( "TotalMemory = %u\n", vib.TotalMemory );

    _getch();

    printf( "\nFunction 4F01h ..." );
    regs.x.ax = 0x4F01;
    regs.x.cx = usMode;
    sregs.es = _FP_SEG(lpMIB);
    regs.x.di = _FP_OFF(lpMIB);
    _int86( 0x10, &regs, &regs );
    printf( " returns AX = %04X:\n", regs.x.ax );

    printf( "ModeAttributes = %04X\n", mib.ModeAttributes );
    printf( "WinAAttributes = %02X\n", mib.WinAAttributes );
    printf( "WinBAttributes = %02X\n", mib.WinBAttributes );
    printf( "WinGranularity = %04X\n", mib.WinGranularity );
    printf( "WinSize = %04X\n", mib.WinSize );
    printf( "WinASegment = %04X\n", mib.WinASegment );
    printf( "WinBSegment = %04X\n", mib.WinBSegment );
    printf( "WinFuncPtr = %08lX\n", mib.WinFuncPtr );
    printf( "BytesPerScanLine = %u\n", mib.BytesPerScanLine );
    printf( "XResolution = %u\n", mib.XResolution );
    printf( "YResolution = %u\n", mib.YResolution );
    printf( "XCharSize = %02X\n", mib.XCharSize );
    printf( "YCharSize = %02X\n", mib.YCharSize );
    printf( "NumberOfPlanes = %02X\n", mib.NumberOfPlanes );
    printf( "BitsPerPixel = %02X\n", mib.BitsPerPixel );
    printf( "NumberOfBanks = %02X\n", mib.NumberOfBanks );
    printf( "MemoryModel = %02X\n", mib.MemoryModel );
    printf( "BankSize = %02X\n", mib.BankSize );
    printf( "NumberOfImagePages = %02X\n", mib.NumberOfImagePages );
    printf( "RedMaskSize = %02X\n", mib.RedMaskSize );
    printf( "RedFieldPosition = %02X\n", mib.RedFieldPosition );
    printf( "GreenMaskSize = %02X\n", mib.GreenMaskSize );
    printf( "GreenFieldPosition = %02X\n", mib.GreenFieldPosition );
    printf( "BlueMaskSize = %02X\n", mib.BlueMaskSize );
    printf( "BlueFieldPosition = %02X\n", mib.BlueFieldPosition );
    printf( "RsvdMaskSize = %02X\n", mib.RsvdMaskSize );
    printf( "RsvdFieldPosition = %02X\n", mib.RsvdFieldPosition );
    printf( "DirectColorModeInfo = %02X\n", mib.DirectColorModeInfo );

    _getch();

    printf( "\nFunction 4F03h ..." );
    regs.x.ax = 0x4F03;
    _int86( 0x10, &regs, &regs );
    printf( " returns AX = %04X:\n", regs.x.ax );

    printf( "BX = %04X\n", regs.x.bx );

    _getch();
}


/***********************************************************************

  Test4 -- VBE set/get logical scan line length

***********************************************************************/

static void Test4( void )
{
    int     x, y;


    printf( "Function 4F06h, BL = 1 ...\n" );
    regs.x.ax = 0x4F06;
    regs.h.bl = 0x01;
    _int86( 0x10, &regs, &regs );
    printf( " returns AX = %04X:\n", regs.x.ax );
    printf( "BX = %u  CX = %u  DX = %u\n", regs.x.bx, regs.x.cx, regs.x.dx );
    getch();

    regs.x.ax = 0x4F06;
    regs.h.bl = 0x00;
    regs.x.cx = 500;	    /* logical pixel width (usCX) */
    _int86( 0x10, &regs, &regs );
    printf( "Function 4F06h, BL = 0 ...\n" );
    printf( " returns AX = %04X:\n", regs.x.ax );
    printf( "BX = %u  CX = %u  DX = %u\n", regs.x.bx, regs.x.cx, regs.x.dx );
    getch();

    /* test M256X.EXE BIOS function 6 */
    regs.h.ah = 6;
    regs.h.al = 0;

    for( x=24; x>=1; x-- )
    {
      regs.h.bh = (UCHAR)((x%15)+1);
      regs.h.ch = 0;
      regs.h.cl = 0;
      regs.h.dh = (UCHAR)x;
      regs.h.dl = (UCHAR)x;
      _int86( 0x10, &regs, &regs );
    }
    getch();

    /* test VBE function 0Ch */
    for( x=0; x<320; x++ )
      for( y=0; y<240; y++ )
      {
	regs.h.ah = 0x0C;
	regs.h.al = (UCHAR)((x%250)+1);
	regs.x.cx = x;
	regs.x.dx = y;
	_int86( 0x10, &regs, &regs );
      }
    getch();

    regs.x.ax = 0x4F06;
    regs.h.bl = 0x02;
    regs.x.cx = 90;	/* logical width in bytes */
    _int86( 0x10, &regs, &regs );
    printf( "Function 4F06h, BL = 2 ...\n" );
    printf( " returns AX = %04X:\n", regs.x.ax );
    printf( "BX = %u  CX = %u  DX = %u\n", regs.x.bx, regs.x.cx, regs.x.dx );
    getch();

    /* test error return for row size too large for video buffer */
    regs.x.ax = 0x4F06;
    regs.h.bl = 0x00;
    regs.x.cx = 32767;
    _int86( 0x10, &regs, &regs );
    printf( "Function 4F06h, BL = 0, CX=32767 ...\n" );
    printf( " returns AX = %04X:\n", regs.x.ax );
    printf( "BX = %u  CX = %u  DX = %u\n", regs.x.bx, regs.x.cx, regs.x.dx );
    _getch();
}


/***********************************************************************

  Test5 -- set/get display start

***********************************************************************/

static void Test5()
{
    char    szBuf[80];
    USHORT  us;
    int     n;


    for( n=0; n<25; n++ )
    {
      sprintf( szBuf, "This is row %2d\r\n", n );
      ShowText( szBuf, (UCHAR)((n%15)+1) );
    }

    for( us=0; us<100; us++ )
    {
      regs.x.ax = 0x4F07;
      regs.x.bx = 0x0000;
      regs.x.cx = us;
      regs.x.dx = us;
      _int86( 0x10, &regs, &regs );
      getch();
    }
}


/***********************************************************************

  Test6 -- display character string

***********************************************************************/

static void Test6()
{
    static char szBuf[80];
    int 	f, n, nLen;


    /* subfunctions 0 and 1 */
    for( f=0x1300; f<=0x1301; f++ )
    {
      SetCursorXY( 20, 0 );
      printf( "Function %04X", f );
      SetCursorXY( 0, 0 );	/* leave the cursor at 0,0 */

      for( n=0; n<16; n++ )
      {
	sprintf( szBuf, "This is row %d", n );
	nLen = strlen( szBuf );

	__asm
	{
	  push	bp

	  mov	ax, f		; AH = 13h, AL = subfunction
	  mov	bx, n		; BH = 0, BL = attribute
	  mov	cx, nLen
	  mov	dx, n
	  mov	dh, dl		; DH = n, DL = n
	  mov	bp, ds
	  mov	es, bp
	  mov	bp, offset szBuf
	  int	10h

	  pop	bp
	}
      }

      /* print an asterisk at the current cursor position */
      printf( "*" );
      _getch();
    }

    nLen = 24;
    for( n=0; n<(nLen*2); n+=2 )
    {
      szBuf[n] = (char)('A' + n/2);   /* character */
      szBuf[n+1] = (char)(0x20+n);  /* attribute */
    }

    /* subfunctions 2 and 3 */
    for( f=0x1302; f<=0x1303; f++ )
    {
      SetCursorXY( 20, 0 );
      printf( "Function %04X", f );
      SetCursorXY( 0, 0 );	/* leave the cursor at 0,0 */

      for( n=0; n<16; n++ )
      {
	__asm
	{
	  push	bp

	  mov	ax, f		; AH = 13h, AL = subfunction
	  mov	cx, nLen
	  mov	dx, n
	  mov	dh, dl		; DH = n, DL = n
	  mov	bp, ds
	  mov	es, bp
	  mov	bp, offset szBuf
	  int	10h

	  pop	bp
	}
      }

      /* print an asterisk at the current cursor position */
      printf( "*" );
      _getch();
    }
}
