/***********************************************************************

  VBUFTEST.C

***********************************************************************/

#include    <stdio.h>	    /* sprintf */
#include    <conio.h>	    /* _kbhit _getch */
#include    <stdlib.h>	    /* rand */
#include    <vs2.h>

#define TMAX	3


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

typedef void (* TESTPROC)( void );

void	    main( int, char * * );
void	    Test0( void );
void	    Test1( void );
void	    Test2( void );
void	    Test3( void );
void	    InitVMode( USHORT );
void	    SolidRectangle( USHORT, USHORT, USHORT, USHORT, UCHAR );
static int  MulDiv( int, int, int );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

static TESTPROC Test[TMAX+1] = { Test0, Test1, Test2, Test3 };
void (VS2API * fnLine)( USHORT, USHORT, USHORT, USHORT, UCHAR );


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    USHORT  usMode;
    int     n;


    /* verify command-line syntax */
    if( argc == 1 )
    {
      printf( "Syntax: VBUFTEST <test #> [mode #]\n" );
      return;
    }

    /* set the specified video mode */
    if( argc == 3 )
    {
      sscanf( argv[2], "%x", &usMode );
      SetVMode( usMode );
    }

    /* do the requested test */
    sscanf( argv[1], "%d", &n );
    if( (n < 0) || (n > TMAX) )
    {
      printf( "Specify a value between 0 and %d\n", TMAX );
      return;
    }

    (Test[n])();

    /* set 80-by-25 alphanumeric mode */
    SetVMode( 0x03 );
}


/***********************************************************************

  Test0

***********************************************************************/

void Test0()
{
    USHORT  usMode, usOfs;
    LPCHAR  lpPage1;


    usMode = GetVMode();
    if( (usMode <= 3) || (usMode == 7) )
    {
      usOfs = 0x00A0;				/* 160 */
      lpPage1 = (LPCHAR)(0xB80000A0L);
    }

    else
    {
      usOfs = 0x8000;				/* 32K */
      lpPage1 = (LPCHAR)(0xA0008000L);
    }

    /* store some data at the start of the page */
    lpPage1[0] = 0x55;
    lpPage1[1] = 0x17;

    /* put the start of the page in the upper left corner */
    SetScreenStart( usOfs );
    _getch();
    SetScreenStart( 0 );
}


/***********************************************************************

  Test1

***********************************************************************/

void Test1()
{
    USHORT  usRVal;


    /* 9-dot alphanumeric mode, character coordinates */
    SetVMode( 3 );
    InitVSP( 3 );
    usRVal = SetVBufDims( 81, 40, TRUE );
    printf( "Mode 0x03:  " );
    printf( "SetVBufDims( 81, 40, TRUE ) ...\n" );
    printf( "cbRow=%d usCX=%d usCY=%d size=%04X (%u)\n",
		 vsp.cbRow, vsp.usCX, vsp.usCY, usRVal, usRVal );
    _getch();

    /* 9-dot alphanumeric mode, pixel coordinates */
    SetVMode( 3 );
    InitVSP( 3 );
    usRVal = SetVBufDims( 721, 400, FALSE );
    printf( "Mode 0x03:  " );
    printf( "SetVBufDims( 721, 400, FALSE ) ...\n" );
    printf( "cbRow=%d usCX=%d usCY=%d size=%04X (%u)\n",
		 vsp.cbRow, vsp.usCX, vsp.usCY, usRVal, usRVal );
    _getch();

    /* 8-dot graphics mode, pixel coordinates */
    SetVMode( 0x12 );
    InitVSP( 0x12 );
    usRVal = SetVBufDims( 641, 481, FALSE );
    printf( "Mode 0x12:  " );
    printf( "SetVBufDims( 641, 481, FALSE ) ...\n" );
    printf( "cbRow=%d usCX=%d usCY=%d size=%04X (%u)\n",
		 vsp.cbRow, vsp.usCX, vsp.usCY, usRVal, usRVal );
    Line10( 0, 0, 100, 100, 0x0E );
    Line10( 0, 0, (USHORT)(vsp.usCX-1), (USHORT)(vsp.usCY-1), 0x01 );
    _getch();

    /* 8-dot graphics mode, character coordinates */
    SetVMode( 0x12 );
    InitVSP( 0x12 );
    usRVal = SetVBufDims( 81, 48, TRUE );
    printf( "Mode 0x12:  " );
    printf( "SetVBufDims( 81, 48, TRUE ) ...\n" );
    printf( "cbRow=%d usCX=%d usCY=%d size=%04X (%u)\n",
		 vsp.cbRow, vsp.usCX, vsp.usCY, usRVal, usRVal );
    Line10( 0, 0, 100, 100, 0x0E );
    Line10( 0, 0, (USHORT)(vsp.usCX-1), (USHORT)(vsp.usCY-1), 0x01 );
    _getch();

    /* 8-dot double-scan graphics, pixel coordinates */
    SetVMode( 0x13 );
    InitVSP( 0x13 );
    usRVal = SetVBufDims( 321, (USHORT)-1, FALSE );
    printf( "Mode 0x13:  " );
    printf( "SetVBufDims( 321, -1, FALSE ) ...\n" );
    printf( "cbRow=%d usCX=%d usCY=%d size=%04X (%u)\n",
		 vsp.cbRow, vsp.usCX, vsp.usCY, usRVal, usRVal );
    Line13( 0, 0, 100, 100, 0x0E );
    Line13( 0, 0, (USHORT)(vsp.usCX-1), (USHORT)(vsp.usCY-1), 0x01 );
    _getch();

    /* 8-dot double-scan graphics, character coordinates */
    SetVMode( 0x13 );
    InitVSP( 0x13 );
    usRVal = SetVBufDims( 41, 25, TRUE );
    printf( "Mode 0x13:  " );
    printf( "SetVBufDims( 41, 25, TRUE ) ...\n" );
    printf( "cbRow=%d usCX=%d usCY=%d size=%04X (%u)\n",
		 vsp.cbRow, vsp.usCX, vsp.usCY, usRVal, usRVal );
    Line13( 0, 0, 100, 100, 0x0E );
    Line13( 0, 0, (USHORT)(vsp.usCX-1), (USHORT)(vsp.usCY-1), 0x01 );
    _getch();

    /* 8-dot double-scan graphics, pixel coordinates */
    SetVMode( 0x845 );
    InitVSP( 0x845 );
    usRVal = SetVBufDims( 367, 481, FALSE );
    __asm
    {
      mov ax, 4F06h	    ; VBE Set Logical Scan Line Length
      mov bl, 0
      mov cx, 367
      int 10h
    }

    printf( "Mode 0x845:  " );
    printf( "SetVBufDims( 367, 481, FALSE ) ...\n" );
    printf( "cbRow=%d usCX=%d usCY=%d size=%04X (%u)\n",
		 vsp.cbRow, vsp.usCX, vsp.usCY, usRVal, usRVal );
    Line256( 0, 0, 100, 100, 0x0E );
    Line256( 0, 0, (USHORT)(vsp.usCX-1), (USHORT)(vsp.usCY-1), 0x01 );
    _getch();

    /* 8-dot double-scan graphics, character coordinates */
    SetVMode( 0x842 );
    InitVSP( 0x842 );
    __asm
    {
      mov ax, 4F06h	    ; VBE Set Logical Scan Line Length
      mov bl, 0
      mov cx, 568	    ; 8*71
      int 10h
    }

    usRVal = SetVBufDims( 71, 25, TRUE );
    printf( "Mode 0x842:  " );
    printf( "SetVBufDims( 71, 25, TRUE ) ...\n" );
    printf( "cbRow=%d usCX=%d usCY=%d size=%04X (%u)\n",
		 vsp.cbRow, vsp.usCX, vsp.usCY, usRVal, usRVal );
    Line256( 0, 0, 100, 100, 0x0E );
    Line256( 0, 0, (USHORT)(vsp.usCX-1), (USHORT)(vsp.usCY-1), 0x01 );
    _getch();
}


/***********************************************************************

  Test2

***********************************************************************/

void Test2()
{
    USHORT  y;


    InitVSP( GetVMode() );

    printf( "About to split at y = -1 ..." );
    _getch();
    SetScreenSplit( 0xFFFF, 0, FALSE );
    printf( " done.\n" );
    _getch();

    for( y=0; y<vsp.usSY; y++ )
    {
      SetScreenSplit( y, vsp.cbRow, FALSE );
      printf( "y=%4u ofs=%04X\r", y, vsp.cbRow );
      _getch();
    }
}


/***********************************************************************

  Test3

***********************************************************************/

void Test3()
{
    USHORT  usX1, usY1, usX2, usY2;
    short   OrgX = 0, OrgY = 0, MaxX, MaxY;
    UCHAR   ucPixel;
    int     nColors, bDone = FALSE;


    InitVMode( GetVMode() );
    SetScreenOrg( OrgX, OrgY, FALSE );

    nColors = (1 << vsp.cBPP);
    MaxX = vsp.usCX - vsp.usSX;
    MaxY = vsp.usCY - vsp.usSY;

    do
    {
      if( _kbhit() )
      {
	switch( _getch() )
	{
	  case 0x1B:		    /* Esc */
	    bDone = TRUE;
	    break;

	  case 0x00:		    /* arrow or function key */
	    switch( _getch() )
	    {
	      case 0x47:	    /* Home */
		OrgX = 0;
		OrgY = 0;
		break;

	      case 0x48:	    /* Up */
		if( OrgY )
		  --OrgY;
		break;

	      case 0x4B:	    /* Left */
		if( OrgX )
		  --OrgX;
		break;

	      case 0x4D:	    /* Right */
		if( OrgX < MaxX )
		  ++OrgX;
		break;

	      case 0x50:	    /* Down */
		if( OrgY < MaxY )
		  ++OrgY;
		break;

	      default:
		break;
	    }

	    VNWait();
	    SetScreenOrg( OrgX, OrgY, FALSE );
	    break;

	  default:
	    break;
	}
      }

      else
      {
	/* pick some random coordinates */
	usX1 = (USHORT)MulDiv( rand(), vsp.usCX-1, RAND_MAX );
	usX2 = (USHORT)MulDiv( rand(), vsp.usCX-1, RAND_MAX );
	usY1 = (USHORT)MulDiv( rand(), vsp.usCY-1, RAND_MAX );
	usY2 = (USHORT)MulDiv( rand(), vsp.usCY-1, RAND_MAX );
	ucPixel = (UCHAR)MulDiv( rand(), nColors, RAND_MAX );

	/* call with the y-coordinates in ascending order */
	if( usY1 < usY2 )
	  SolidRectangle( usX1, usY1, usX2, usY2, ucPixel );
	else
	  SolidRectangle( usX2, usY2, usX1, usY1, ucPixel );
      }
    }
    while( !bDone );

    /* restore the default screen dimensions */
    SetVBufDims( vsp.usSX, 0xFFFF, FALSE );
    SetScreenOrg( 0, 0, FALSE );
}


/***********************************************************************

  InitVMode

***********************************************************************/

void InitVMode( USHORT usMode )
{
    InitVSP( usMode );

    /* select the appropriate line routine */
    switch( usMode )
    {
      case 0x0D:
      case 0x0E:
      case 0x0F:
      case 0x10:
      case 0x11:
      case 0x12:
	fnLine = Line10;
	SetVBufDims( 3*vsp.usSX/2, 0xFFFF, FALSE );
	break;

      case 0x840:
      case 0x841:
      case 0x842:
      case 0x843:
      case 0x844:
      case 0x845:
	fnLine = Line256;
	SetVBufDims( 4*vsp.usSX/3, 0xFFFF, FALSE );
	break;

      default:
	printf( "*** unsupported video mode ***" );
	exit( 3 );
	return;
    }
}


/***********************************************************************

  MulDiv

  Notes:
    Leaves its result in register AX.  (The MS C compiler generates
    a warning message anyway.)

***********************************************************************/

static int MulDiv( int n1, int n2, int n3 )
{
    __asm
    {
      mov   ax, n1
      mul   n2
      div   n3
    }
}


/***********************************************************************

  SolidRectangle

***********************************************************************/

void SolidRectangle( USHORT x1, USHORT y1,
			USHORT x2, USHORT y2, UCHAR ucPixel )
{
    USHORT   y;


    for( y=y1; y<=y2; y++ )
      (*fnLine)( x1, y, x2, y, ucPixel );
}
