/***********************************************************************

  VNTTEST.C

***********************************************************************/

#include    <stdio.h>	    /* sprintf */
#include    <conio.h>	    /* _getch _kbhit */
#include    <math.h>	    /* sqrt */
#include    <string.h>	    /* strlen */
#include    <vs2.h>


/***********************************************************************

  FUNCTION PROTOTYPES

***********************************************************************/

void	    main( int, char * * );
void VS2API CountVNT( void );
void VS2API RotateColors( void );
static void ShowColors( void );


/***********************************************************************

  GLOBAL VARIABLES

***********************************************************************/

extern VIDEOSTATEPARAMS vsp;

static ULONG	ulVNI = 0;
static USHORT	usLoops;
static UCHAR	RGBTable[256][3];
static int	bWait;
static char *	szCaption[] = { " DAC animation by synchronizing ",
				" the system timer with ",
				" the vertical nondisplay interval "
			      };


/***********************************************************************

  main

***********************************************************************/

void main( int argc, char * argv[] )
{
    char    Buf[48];


    /* verify command-line syntax */
    if( argc > 2 )
    {
      printf( "Syntax: VNTTEST [wait]\n" );
      return;
    }

    /* if "wait" (or anything else) is specified, set a flag */
    bWait = (argc == 2);

    /* hide the cursor */
    SetCursorSize( 0x20, 0 );

    /* initialize global variables for the current video mode */
    InitVSP( GetVMode() );

    /* count vertical nondisplay intervals */
    EnableVNTimer( CountVNT );
    while( !_kbhit() )		/* do until a key is pressed */
    {
      sprintf( Buf, "interrupt count: %ld  VN->VR loops: %04X",
						ulVNI, usLoops );
      TextAt( 0, 0x8000, Buf, 0x0E );
    }

    _getch();			/* discard the keypress */
    DisableVNTimer();
    SetCursorSize( 6, 7 );	/* restore the cursor */

    /* "animate" the DAC register values */
    ShowColors();
    EnableVNTimer( RotateColors );
    _getch();
    DisableVNTimer();

    /* restore 80-by-25 alphanumeric mode */
    SetVMode( 3 );
}


/***********************************************************************

  CountVNT

  Notes:
    This function is called by the VNI handler (ISR0A).

***********************************************************************/

void VS2API CountVNT()
{
    /* increment a count of vertical nondisplay intervals */
    ulVNI++ ;

    /* this interrupt occurred at the start of the vertical */
    /*	nondisplay interval, so the following loop will execute until */
    /*	vertical retrace starts */
    __asm
    {
      mov   cx, 0FFFFh	    ; loop counter
      mov   dx, 3DAh	    ; Input Status reg 1

L01:  in    al, dx	    ; wait for vertical retrace
      test  al, 8
      loopz L01

      neg   cx		    ; save the number of loops
      dec   cx
      mov   usLoops, cx
    }
}


/***********************************************************************

  RotateColors

  Notes:
    This function is called by the VNI handler (ISR0A).

***********************************************************************/

void VS2API RotateColors()
{
    /* rotate the colors in the RGB table */
    RotateDACTable( RGBTable, 46, 1 );

    /* wait for vertical retrace */
    if( bWait )
      VRWait();

    /* update the DAC */
    SetDACRegs( RGBTable, 16, 46 );
}


/***********************************************************************

  ShowColors

***********************************************************************/

static void ShowColors()
{
    USHORT  x, y;
    UCHAR   ucPixel;


    /* select 320-by-200 256-color mode */
    SetVMode( 0x13 );
    InitVSP( 0x13 );

    /* create an RGB table */
    GetDACRegs( RGBTable, 32, 24 );	    /* 24 bright hues */
    for( x=24; x<47; x++ )		    /* same 24 in reverse */
      GetDACRegs( RGBTable[x], 79-x, 1 );

    /* update the DAC */
    SetDACRegs( RGBTable, 16, 46 );

    /* draw some colors */
    for( x=0; x<vsp.usSX; x++ )
      for( y=0; y<vsp.usSY; y++ )
      {
	ucPixel =
	  (UCHAR)( sqrt( (x-vsp.usSX/2)*(x-vsp.usSX/2) +
				    (y-vsp.usSY/2)*(y-vsp.usSY/2) ) );

	ucPixel = (UCHAR)( (ucPixel % 46) + 16 );
	SetPixel13( x, y, ucPixel );
      }

    /* add a caption */
    for( y=0; y<3; y++ )
    {
      x = ((vsp.usSX/8)-strlen(szCaption[y])) / 2;
      TextAt( x, 19+y, szCaption[y], 0x0F );
    }
}
